@@ -39,7 +39,9 @@
 
 using namespace std;
 
-enum { ALL_RW = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH };
+bool debug_log = false;;
+
+        enum { ALL_RW = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH };
 
 struct SVAR_context
 {
@@ -235,13 +237,13 @@ const CDR_string & cdr = *var_D.data;
       {
         const int len = cdr.header().get_nb();
         const uint8_t * data = cdr.get_items();
-        const size_t written = fwrite(data, len, 1, file);
+        const int written = fwrite(data, len, 1, file);
         if (written == len)   return -48;   // partial write
 
         ++filepos;
         return 0;
       }
-   else if (code == 'D')   // char (var size
+   else if (code == 'D')   // char (var size)
       {
       }
    else if (code == 'T')   // translate
@@ -395,8 +397,16 @@ open_pipe(const char * filename, const char * mode)
 struct stat st;
    if (stat(filename, &st) == 0)   // file exists
       {
-        if (S_ISFIFO(st.st_mode))   return fopen(filename, mode);
-        return 0;   // file exists but is not a pipe
+        FILE * pipe = 0;
+        if (S_ISFIFO(st.st_mode))
+           {
+             pipe = fopen(filename, mode);
+             if (pipe == 0 && debug_log)
+                get_CERR() << "Could not open pipe " << filename << ": "
+                           << strerror(errno) << endl;
+           }
+
+        return pipe;   // file exists but is not a pipe
       }
 
    // stat failed: create pipe
@@ -455,7 +465,8 @@ SVAR_context * ctx = new SVAR_context(f, code, var_C, var_D, true);
 }
 //-----------------------------------------------------------------------------
 void
-read_file(const char * filename, int code, Coupled_var & var_C, Coupled_var & var_D)
+read_file(const char * filename, int code, Coupled_var & var_C,
+          Coupled_var & var_D)
 {
    if (code == 0)   code = 'A';    // default if no code is provided
    if (!strchr("ABCDT", code))
@@ -467,6 +478,10 @@ read_file(const char * filename, int code, Coupled_var & var_C, Coupled_var & va
 FILE * f = fopen(filename, "r");
    if (f == 0)
       {
+        if (debug_log)
+           get_CERR() << "Could not open file " << filename << " for reading: "
+                      << strerror(errno) << endl;
+
        set_ACK(var_C, 2);          //  2 := FILE NOT FOUND
        return;
       }
@@ -493,6 +508,10 @@ write_file(const char * filename, int code, Coupled_var & var_C, Coupled_var & v
 FILE * f = fopen(filename, "w");
    if (f == 0)
       {
+        if (debug_log)
+           get_CERR() << "Could not open file " << filename << " for writing: "
+                      << strerror(errno) << endl;
+
        set_ACK(var_C, 2);          //  2 := FILE NOT FOUND
        return;
       }
@@ -594,7 +613,7 @@ bool
 initialize(Coupled_var & var)
 {
    coupled_vars.push_back(var);
-const uint32_t * varname = Svar_DB::get_varname(var.key);
+const uint32_t * varname = Svar_DB::get_svar_name(var.key);
    if (!varname)   return true;
 
    if (*varname == 'C')
@@ -627,7 +646,7 @@ const CDR_header * header = (const CDR_header *)data.c_str();
 
 const int rank = header->rank;
 
-const uint32_t * varname = Svar_DB::get_varname(var.key);
+const uint32_t * varname = Svar_DB::get_svar_name(var.key);
    if (!varname)   return E_VALUE_ERROR;
 
    if (*varname == 'D')   // e.g. DAT←value
@@ -645,7 +664,7 @@ const uint32_t * varname = Svar_DB::get_varname(var.key);
 SV_key key_D = Svar_DB::find_pairing_key(var.key);
    if (key_D == 0)   { error_loc = LOC;   return E_VALUE_ERROR; }
 Coupled_var * var_D = 0;
-   for (int c = 0; c < coupled_vars.size(); ++c)
+   for (size_t c = 0; c < coupled_vars.size(); ++c)
        {
          if (coupled_vars[c].key == key_D)
             {
@@ -653,11 +672,16 @@ Coupled_var * var_D = 0;
               break;
             }
        }
+
    if (var_D == 0)
       {
-        get_CERR() << "key_D = " << (key_D & 0xFFFF) << endl;
-        for (int c = 0; c < coupled_vars.size(); ++c)
-            get_CERR() << "key = " << (coupled_vars[c].key & 0xFFFF) << endl;
+        get_CERR() << "Could not find matching data variable for key "
+                   << var.key << " (svar " << varname
+                   << ")" << endl
+                   << "unknown key_D = " << key_D << endl
+                   << "known keys:" << endl;
+        for (size_t c = 0; c < coupled_vars.size(); ++c)
+            get_CERR() << " key[" << c << "] = " << coupled_vars[c].key << endl;
 
         error_loc = LOC;   return E_VALUE_ERROR;
       }
@@ -679,8 +703,8 @@ SVAR_context * ctx = var.context;
 
         if (ctx->state_D != SVS_NOT_SHARED)   // var D read or written
            {
-             if (!Svar_DB::get_varname(key_D)) { error_loc = LOC;
-                                                 return E_VALUE_ERROR; }
+             if (!Svar_DB::get_svar_name(key_D)) { error_loc = LOC;
+                                                   return E_VALUE_ERROR; }
              Svar_DB::set_state(key_D, ctx->state_D == SVS_IDLE, LOC);
            }
       }
@@ -688,7 +712,7 @@ SVAR_context * ctx = var.context;
       {
         const char * cmd = (const char *)(header + 1) + 4*rank;
         handle_cmd(cmd, data.size(), var, *var_D);
-        if (!Svar_DB::get_varname(key_D)) { error_loc = LOC;
+        if (!Svar_DB::get_svar_name(key_D)) { error_loc = LOC;
                                             return E_VALUE_ERROR; }
    Svar_DB::set_state(var.key, false, LOC);
    Svar_DB::set_state(key_D, false, LOC);
