@@ -92,7 +92,7 @@ uint64_t ret = current.tv_sec;
 bool
 add_var(SV_key key)
 {
-   for (int c = 0; c < coupled_vars.size(); ++c)
+   for (size_t c = 0; c < coupled_vars.size(); ++c)
        {
          Coupled_var & cv = coupled_vars[c];
          if (key == cv.key)   return true;   // key already exists
@@ -110,11 +110,11 @@ Coupled_var cv = { key, 0, 0 };
 void
 print_vars(ostream & out)
 {
-   for (int c = 0; c < coupled_vars.size(); ++c)
+   for (size_t c = 0; c < coupled_vars.size(); ++c)
        {
          Coupled_var & cv = coupled_vars[c];
          get_CERR() << "   key: 0x" << hex << cv.key << dec << " ";
-         const uint32_t * varname = Svar_DB::get_varname(cv.key);
+         const uint32_t * varname = Svar_DB::get_svar_name(cv.key);
          if (varname)
             {
               while (*varname)   get_CERR() << (Unicode)(*varname++);
@@ -143,8 +143,6 @@ AP_num3 this_proc = ProcessorID::get_id();
    if (verbose)   get_CERR() << pref << " unregistering processor "
                        << this_proc << endl;
 
-const TCP_socket tcp = Svar_DB::get_DB_tcp();
-
    if (verbose)   get_CERR() << pref << " done (got ^C)" << endl;
 
    exit(0);
@@ -197,7 +195,8 @@ char * slash = strrchr(bin_path, '/');
         *slash = 0;
       }
 
-   Svar_DB::init(bin_path, prog, false, true);
+   Svar_DB::init(bin_path, prog, /* retry_max */ 10, /* logit */ false,
+                 /* do_svars */  true);
 
    if (strrchr(prog, '/'))   prog = strrchr(prog, '/') + 1;
    snprintf(pref, sizeof(pref) - 1, "%s(%u) ", prog, getpid());
@@ -254,7 +253,8 @@ Svar_partner this_proc(ProcessorID::get_id(), NO_TCP_SOCKET);
 const TCP_socket tcp = Svar_DB::get_DB_tcp();
 
    if (ProcessorID::get_id().proc < AP_FIRST_USER)
-      tcp2 = Svar_DB::connect_to_APserver(0, prog_name(), verbose);
+      tcp2 = Svar_DB::connect_to_APserver(0, prog_name(), /* retry_max */ 10,
+                                          verbose);
 
    if (tcp == NO_TCP_SOCKET || tcp2 == NO_TCP_SOCKET)
       {
@@ -262,8 +262,6 @@ const TCP_socket tcp = Svar_DB::get_DB_tcp();
         return 3;
       }
 
-const int nfds = 1 + tcp2;
-
 string progname(prog_name());
 
       { REGISTER_PROCESSOR_c request(tcp, this_proc.id.proc,
@@ -309,7 +307,7 @@ cerr << "APnnn got " << signal->get_sigName() << endl;
                                            << " got MAKE_OFFER" << endl;
                  {
                    const SV_key key = signal->get__MAKE_OFFER__key();
-                   const uint32_t * varname = Svar_DB::get_varname(key);
+                   const uint32_t * varname = Svar_DB::get_svar_name(key);
                   if (varname == 0)
                      {
                        get_CERR() << "Could not find svar for key "
@@ -344,7 +342,7 @@ cerr << "APnnn got " << signal->get_sigName() << endl;
                                        << " got RETRACT_OFFER" << endl;
                  {
                    const SV_key key = signal->get__RETRACT_OFFER__key();
-                   for (int c = 0; c < coupled_vars.size(); ++c)
+                   for (size_t c = 0; c < coupled_vars.size(); ++c)
                        {
                          Coupled_var & cv = coupled_vars[c];
                          if (key == cv.key)
@@ -374,7 +372,7 @@ cerr << "APnnn got " << signal->get_sigName() << endl;
                    APL_error_code error = E_VALUE_ERROR;
                    bool found = false;
                    string data;
-                   for (int c = 0; c < coupled_vars.size(); ++c)
+                   for (size_t c = 0; c < coupled_vars.size(); ++c)
                        {
                          Coupled_var & cv = coupled_vars[c];
                          if (key == cv.key)
@@ -405,7 +403,7 @@ cerr << "APnnn got " << signal->get_sigName() << endl;
                    const SV_key key = signal->get__ASSIGN_VALUE__key();
                    APL_error_code error = E_VALUE_ERROR;
                    bool found = false;
-                   for (int c = 0; c < coupled_vars.size(); ++c)
+                   for (size_t c = 0; c < coupled_vars.size(); ++c)
                        {
                          Coupled_var & cv = coupled_vars[c];
                          if (key == cv.key)
