@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include "IndexExpr.hh"
 #include "IntCell.hh"
 #include "LvalCell.hh"
+#include "Macro.hh"
 #include "NativeFunction.hh"
 #include "Output.hh"
 #include "PointerCell.hh"
@@ -52,8 +53,8 @@ using namespace std;
 
 /// if there is less than x chars left on the current line then leave
 /// char mode and start a new line indented.
-#define NEED(x)   if (space < (x))   { leave_char_mode();   out << "\n";   \
-                                       space = do_indent(); } out
+#define NEED(x)   if (space < (int)(x))   { leave_char_mode();   out << "\n"; \
+                                            space = do_indent(); } out
 
 //-----------------------------------------------------------------------------
 bool
@@ -152,29 +153,20 @@ int space = do_indent();
 XML_Saving_Archive &
 XML_Saving_Archive::save_shape(const Value & v)
 {
-char cc[80];
-
    do_indent();
-   snprintf(cc, sizeof(cc), "<Value flg=\"%X\" vid=\"%d\"",
-                 v.get_flags(), vid);
-   out << cc;
+   out << "<Value flg=\"" << HEX(v.get_flags()) << "\" vid=\"" << vid << "\"";
 
-const unsigned int sub_vid = find_vid(v);
-   Assert(sub_vid < values.size());
+const int sub_vid = find_vid(v);
+   Assert(sub_vid < (int)values.size());
 unsigned int parent_vid = -1;
 
-   if (&values[sub_vid]._par)   parent_vid = find_vid(values[sub_vid]._par);
+   if (values[sub_vid]._par)   parent_vid = find_vid(*values[sub_vid]._par);
 
-   snprintf(cc, sizeof(cc), " parent=\"%d\"", parent_vid);
-   out << cc;
-   snprintf(cc, sizeof(cc), " rk=\"%u\"", v.get_rank());
-   out << cc;
+   out << " parent=\"" << parent_vid << "\" rk=\"" << v.get_rank()<< "\"";
 
    loop (r, v.get_rank())
       {
-        snprintf(cc, sizeof(cc), " sh-%llu=\"%llu\"", (unsigned long long)r,
-                 (unsigned long long)(v.get_shape_item(r)));
-        out << cc;
+        out << " sh-" << r << "=\"" << v.get_shape_item(r) << "\"";
       }
 
    out << "/>" << endl;
@@ -182,7 +174,7 @@ unsigned int parent_vid = -1;
 }
 //-----------------------------------------------------------------------------
 XML_Saving_Archive &
-XML_Saving_Archive::save_ravel(const Value & v)
+XML_Saving_Archive::save_Ravel(const Value & v)
 {
 int space = do_indent();
 
@@ -222,13 +214,13 @@ char cc[80];
 
             case CT_FLOAT:   // uses UNI_PAD_U4
                  space -= leave_char_mode();
-                 snprintf(cc, sizeof(cc), "%g", cell.get_real_value());
+                 snprintf(cc, sizeof(cc), "%.17g", cell.get_real_value());
                  NEED(1 + strlen(cc)) << UNI_PAD_U4 << decr(--space, cc);
                  break;
 
             case CT_COMPLEX:   // uses UNI_PAD_U5
                  space -= leave_char_mode();
-                 snprintf(cc, sizeof(cc), "%gJ%g",
+                 snprintf(cc, sizeof(cc), "%17gJ%17g",
                           cell.get_real_value(), cell.get_imag_value());
                  NEED(1 + strlen(cc)) << UNI_PAD_U5 << decr(--space, cc);
                  break;
@@ -246,6 +238,13 @@ char cc[80];
                  space -= leave_char_mode();
                  {
                    Cell * cp = cell.get_lval_value();
+                   if (cp == 0)
+                      {
+                        // 0-cell-pointer
+                        snprintf(cc, sizeof(cc), "0");
+                        NEED(2) << UNI_PAD_U7 << "0" << decr(--space, cc);
+                        break;
+                      }
                    const int vid = find_owner(cp);
                    const Value & val = values[vid]._val;
                    const ShapeItem offset = val.get_offset(cp);
@@ -259,7 +258,7 @@ char cc[80];
 }
 //-----------------------------------------------------------------------------
 void
-XML_Saving_Archive::save_function(const Function & fun)
+XML_Saving_Archive::save_Function(const Function & fun)
 {
 const int * eprops = fun.get_exec_properties();
 const APL_time_us creation_time = fun.get_creation_time();
@@ -281,6 +280,39 @@ const APL_time_us creation_time = fun.get_creation_time();
    out << "</Function>" << endl;
 }
 //-----------------------------------------------------------------------------
+int
+XML_Saving_Archive::save_Function_name(const char * ufun_prefix,
+                                       const char * level_prefix,
+                                       const char * id_prefix,
+                                       const Function & fun)
+{
+   if (fun.is_derived())
+      {
+        CERR << endl <<
+"WARNING: The )SI stack contains a derived function. )SAVEing a workspace in\n"
+"         such a state is currently not supported and WILL cause problems\n"
+"         when )LOADing the workspace. Please perform )SIC (or → and then\n"
+"         )SAVE this workspace again." << endl;
+      }
+
+const UserFunction * ufun = fun.get_ufun1();
+   if (ufun)   // user defined function
+      {
+        const UCS_string & fname = ufun->get_name();
+        Symbol * sym = Workspace::lookup_symbol(fname);
+        Assert(sym);
+        const int sym_depth = sym->get_ufun_depth(ufun);
+        out << " " << ufun_prefix << "-name=\""  << fname     << "\""
+            << " " << level_prefix << "-level=\"" << sym_depth << "\"";
+        return 2;   // two attributes
+      }
+   else        // primitive or quad function
+      {
+        out << " " << id_prefix << "-id=\"" << HEX(fun.get_Id());
+        return 1;   // one attribute
+      }
+}
+//-----------------------------------------------------------------------------
 void
 XML_Saving_Archive::save_prefix(const Prefix & prefix)
 {
@@ -377,8 +409,12 @@ const Executable & exec = *si.get_executable();
                Assert(ufun);
                const int sym_depth = sym->get_ufun_depth(ufun);
 
-               out << "<UserFunction ufun-name=\"" << sym->get_name()
-                 << "\" symbol-level=\"" << sym_depth << "\"/>" << endl;
+               if (ufun->is_macro())
+                  out << "<UserFunction macro-num=\"" << ufun->get_macnum()
+                      << "\"/>" << endl;
+               else
+                  out << "<UserFunction ufun-name=\"" << sym->get_name()
+                      << "\" symbol-level=\"" << sym_depth << "\"/>" << endl;
              }
              break;
 
@@ -431,12 +467,9 @@ XML_Saving_Archive::save_symbol(const Symbol & sym)
 void
 XML_Saving_Archive::save_token_loc(const Token_loc & tloc)
 {
-char cc[80];
-   snprintf(cc, sizeof(cc), "%X", tloc.tok.get_tag());
-
    do_indent();
    out << "<Token pc=\"" << tloc.pc
-       << "\" tag=\"" << cc << "\"";
+       << "\" tag=\"" << HEX(tloc.tok.get_tag()) << "\"";
    emit_token_val(tloc.tok);
 
    out << "/>" << endl;
@@ -478,14 +511,14 @@ XML_Saving_Archive::emit_token_val(const Token & tok)
                        break;
 
         case TV_INDEX: { const IndexExpr & idx = tok.get_index_val();
-                         const size_t rank = idx.value_count();
+                         const int rank = idx.value_count();
                          out << " index=\"";
                          loop(i, rank)
                              {
                                if (i)   out << ",";
-                               const Value & val = *idx.values[i].get();
-                               if (&val)   out << "vid_" << find_vid(val);
-                               else        out << "-";
+                               const Value * val = idx.values[i].get();
+                               if (val)   out << "vid_" << find_vid(*val);
+                               else       out << "-";
                                 out << "\"";
                              }
                        }
@@ -494,23 +527,7 @@ XML_Saving_Archive::emit_token_val(const Token & tok)
         case TV_FUN:   {
                          Function * fun = tok.get_function();
                          Assert1(fun);
-                         const UserFunction * ufun = fun->get_ufun1();
-                         if (ufun)   // user defined function
-                            {
-                              const UCS_string & fname = ufun->get_name();
-                              Symbol * sym = Workspace::lookup_symbol(fname);
-                              Assert(sym);
-                              const int sym_depth = sym->get_ufun_depth(ufun);
-                              out << " ufun-name=\"" << fname
-                                  << "\" symbol-level=\"" << sym_depth
-                                  << "\"" << endl;
-                            }
-                         else        // primitive or quad function
-                            {
-                              char cc[40];
-                              snprintf(cc, sizeof(cc), "%X", fun->get_Id());
-                              out << " fun-id=\"" << cc;
-                            }
+                         save_Function_name("ufun", "symbol", "fun", *fun);
                        }
                        out << "\"";
                        break;
@@ -543,7 +560,7 @@ XML_Saving_Archive::save_vstack_item(const ValueStackItem & vsi)
 
         case NC_FUNCTION:
         case NC_OPERATOR:
-             save_function(*vsi.sym_val.function);
+             save_Function(*vsi.sym_val.function);
              break;
 
         case NC_SHARED_VAR:
@@ -568,20 +585,23 @@ tm * t;
 
 const int offset = Workspace::get_v_Quad_TZ().get_offset();   // timezone offset
 
+   // check with: xmllint --valid workspace.xml >/dev/null
+   //
    out <<
 "<?xml version='1.0' encoding='UTF-8' standalone='yes'?>\n"
 "\n"
 "<!DOCTYPE Workspace\n"
 "[\n"
 "    <!ELEMENT Workspace (Value*,Ravel*,SymbolTable,Symbol*,StateIndicator)>\n"
-"    <!ATTLIST Workspace  wsid     CDATA #REQUIRED>\n"
-"    <!ATTLIST Workspace  year     CDATA #REQUIRED>\n"
-"    <!ATTLIST Workspace  month    CDATA #REQUIRED>\n"
-"    <!ATTLIST Workspace  day      CDATA #REQUIRED>\n"
-"    <!ATTLIST Workspace  hour     CDATA #REQUIRED>\n"
-"    <!ATTLIST Workspace  minute   CDATA #REQUIRED>\n"
-"    <!ATTLIST Workspace  second   CDATA #REQUIRED>\n"
-"    <!ATTLIST Workspace  timezone CDATA #REQUIRED>\n"
+"    <!ATTLIST Workspace  wsid       CDATA #REQUIRED>\n"
+"    <!ATTLIST Workspace  year       CDATA #REQUIRED>\n"
+"    <!ATTLIST Workspace  month      CDATA #REQUIRED>\n"
+"    <!ATTLIST Workspace  day        CDATA #REQUIRED>\n"
+"    <!ATTLIST Workspace  hour       CDATA #REQUIRED>\n"
+"    <!ATTLIST Workspace  minute     CDATA #REQUIRED>\n"
+"    <!ATTLIST Workspace  second     CDATA #REQUIRED>\n"
+"    <!ATTLIST Workspace  timezone   CDATA #REQUIRED>\n"
+"    <!ATTLIST Workspace  saving_SVN CDATA #REQUIRED>\n"
 "\n"
 "        <!ELEMENT Value (#PCDATA)>\n"
 "        <!ATTLIST Value flg    CDATA #REQUIRED>\n"
@@ -614,6 +634,8 @@ const int offset = Workspace::get_v_Quad_TZ().get_offset();   // timezone offset
 "                <!ATTLIST Variable vid CDATA #REQUIRED>\n"
 "\n"
 "                <!ELEMENT Function (UCS)>\n"
+"                <!ATTLIST Function creation-time   CDATA #IMPLIED>\n"
+"                <!ATTLIST Function exec-properties CDATA #IMPLIED>\n"
 "\n"
 "                <!ELEMENT Label (#PCDATA)>\n"
 "                <!ATTLIST Label value CDATA #REQUIRED>\n"
@@ -639,8 +661,6 @@ const int offset = Workspace::get_v_Quad_TZ().get_offset();   // timezone offset
 "                <!ELEMENT UserFunction (#PCDATA)>\n"
 "                <!ATTLIST UserFunction ufun-name       CDATA #REQUIRED>\n"
 "                <!ATTLIST UserFunction symbol-level    CDATA #REQUIRED>\n"
-"                <!ATTLIST UserFunction creation-time   CDATA #IMPLIED>\n"
-"                <!ATTLIST UserFunction exec-properties CDATA #IMPLIED>\n"
 "\n"
 "                <!ELEMENT Parser (Token*)>\n"
 "                <!ATTLIST Parser assign-pending CDATA #REQUIRED>\n"
@@ -661,7 +681,7 @@ const int offset = Workspace::get_v_Quad_TZ().get_offset();   // timezone offset
 "                    <!ATTLIST Token fun-id       CDATA #IMPLIED>\n"
 "                    <!ATTLIST Token ufun-name    CDATA #IMPLIED>\n"
 "                    <!ATTLIST Token symbol-level CDATA #IMPLIED>\n"
-"                    <!ATTLIST Token comment  CDATA #IMPLIED>\n"
+"                    <!ATTLIST Token comment      CDATA #IMPLIED>\n"
 "\n"
 "]>\n"
 "\n"
@@ -712,14 +732,15 @@ const int offset = Workspace::get_v_Quad_TZ().get_offset();   // timezone offset
                  {
                    const Value & sub = *cP->get_pointer_value().get();
                    Assert1(&sub);
-                   unsigned int sub_idx = find_vid(sub);
-                   Assert(sub_idx < values.size());
-                   Assert(!&values[sub_idx]._par);
-                   values[sub_idx] = _val_par(values[sub_idx]._val,  parent);
+                   const int sub_idx = find_vid(sub);
+                   Assert(sub_idx < (int)values.size());
+                   Assert(!values[sub_idx]._par);
+                   values[sub_idx] = _val_par(values[sub_idx]._val, &parent);
                  }
               else if (cP->is_lval_cell())
                  {
-CERR << "LVAL CELL in " << p << " at " LOC << endl;
+                   Log(LOG_archive)
+                      CERR << "LVAL CELL in " << p << " at " LOC << endl;
                  }
               ++cP;
             }
@@ -727,11 +748,11 @@ CERR << "LVAL CELL in " << p << " at " LOC << endl;
 
    // save all values (without their ravel)
    //
-   for (vid = 0; vid < values.size(); ++vid)   save_shape(values[vid]._val);
+   for (vid = 0; vid < (int)values.size(); ++vid)  save_shape(values[vid]._val);
 
    // save ravels of all values
    //
-   for (vid = 0; vid < values.size(); ++vid)   save_ravel(values[vid]._val);
+   for (vid = 0; vid < (int)values.size(); ++vid)   save_Ravel(values[vid]._val);
 
    // save user defined symbols
    //
@@ -739,8 +760,8 @@ CERR << "LVAL CELL in " << p << " at " LOC << endl;
 
    // save certain system variables
    //
-#define rw_sv_def(x, txt) save_symbol(Workspace::get_v_ ## x());
-#define ro_sv_def(x, txt) save_symbol(Workspace::get_v_ ## x());
+#define rw_sv_def(x, _str, _txt) save_symbol(Workspace::get_v_ ## x());
+#define ro_sv_def(x, _str, _txt) save_symbol(Workspace::get_v_ ## x());
 #include "SystemVariable.def"
 
    // save state indicator
@@ -832,8 +853,9 @@ struct stat st;
 
    reset();
 
-   if (!strncmp((const char *)file_start, "#!", 2) ||    // a )DUMP file
-       !strncmp((const char *)file_start, "⍝!", 4))      // a library
+   if (!strncmp((const char *)file_start, "#!", 2) ||   // )DUMP file
+       !strncmp((const char *)file_start, "<!", 2) ||   // )DUMP-HTML file
+       !strncmp((const char *)file_start, "⍝!", 4))     // a library
       {
         // the file was either written with )DUMP or is a library.
         // Return the open file descriptor (the destructor will unmap())
@@ -920,7 +942,7 @@ XML_Loading_Archive::get_uni()
    if (data > file_end)   return true;   // EOF
 
 int len = 0;
-   current_char = UTF8_string::toUni(data, len);
+   current_char = UTF8_string::toUni(data, len, true);
    data += len;
    if (current_char == 0x0A)   { ++line_no;   line_start = data; }
    return false;
@@ -1065,6 +1087,37 @@ int hour  = find_int_attr("hour",     false, 10);
 int min   = find_int_attr("minute",   false, 10);
 int sec   = find_int_attr("second",   false, 10);
 
+UCS_string saving_SVN;
+UCS_string current_SVN(ARCHIVE_SVN);
+   {
+     const UTF8 * saving = find_attr("saving_SVN", true);
+     while (saving && *saving != '"')   saving_SVN.append(Unicode(*saving++));
+   }
+bool mismatch = false;
+
+   if (saving_SVN.size() == 0)   // saved with very old version
+      {
+        mismatch = true;
+        CERR << "WARNING: this workspace was )SAVEd with a VERY "
+             << "old SVN version of GNU APL." << endl;
+      }
+   else if (saving_SVN != current_SVN)   // saved with different version
+      {
+        mismatch = true;
+        CERR << "WARNING: this workspace was )SAVEd with SVN version "
+             << saving_SVN << endl <<
+        "          but is now being )LOADed with a SVN version "
+             << current_SVN << " or greater" << endl;
+      }
+
+   if (mismatch)
+      {
+        CERR << "Expect problems, in particular when the )SI was not clear.\n";
+        if (!copying)
+           CERR << "In case of problems, please try )COPY instead of )LOAD."
+                << endl;
+      }
+
    sec  += offset          % 60;
    min  += (offset /   60) % 60;
    hour += (offset / 3600) % 60;
@@ -1250,7 +1303,7 @@ bool no_copy = false;   // assume the value is needed
         int parent = vid;
         for (;;)
             {
-              Assert(parent < parents.size());
+              Assert(parent < (int)parents.size());
               if (parents[parent] == -1)   break;   // topmost owner found
               parent = parents[parent];
             }
@@ -1272,7 +1325,7 @@ bool no_copy = false;   // assume the value is needed
       }
    else
       {
-        Assert(vid == values.size());
+        Assert(vid == (int)values.size());
 
         Value_P val(sh_value, LOC);
         values.push_back(val);
@@ -1286,7 +1339,7 @@ XML_Loading_Archive::read_Cells(Cell * & C, Value & C_owner,
    while (*first <= ' ')   ++first;
 
 int len;
-const Unicode type = UTF8_string::toUni(first, len);
+const Unicode type = UTF8_string::toUni(first, len, true);
 
    switch (type)
       {
@@ -1344,7 +1397,7 @@ const Unicode type = UTF8_string::toUni(first, len);
                char * end = 0;
                const int vid = strtol((const char *)first, &end, 10);
                Assert(vid >= 0);
-               Assert(vid < values.size());
+               Assert(vid < (int)values.size());
                new (C++) PointerCell(values[vid], C_owner);
                first = (const UTF8 *)end;
              }
@@ -1352,18 +1405,24 @@ const Unicode type = UTF8_string::toUni(first, len);
 
         case UNI_PAD_U7: // cellref
              first += len;
-             {
-               char * end = 0;
-               const int vid = strtol((const char *)first, &end, 16);
-               Assert(vid >= 0);
-               Assert(vid < values.size());
-               Assert(*end == '[');   ++end;
-               const ShapeItem offset = strtoll(end, &end, 16);
-               Assert(*end == ']');   ++end;
-               new (C++) LvalCell(&values[vid]->get_ravel(offset),
-                                  values[vid].get());
-               first = (const UTF8 *)end;
-             }
+             if (first[0] == '0')    // 0-cell-pointer
+                {
+                  new (C++) LvalCell(0, 0);
+                  first++;
+                }
+             else
+                {
+                  char * end = 0;
+                  const int vid = strtol((const char *)first, &end, 16);
+                  Assert(vid >= 0);
+                  Assert(vid < (int)values.size());
+                  Assert(*end == '[');   ++end;
+                  const ShapeItem offset = strtoll(end, &end, 16);
+                  Assert(*end == ']');   ++end;
+                  new (C++) LvalCell(&values[vid]->get_ravel(offset),
+                                     values[vid].get());
+                  first = (const UTF8 *)end;
+                }
              break;
 
         default: Q1(type) Q1(line_no) DOMAIN_ERROR;
@@ -1386,7 +1445,7 @@ XML_Loading_Archive::read_chars(UCS_string & ucs, const UTF8 * & utf)
              }
 
          int len;
-         const Unicode type = UTF8_string::toUni(utf, len);
+         const Unicode type = UTF8_string::toUni(utf, len, true);
 
          if (type == UNI_PAD_U2)   // start of char_mode
             {
@@ -1441,7 +1500,7 @@ const UTF8 * cells = find_attr("cells", false);
 
    Log(LOG_archive)   CERR << "    read_Ravel() vid=" << vid << endl;
 
-   Assert(vid < values.size());
+   Assert(vid < (int)values.size());
 Value_P val = values[vid];
 
    if (!val)   // )COPY with vids_COPY or static value
@@ -1467,7 +1526,7 @@ Cell * end = C + count;
 
    {
      int len = 0;
-     const Unicode next = UTF8_string::toUni(cells, len);
+     const Unicode next = UTF8_string::toUni(cells, len, true);
      Assert(next == '"');
    }
 
@@ -1485,8 +1544,8 @@ XML_Loading_Archive::read_unused_name(int d, Symbol & symbol)
 void
 XML_Loading_Archive::read_Variable(int d, Symbol & symbol)
 {
-const unsigned int vid = find_int_attr("vid", false, 10);
-   Assert(vid < values.size());
+const int vid = find_int_attr("vid", false, 10);
+   Assert(vid < (int)values.size());
 
    Log(LOG_archive)   CERR << "      read_Variable() vid=" << vid
                            << " name=" << symbol.get_name() << endl;
@@ -1506,7 +1565,7 @@ const unsigned int vid = find_int_attr("vid", false, 10);
 
    try
       {
-        symbol.assign(values[vid], LOC);
+        symbol.assign(values[vid], true, LOC);
       }
    catch (...)
       {
@@ -1536,7 +1595,9 @@ const UTF8 * ep = find_attr("exec-properties", true);
                eprops, eprops + 1, eprops + 2, eprops+ 3);
       }
 
-   Log(LOG_archive)   CERR << "      read_Function() native=" << native << endl;
+   Log(LOG_archive)
+      CERR << "      read_Function(" << symbol.get_name()
+           << ") native=" << native << endl;
 
    next_tag(LOC);
    expect_tag("UCS", LOC);
@@ -1567,13 +1628,21 @@ UCS_string text;
    else
       {
         int err = 0;
-        UCS_string creator(filename);
-        creator.append(UNI_ASCII_COLON);
-        creator.append_number(line_no);
-        UTF8_string creator_utf8(creator);
+        UCS_string creator_UCS(filename);
+        creator_UCS.append(UNI_ASCII_COLON);
+        creator_UCS.append_number(line_no);
+        UTF8_string creator(creator_UCS);
 
-        UserFunction * ufun = UserFunction::fix(text, err, false,
-                                               LOC, creator_utf8);
+        UserFunction * ufun = 0;
+        if (text[0] == UNI_LAMBDA)
+           {
+             ufun = UserFunction::fix_lambda(symbol, text);
+             ufun->increment_refcount(LOC);   // since we push it below
+           }
+        else
+           {
+             ufun = UserFunction::fix(text, err, false, LOC, creator, false);
+           }
 
         if (d == 0)   symbol.pop();
         if (ufun)
@@ -1639,7 +1708,8 @@ UCS_string name_ucs;
 
    // ⎕NLT was removed, but could lurk around in old workspaces.
    //
-   if (name_ucs == UCS_string(UTF8_string("⎕NLT")))
+   if (name_ucs == UCS_string(UTF8_string("⎕NLT")) ||
+       name_ucs == UCS_string(UTF8_string("⎕PT")))
       {
         Log(LOG_archive)   CERR << "        skipped at " << LOC << endl;
         skip_to_tag("/Symbol");
@@ -1678,7 +1748,7 @@ bool no_copy = is_protected || (have_allowed_objects && !is_selected);
         next_tag(LOC);
         if (is_tag("Variable"))
            {
-             const unsigned int vid = find_int_attr("vid", false, 10);
+             const int vid = find_int_attr("vid", false, 10);
              vids_COPY.push_back(vid);
            }
         skip_to_tag("/Symbol");
@@ -1785,7 +1855,24 @@ const int levels = find_int_attr("levels", false, 10);
         next_tag(LOC);
         expect_tag("SI-entry", LOC);
 
-        read_SI_entry(l);
+        try
+           {
+             read_SI_entry(l);
+           }
+        catch (...)
+           {
+             CERR <<
+"\n"
+"*** SORRY! An error occured while reading the )SI stack of the )SAVEd\n"
+"    workspace. The )SI stack was reconstructed to the extent possible.\n"
+"    We stronly recommend to perform )SIC and then )DUMP the workspace under\n"
+"    a different name.\n" << endl;
+
+             // skip rest of <StateIndicator>
+             //
+             skip_to_tag("/StateIndicator");
+             return;
+           }
 
         // the parsers loop eats the terminating /SI-entry
       }
@@ -1797,9 +1884,9 @@ const int levels = find_int_attr("levels", false, 10);
 void
 XML_Loading_Archive::read_SI_entry(int lev)
 {
-const int level     = find_int_attr("level",     false, 10);
+const int level = find_int_attr("level", false, 10);
 
-   Log(LOG_archive)   CERR << "read_SI_entry() level=" << level << endl;
+   Log(LOG_archive)   CERR << "    read_SI_entry() level=" << level << endl;
 
    next_tag(LOC);
 Executable * exec = 0;
@@ -1814,7 +1901,15 @@ Executable * exec = 0;
    Workspace::push_SI(exec, LOC);
 StateIndicator * si = Workspace::SI_top();
    Assert(si);
-   read_Parsers(*si);
+   read_Parser(*si);
+
+   for (;;)
+       {
+         // skip old EOC tags
+         //
+         next_tag(LOC);
+         if (is_tag("/SI-entry"))   break;
+       }
 }
 //-----------------------------------------------------------------------------
 Executable *
@@ -1822,6 +1917,7 @@ XML_Loading_Archive::read_Execute()
 {
    next_tag(LOC);
    expect_tag("UCS", LOC);
+
 const UTF8 * uni = find_attr("uni", false);
 UCS_string text;
    while (*uni != '"')   read_chars(text, uni);
@@ -1851,8 +1947,11 @@ Executable * exec = new StatementList(text, LOC);
 Executable *
 XML_Loading_Archive::read_UserFunction()
 {
-const int level = find_int_attr("symbol-level", false, 10);
-const UTF8 * name = find_attr("ufun-name", false);
+const int macro_num = find_int_attr("macro-num", true, 10);
+   if (macro_num != -1)   return Macro::get_macro((Macro::Macro_num)macro_num);
+
+const int level     = find_int_attr("symbol-level", false, 10);
+const UTF8 * name   = find_attr("ufun-name", false);
 const UTF8 * n  = name;
    while (*n != '"')   ++n;
 UTF8_string name_utf(name, n - name);
@@ -1873,7 +1972,7 @@ UserFunction * ufun = fun->get_ufun1();
 }
 //-----------------------------------------------------------------------------
 void
-XML_Loading_Archive::read_Parsers(StateIndicator & si)
+XML_Loading_Archive::read_Parser(StateIndicator & si)
 {
    next_tag(LOC);
    expect_tag("Parser", LOC);
@@ -1895,7 +1994,7 @@ Prefix & parser = si.current_stack;
          parser.push(tloc);
        }
 
-   next_tag(LOC);
+   expect_tag("/Parser", LOC);
 }
 //-----------------------------------------------------------------------------
 bool
@@ -1951,7 +2050,10 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
                while (*end != '"')   ++end;
                UTF8_string name_utf(sym_name, end - sym_name);
                UCS_string name_ucs(name_utf);
-               Symbol * symbol = Workspace::lookup_symbol(name_ucs);
+
+               Symbol * symbol = Avec::is_quad(name_ucs[0])
+                               ? Workspace::lookup_existing_symbol(name_ucs)
+                               : Workspace::lookup_symbol(name_ucs);
                new (&tloc.tok) Token(tag, symbol);
              }
              break;
@@ -1965,8 +2067,8 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
 
         case TV_VAL:   
              {
-               const unsigned int vid = find_int_attr("vid", false, 10);
-               Assert(vid < values.size());
+               const int vid = find_int_attr("vid", false, 10);
+               Assert(vid < (int)values.size());
                new (&tloc.tok) Token(tag, values[vid]);
              }
              break;
@@ -1988,8 +2090,8 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
                          Assert1(*vids == 'i');   ++vids;
                          Assert1(*vids == 'd');   ++vids;
                          Assert1(*vids == '_');   ++vids;
-                         const unsigned int vid = strtol(vids, &vids, 10);
-                         Assert(vid < values.size());
+                         const int vid = strtol(vids, &vids, 10);
+                         Assert(vid < (int)values.size());
                          idx.add(values[vid]);
                        }
                   }
@@ -1999,32 +2101,10 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
 
         case TV_FUN:
              {
-               const UTF8 * fun_name = find_attr("ufun-name", true);
-
-               if (fun_name == 0)   // primitive or Quad function
-                  {
-                    const int fun_id = find_int_attr("fun-id", false, 16);
-                    Function * sysfun = ID::get_system_function(ID::Id(fun_id));
-                    Assert1(sysfun);
-                    new (&tloc.tok) Token(tag, sysfun);
-                  }
-               else
-                  {
-                    const int level = find_int_attr("symbol-level", false, 10);
-
-                    const UTF8 * end = fun_name;
-                    while (*end != '"')   ++end;
-                    UTF8_string name_utf(fun_name, end - fun_name);
-                    UCS_string ucs(name_utf);
-                    const Symbol & symbol = *Workspace::lookup_symbol(ucs);
-                    Assert(level >= 0);
-                    Assert(level < symbol.value_stack_size());
-                    const ValueStackItem & vsi = symbol[level];
-                    Assert(vsi.name_class == NC_FUNCTION);
-                    Function * fun = vsi.sym_val.function;
-                    Assert(fun);
-                    new (&tloc.tok) Token(tag, fun);
-                  }
+               Function * fun = read_Function_name("ufun-name",
+                                                   "symbol-level", "fun-id");
+               Assert(fun);
+               new (&tloc.tok) Token(tag, fun);
              }
              break;
 
@@ -2033,5 +2113,44 @@ const TokenTag tag = TokenTag(find_int_attr("tag", false, 16));
 
    return true;
 }
+//-----------------------------------------------------------------------------
+Function *
+XML_Loading_Archive::read_Function_name(const char * name_attr,
+                                        const char * level_attr,
+                                        const char * id_attr)
+{
+const UTF8 * fun_name = find_attr(name_attr, true);
+
+   if (fun_name)   // user defined function
+      {
+        int level = find_int_attr(level_attr, false, 10);
+        const UTF8 * end = fun_name;
+        while (*end != '"')   ++end;
+        UTF8_string name_utf(fun_name, end - fun_name);
+        UCS_string ucs(name_utf);
+        const Symbol & symbol = *Workspace::lookup_symbol(ucs);
+        if (level == -1)   level = 0;   // named lambda?
+
+        Assert(level >= 0);
+        Assert(level < symbol.value_stack_size());
+        const ValueStackItem & vsi = symbol[level];
+        Assert(vsi.name_class == NC_FUNCTION);
+        Function * fun = vsi.sym_val.function;
+        Assert(fun);
+        return fun;
+      }
+
+const int fun_id = find_int_attr(id_attr, true, 16);
+   if (fun_id != -1)
+      {
+        Function * sysfun = ID::get_system_function(ID::Id(fun_id));
+        Assert(sysfun);
+        return sysfun;
+      }
+
+   // not found. This can happen when the function is optional.
+   //
+   return 0;
+}
 //=============================================================================
 
