@@ -43,7 +43,32 @@ Bif_F12_FORMAT::eval_B(Value_P B)
    if (B->is_empty())
       {
         Value_P Z(B->get_shape(), LOC);
-        new (&Z->get_ravel(0)) CharCell(UNI_ASCII_SPACE);
+        Z->set_proto_Spc();
+        return Token(TOK_APL_VALUE1, Z);
+      }
+
+   if (!B->is_simple())
+      {
+        PrintBuffer pb(*B, Workspace::get_PrintContext(PR_APL), 0);
+        Assert(pb.is_rectangular());
+        const ShapeItem cols = pb.get_width(0);
+        const ShapeItem rows = pb.get_height();
+        Shape shape_Z(rows, cols);
+        Value_P Z(shape_Z, LOC);
+        loop(y, rows)
+            {
+              UCS_string row = pb.get_line(y);
+              row.map_pad();
+              loop(x, cols)   new (Z->next_ravel()) CharCell(row[x]);
+            }
+
+        if (Z->get_rank() == 2 && Z->get_shape().get_shape_item(0) == 1)
+           {
+             Shape sh(Z->get_shape().get_shape_item(1));
+             Z->set_shape(sh);
+           }
+
+        Z->check_value(LOC);
         return Token(TOK_APL_VALUE1, Z);
       }
 
@@ -130,14 +155,13 @@ Value_P Z;
       {
         Z = format_by_example(A, B);
       }
-   else if (A->is_int_vector(Workspace::get_CT()) ||
-            A->is_int_scalar(Workspace::get_CT()))
+   else if (A->is_int_vector() || A->is_int_scalar())
       {
         Z = format_by_specification(A, B);
       }
    else
       {
-        A->print_boxed(CERR, "left argument of ⍕: ");
+        Workspace::more_error() = UCS_string("Bad left argument of ⍕");
         DOMAIN_ERROR;
       }
 
@@ -151,8 +175,8 @@ Bif_F12_FORMAT::monadic_format(Value_P B)
 {
    Assert(B->get_rank() <= 2);
 
-PrintContext pctx = Workspace::get_PrintContext();
-   pctx.set_style(PrintStyle(pctx.get_style() | PST_NO_FRACT_0));
+const PrintStyle style((PrintStyle)(PR_APL | PST_NO_FRACT_0));
+const PrintContext pctx = Workspace::get_PrintContext(style);
 
 const PrintBuffer pb(*B, pctx, 0);
 
@@ -207,7 +231,7 @@ vector<UCS_string> col_formats;
            }
       }
 
-   if (col_formats.size() != cols)   LENGTH_ERROR;
+   if (cols != (ShapeItem)(col_formats.size()))   LENGTH_ERROR;
 
    // convert each column format string into a Format_LIFER
    //
@@ -649,7 +673,7 @@ char * fract_end = 0;
         //
         const int flen = snprintf(format, sizeof(format), "%%.%luE",
                                   (unsigned long)fract_part.size());
-        Assert(flen < sizeof(format));   // format was big enough.
+        Assert(flen < (int)sizeof(format));   // format was big enough.
 
         const int dlen = snprintf(&data_buf[0], data_buf_len, format, value);
         Assert(dlen < data_buf_len);
@@ -672,15 +696,16 @@ char * fract_end = 0;
       {
         const int flen = snprintf(format, sizeof(format), "%%.%luf",
                                   (unsigned long)fract_part.size());
-        Assert(flen < sizeof(format));   // assume no snprintf() overflow
+        Assert(flen < (int)sizeof(format));   // assume no snprintf() overflow
 
         const int dlen = snprintf(&data_buf[0], data_buf_len, format, value);
         data_buf[data_buf_len - 1] = 0;
 
         // the int part could be longer than allowed by the exaple string.
         //
-        const char * dot = strchr(data_buf, '.');
-        const int ilen = dot ? (dot - data_buf) : strlen(data_buf);
+        const char * dot = strchr(data_buf.get_data(), '.');
+        const int ilen = dot ? (dot - data_buf.get_data())
+                             : strlen(data_buf.get_data());
         if (ilen > int_part.out_len)
            {
              if (Workspace::get_FC(3) == UNI_ASCII_0)   DOMAIN_ERROR;
@@ -889,8 +914,8 @@ PrintBuffer pb;
 
    loop(col, cols_B)
        {
-         uint32_t col_width;
-         uint32_t precision;
+         APL_Integer col_width;
+         APL_Integer precision;
 
          if (len_A == 1)
             {
@@ -989,7 +1014,8 @@ bool has_num = false;
            }
         else                  // exponential format
            {
-             UCS_string data = format_spec_expo(value, - precision);
+             UCS_string data = UCS_string::from_double_expo_prec(value,
+                                            -precision - 1);
              add_row(ret, r, has_char, has_num, UNI_ASCII_E, data);
           }
       }
@@ -1063,12 +1089,6 @@ UCS_string ret = UCS_string::from_double_fixed_prec(value, precision);
    return ret;
 }
 //-----------------------------------------------------------------------------
-UCS_string
-Bif_F12_FORMAT::format_spec_expo(APL_Float value, int precision)
-{
-  return UCS_string::from_double_expo_prec(value, precision - 1);
-}
-//-----------------------------------------------------------------------------
 ostream &
 operator<<(ostream & out, const Format_sub & fmt)
 {
