@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 */
 
 #include "Bif_OPER1_REDUCE.hh"
+#include "Macro.hh"
+#include "PointerCell.hh"
 #include "Workspace.hh"
 
 Bif_OPER1_REDUCE    Bif_OPER1_REDUCE ::_fun;
@@ -91,7 +93,7 @@ const Shape3 shape_B3(shape_B, axis);
                   loop(l, shape_B3.l())
                      {
                        const ShapeItem src = shape_B3.hml(h, bm, l);
-                       Z->next_ravel()->init(B->get_ravel(src), Z.getref());
+                       Z->next_ravel()->init(B->get_ravel(src), Z.getref(),LOC);
                      }
                   if (shape_B3.m() > 1)   ++bm;
                 }
@@ -101,7 +103,8 @@ const Shape3 shape_B3(shape_B, axis);
                   loop(l, shape_B3.l())
                      {
                        const ShapeItem src = shape_B3.hml(h, 0, l);
-                       Z->next_ravel()->init_type(B->get_ravel(src), Z.getref());
+                       Z->next_ravel()
+                        ->init_type(B->get_ravel(src), Z.getref(), LOC);
                      }
 
                   // cB is not incremented when fill item is used.
@@ -109,16 +112,18 @@ const Shape3 shape_B3(shape_B, axis);
            }
       }
 
-   Z->set_default(*B.get());
+   Z->set_default(*B.get(), LOC);
    Z->check_value(LOC);
    return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 Token
-Bif_REDUCE::reduce(Function * LO, Value_P B, Axis axis)
+Bif_REDUCE::reduce(Token & _LO, Value_P B, Axis axis)
 {
+Function * LO = _LO.get_function();
    Assert1(LO);
-   if (!LO->has_result())   DOMAIN_ERROR;
+   if (!LO->has_result())            DOMAIN_ERROR;
+   if (LO->get_fun_valence() != 2)   SYNTAX_ERROR;
 
    // if B is a scalar, then Z is B.
    //
@@ -137,23 +142,56 @@ const Shape shape_Z = B->get_shape().without_axis(axis);
 
    if (m_len == 1)   return Bif_F12_RHO::do_reshape(shape_Z, *B);
 
+   // non-trivial reduce (len > 1)
+   //
 const Shape3 B3(B->get_shape(), axis);
+   if (LO->may_push_SI())   // user defined LO
+      {
+        Value_P X4(4, LOC);
+        new (X4->next_ravel())   IntCell(axis + Workspace::get_IO());
+        new (X4->next_ravel())   IntCell(B3.h());
+        new (X4->next_ravel())   IntCell(B3.m());
+        new (X4->next_ravel())   IntCell(B3.l());
+        X4->check_value(LOC);
+        return Macro::Z__LO_REDUCE_X4_B->eval_LXB(_LO, X4, B);
+      }
+
+   if (shape_Z.is_empty())   return LO->eval_identity_fun(B, axis);
+
 const Shape3 Z3(B3.h(), 1, B3.l());
-   return do_reduce(shape_Z, Z3, B3.m(), LO, axis, B, B->get_shape_item(axis));
+   return do_reduce(shape_Z, Z3, B3.m(), LO, B, B->get_shape_item(axis));
 }
 //-----------------------------------------------------------------------------
 Token
-Bif_REDUCE::reduce_n_wise(Value_P A, Function * LO, Value_P B, Axis axis)
+Bif_REDUCE::reduce_n_wise(Value_P A, Token & _LO, Value_P B, Axis axis)
 {
+Function * LO = _LO.get_function();
+   Assert(LO);
    if (!LO->has_result())   DOMAIN_ERROR;
 
    if (A->element_count() != 1)   LENGTH_ERROR;
 const APL_Integer A0 = A->get_ravel(0).get_int_value();
-const int n_wise = A0 < 0 ? -A0 : A0;   // the number of items
+const int n_wise = A0 < 0 ? -A0 : A0;   // the number of items (= M1 in iso)
 
    if (B->is_scalar())
       {
-        if (n_wise > 1)                               DOMAIN_ERROR;
+        if (n_wise > 2)    DOMAIN_ERROR;
+        if (n_wise == 0)
+           {
+              Token ident = LO->eval_identity_fun(B, axis);
+              Value_P val(2, LOC);
+              val->next_ravel()->init(ident.get_apl_val()->get_ravel(0),
+                                      val.getref(), LOC);
+              val->next_ravel()->init(ident.get_apl_val()->get_ravel(0),
+                                      val.getref(), LOC);
+              return Token(TOK_APL_VALUE1, val);
+           }
+
+        // n_wise is 1 or 2: return (2 - n_wise) ⍴ B
+        //
+        Shape sh;
+        if (n_wise == 2)   sh.add_shape_item(0);
+        return Bif_F12_RHO::do_reshape(sh, B.getref());
       }
    else
       {
@@ -171,7 +209,7 @@ const int n_wise = A0 < 0 ? -A0 : A0;   // the number of items
         Shape shape_B1 = B->get_shape().insert_axis(axis, 0);
         shape_B1.increment_shape_item(axis + 1);
         Value_P val(shape_B1, LOC);
-        val->get_ravel(0).init(B->get_ravel(0), val.getref());   // prototype
+        val->get_ravel(0).init(B->get_ravel(0), val.getref(), LOC); // prototype
 
         Token result = LO->eval_identity_fun(val, axis);
         return result;
@@ -187,190 +225,143 @@ const int n_wise = A0 < 0 ? -A0 : A0;   // the number of items
         Shape shape_Z = B->get_shape();
         shape_Z.set_shape_item(axis, 0);
         Value_P Z(shape_Z, LOC);
-        Z->get_ravel(0).init(B->get_ravel(0), Z.getref());
+        Z->get_ravel(0).init(B->get_ravel(0), Z.getref(), LOC);
         Z->check_value(LOC);
         return Token(TOK_APL_VALUE1, Z);
       }
 
 Shape shape_Z(B->get_shape());
    shape_Z.set_shape_item(axis, shape_Z.get_shape_item(axis) - n_wise + 1);
+   if (shape_Z.is_empty())   return LO->eval_identity_fun(B, axis);
 
    if (n_wise == 1)   return Bif_F12_RHO::do_reshape(shape_Z, *B);
 
 const Shape3 Z3(shape_Z, axis);
-   return do_reduce(shape_Z, Z3, A0, LO, axis, B, B->get_shape_item(axis));
-}
-//-----------------------------------------------------------------------------
-Token
-Bif_REDUCE::do_reduce(const Shape & shape_Z, const Shape3 & Z3, ShapeItem a,
-                      Function * LO, Axis axis, Value_P B, ShapeItem bm)
-{
-   if (shape_Z.is_empty())   return LO->eval_identity_fun(B, axis);
-
-Value_P Z(shape_Z, LOC);
-EOC_arg arg(Z, B);
-REDUCTION & _arg = arg.u.u_REDUCTION;
-
-const bool scan = (a == -1);
-const bool reverse = (a < -1);
-
-   _arg.LO         = LO;
-   _arg.scan         = scan;
-   _arg.len_L        = Z3.get_last_shape_item();
-   _arg.len_L_s       = scan ? 0 : _arg.len_L;
-   _arg.len_BML      = bm * _arg.len_L;
-   _arg.len_ZM       = Z3.m();
-   _arg.len_Z        = shape_Z.get_volume();
-   _arg.beam_len     = reverse ? -a : a;
-   _arg.dB           = reverse ? _arg.len_L : - _arg.len_L;
-   _arg.start_offset = reverse ? (_arg.beam_len - 1) * _arg.dB : 0;
-
-   _arg.todo_B = 0;   // new beam
-   _arg.z = -1;
-
-   return finish_REDUCE(arg, true);
-}
-//-----------------------------------------------------------------------------
-Token
-Bif_REDUCE::finish_REDUCE(EOC_arg & arg, bool first)
-{
-REDUCTION & _arg = arg.u.u_REDUCTION;
-
-   for (;;)
+const Shape3 B3(B->get_shape(), axis);
+   if (LO->may_push_SI())   // user defined LO
       {
-        if (_arg.todo_B == 0)   // new beam
-           {
-             if (++_arg.z >= _arg.len_Z)   break;   // all beams done
-             
-             _arg.z_L =  _arg.z % _arg.len_L;
-             _arg.z_M = (_arg.z / _arg.len_L) % _arg.len_ZM;
-             _arg.z_H =  _arg.z / (_arg.len_L * _arg.len_ZM);
-
-             if (_arg.scan)   _arg.todo_B = 1 + _arg.z_M;
-             else             _arg.todo_B = _arg.beam_len;
-
-            _arg.b = _arg.z_L + _arg.z_H * _arg.len_BML   // start of row
-                   + _arg.z_M * _arg.len_L_s              // start of beam
-                   + (_arg.todo_B - 1) * _arg.len_L       // end of beam
-                   - _arg.start_offset;                   // or start (reverse)
-
-             // Z[z] ? B[b]
-             //
-             arg.Z->next_ravel()->init(arg.B->get_ravel(_arg.b),
-                                       arg.Z.getref());
-           }
-
-        // update position and check for end of beam
-        //
-        Assert(_arg.todo_B > 0);
-        _arg.b += _arg.dB;
-        --_arg.todo_B;
-        if (_arg.todo_B == 0)
-           {
-             continue;
-           }
-
-        // reduction step (one call of LO)
-        //
-        {
-          Value_P LO_A = arg.B->get_ravel(_arg.b).to_value(LOC);
-          Value_P LO_B = arg.Z->get_ravel(_arg.z).to_value(LOC);
-          Token result = _arg.LO->eval_AB(LO_A, LO_B);
-          arg.Z->get_ravel(_arg.z).release(LOC);
-
-          if (result.get_Class() == TC_VALUE)
-             {
-               Value_P ZZ = result.get_apl_val();
-               arg.Z->get_ravel(_arg.z).init_from_value(ZZ, arg.Z.getref(),LOC);
-               continue;
-             }
-
-          if (result.get_tag() == TOK_ERROR)   return result;
-
-          if (result.get_tag() == TOK_SI_PUSHED)
-             {
-               // LO was a user defined function
-               //
-               if (first)   // first call
-                  Workspace::SI_top()->add_eoc_handler(eoc_REDUCE, arg, LOC);
-               else           // subsequent call
-                  Workspace::SI_top()->move_eoc_handler(eoc_REDUCE, &arg, LOC);
-
-               return result;   // continue in user defined function...
-             }
-
-          Q1(result);   FIXME;
-        }
+        Value_P A1 = IntScalar(A0 < 0 ?  A0 + 1 : 1 - A0, LOC);
+        Value_P vsh_Z(LOC, &shape_Z);
+        Value_P vsh_Z3(LOC, &Z3);
+        Value_P vsh_B3(LOC, &B3);
+        Value_P X4(4, LOC);
+        new (X4->next_ravel())   IntCell(axis + Workspace::get_IO());   // X
+        new (X4->next_ravel())   PointerCell(vsh_Z,  X4.getref());      // ⍴Z
+        new (X4->next_ravel())   PointerCell(vsh_Z3, X4.getref());      // ⍴Z3
+        new (X4->next_ravel())   PointerCell(vsh_B3, X4.getref());      // ⍴B3
+        X4->check_value(LOC);
+        if (A0<0)   return Macro::Z__nA_LO_REDUCE_X4_B->eval_ALXB(A1,_LO,X4,B);
+        else        return Macro::Z__pA_LO_REDUCE_X4_B->eval_ALXB(A1,_LO,X4,B);
       }
 
-   arg.Z->set_default(*arg.B.get());
-
-   arg.Z->check_value(LOC);
-   return Token(TOK_APL_VALUE1, arg.Z);
+   return do_reduce(shape_Z, Z3, A0, LO, B, B->get_shape_item(axis));
 }
 //-----------------------------------------------------------------------------
-bool
-Bif_REDUCE::eoc_REDUCE(Token & token)
+Token
+Bif_REDUCE::do_reduce(const Shape & shape_Z, const Shape3 & Z3, ShapeItem nwise,
+                      Function * LO, Value_P B, ShapeItem bm)
 {
-EOC_arg * arg = Workspace::SI_top()->remove_eoc_handlers();
-EOC_arg * next = arg->next;
-REDUCTION & _arg = arg->u.u_REDUCTION;
+Value_P Z(shape_Z, LOC);
 
-   if (token.get_Class() != TC_VALUE)  return false;   // stop it
-
-   // the user defined function has returned a value. Store it.
+   // constants that are valid for the entire reduction
    //
-   {
-     Value_P ZZ = token.get_apl_val();
-     arg->Z->get_ravel(_arg.z).init_from_value(ZZ, arg->Z.getref(), LOC);
-   }
+const ShapeItem len_Z   = Z->element_count();
+const ShapeItem len_L   = Z3.get_last_shape_item();
+const ShapeItem len_BML = bm * len_L;
+const ShapeItem len_ZM  = Z3.m();
+const ShapeItem len_ZML = len_L * len_ZM;
+prim_f2 scalar_LO       = LO->get_scalar_f2();
 
-   // pop the SI unless this is the last reduction of the last 
-   // ravel element of Z to be computed
-   //
-   if (_arg.z < (_arg.len_Z - 1) || _arg.todo_B > 1 )   Workspace::pop_SI(LOC);
+   loop(z, len_Z)
+      {
+        // break down z into H, M, and L components
+        //
+        const ShapeItem z_L =  z % len_L;
+        const ShapeItem z_M = (z / len_L) % len_ZM;
+        const ShapeItem z_H =  z / len_ZML;
 
-   copy_1(token, finish_REDUCE(*arg, false), LOC);
-   if (token.get_tag() == TOK_SI_PUSHED)   return true;   // continue
+        // compute LO_count (= the number of LO-reductions), and b (= the
+        // starting point of the LO-reductions). They depend on z and nwise
+        //
+        const ShapeItem LO_count = (nwise == -1) ? z_M
+                                 : (nwise < -1)  ? - nwise - 1
+                                 :                 nwise - 1;
 
-   delete arg;
-   Workspace::SI_top()->set_eoc_handlers(next);
-   if (next)   return next->handler(token);
+        ShapeItem b = z_L + z_H * len_BML       // start of row in B
+                          + LO_count * len_L;   // end of beam in B
 
-   return false;   // stop it
+        // for reverse (i,e. nwise < -1) reduction direction go to the
+        // start of the beam instead of the end of the beam.
+        //
+        if (nwise < -1)    b += (nwise + 1) * len_L;
+        if (nwise != -1)   b += z_M * len_L; // start of beam
+
+        // Z[z] = B[b]
+        //
+        Cell * cZ = Z->next_ravel();
+        cZ->init(B->get_ravel(b), Z.getref(), LOC);
+
+        loop(lo, LO_count)
+           {
+             if (nwise < -1)   b += len_L;   // move forward in B
+             else              b -= len_L;   // move backward in B
+
+             // one reduction step (one call of LO)
+             //
+             const Cell & cB = B->get_ravel(b);
+             if (scalar_LO && cZ->is_simple_cell() && cB.is_simple_cell())
+                {
+                  ErrorCode ec = (cZ->*scalar_LO)(cZ, &cB);
+                  if (ec)   throw_apl_error(ec, LOC);
+                }
+             else
+                {
+                  Value_P LO_A = cB.to_value(LOC);
+                  Value_P LO_B = cZ->to_value(LOC);
+                  Token result = LO->eval_AB(LO_A, LO_B);
+                  cZ->release(LOC);
+
+                  if (result.get_tag() == TOK_ERROR)
+                    throw_apl_error(result.get_ErrorCode(), LOC);
+
+                  Assert(result.get_Class() == TC_VALUE);
+                  Value_P ZZ = result.get_apl_val();
+                  cZ->init_from_value(ZZ, Z.getref(), LOC);
+                }
+           }
+      }
+
+   Z->set_default(*B.get(), LOC);
+   Z->check_value(LOC);
+   return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 Token
 Bif_OPER1_REDUCE::eval_AXB(Value_P A, Value_P X, Value_P B)
 {
-const Rank axis = X->get_single_axis(B->get_rank());
-
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
    return replicate(A, B, axis);
 }
 //-----------------------------------------------------------------------------
 Token
 Bif_OPER1_REDUCE::eval_LXB(Token & _LO, Value_P X, Value_P B)
 {
-Function * LO = _LO.get_function();
-
-const Rank axis = X->get_single_axis(B->get_rank());
-   return reduce(LO, B, axis);
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
+   return reduce(_LO, B, axis);
 }
 //-----------------------------------------------------------------------------
 Token
 Bif_OPER1_REDUCE::eval_ALXB(Value_P A, Token & _LO, Value_P X, Value_P B)
 {
-Function * LO = _LO.get_function();
-
-const Rank axis = X->get_single_axis(B->get_rank());
-   return reduce_n_wise(A, LO, B, axis);
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
+   return reduce_n_wise(A, _LO, B, axis);
 }
 //-----------------------------------------------------------------------------
 Token
 Bif_OPER1_REDUCE1::eval_AXB(Value_P A,
                             Value_P X, Value_P B)
 {
-const Rank axis = X->get_single_axis(B->get_rank());
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
 
    return replicate(A, B, axis);
 }
@@ -378,14 +369,14 @@ const Rank axis = X->get_single_axis(B->get_rank());
 Token
 Bif_OPER1_REDUCE1::eval_LXB(Token & LO, Value_P X, Value_P B)
 {
-const Rank axis = X->get_single_axis(B->get_rank());
-   return reduce(LO.get_function(), B, axis);
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
+   return reduce(LO, B, axis);
 }
 //-----------------------------------------------------------------------------
 Token
 Bif_OPER1_REDUCE1::eval_ALXB(Value_P A, Token & LO, Value_P X, Value_P B)
 {
-const Rank axis = X->get_single_axis(B->get_rank());
-   return reduce_n_wise(A, LO.get_function(), B, axis);
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
+   return reduce_n_wise(A, LO, B, axis);
 }
 //-----------------------------------------------------------------------------
