@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "Bif_OPER1_REDUCE.hh"
 #include "Bif_OPER1_SCAN.hh"
 #include "LvalCell.hh"
+#include "Macro.hh"
 #include "Workspace.hh"
 
 Bif_OPER1_SCAN    Bif_OPER1_SCAN ::_fun;
@@ -67,7 +68,7 @@ Value_P Z(shape_Z, LOC);
       {
         if (shape_B.get_shape_item(axis) > 1)   LENGTH_ERROR;
 
-        Z->set_default(*B.get());
+        Z->set_default(*B.get(), LOC);
         Z->check_value(LOC);
         return Token(TOK_APL_VALUE1, Z);
       }
@@ -96,7 +97,7 @@ ShapeItem inc_2 = 0;              // increment after result m*l items
              if (rep_counts[m] == 1)   // copy items from B
                 {
                   loop(l, shape_Z3.l())
-                      Z->next_ravel()->init(cB[l], Z.getref());
+                      Z->next_ravel()->init(cB[l], Z.getref(), LOC);
                   cB += inc_1;
                 }
              else                      // init items
@@ -109,7 +110,7 @@ ShapeItem inc_2 = 0;              // increment after result m*l items
                   else
                      {
                        loop(l, shape_Z3.l())
-                           Z->next_ravel()->init_type(fill[l], Z.getref());
+                           Z->next_ravel()->init_type(fill[l], Z.getref(), LOC);
                      }
                 }
            }
@@ -117,16 +118,18 @@ ShapeItem inc_2 = 0;              // increment after result m*l items
         cB += inc_2;
       }
 
-   Z->set_default(*B.get());
+   Z->set_default(*B.get(), LOC);
 
    Z->check_value(LOC);
    return Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 Token
-Bif_SCAN::scan(Function * LO, Value_P B, Axis axis)
+Bif_SCAN::scan(Token & _LO, Value_P B, Axis axis)
 {
+Function * LO = _LO.get_function();
    Assert(LO);
+
    if (!LO->has_result())   DOMAIN_ERROR;
 
    if (B->get_rank() == 0)      return Token(TOK_APL_VALUE1, B->clone(LOC));
@@ -160,61 +163,76 @@ ErrorCode (Cell::*assoc_f2)(Cell *, const Cell *) const = LO->get_assoc();
               Cell * cZ = Z->next_ravel();
               if (m == 0)
                  {
-                   cZ->init(*cB++, Z.getref());
+                   cZ->init(*cB++, Z.getref(), LOC);
                  }
               else
                  {
 
                    Value_P AA(LOC);
-                   AA->next_ravel()->init(cZ[-shape_Z3.l()], AA.getref());
+                   AA->next_ravel()->init(cZ[-shape_Z3.l()], AA.getref(), LOC);
+                   AA->set_complete();
 
                    Value_P BB(LOC);
-                   BB->next_ravel()->init(*cB++, BB.getref());
+                   BB->next_ravel()->init(*cB++, BB.getref(), LOC);
+                   BB->set_complete();
 
                    Token tok = LO->eval_AB(AA, BB);
                    if (!tok.is_apl_val())   return tok;
 
                    Value_P ZZ = tok.get_apl_val();
-                   cZ->init(ZZ->get_ravel(0), ZZ.getref());
+                   cZ->init(ZZ->get_ravel(0), ZZ.getref(), LOC);
+                   ZZ->set_complete();
                  }
             }
+        Z->check_value(LOC);
         return Token(TOK_APL_VALUE1, Z);
       }
 
+   // non-trivial reduce (len > 1)
+   //
 const Shape3 Z3(B->get_shape(), axis);
+   if (LO->may_push_SI())   // user defined LO
+      {
+        Value_P X4(4, LOC);
+        new (X4->next_ravel())   IntCell(axis + Workspace::get_IO());
+        new (X4->next_ravel())   IntCell(Z3.h());
+        new (X4->next_ravel())   IntCell(Z3.m());
+        new (X4->next_ravel())   IntCell(Z3.l());
+        X4->check_value(LOC);
+        return Macro::Z__LO_SCAN_X4_B->eval_LXB(_LO, X4, B);
+      }
 
-   return Bif_REDUCE::do_reduce(B->get_shape(), Z3, -1, LO, axis, B, m_len);
+   if (B->get_shape().is_empty())   return LO->eval_identity_fun(B, axis);
+
+   return Bif_REDUCE::do_reduce(B->get_shape(), Z3, -1, LO, B, m_len);
 }
 //-----------------------------------------------------------------------------
 Token
 Bif_OPER1_SCAN::eval_AXB(Value_P A,
                              Value_P X, Value_P B)
 {
-const Rank axis = X->get_single_axis(B->get_rank());
-
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
    return expand(A, B, axis);
 }
 //-----------------------------------------------------------------------------
 Token
 Bif_OPER1_SCAN::eval_LXB(Token & LO, Value_P X, Value_P B)
 {
-const Rank axis = X->get_single_axis(B->get_rank());
-   return scan(LO.get_function(), B, axis);
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
+   return scan(LO, B, axis);
 }
 //-----------------------------------------------------------------------------
 Token
-Bif_OPER1_SCAN1::eval_AXB(Value_P A,
-                             Value_P X, Value_P B)
+Bif_OPER1_SCAN1::eval_AXB(Value_P A, Value_P X, Value_P B)
 {
-const Rank axis = X->get_single_axis(B->get_rank());
-
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
    return expand(A, B, axis);
 }
 //-----------------------------------------------------------------------------
 Token
 Bif_OPER1_SCAN1::eval_LXB(Token & LO, Value_P X, Value_P B)
 {
-const Rank axis = X->get_single_axis(B->get_rank());
-   return scan(LO.get_function(), B, axis);
+const Rank axis = Value::get_single_axis(X.get(), B->get_rank());
+   return scan(LO, B, axis);
 }
 //-----------------------------------------------------------------------------
