@@ -238,7 +238,7 @@ const uint32_t nelm = val.element_count();
              result.append(Unicode(offset >> 24 & 0xFF));
 
              const Cell & cell = val.get_ravel(e);
-             if (cell.is_character_cell() || cell.is_numeric())
+             if (cell.is_simple_cell())
                 {
                   // a non-pointer sub value: 16 byte header,
                   // and 1-16 data bytes, padded up to 16 bytes,
@@ -265,10 +265,10 @@ const uint32_t nelm = val.element_count();
         loop(e, nelm)
            {
              const Cell & cell = val.get_ravel(e);
-             if (cell.is_character_cell() || cell.is_numeric())
+             if (cell.is_simple_cell())
                 {
                   Value_P sub_val(LOC);
-                  sub_val->get_ravel(0).init(cell, sub_val.getref());
+                  sub_val->get_ravel(0).init(cell, sub_val.getref(), LOC);
 
                   const CDR_type sub_type = sub_val->get_CDR_type();
                   const int sub_len = sub_val->total_size_brutto(sub_type);
@@ -482,7 +482,6 @@ const uint8_t * ravel = data + 16 + 4*rank;
         Assert(0 && "Bad/unsupported CDR type");
       }
 
-   ret->set_default_Zero();
    ret->check_value(LOC);
    return ret;
 }
