@@ -37,6 +37,7 @@
 #include "Output.hh"
 #include "Parser.hh"
 #include "Prefix.hh"
+#include "Quad_FX.hh"
 #include "Quad_TF.hh"
 #include "StateIndicator.hh"
 #include "Svar_DB.hh"
@@ -47,11 +48,8 @@
 #include "Value.icc"
 #include "Workspace.hh"
 
-#ifndef PATH_MAX
-#define PATH_MAX 4096
-#endif
-
 int Command::boxing_format = -1;
+ShapeItem Command::APL_expression_count = 0;
 
 vector<Command::user_command> Command::user_commands;
 
@@ -63,11 +61,8 @@ UCS_string line;
 bool eof = false;
    InputMux::get_line(LIM_ImmediateExecution, Workspace::get_prompt(),
                       line, eof, LineInput::get_history());
-   if (eof)
-      {
-        CERR << "EOF at " << LOC << endl;
-        return;
-      }
+
+   if (eof) CERR << "EOF at " << LOC << endl;
 
    process_line(line);
 }
@@ -101,6 +96,7 @@ Command::process_line(UCS_string & line)
         default: ;
       }
 
+   ++APL_expression_count;
    do_APL_expression(line);
 }
 //-----------------------------------------------------------------------------
@@ -127,7 +123,7 @@ vector<UCS_string> args;
       }
 
 #define cmd_def(cmd_str, code, _arg, _hint) \
-   if (cmd.starts_iwith(cmd_str)) { code; return true; }
+   if (cmd.starts_iwith(cmd_str)) { code; return false; }
 #include "Command.def"
 
    // check for user defined commands...
@@ -171,7 +167,7 @@ Executable * statements = 0;
       {
         CERR << "*** Command::process_line() caught other exception ***"
              << endl;
-        return;
+        cmd_OFF(0);
       }
 
    if (statements == 0)
@@ -200,9 +196,7 @@ Executable * statements = 0;
                 StateIndicator * si = Workspace::SI_top();
                 if (si == 0)   break;   // SI empty
 
-                const Executable * exec = si->get_executable();
-                Assert(exec);
-                goon = exec->get_parse_mode() != PM_STATEMENT_LIST;
+                goon = si->get_parse_mode() != PM_STATEMENT_LIST;
                 si->escape();   // pop local vars of user defined functions
                 Workspace::pop_SI(LOC);
               }
@@ -230,23 +224,23 @@ Executable * statements = 0;
          //
          if (token.get_tag() == TOK_SI_PUSHED)   continue;
 
-         // maybe call EOC handler and repeat if true returned
-         //
-       check_EOC:
-         if (Workspace::SI_top()->call_eoc_handler(token))   continue;
+check_EOC:
+         if (Workspace::SI_top()->is_safe_execution_start())
+            {
+              Quad_EC::eoc(token);
+            }
 
          // the far most frequent cases are TC_VALUE and TOK_VOID
          // so we handle them first.
          //
          if (token.get_Class() == TC_VALUE || token.get_tag() == TOK_VOID )
             {
-              if (Workspace::SI_top()->get_executable()
-                             ->get_parse_mode() == PM_STATEMENT_LIST)
+              if (Workspace::SI_top()->get_parse_mode() == PM_STATEMENT_LIST)
                  {
-                   if (attention_raised)
+                   if (attention_is_raised())
                       {
-                        attention_raised = false;
-                        interrupt_raised = false;
+                        clear_attention_raised(LOC);
+                        clear_interrupt_raised(LOC);
                         ATTENTION;
                       }
 
@@ -266,10 +260,10 @@ Executable * statements = 0;
 
                 copy_1(prefix.tos().tok, token, LOC);
               }
-              if (attention_raised)
+              if (attention_is_raised())
                  {
-                   attention_raised = false;
-                   interrupt_raised = false;
+                   clear_attention_raised(LOC);
+                   clear_interrupt_raised(LOC);
                    ATTENTION;
                  }
 
@@ -278,6 +272,16 @@ Executable * statements = 0;
 
          if (token.get_tag() == TOK_BRANCH)
             {
+              const Function_Line line = Function_Line(token.get_int_val());
+              if (line == Function_Retry                                     &&
+                  Workspace::SI_top()->get_parse_mode() == PM_STATEMENT_LIST &&
+                  Workspace::SI_top()->get_parent())
+                 {
+                   Workspace::pop_SI(LOC);
+                   Workspace::SI_top()->retry(LOC);
+                   continue;
+                 }
+
               StateIndicator * si = Workspace::SI_top_fun();
 
               if (si == 0)
@@ -292,10 +296,7 @@ Executable * statements = 0;
               //
               while (si != Workspace::SI_top())   Workspace::pop_SI(LOC);
 
-              const Function_Line line = Function_Line(token.get_int_val());
-
-              if (line == Function_Retry)   si->retry(LOC);
-              else                          si->goon(line, LOC);
+              si->goon(line, LOC);
               continue;
             }
 
@@ -309,15 +310,11 @@ Executable * statements = 0;
                     StateIndicator * si = Workspace::SI_top();
                     if (si == 0)   break;   // SI empty
 
-                    const Executable * exec = si->get_executable();
-                    Assert(exec);
-                    goon = exec->get_parse_mode() != PM_STATEMENT_LIST;
+                    goon = si->get_parse_mode() != PM_STATEMENT_LIST;
                     si->escape();   // pop local vars of user defined functions
                     Workspace::pop_SI(LOC);
-              }
+                  }
               return;
-
-              Assert(0 && "not reached");
             }
 
          if (token.get_tag() == TOK_ERROR)
@@ -334,27 +331,24 @@ Executable * statements = 0;
 
               // clear attention and interrupt flags
               //
-              attention_raised = false;
-              interrupt_raised = false;
+              clear_attention_raised(LOC);
+              clear_interrupt_raised(LOC);
 
-              // check for safe execution mode. Entries in safe execution mode
-              // can be far above the current SI entry. The EOC handler will
+              // check for safe execution mode. Unroll all SI entries that
+              // have the same safe_execution_count, except the last
               // unroll the SI stack.
               //
-              for (StateIndicator * si = Workspace::SI_top();
-                   si; si = si->get_parent())
-                  {
-                    if (si->get_safe_execution())
-                       {
-                         // pop SI entries above the entry with safe_execution
-                         //
-                         while (Workspace::SI_top() != si)
-                            {
-                              Workspace::pop_SI(LOC);
-                            }
+              if (Workspace::SI_top()->get_safe_execution())
+                 {
+                  StateIndicator * si = Workspace::SI_top();
+                   while (si->get_parent() && si->get_safe_execution() ==
+                          si->get_parent()->get_safe_execution())
+                      {
+                        si = si->get_parent();
+                        Workspace::pop_SI(LOC);
+                      }
 
-                         goto check_EOC;
-                       }
+                    goto check_EOC;
                   }
 
               // if suspend is not allowed then pop all SI entries that
@@ -442,13 +436,13 @@ int format = arg.atoi();
         case  7: case   8: case   9:
         case 20: case  21: case  22:
         case 23: case  24: case  25:
+        case 29:
                  boxing_format = format;
                  return;
-
-        default:   out << "Bad ]BOXING parameter " << arg
-                       << " (valid values are: OFF, 2...4, 7...9, and 20...25)"
-                       << endl;
       }
+
+   out << "Bad ]BOXING parameter " << arg
+       << ". Valid values are: OFF, 2-4, 7-9, 20-25, and 29." << endl;
 }
 //-----------------------------------------------------------------------------
 void 
@@ -461,19 +455,17 @@ Command::cmd_CHECK(ostream & out)
      int stale = Workspace::cleanup_expunged(CERR, erased);
      if (stale)
         {
-          CERR << "WARNING - " << stale << " stale functions ("
+          out << "WARNING - " << stale << " stale functions ("
                << (erased ? "" : "not ") << "erased)" << endl;
         }
-     else CERR << "OK      - no stale functions" << endl;
+     else out << "OK      - no stale functions" << endl;
    }
 
    {
      const int stale = Value::print_stale(CERR);
      if (stale)
         {
-          char cc[200];
-          snprintf(cc, sizeof(cc), "ERROR   - %d stale values", stale);
-          out << cc << endl;
+          out << "ERROR   - " << stale << " stale values" << endl;
           IO_Files::apl_error(LOC);
         }
      else out << "OK      - no stale values" << endl;
@@ -482,9 +474,7 @@ Command::cmd_CHECK(ostream & out)
      const int stale = IndexExpr::print_stale(CERR);
      if (stale)
         {
-          char cc[200];
-          snprintf(cc, sizeof(cc), "ERROR   - %d stale indices", stale);
-          out << cc << endl;
+          out << "ERROR   - " << stale << " stale indices" << endl;
           IO_Files::apl_error(LOC);
         }
      else out << "OK      - no stale indices" << endl;
@@ -494,7 +484,7 @@ Command::cmd_CHECK(ostream & out)
 void 
 Command::cmd_CONTINUE(ostream & out)
 {
-   Workspace::wsid(out, UCS_string("CONTINUE"));
+   Workspace::wsid(out, UCS_string("CONTINUE"), false);
 
 vector<UCS_string> vcont;
    Workspace::save_WS(out, vcont);
@@ -533,7 +523,7 @@ UCS_string wname = lib_ws.back();
 UTF8_string filename = LibPaths::get_lib_filename(libref, wname, true,
                                                   ".xml", ".apl");
 
-int result = unlink(filename.c_str());
+const int result = unlink(filename.c_str());
    if (result)
       {
         out << wname << " NOT DROPPED: " << strerror(errno) << endl;
@@ -586,7 +576,7 @@ Command::cmd_KEYB(ostream & out)
 "║       ║ Q  ║ W⍹ ║ E⋸ ║ R  ║ T⍨ ║ Y¥ ║ U  ║ I⍸ ║ O⍥ ║ P⍣ ║ {⍞ ║ }⍬ ║  |⊣  ║\n"
 "║  TAB  ║ q? ║ w⍵ ║ e∈ ║ r⍴ ║ t∼ ║ y↑ ║ u↓ ║ i⍳ ║ o○ ║ p⋆ ║ [← ║ ]→ ║  \\⊢  ║\n"
 "╠═══════╩═╦══╩═╦══╩═╦══╩═╦══╩═╦══╩═╦══╩═╦══╩═╦══╩═╦══╩═╦══╩═╦══╩═╦══╩══════╣\n"
-"║ (CAPS   ║ A⍶ ║ S  ║ D  ║ F  ║ G  ║ H⍙ ║ J⍤ ║ K  ║ L⌷ ║ :≡ ║ \"≢ ║         ║\n"
+"║ (CAPS   ║ A⍶ ║ S  ║ D  ║ F  ║ G  ║ H  ║ J⍤ ║ K  ║ L⌷ ║ :≡ ║ \"≢ ║         ║\n"
 "║  LOCK)  ║ a⍺ ║ s⌈ ║ d⌊ ║ f_ ║ g∇ ║ h∆ ║ j∘ ║ k' ║ l⎕ ║ ;⍎ ║ '⍕ ║ RETURN  ║\n"
 "╠═════════╩═══╦╩═══╦╩═══╦╩═══╦╩═══╦╩═══╦╩═══╦╩═══╦╩═══╦╩═══╦╩═══╦╩═════════╣\n"
 "║             ║ Z  ║ Xχ ║ C¢ ║ V  ║ B£ ║ N  ║ M  ║ <⍪ ║ >⍙ ║ ?⍠ ║          ║\n"
@@ -668,20 +658,20 @@ Command::cmd_HELP(ostream & out)
      }
 
    out << endl << "System variables:" << endl;
-#define ro_sv_def(x, txt)                                           \
+#define ro_sv_def(x, _str, txt)                                           \
    { const UCS_string & ucs = Workspace::get_v_ ## x().get_name();  \
-        out << "      " << setw(8) << ucs << #txt << endl; }
-#define rw_sv_def(x, txt)                                           \
+        out << "      " << setw(8) << ucs << txt << endl; }
+#define rw_sv_def(x, _str, txt)                                           \
    { const UCS_string & ucs = Workspace::get_v_ ## x().get_name();  \
-        out << "      " << setw(8) << ucs << #txt << endl; }
+        out << "      " << setw(8) << ucs << txt << endl; }
 #include "SystemVariable.def"
 
    out << endl << "System functions:" << endl;
-#define ro_sv_def(x, txt)
-#define rw_sv_def(x, txt)
-#define sf_def(q, txt) { const char * qu = #q;                      \
-   if (!strncmp(qu, "Quad_", 5))                                    \
-        out << "      ⎕" << setw(8) << UCS_string(qu + 5) << #txt << endl; }
+#define ro_sv_def(x, _str, _txt)
+#define rw_sv_def(x, _str, _txt)
+#define sf_def(q, _str, txt) \
+        out << "      " << setw(8) << q::fun->get_name() << txt << endl;
+
 #include "SystemVariable.def"
 }
 //-----------------------------------------------------------------------------
@@ -994,15 +984,15 @@ vector<UCS_string> directories;
    //
 DynArray(const UCS_string *, directory_names, directories.size());
    loop(a, directories.size())   directory_names[a] = &directories[a];
-   UCS_string::sort_names(directory_names, directories.size());
+   UCS_string::sort_names(directory_names.get_data(), directories.size());
 
 DynArray(const UCS_string *, apl_filenames, apl_files.size());
    loop(a, apl_files.size())   apl_filenames[a] = &apl_files[a];
-   UCS_string::sort_names(apl_filenames, apl_files.size());
+   UCS_string::sort_names(apl_filenames.get_data(), apl_files.size());
 
 DynArray(const UCS_string *, xml_filenames, xml_files.size());
    loop(x, xml_files.size())   xml_filenames[x] = &xml_files[x];
-   UCS_string::sort_names(xml_filenames, xml_files.size());
+   UCS_string::sort_names(xml_filenames.get_data(), xml_files.size());
 
    // 4. list directories first, then files
    //
@@ -1016,14 +1006,14 @@ int count = 0;
 
    for (int a = 0, x = 0;;)
        {
-         if (a >= apl_files.size())   // end of apl_files reached
+         if (a >= (int)apl_files.size())   // end of apl_files reached
             {
               loop(xx, xml_files.size() - x)
                   filenames[count++] = xml_filenames[x + xx];
               break;
             }
 
-         if (x >= xml_files.size())   // end of xml_files reached
+         if (x >= (int)xml_files.size())   // end of xml_files reached
             {
               loop(aa, apl_files.size() - a)
                   filenames[count++] = apl_filenames[a + aa];
@@ -1052,12 +1042,12 @@ int count = 0;
    //
    enum { tabsize = 4 };
 vector<int> col_width;
-   UCS_string::compute_column_width(col_width, filenames, count, tabsize,
-                                    Workspace::get_PrintContext().get_PW());
+   UCS_string::compute_column_width(col_width, filenames.get_data(), count,
+                                    tabsize, Workspace::get_PW());
 
    loop(c, count)
       {
-        const int col = c % col_width.size();
+        const size_t col = c % col_width.size();
         out << *filenames[c];
         if (col == (col_width.size() - 1) || c == (count - 1))
            {
@@ -1556,7 +1546,7 @@ Symbol * sym = 0;
 Token_string tos;
    {
      UCS_string data1(data, idx, data.size() - idx);
-     Tokenizer tokenizer(PM_EXECUTE, LOC);
+     Tokenizer tokenizer(PM_EXECUTE, LOC, false);
      if (tokenizer.tokenize(data1, tos) != E_NO_ERROR)   return;
    }
  
@@ -1569,7 +1559,7 @@ const ShapeItem ec = val->element_count();
    loop(e, ec)
       {
         const TokenTag tag = tos[e].get_tag();
-        Cell * C = &val->get_ravel(e);
+        Cell * C = val->next_ravel();
         if      (tag == TOK_INTEGER)  new (C) IntCell(tos[e].get_int_val());
         else if (tag == TOK_REAL)     new (C) FloatCell(tos[e].get_flt_val());
         else if (tag == TOK_COMPLEX)  new (C)
@@ -1578,8 +1568,10 @@ const ShapeItem ec = val->element_count();
         else FIXME;
       }
 
+   val->check_value(LOC);
+
    Assert(sym);
-   sym->assign(val, LOC);
+   sym->assign(val, false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -1607,19 +1599,34 @@ Symbol * sym = 0;
    
    Log(LOG_command_IN)
       {
-        CERR << endl << var_name << " rank " << shape.get_rank() << " IS '";
+        CERR << endl << var_name << " shape " << shape << " IS: '";
         loop(j, data.size() - idx)   CERR << data[idx + j];
         CERR << "'" << endl;
       }
 
 Value_P val(shape, LOC);
+const ShapeItem ec = val->element_count();
    new (&val->get_ravel(0)) CharCell(UNI_ASCII_SPACE);   // prototype
 
-const ShapeItem ec = val->element_count();
-   loop(e, ec)   new (&val->get_ravel(e)) CharCell(data[idx + e]);
+ShapeItem padded = 0;
+   loop(e, ec)
+      {
+        Unicode uni = UNI_ASCII_SPACE;
+        if (e < (data.size() - idx))   uni = data[e + idx];
+        else                           ++padded;
+         new (&val->get_ravel(e)) CharCell(uni);
+      }
+
+   if (padded)
+      {
+        CERR << "WARNING: ATF Record for " << var_name << " is broken ("
+             << padded << " spaces added)" << endl;
+      }
+
+   val->check_value(LOC);
 
    Assert(sym);
-   sym->assign(val, LOC);
+   sym->assign(val, false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -1647,7 +1654,10 @@ UCS_string var_or_fun;
 
    var_or_fun = Quad_TF::tf2_inv(data1);
 
-   Assert(var_or_fun.size());
+   if (var_or_fun.size() == 0)
+      {
+        CERR << "ERROR: inverse 2 ⎕TF failed for '" << data1 << ";" << endl;
+      }
 }
 //-----------------------------------------------------------------------------
 void
@@ -1698,12 +1708,9 @@ Command::transfer_context::add(const UTF8 * str, int len)
 
 #if 0
    // helper to print the uni_to_cp_map when given a cp_to_uni_map.
-   // The print-out can be triggered by )IN file.atf and then sorted
-   // with the GNU/Linux command 'sort'.
    //
-   loop(q, 128)
-       fprintf(stderr, "  { 0x%4.4X, %u },\n", cp_to_uni_map[q], q + 128);
-   exit(0);
+   Avec::print_inverse_IBM_quad_AV();
+   DOMAIN_ERROR;
 #endif
 
 const Unicode * cp_to_uni_map = Avec::IBM_quad_AV();
@@ -1826,6 +1833,7 @@ const bool have_trailing_blank = replace_count && user.last() == ' ';
             {
              const Symbol * sym = all_symbols[s];
              if (sym->is_erased())   continue;
+             if (!sym->is_used())    continue;
 
               const UCS_string & sym_name = sym->get_name();
               if (!sym_name.starts_with(user))   continue;
@@ -1951,17 +1959,14 @@ int qpos = -1;
         UCS_string qxx(user, qpos, user.size() - qpos);
         vector<UCS_string>matches;
 
-#define ro_sv_def(q, txt) { const char * qu = #q;                  \
-   if (!strncmp(qu, "Quad_", 5)) { UCS_string ustr(qu + 5);   \
-        if (ustr.starts_iwith(qxx)) matches.push_back(ustr); } }
+#define ro_sv_def(_q, str, _txt) { UCS_string ustr(str);   \
+   if (ustr.size() && ustr.starts_iwith(qxx)) matches.push_back(ustr); }
 
-#define rw_sv_def(q, txt) { const char * qu = #q;                  \
-   if (!strncmp(qu, "Quad_", 5)) { UCS_string ustr(qu + 5);   \
-        if (ustr.starts_iwith(qxx)) matches.push_back(ustr); } }
+#define rw_sv_def(_q, str, _txt) { UCS_string ustr(str);   \
+   if (ustr.size() && ustr.starts_iwith(qxx)) matches.push_back(ustr); }
 
-#define sf_def(q, txt) { const char * qu = #q;                  \
-   if (!strncmp(qu, "Quad_", 5)) { UCS_string ustr(qu + 5);   \
-        if (ustr.starts_iwith(qxx)) matches.push_back(ustr); } }
+#define sf_def(_q, str, _txt) { UCS_string ustr(str);   \
+   if (ustr.size() && ustr.starts_iwith(qxx)) matches.push_back(ustr); }
 
 #include "SystemVariable.def"
 
