@@ -42,13 +42,38 @@
 
 uint64_t total_memory = 0;
 
+//-----------------------------------------------------------------------------
+static bool attention_raised = false;
+static uint64_t attention_count = 0;
+
+extern void set_attention_raised(const char * loc)
+{
+   attention_raised = true;
+}
+extern void clear_attention_raised(const char * loc)
+{
+   attention_raised = false;
+}
+bool attention_is_raised()
+{
+   return attention_raised;
+}
+//-----------------------------------------------------------------------------
 APL_time_us interrupt_when = 0;
 uint64_t interrupt_count = 0;
-bool interrupt_raised = false;
-
-bool attention_raised = false;
-uint64_t attention_count = 0;
-
+static bool interrupt_raised = false;
+void set_interrupt_raised(const char * loc)
+{
+   interrupt_raised = true;
+}
+void clear_interrupt_raised(const char * loc)
+{
+   interrupt_raised = false;
+}
+bool interrupt_is_raised()
+{
+   return interrupt_raised;
+}
 //-----------------------------------------------------------------------------
 void
 init_1(const char * argv0, bool log_startup)
@@ -83,10 +108,11 @@ rlimit rl;
 void
 init_2(bool log_startup)
 {
+const int retry_max = uprefs.emacs_mode ? 15 : 5;
+
    Output::init(log_startup);
-   Svar_DB::init(LibPaths::get_APL_bin_path(),
-                 LibPaths::get_APL_bin_name(),
-                 log_startup, uprefs.system_do_svars);
+   Svar_DB::init(LibPaths::get_APL_bin_path(), LibPaths::get_APL_bin_name(),
+                 retry_max, log_startup, uprefs.system_do_svars);
 
    LineInput::init(true);
 
@@ -264,6 +290,7 @@ IntScalar(APL_Integer val, const char * loc)
 {
 Value_P Z(loc);
    new (Z->next_ravel())   IntCell(val);
+   Z->check_value(LOC);
    return Z;
 }
 //-----------------------------------------------------------------------------
@@ -272,6 +299,7 @@ FloatScalar(APL_Float val, const char * loc)
 {
 Value_P Z(loc);
    new (Z->next_ravel())   FloatCell(val);
+   Z->check_value(LOC);
    return Z;
 }
 //-----------------------------------------------------------------------------
@@ -280,6 +308,7 @@ ComplexScalar(APL_Complex val, const char * loc)
 {
 Value_P Z(loc);
    new (Z->next_ravel()) ComplexCell(val);
+   Z->check_value(LOC);
    return Z;
 }
 //-----------------------------------------------------------------------------
@@ -288,6 +317,7 @@ CharScalar(Unicode uni, const char * loc)
 {
 Value_P Z(loc);
    new (Z->next_ravel()) CharCell(uni);
+   Z->check_value(LOC);
    return Z;
 }
 //-----------------------------------------------------------------------------
@@ -295,7 +325,7 @@ Value_P
 Idx0(const char * loc)
 {
 Value_P Z((ShapeItem)0, loc);
-   new (&Z->get_ravel(0))   IntCell(0);
+   Z->check_value(LOC);
    return Z;
 }
 //-----------------------------------------------------------------------------
@@ -303,7 +333,8 @@ Value_P
 Str0(const char * loc)
 {
 Value_P Z((ShapeItem)0, loc);
-   new (&Z->get_ravel(0))   CharCell(UNI_ASCII_SPACE);
+   Z->set_proto_Spc();
+   Z->check_value(LOC);
    return Z;
 }
 //-----------------------------------------------------------------------------
@@ -312,7 +343,17 @@ Str0_0(const char * loc)
 {
 Shape sh((ShapeItem)0, (ShapeItem)0);
 Value_P Z(sh, loc);
-   new (&Z->get_ravel(0))   CharCell(UNI_ASCII_SPACE);
+   Z->set_proto_Spc();
+   Z->check_value(LOC);
+   return Z;
+}
+//-----------------------------------------------------------------------------
+Value_P
+Idx0_0(const char * loc)
+{
+Shape sh((ShapeItem)0, (ShapeItem)0);
+Value_P Z(sh, loc);
+   Z->check_value(LOC);
    return Z;
 }
 //-----------------------------------------------------------------------------
