@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,34 +28,77 @@
 #include "Workspace.hh"
 
 //-----------------------------------------------------------------------------
-Executable::Executable(const UCS_string & ucs, const char * loc)
-   : alloc_loc(loc)
+Executable::Executable(const UCS_string & ucs,  bool multi_line,
+                       ParseMode pm, const char * loc)
+   : alloc_loc(loc),
+     pmode(pm),
+     refcount(0)
 {
 // { cerr << "Executable " << (void *)this << " created at " << loc << endl; }
+
    // initialize text, stripping carriage returns and line feeds.
    //
-UCS_string line;
-
-   loop(t, ucs.size())
+   if (multi_line)
       {
-        const Unicode uni = ucs[t];
-        switch (uni)
-           {
-             case UNI_ASCII_CR:                                         break;
-             case UNI_ASCII_LF: text.push_back(line);   line.clear();   break;
-             default: line.append(uni);
-          }
-      }
+        UCS_string line;
 
-   if (line.size())   text.push_back(line);
+        loop(t, ucs.size())
+           {
+             const Unicode uni = ucs[t];
+             switch (uni)
+                {
+                  case UNI_ASCII_CR:                         break;
+                  case UNI_ASCII_LF: text.push_back(line);
+                                     line.clear();           break;
+                  default:           line.append(uni);
+               }
+           }
+
+        if (line.size())   text.push_back(line);
+      }
+   else
+      {
+        text.push_back(ucs);
+      }
 }
 //-----------------------------------------------------------------------------
-Executable::Executable(Fun_signature sig, const UCS_string & fname,
-           const UCS_string & lambda_text, const char * loc)
-   : alloc_loc(loc)
+Executable::Executable(Fun_signature sig, int lambda_num,
+                       const UCS_string & lambda_text, const char * loc)
+   : alloc_loc(loc),
+     pmode(PM_FUNCTION),
+     refcount(0)
 {
-   text.push_back(UserFunction_header::lambda_header(sig, fname));
-   text.push_back(lambda_text);
+UCS_string header = UserFunction_header::lambda_header(sig, lambda_num);
+
+   // remove local vars from lambda_text and add them to the header
+   //
+ShapeItem last_semi = -1;
+   for (ShapeItem t = lambda_text.size() - 1; t >= 0; --t)
+       {
+         const Unicode cc = lambda_text[t];
+         if (cc == UNI_ASCII_SPACE)      continue;
+         if (Avec::is_symbol_char(cc))   continue;
+         if (Avec::is_quad(cc))          continue;
+         if (cc == UNI_ASCII_SEMICOLON)   last_semi = t;
+         else                             break;
+       }
+
+   if (last_semi != -1)
+      {
+        for (ShapeItem t = last_semi; t < lambda_text.size(); ++t)
+            header.append(lambda_text[t]);
+        text.push_back(header);
+
+        UCS_string new_body(lambda_text);
+        new_body.shrink(last_semi);
+        new_body.remove_trailing_whitespaces();
+        text.push_back(new_body);
+      }
+   else
+      {
+        text.push_back(header);
+        text.push_back(lambda_text);
+     }
 }
 //-----------------------------------------------------------------------------
 Executable::~Executable()
@@ -75,90 +118,52 @@ Executable::clear_body()
    loop(b, body.size())
       {
         body[b].extract_apl_val(LOC);
-      }
 
-   clear_lambdas();
-   body.clear();
-}
-//-----------------------------------------------------------------------------
-void
-Executable::clear_lambdas()
-{
-   while (named_lambdas.size())
-      {
-        // remove named lambda, unless it is bound to a symbol
-        const UserFunction * ufun = named_lambdas.back();
-        if (ufun == 0)   continue;
-
-        const Function * fun = ufun;
-
-        const UCS_string & ufun_name = ufun->get_name();
-        const Symbol * ufun_sym = Workspace::lookup_existing_symbol(ufun_name);
-
-        bool used = false;
-        if (ufun_sym)
+        if (body[b].is_function())
            {
-              loop(v, ufun_sym->value_stack_size())
-                 {
-                   const ValueStackItem vs = (*ufun_sym)[v];
-                   if ((vs.name_class == NC_FUNCTION ||
-                        vs.name_class == NC_OPERATOR) &&
-                        vs.sym_val.function == fun)
-                      {
-                        // the named lambda was assigned to ufun_name so we
-                        // should not delete it.
-                        //
-                        used = true;
-                        break;
-                      }
-                 }
+              Function * fun = body[b].get_function();
+              UserFunction * ufun = fun->get_ufun1();
+              if (ufun && ufun->is_lambda())   ufun->decrement_refcount(LOC);
            }
-
-        if (!used)   delete named_lambdas.back();
-        named_lambdas.pop_back();
       }
 
-   while (unnamed_lambdas.size())
-      {
-        delete unnamed_lambdas.back();
-        unnamed_lambdas.pop_back();
-      }
+   body.clear();   // resize to 0
 }
 //-----------------------------------------------------------------------------
-void
+ErrorCode
 Executable::parse_body_line(Function_Line line, const UCS_string & ucs_line,
-                            bool trace, const char * loc)
+                            bool trace, bool tolerant, const char * loc,
+                            bool macro)
 {
    Log(LOG_UserFunction__set_line)
       CERR << "[" << line << "]" << ucs_line << endl;
 
 Token_string in;
-const Parser parser(get_parse_mode(), loc);
-   {
-     ErrorCode ec = parser.parse(ucs_line, in);
-     if (ec)
-        {
-          if (ec == E_NO_TOKEN)
-             {
-               Error error(ec, LOC);
-               throw error;
+const Parser parser(get_parse_mode(), loc, macro);
+ErrorCode ec = parser.parse(ucs_line, in);
+   if (ec)
+      {
+        if (tolerant)   return ec;
 
-             }
+        if (ec == E_NO_TOKEN)
+           {
+             Error error(ec, LOC);
+             throw error;
+           }
 
-     if (ec != E_NO_ERROR)   throw_parse_error(ec, LOC, LOC);
-        }
-   }
+        if (ec != E_NO_ERROR)   throw_parse_error(ec, LOC, LOC);
+      }
 
-   parse_body_line(line, in, trace, loc);
+   return parse_body_line(line, in, trace, tolerant, loc);
 } 
 //-----------------------------------------------------------------------------
-void
+ErrorCode
 Executable::parse_body_line(Function_Line line, const Token_string & in,
-                            bool trace, const char * loc)
+                            bool trace,  bool tolerant, const char * loc)
 {
 Source<Token> src(in);
 
-   // handle labs (if any)
+   // handle labels (if any)
    //
    if (get_parse_mode() == PM_FUNCTION &&
        src.rest() >= 2                 &&
@@ -205,6 +210,8 @@ Token_string out;
                   tok.get_tag() != TOK_L_CURLY &&
                   tok.get_tag() != TOK_R_CURLY)
                  {
+                   if (tolerant)   return E_SYNTAX_ERROR;
+
                    CERR << "Line " << line << endl
                         << "Offending token: (tag > TC_MAX_PERM) "
                         << tok.get_tag() << " " << tok << endl
@@ -237,6 +244,7 @@ Token_string out;
       } 
 
    loop(t, out.size())   body.append(out[t], LOC);
+   return E_NO_ERROR;
 }
 //-----------------------------------------------------------------------------
 Token
@@ -365,7 +373,7 @@ Function_PC end = start;
 
    Assert(start   <= pc_from_to.low);
    Assert(pc_from_to.low  <= pc_from_to.high);
-   Assert(pc_from_to.high <= end);
+   if (pc_from_to.high > end)   pc_from_to.high = end;
 
    // if the error was caused by some function being executed then
    // the right arg of the function was OK and we skip it
@@ -389,6 +397,11 @@ int len_left = 0;      // characters before the first caret
 int len_between = 0;   // distance between the carets
    for (int q = end - 1; q >= start; --q)
        {
+         // avoid duplicate ∘ in ∘.f
+         //
+         if (body[q].get_tag() == TOK_JOT && q > 0 &&
+             body[q - 1].get_tag() == TOK_OPER2_OUTER)   continue;
+
          // Note: Token::error_info returns -len if it inserts a space.
          // Such an inserted space counts for the previous token.
          // The previous token is q + 1 since we count down.
@@ -445,20 +458,21 @@ Executable::unmark_all_values() const
    loop (b, body.size())
       {
         const Token & tok = body[b];
-        if (tok.get_ValueType() != TV_VAL)      continue;
+        if (tok.get_ValueType() == TV_VAL)
+           {
+             Value_P value = tok.get_apl_val();
+             if (!!value)   value->unmark();
+           }
 
-        Value_P value = tok.get_apl_val();
-        if (!!value)   value->unmark();
-      }
-
-   loop (n, named_lambdas.size())
-      {
-        named_lambdas[n]->unmark_all_values();
-      }
-
-   loop (u, unnamed_lambdas.size())
-      {
-        unnamed_lambdas[u]->unmark_all_values();
+        if (tok.get_ValueType() == TV_FUN)
+           {
+             Function * fun = tok.get_function();
+             UserFunction * ufun = fun->get_ufun1();
+             if (ufun && ufun->is_lambda())
+                {
+                  ufun->unmark_all_values();
+                }
+           }
       }
 }
 //-----------------------------------------------------------------------------
@@ -472,7 +486,7 @@ int count = 0;
         const Token & tok = body[b];
         if (tok.get_ValueType() != TV_VAL)      continue;
 
-        if (tok.get_apl_val()->is_or_contains(value))
+        if (Value::is_or_contains(tok.get_apl_val().get(), value))
            {
              out << prefix << get_name() << "[" << b << "]" << endl;
              ++count;
@@ -501,22 +515,17 @@ bool have_curly = false;
 
    if (!have_curly)   return;   // no { ... } in this body
 
-   // setup_lambdas() may be called multiple times for the same executable.
-   // remove old lambdas...
-   //
-   clear_lambdas();
-
    // undo the token reversion of the body so that the body token and the
    // text run in the same direction.
    //
-
    reverse_statement_token(body);
 
+int lambda_num = 0;
    loop(b, body.size())
        {
          if (body[b].get_tag() != TOK_L_CURLY)   continue;   // not {
 
-         b = setup_one_lambda(b) - 1;   // -1 due to ++b in loop(b) above
+         b = setup_one_lambda(b, ++lambda_num) - 1;   // -1 due to ++b in loop(b) above
        }
 
    // redo the token reversion of the body so that the body token and the
@@ -524,101 +533,74 @@ bool have_curly = false;
    //
    reverse_statement_token(body);
 
-   adjust_line_starts();
-   Parser::remove_void_token(body);
    Parser::match_par_bra(body, true);
+   adjust_line_starts();
+   Parser::remove_void_token(body);   // do this AFTER adjust_line_starts() !!!
 }
 //-----------------------------------------------------------------------------
 ShapeItem
-Executable::setup_one_lambda(ShapeItem b)
+Executable::setup_one_lambda(ShapeItem b, int lambda_num)
 {
 const ShapeItem bend = b + body[b].get_int_val2();   // the corresponding }
    Assert(bend < body.size());
    Assert(body[bend].get_tag() == TOK_R_CURLY);
 
-   // a named lambda has the form Q←{ ... }
-   //
-   // however, eg. Q←{ ... } / ... is NOT a named lambda but an unnamed
-   // lambda for / whose result is assigned to Q. We need to distinguish
-   // the two cases and remember if the token after } is the start of
-   // the next statement or not.
-   //
-const bool right_curly_at_end = (bend >= body.size() - 1)
-                             ||  body[bend + 1].get_tag() == TOK_DIAMOND
-                             ||  body[bend + 1].get_tag() == TOK_END
-                             ||  body[bend + 1].get_tag() == TOK_ENDL;
-
-const bool is_named = right_curly_at_end                  && 
-                      b >=   2                            &&
-                      body[b - 1].get_tag() == TOK_ASSIGN &&
-                      body[b - 2].get_tag() == TOK_LSYMB;
-
-Symbol * lambda_sym = is_named ? body[b - 2].get_sym_ptr() : 0;
-
    body[b++].clear(LOC);    // invalidate {
    body[bend].clear(LOC);   // invalidate }
 
 Token_string lambda_body;
 
 const Fun_signature signature =
-      compute_lambda_body(lambda_body, b, bend, is_named);
+      compute_lambda_body(lambda_body, b, bend);
 
    // at this point { ... } was copied from body to lambda_body and
    // bend is at the (now invalidated) { token.
-   // check if lambda is named, i.e. Name ← { ... }
    //
-UCS_string lambda_name;
-   if (is_named)   // named lambda
-      {
-        lambda_name = lambda_sym->get_name();
-      }
-   else            // unnamed lambda
-      {
-        lambda_name.append(UNI_LAMBDA);
-        lambda_name.append_number(unnamed_lambdas.size() + 1);
-      }
-
-const UCS_string lambda_text = extract_lambda_text(signature);
+const UCS_string lambda_text = extract_lambda_text(signature, lambda_num - 1);
 
    reverse_statement_token(lambda_body);
    reverse_all_token(lambda_body);
-UserFunction * ufun = new UserFunction(signature, lambda_name,
+
+#if 0  // not yet working
+
+UCS_string ufun_text = UserFunction_header::lambda_header(signature,lambda_num);
+   ufun_text.append(UNI_ASCII_LF);
+   ufun_text.append(lambda_text);
+   ufun_text.append(UNI_ASCII_LF);
+
+Q(ufun_text)
+
+int error_line = -1;
+const char * error_loc = 0;
+const UTF8_string creator("Executable::setup_one_lambda()");
+
+UserFunction * ufun = new UserFunction(ufun_text, LOC, creator, false, false);
+
+   ufun->increment_refcount(LOC);
+
+#else
+
+UserFunction * ufun = new UserFunction(signature, lambda_num,
                                        lambda_text, lambda_body);
+   ufun->increment_refcount(LOC);
 
-   if (signature & SIG_FUN)   // named lambda
-      {
-        named_lambdas.push_back(ufun);
-
-        // put a token for the lambda at thr place where the { was.
-        // That means we replace (in forward notation) e.g.:
-        //
-        // A←{ ... }   by:
-        // A←UFUN      UFUN being a user-defined function with body { ... }
-        //
-        // This is to bind UFUN to A when A←UFUN is executed
-        //
-        move_2(body[bend], ufun->get_token(), LOC);
-
-        // UserFunction::UserFunction has bound the lambda body to
-        // the lambda name. However, we want that to happen when
-        // V←{ ...} is executed, not now. We therefore unbind it here
-        //
-        lambda_sym->set_nc(NC_UNUSED_USER_NAME, 0);
-      }
-   else                       // unnamed lambda
-      {
-        unnamed_lambdas.push_back(ufun);
-        move_2(body[bend], ufun->get_token(), LOC);
-      }
+#endif
+   // put a token for the lambda at the place where the { was.
+   // That replaces, for example, (in forward notation):
+   //
+   // A←{ ... }   by:
+   // A←UFUN      UFUN being a user-defined function with body { ... }
+   //
+   move_2(body[bend], ufun->get_token(), LOC);
 
    return bend + 1;
 }
 //-----------------------------------------------------------------------------
 Fun_signature
 Executable::compute_lambda_body(Token_string & lambda_body,
-                                ShapeItem b, ShapeItem bend, bool is_named)
+                                ShapeItem b, ShapeItem bend)
 {
-int signature = is_named ? SIG_FUN : SIG_NONE;
+int signature = SIG_FUN;
 int level = 0;
 
    for (; b < bend; ++b)
@@ -634,8 +616,8 @@ int level = 0;
               case TOK_ALPHA:   if (!level)   signature |= SIG_A;   // no break
               case TOK_OMEGA:   if (!level)   signature |= SIG_B;         break;
               case TOK_CHI:     if (!level)   signature |= SIG_X;         break;
-              case TOK_ALPHA_U: if (!level)   signature |= SIG_LO;  // no break
-              case TOK_OMEGA_U: if (!level)   signature |= SIG_RO;        break;
+              case TOK_OMEGA_U: if (!level)   signature |= SIG_RO;  // no break
+              case TOK_ALPHA_U: if (!level)   signature |= SIG_LO;        break;
 
               case TOK_DIAMOND: DEFN_ERROR;
               case TOK_BRANCH:  DEFN_ERROR;
@@ -652,7 +634,7 @@ int level = 0;
               //
               Symbol * sym_Z = &Workspace::get_v_LAMBDA();
               lambda_body.append(Token(TOK_LAMBDA, sym_Z), LOC);
-              lambda_body.append(Token(TOK_ASSIGN), LOC);
+              lambda_body.append(Token(TOK_ASSIGN1), LOC);
 
               signature |= SIG_Z;
             }
@@ -660,8 +642,8 @@ int level = 0;
          lambda_body.append(t, LOC);
        }
 
-   // if the lambda has at least one token, then it returns λ.
-   // Otherwise the lambda is result-less
+   // if the lambda has at least one token, then it (is supposed to) return λ.
+   // Otherwise the lambda is empty (and result-less)
    //
    if (signature & SIG_Z)
       {
@@ -681,17 +663,17 @@ int level = 0;
 }
 //-----------------------------------------------------------------------------
 UCS_string
-Executable::extract_lambda_text(Fun_signature signature) const
+Executable::extract_lambda_text(Fun_signature signature, int skip) const
 {
-int skip = unnamed_lambdas.size() + named_lambdas.size();
 UCS_string lambda_text;
    if (signature & SIG_Z)   lambda_text.append_utf8("λ←");
 
 int level = 0;   // {/} nesting level
 bool copying = false;
-
-int tidx = 0;
-int tcol = 0;
+size_t tidx = 0;   // the current line in text[]
+int tcol = 0;      // the current column in text[tidx];
+bool in_single_quotes = false;
+bool in_double_quotes = false;
 
    // skip over the first skip lambdas and copy the next one to lambda_text
    //
@@ -699,40 +681,79 @@ int tcol = 0;
        {
          if (tidx >= text.size())
             {
-              Q(copying)   Q(tidx)   FIXME;
+              Q1(copying)   Q1(tidx)   Q(skip)   FIXME;
             }
 
          const UCS_string & line = text[tidx];
 
          if (tcol >= line.size())   // end of line: wrap to next line
             {
-              ++tidx;
-              tcol = 0;
+              ++tidx;     // next line
+              tcol = 0;   // first column
+              in_single_quotes = false;
+              in_double_quotes = false;
               continue;
             }
 
          const Unicode uni = line[tcol++];
+         if (in_single_quotes)
+            {
+              if (uni == UNI_SINGLE_QUOTE)   in_single_quotes = false;
+              if (copying)   lambda_text.append(uni);
+              continue;
+            }
+
+         if (in_double_quotes)
+            {
+              if (uni == UNI_ASCII_DOUBLE_QUOTE     // either \" or string end
+                 && !(tcol >= 2 && line[tcol - 2] == UNI_ASCII_BACKSLASH))
+                 in_double_quotes = false;
+              if (copying)   lambda_text.append(uni);
+              continue;
+            }
+
+         // at this point uni is outside strings
+         //
+         if (uni == UNI_COMMENT || uni == UNI_ASCII_NUMBER_SIGN)
+            {
+              ++tidx;
+              tcol = 0;
+              in_single_quotes = false;
+              in_double_quotes = false;
+              continue;
+            }
+
          if (copying)   lambda_text.append(uni);
 
-         if (uni == UNI_ASCII_L_CURLY)
+         switch(uni)
             {
-              if (level++ == 0)   // top-level {
-                 {
-                   if (skip == 0)   copying = true;
-                 }
-            }
-         else if (uni == UNI_ASCII_R_CURLY)
-            {
-              if (--level == 0)   // top-level }
-                 {
-                   if (copying)   break;   // done
-                   --skip;
-                 }
+              case UNI_SINGLE_QUOTE:             // start of a new '...' string
+                   in_single_quotes = true;
+                   continue;
+
+              case UNI_ASCII_DOUBLE_QUOTE:       // start of a new "..." string
+                   in_double_quotes = true;
+                   continue;
+
+              case UNI_ASCII_L_CURLY:            // start of a new { ... }
+                   if (level++ == 0 && skip == 0)   copying = true;
+                   continue;
+
+              case UNI_ASCII_R_CURLY:            // end of { ... }
+                   if (--level)   continue;      // but not top-level }
+                   --skip;                       // next {...} at top-level
+                   if (!copying)  continue;
+
+                   lambda_text.pop();            // the last }
+                   return lambda_text;
+
+              default: continue;
             }
        }
 
-   lambda_text.pop();   // the last }
-   return lambda_text;
+   // not reached
+   //
+   FIXME;
 }
 //-----------------------------------------------------------------------------
 void
@@ -744,19 +765,15 @@ ShapeItem from = 0;
        {
          if (tos[to].get_Class() == TC_END)
             {
-              Token * t1 = &tos[from];
-              Token * t2 = &tos[to - 1];
-              for (;t1 < t2; ++t1, --t2)
-                  {
-                    char tt[sizeof(Token)];
-                    memcpy(tt, t1, sizeof(Token));
-                    memcpy(t1, t2,   sizeof(Token));
-                    memcpy(t2, tt, sizeof(Token));
-                  }
+              tos.reverse_from_to(from, to - 1);   // except TC_END
+              from = to + 1;
+            }
+         else if (to == (tos.size() - 1))
+            {
+              tos.reverse_from_to(from, to);
               from = to + 1;
             }
        }
-
 }
 //-----------------------------------------------------------------------------
 void
@@ -773,9 +790,48 @@ Token * t2 = &tos[tos.size() - 1];
          memcpy(t2, tt, sizeof(Token));
        }
 }
+//-----------------------------------------------------------------------------
+void
+Executable::increment_refcount(const char * loc)
+{
+   Assert1(get_ufun());
+   Assert(get_ufun()->is_lambda());
+
+   ++refcount;
+
+// CERR << "*** increment_refcount() of " << get_name()
+//      << " to " << refcount << " at " << loc << endl;A
+}
+//-----------------------------------------------------------------------------
+void
+Executable::decrement_refcount(const char * loc)
+{
+   Assert1(get_ufun());
+   Assert(get_ufun()->is_lambda());
+
+   if (refcount <= 0)
+      {
+        CERR << "*** Warning: refcount of " << get_name() << " is " << refcount
+             << ":" << endl;
+        print_text(CERR);
+//     FIXME;
+      }
+
+   --refcount;
+
+// CERR << "*** decrement_refcount() of " << get_name()
+//      << " to " << refcount << " at " << loc << endl;
+
+
+   if (refcount <= 0)
+      {
+//      CERR << "*** lambda died" << endl;
+        clear_body();
+      }
+}
 //=============================================================================
 ExecuteList *
-ExecuteList::fix(const UCS_string & data, bool is_cmd, const char * loc)
+ExecuteList::fix(const UCS_string & data, const char * loc)
 {
    // clear errors that may have occured before
    {
@@ -783,15 +839,6 @@ ExecuteList::fix(const UCS_string & data, bool is_cmd, const char * loc)
      if (err) err->error_code = E_NO_ERROR;
    }
 
-   if (is_cmd)
-      {
-         UCS_string empty;
-         ExecuteList * fun = new ExecuteList(empty, loc);
-         Assert(fun);
-         fun->text.push_back(data);
-         return fun;
-      }
-
 ExecuteList * fun = new ExecuteList(data, loc);
 
    Log(LOG_UserFunction__fix)
@@ -819,7 +866,7 @@ ExecuteList * fun = new ExecuteList(data, loc);
 
    try
       {
-        fun->parse_body_line(Function_Line_0, data, false, loc);
+        fun->parse_body_line(Function_Line_0, data, false, false, loc, false);
       }
    catch (Error err)
       {
@@ -860,7 +907,7 @@ StatementList * fun = new StatementList(data, loc);
      if (err)   err->parser_loc = 0;
    }
 
-   fun->parse_body_line(Function_Line_0, data, false, loc);
+   fun->parse_body_line(Function_Line_0, data, false, false, loc, false);
    fun->setup_lambdas();
 
    Log(LOG_UserFunction__fix)
