@@ -161,7 +161,7 @@ ErrorCode
 FloatCell::bif_reciprocal(Cell * Z) const
 {
 const APL_Float z = 1.0/value.fval;
-   if (!isfinite(z))   return E_DOMAIN_ERROR;;
+   if (!isfinite(z))   return E_DOMAIN_ERROR;
 
    new (Z) FloatCell(1.0/value.fval);
    return E_NO_ERROR;
@@ -196,33 +196,78 @@ FloatCell::bif_pi_times_inverse(Cell * Z) const
 ErrorCode
 FloatCell::bif_ceiling(Cell * Z) const
 {
-const APL_Float qct = Workspace::get_CT();
-const APL_Float c = ceil(value.fval);   // c >= value.fval
+   // see comments for bif_floor below.
 
-   // if the distance between value.fval and c is close to 1 (within ⎕CT)
-   // then value.fval is rounded down rather than up. In that decision we
-   // compute:  tolerantly_equal(c - value.fval, 1)
-   // and NOT   tolerantly_equal(value.fval, c - 1) !
+const double b = value.fval;
+   // if b is large then return it as is.
    //
-   if (tolerantly_equal(c - value.fval, 1.0, qct))   return zv(Z, c - 1);
+   if (b >= LARGE_INT)   return zv(Z, b);
+   if (b <= SMALL_INT)   return zv(Z, b);
 
-   return zv(Z, c);
+APL_Integer bi = b;
+   while (bi < b)         ++bi;
+   while ((bi - 1) > b)   --bi;
+   if (bi == b)   return IntCell::zv(Z, bi);   // b already equal to its floor
+
+const double D = bi - b;
+
+   if (D >= (1 - Workspace::get_CT()))   --bi;
+   return IntCell::zv(Z, bi);
 }
 //-----------------------------------------------------------------------------
 ErrorCode
 FloatCell::bif_floor(Cell * Z) const
 {
-const APL_Float qct = Workspace::get_CT();
-const APL_Float f = floor(value.fval);   // f <= value.fval
+/* Informal description (iso p. 78):
+   For real-numbers, Z is the greatest integer tolerantly less than
+   or equal to B. Uses comparison-tolerance.
 
-   // if the distance between value.fval and f is close to 1 (within ⎕CT)
-   // then value.fval is rounded up rather than down. In that decision we
-   // compute:  tolerantly_equal(value.fval - f, 1)
-   // and NOT   tolerantly_equal(value.fval, f + 1) !
+   Formal description:
+   Return the tolerant-floor of B within comparison-tolerance.
+
+   tolerant-floor (p.19) is defined for complex A:
+   Let A be a member of the set of numbers in the unit-square at the
+   complex-integer C, and let D be A minus C.
+   If the sum of the real and imaginary parts of D is tolerantly-less-than
+   one within B, then Z is C.
+   Otherwise, if the imaginary-part of D is greater-than the real-part of D,
+   then Z is C plus imaginary-one.
+   Otherwise, Z is C plus one.
+
+   Unfortunately tolerantly-less-than is not defined in the standard. We
+   interpret it as meaning 'less than and not tolerrantly-equal'.
+
+   Replacing B with ⎕CT, and A with B, and considering that the imaginary
+   part of B is always 0 if B is real this becomes:
+
+   tolerant-floor of (real) B within ⎕CT:
+   Let B be a member of the set of numbers in the hals-open interval [C, C+1),
+   and let D be B minus C.
+   If D is tolerantly-less-than one within ⎕CT, then Z is C.
+   Otherwise, Z is C plus one.
+
+   In other word, Let RB be B rounded down. Then Z is RB if B < RB + 1 - ⎕CT
+   and RB+1 otherwise.
+
+   Note: if B cannot fit into int64_t then, due to the smaller precision
+   of double, it is already equal to its floor and we return it unchanged.
+*/
+
+const double b = value.fval;
+   // if b is large then return it as is.
    //
-   if (tolerantly_equal(value.fval - f, 1.0, qct))   return zv(Z, f + 1);
+   if (b >= LARGE_INT)   return zv(Z, b);
+   if (b <= SMALL_INT)   return zv(Z, b);
 
-   return zv(Z, f);
+APL_Integer bi = b;
+   while (bi > b)         --bi;
+   while ((bi + 1) < b)   ++bi;
+   if (bi == b)   return IntCell::zv(Z, bi);   // b already equal to its floor
+
+const double D = b - bi;
+
+   if (D >= (1 - Workspace::get_CT()))   ++bi;
+   return IntCell::zv(Z, bi);
 }
 //-----------------------------------------------------------------------------
 ErrorCode
@@ -237,20 +282,18 @@ FloatCell::bif_exponential(Cell * Z) const
 ErrorCode
 FloatCell::bif_nat_log(Cell * Z) const
 {
+   if (value.fval == 0)     return E_DOMAIN_ERROR;
+
    if (value.fval > 0)
       {
         new (Z) FloatCell(log(value.fval));
       }
    else if (value.fval < 0)
       {
-        const APL_Float real = log(-value.fval);
-        const APL_Float imag = M_PI;   // argz(-1)
-        new (Z) ComplexCell(real, imag);
-      }
-   else
-      {
-        return E_DOMAIN_ERROR;
+        const APL_Complex bb(value.fval, 0);
+        new (Z) ComplexCell(log(bb));
       }
+
    return E_NO_ERROR;
 }
 //-----------------------------------------------------------------------------
@@ -278,15 +321,17 @@ FloatCell::bif_add(Cell * Z, const Cell * A) const
 ErrorCode
 FloatCell::bif_subtract(Cell * Z, const Cell * A) const
 {
-   if (A->is_real_cell())
+   if (A->is_real_cell())   // real result
       {
-        new (Z) FloatCell(A->get_real_value() - get_real_value());
-        return E_NO_ERROR;
+       return zv(Z, A->get_real_value() - get_real_value());
       }
 
-   // delegate to A
-   //
-   return A->bif_subtract(Z, this);
+   if (A->is_complex_cell())   // complex result
+      {
+       return ComplexCell::zv(Z, A->get_complex_value() - get_real_value());
+      }
+
+   return E_DOMAIN_ERROR;
 }
 //-----------------------------------------------------------------------------
 ErrorCode
@@ -300,7 +345,7 @@ const APL_Float ai = A->get_imag_value();
    if (ai == 0.0)   // real result
       {
         const APL_Float z = ar * value.fval;
-        if (!isfinite(z))   return E_DOMAIN_ERROR;;
+        if (!isfinite(z))   return E_DOMAIN_ERROR;
         new (Z) FloatCell(z);
         return E_NO_ERROR;
       } 
@@ -309,8 +354,8 @@ const APL_Float ai = A->get_imag_value();
    //
 const APL_Complex a(ar, ai);
 const APL_Complex z = a * value.fval;
-   if (!isfinite(z.real()))   return E_DOMAIN_ERROR;;
-   if (!isfinite(z.imag()))   return E_DOMAIN_ERROR;;
+   if (!isfinite(z.real()))   return E_DOMAIN_ERROR;
+   if (!isfinite(z.imag()))   return E_DOMAIN_ERROR;
    new (Z) ComplexCell(z);
    return E_NO_ERROR;
 }     
@@ -343,8 +388,8 @@ const APL_Float ai = A->get_imag_value();
    //
 const APL_Complex a(ar, ai);
 const APL_Complex z = a / value.fval;
-   if (!isfinite(z.real()))   return E_DOMAIN_ERROR;;
-   if (!isfinite(z.imag()))   return E_DOMAIN_ERROR;;
+   if (!isfinite(z.real()))   return E_DOMAIN_ERROR;
+   if (!isfinite(z.imag()))   return E_DOMAIN_ERROR;
    new (Z) ComplexCell(z);
    return E_NO_ERROR;
 }
@@ -365,7 +410,7 @@ const APL_Float ai = A->get_imag_value();
        {
          if (value.fval == 0.0)   return IntCell::z1(Z);   // 0⋆0 is 1
          if (value.fval > 0)      return IntCell::z0(Z);   // 0⋆N is 0
-         return E_DOMAIN_ERROR;;                           // 0⋆¯N = 1÷0
+         return E_DOMAIN_ERROR;                            // 0⋆¯N = 1÷0
        }
 
    // 2. real A > 0   (real result)
@@ -373,28 +418,20 @@ const APL_Float ai = A->get_imag_value();
    if (ai == 0.0)
       {
         if (ar  == 1.0)   return IntCell::z1(Z);   // 1⋆b = 1
-        if (ar > 0)
-           {
-             const APL_Float z = pow(ar, value.fval);
-             if (!isfinite(z))   return E_DOMAIN_ERROR;;
-             new (Z) FloatCell(z);
-             return E_NO_ERROR;
-           }
+        const APL_Float z = pow(ar, value.fval);
+        if (isfinite(z)) return zv(Z, z);
 
         /* fall through */
       }
 
    // 3. complex result
    //
-   {
-     const APL_Complex a(ar, ai);
-     const APL_Complex z = pow(a, value.fval);
-     if (!isfinite(z.real()))   return E_DOMAIN_ERROR;;
-     if (!isfinite(z.imag()))   return E_DOMAIN_ERROR;;
+const APL_Complex a(ar, ai);
+const APL_Complex z = pow(a, value.fval);
+   if (!isfinite(z.real()))   return E_DOMAIN_ERROR;
+   if (!isfinite(z.imag()))   return E_DOMAIN_ERROR;
 
-     new (Z) ComplexCell(z);
-     return E_NO_ERROR;
-   }
+   return ComplexCell::zv(Z, z);
 }
 //-----------------------------------------------------------------------------
 ErrorCode
@@ -567,6 +604,7 @@ static const APL_Float big[MAX_Quad_PP + 1] =
     100000000000000ULL,
    1000000000000000ULL,
   10000000000000000ULL,
+ 1000000000000000000ULL,
 };
 
    return val >= big[quad_pp] || val <= -big[quad_pp];
