@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -55,21 +55,21 @@ IO_Files::get_file_line(UTF8_string & line, bool & eof)
              if (!InputFile::current_file()->file)   break;   // no more files
            }
 
+        // At this point current_file and current_file->file are valid
         // read a line with CR and LF removed.
         //
         eof = false;
         read_file_line(line, eof);
         if (eof)   // end of file reached: do some global checks
            {
-             if (InputFile::current_file() &&
-                 InputFile::current_file()->with_LX)
+             if (InputFile::current_file()->with_LX == do_LX)
                 {
-                  InputFile::current_file()->with_LX = false;
+                  InputFile::current_file()->with_LX = no_LX;
+                  InputFile::current_file()->echo = true;
                   UCS_string LX = Workspace::get_LX();
                   if (LX.size())   // ⎕LX pending
                      {
-                       LX.append_utf8(" ⍝ ⎕LX");
-                       line = UTF8_string(LX);
+                       Command::process_line(LX);
                        eof = false;
                        return;
                      }
@@ -106,9 +106,10 @@ IO_Files::get_file_line(UTF8_string & line, bool & eof)
 void
 IO_Files::read_file_line(UTF8_string & file_line, bool & eof)
 {
+InputFile * input = InputFile::current_file();
    for (;;)
        {
-         int cc = fgetc(InputFile::current_file()->file);
+         const int cc = fgetc(input->file);
          if (cc == EOF)   // end of file
             {
               if (file_line.size())   break;   // EOF, but we have chars
@@ -119,10 +120,34 @@ IO_Files::read_file_line(UTF8_string & file_line, bool & eof)
 
          if (cc == '\n' || cc == 2)   // end of line or ^B
             {
+              if (input->current_line_no() == 1 &&
+                  file_line.starts_with("<!"))
+                 {
+                   // first line of the file starts with <! (so we assume that
+                   // this file is a HTML tagged file
+                   //
+                   input->set_html(1);
+                 }
+
+              if (input->get_html() > 0)   // HTML file
+                 {
+                  input->set_html(file_line.un_HTML(input->get_html()));
+                  if (file_line.size() == 0)   continue;   // line with tag(s)
+                 }
+
+              if (input->has_object_filter())
+                 {
+                    const bool allowed = input->check_filter(file_line);
+                    if (!allowed)
+                       {
+                         file_line.clear();
+                         continue;
+                       }
+                 }
               break;
             }
 
-         if (cc == '\r')   continue;   // ignore carrige returns
+         if (cc == '\r')   continue;   // ignore carriage returns
 
           file_line.append((UTF8)cc);
        }
@@ -284,13 +309,13 @@ IO_Files::open_next_file()
 {
    if (InputFile::current_file() == 0)
       {
-        CERR << "Workspace::open_next_file(): no more files" << endl;
+        CERR << "IO_Files::open_next_file(): no more files" << endl;
         return;
       }
 
    if (InputFile::current_file()->file)
       {
-        CERR << "Workspace::open_next_file(): already open" << endl;
+        CERR << "IO_Files::open_next_file(): already open" << endl;
         return;
       }
 
