@@ -363,7 +363,7 @@ IntCell::bif_conjugate(Cell * Z) const
 ErrorCode
 IntCell::bif_negative(Cell * Z) const
 {
-   if (value.ival == 0x8000000000000000LL)   // integer overflow
+   if ((uint64_t)value.ival == 0x8000000000000000LL)   // integer overflow
       new (Z) FloatCell(- value.ival);
    else
       new (Z) IntCell(- value.ival);
@@ -450,20 +450,18 @@ IntCell::bif_exponential(Cell * Z) const
 ErrorCode
 IntCell::bif_nat_log(Cell * Z) const
 {
+   if (value.ival == 0)     return E_DOMAIN_ERROR;
+
    if (value.ival > 0)
       {
         new (Z) FloatCell(log((APL_Float)value.ival));
       }
-   else if (value.ival < 0)
-      {
-        const APL_Float real = log(-((APL_Float)value.ival));
-        const APL_Float imag = M_PI;   // argz(-1)
-        new (Z) ComplexCell(real, imag);
-      }
    else
       {
-        return E_DOMAIN_ERROR;
+        const APL_Complex bb(value.ival, 0);
+        new (Z) ComplexCell(log(bb));
       }
+
    return E_NO_ERROR;
 }
 //-----------------------------------------------------------------------------
@@ -625,7 +623,7 @@ const bool invert_Z = b < 0;
            {
              if (b1 & 1)
                 {
-                  if (zi >= 0x7FFFFFFFFFFFFFFFULL / a_2_n)
+                  if ((uint64_t)zi >= 0x7FFFFFFFFFFFFFFFULL / a_2_n)
                      {
                        overflow = true;
                        break;
@@ -634,7 +632,7 @@ const bool invert_Z = b < 0;
                   if (b1 == 1)   break;
                 }
 
-             if (a_2_n >= 100000000ULL)
+             if (a_2_n >= 100000000LL)
                 {
                   overflow = true;
                   break;
@@ -689,7 +687,6 @@ const bool invert_Z = b < 0;
    //
    {
      APL_Complex a = A->get_complex_value();
-     if (a.imag() == 0 && a.real() < 0)   return E_DOMAIN_ERROR;
 
      APL_Complex z = pow(a, (APL_Complex)b);
      if (invert_Z)
