@@ -18,6 +18,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "malloc.h"
+
 #include <iostream>
 
 #include "Common.hh"
@@ -25,6 +27,7 @@
 #include "FloatCell.hh"
 #include "Output.hh"
 #include "Value.icc"
+#include "Workspace.hh"
 
 using namespace std;
 
@@ -35,13 +38,19 @@ using namespace std;
 void
 divide_matrix(Cell * cZ, bool need_complex,
               ShapeItem rows, ShapeItem cols_A, const Cell * cA,
-              ShapeItem cols_B, const Cell * cB, ShapeItem nB2)
+              ShapeItem cols_B, const Cell * cB)
 {
+   // the following has been checked by the caller:
+   //
+   // rows_B >= cols_B and
+   // rows_A == rows_B  (aka. rows below)
+   //
    loop(c, cols_A)
        {
          if (need_complex)
             {
-              DynArray(double, ad, 2*rows);
+              double * ad = (double *)malloc(rows * cols_A * 2*sizeof(double));
+              if (ad == 0)   WS_FULL
               ZZ * const a = (ZZ *)ad;
               loop(r, rows)
                   {
@@ -49,7 +58,9 @@ divide_matrix(Cell * cZ, bool need_complex,
                                    cA[r*cols_A + c].get_imag_value());
                   }
 
-              DynArray(double, bd, 2*nB2*nB2);
+              double * bd = (double *)malloc(rows * cols_B * 2*sizeof(double));
+              if (bd == 0)   { free(a);   WS_FULL }
+
               ZZ * const b = (ZZ *)bd;
               ZZ * bb = b;
               loop(rr, cols_B)
@@ -60,13 +71,15 @@ divide_matrix(Cell * cZ, bool need_complex,
                  }
 
               {
-                const double rcond = 1E-10;
+                const APL_Float rcond = Workspace::get_CT();
 
                 Matrix<ZZ> B(b, rows, cols_B, /* LDB */ rows);
                 Matrix<ZZ> A(a, rows, 1,      /* LDA */ rows);
                 const ShapeItem rk = gelsy<ZZ>(B, A, rcond);
+                free(bd);
                 if (rk != cols_B)
                    {
+                     free(ad);
                      DOMAIN_ERROR;
                    }
               }
@@ -76,16 +89,20 @@ divide_matrix(Cell * cZ, bool need_complex,
               //
               loop(r, cols_B)
                   new (cZ + r*cols_A + c) ComplexCell(a[r].real(), a[r].imag());
+              free(ad);
             }
          else   // real
             {
-              DynArray(double, a, rows);
+              double * a = (double *)malloc(rows * cols_A * sizeof(double));
+              if (a == 0)   WS_FULL
               loop(r, rows)
                  {
                    a[r] = cA[r*cols_A + c].get_real_value();
                  }
 
-              DynArray(double, b, nB2*nB2);
+              double * b = (double *)malloc(rows * cols_B * sizeof(double));
+              if (b == 0)   { free(a);   WS_FULL }
+
               double * bb = b;
               loop(rr, cols_B)
               loop(cc, rows)
@@ -94,13 +111,15 @@ divide_matrix(Cell * cZ, bool need_complex,
                  }
 
               {
-                const double rcond = 1E-10;
+                const APL_Float rcond = Workspace::get_CT();
 
                 Matrix<DD> B(b, rows, cols_B, /* LDB */ rows);
                 Matrix<DD> A(a, rows, 1,      /* LDA */ rows);
                 const ShapeItem rk = gelsy<DD>(B, A, rcond);
+                free(b);
                 if (rk != cols_B)
                    {
+                     free(a);
                      DOMAIN_ERROR;
                    }
               }
@@ -111,6 +130,7 @@ divide_matrix(Cell * cZ, bool need_complex,
 
               loop(r, cols_B)
                   new (cZ + r*cols_A + c)   FloatCell(a[r]);
+              free(a);
             }
        }
 }
