@@ -195,7 +195,7 @@ ofstream outf(filename);
       }
 
 int count = 0;
-   for (int p = put + 1; p < hist_lines.size(); ++p)
+   for (size_t p = put + 1; p < hist_lines.size(); ++p)
       {
         outf << hist_lines[p] << endl;
         ++count;
@@ -223,7 +223,7 @@ UCS_string u("xxx");
 void
 LineHistory::print_history(ostream & out)
 {
-   for (int p = put + 1; p < hist_lines.size(); ++p)
+   for (size_t p = put + 1; p < hist_lines.size(); ++p)
       {
         out << "      " << hist_lines[p] << endl;
       }
@@ -238,14 +238,14 @@ LineHistory::add_line(const UCS_string & line)
 {
    if (!line.has_black())   return;
 
-   if (hist_lines.size() < max_lines)   // append
+   if ((int)hist_lines.size() < max_lines)   // append
       {
         hist_lines.push_back(line);
         put = 0;
       }
    else                            // override
       {
-        if (put >= hist_lines.size())   put = 0;   // wrap
+        if (put >= (int)hist_lines.size())   put = 0;   // wrap
         hist_lines[put++] = line;
       }
 
@@ -278,7 +278,7 @@ LineHistory::down()
    if (current_line == put)   return 0;
 
 int new_current_line = current_line + 1;
-   if (new_current_line >= hist_lines.size())   new_current_line = 0;   // wrap
+   if (new_current_line >= (int)hist_lines.size())   new_current_line = 0;   // wrap
    if (new_current_line == put)   return 0;
 
    return &hist_lines[current_line = new_current_line];
@@ -726,7 +726,10 @@ InputMux::get_line(LineInputMode mode, const UCS_string & prompt,
 const APL_time_us from = now();
    if (start_input)   (*start_input)();
 
+#if PARALLEL_ENABLED
    Parallel::lock_pool(false);
+#endif
+
    for (int control_D_count = 0;;)
        {
          bool _eof = false;
@@ -741,7 +744,9 @@ const APL_time_us from = now();
               if (control_D_count >= uprefs.control_Ds_to_exit)
                  {
                    CIN << endl;
+#if PARALLEL_ENABLED
                    Thread_context::kill_pool();
+#endif // PARALLEL_ENABLED
                    uprefs.silent = true;   // exit silently
                    Command::cmd_OFF(4);    // exit()s
                    return;  // not reached
@@ -765,12 +770,16 @@ const APL_time_us from = now();
               //
               CIN << endl;
               COUT << "      *** end of input" << endl;
+#if PARALLEL_ENABLED
               Thread_context::kill_pool();
+#endif
               Command::cmd_OFF(2);   // exit()s
               return;  // not reached
             }
       }
+#if PARALLEL_ENABLED
    Parallel::unlock_pool(false);
+#endif
 
    Log(LOG_get_line)   CERR << " '" << line << "'" << endl;
 
@@ -823,8 +832,7 @@ LineInput::edit_line(LineInputMode mode, const UCS_string & prompt,
 
    user_line.clear();
 
-LineEditContext lec(mode, 24, Workspace::get_PrintContext().get_PW(),
-                    hist, prompt);
+LineEditContext lec(mode, 24, Workspace::get_PW(), hist, prompt);
 
    for (;;)
        {
@@ -899,6 +907,9 @@ LineEditContext lec(mode, 24, Workspace::get_PrintContext().get_PW(),
                    lec.paste();
                    continue;
 
+              case Invalid_Unicode:
+                   continue;
+ 
               default:  // regular APL character
                    lec.insert_char(uni);
                    continue;
@@ -1008,9 +1019,9 @@ const int b0 = fgetc(stdin);
               //
               if (ESCmap::need_more(seq, s))   continue;
 
-              CERR << endl << "Unknown ESC sequence: ESC";
-              loop(ss, s)   CERR << " " << HEX2(seq[ss + 1]);
-              CERR << endl;
+//            CERR << endl << "Unknown ESC sequence: ESC";
+//            loop(ss, s)   CERR << " " << HEX2(seq[ss + 1]);
+//            CERR << endl;
 
               return Invalid_Unicode;
             }
