@@ -161,7 +161,7 @@ UCS_string fun_text;
 
         }
 
-        for (int l = 1; l < lines.size(); ++l)
+        for (size_t l = 1; l < lines.size(); ++l)
             {
               UCS_string line_l("[");
               line_l.append_number(l);
@@ -185,7 +185,7 @@ UCS_string creator(InputFile::current_filename());
 UTF8_string creator_utf8(creator);
 
 UserFunction * ufun = UserFunction::fix(fun_text, error_line, false,
-                                        LOC, creator_utf8);
+                                        LOC, creator_utf8, false);
    if (ufun == 0)   NABLA_ERROR;
 
    if (locked)
@@ -206,8 +206,8 @@ int trace_count = 0;
          if (lines[l].trace_flag)   trace_vec[trace_count++] = Function_Line(l);
        }
 
-   ufun->set_trace_stop(stop_vec,  stop_count,  true);
-   ufun->set_trace_stop(trace_vec, trace_count, false);
+   ufun->set_trace_stop(stop_vec.get_data(),  stop_count,  true);
+   ufun->set_trace_stop(trace_vec.get_data(), trace_count, false);
 }
 //-----------------------------------------------------------------------------
 const char *
@@ -258,9 +258,10 @@ UCS_string::iterator c(first_command.begin());
             }
       }
 
-UserFunction_header hdr(fun_header);
+UserFunction_header hdr(fun_header, false);
    if (hdr.get_error())   return "Bad function header";
-   fun_symbol = hdr.FUN();
+
+   fun_symbol = Workspace::lookup_symbol(hdr.get_name());
    Assert(fun_symbol);
 
    if (fun_header.size() == 0)   return "no function name";
@@ -595,6 +596,9 @@ Function * function = fun_symbol->get_function();
    if (function->is_native())
       return "function is native";
 
+   if (function->is_lambda())
+      return "function is a lambda";
+
    if (Workspace::is_called(fun_symbol->get_name()))
       return "function is pendent or suspended";
 
@@ -694,7 +698,7 @@ const LineLabel user_edit_to = edit_to;
 
    if (idx_from == 0)   COUT << "    ∇" << endl;               // if header line
    for (int e = idx_from; e <= idx_to; ++e)   lines[e].print(COUT);
-   if (idx_to == lines.size() - 1)   COUT << "    ∇" << endl;  // if last line
+   if ((size_t)idx_to == lines.size() - 1)   COUT << "    ∇" << endl;  // if last line
 
    if (user_edit_to.valid())   // eg. [⎕42] or [2⎕42]
       {
@@ -773,7 +777,7 @@ Nabla::edit_header_line()
 {
    // parse the header and check that it is valid
    //
-UserFunction_header header(current_text);
+UserFunction_header header(current_text, false);
    if (header.get_error() != E_NO_ERROR)
       {
         CERR << "BAD FUNCTION HEADER";
@@ -784,9 +788,8 @@ UserFunction_header header(current_text);
    // check if the function name has changed
    //
    Assert(fun_symbol);
-   Assert(header.FUN());
 const UCS_string & old_name = fun_symbol->get_name();
-const UCS_string & new_name = header.FUN()->get_name();
+const UCS_string & new_name = header.get_name();
    if (old_name != new_name)
       {
         // the name has changed. This is OK if the new name can be edited.
@@ -794,11 +797,11 @@ const UCS_string & new_name = header.FUN()->get_name();
         // the old symbol shall not be ⎕FXed when closing the editor, so we
         // can simply forget it and continue with the new function
         //
-        // UserFunction_header::UserFunction_header() should have triggered
-        // the creation of header.FUN() so we can assume that it exists. We
-        // need to check that it is not a variable or an existing function.
+        // We need to check that it is not a variable or an existing function.
         //
-        if (header.FUN()->get_nc() != NC_UNUSED_USER_NAME)
+        Symbol * sym = Workspace::lookup_symbol(new_name);   // create if needed
+        Assert(sym);
+        if (sym->get_nc() != NC_UNUSED_USER_NAME)
            {
              CERR << "BAD FUNCTION HEADER";
              COUT << endl;
@@ -816,7 +819,7 @@ const UCS_string & new_name = header.FUN()->get_name();
 const char *
 Nabla::edit_body_line()
 {
-const Parser parser(PM_FUNCTION, LOC);
+const Parser parser(PM_FUNCTION, LOC, false);
 Token_string in;
 ErrorCode ec = parser.parse(current_text, in);
    if (ec)
@@ -836,7 +839,7 @@ const int idx_from = find_line(edit_from);
         // find the largest label before edit_from (if any)
         //
         int before_idx = -1;
-        for (int i = 0; i < lines.size(); ++i)
+        for (size_t i = 0; i < lines.size(); ++i)
             {
               if (lines[i].label < edit_from)   before_idx = i;
               else                                 break;
