@@ -1131,7 +1131,7 @@ NumericCell::do_binomial(Cell * Z, APL_Integer K, APL_Integer N, bool negate)
            }
         else if (K == 2)   // K == 2 or N-2
            {
-             if (N > 4294967296)   goto non_integer;   // K!N too large
+             if (N > 4294967296LL)   goto non_integer;   // K!N too large
 
              // either N or N-1 must be even
              //
@@ -1322,7 +1322,7 @@ NumericCell::do_binomial(Cell * Z, APL_Integer K, APL_Integer N, bool negate)
 
         // the first element in the table is the max. N (including) for K
         //
-        if (N <= table_K[0])
+        if ((uint64_t)N <= table_K[0])
            {
              const APL_Integer z = table_K[1 + (N - K)];
              if (negate)   new (Z) IntCell(-z);
@@ -1380,8 +1380,10 @@ NumericCell::bif_binomial(Cell * Z, const Cell * A) const
         return E_NO_ERROR;
       }
 
-const APL_Integer a = A->get_near_int();
-const APL_Integer b = get_near_int();
+   if (!A->is_near_int())   return E_DOMAIN_ERROR;
+   if (!is_near_int())      return E_DOMAIN_ERROR;
+const APL_Integer a = A->get_checked_near_int();
+const APL_Integer b = get_checked_near_int();
 
 int how = 0;
    if (a < 0)    how |= 4;
@@ -1444,8 +1446,11 @@ const APL_Float qct = Workspace::get_CT();
    //
    if (A->is_near_int() && is_near_int())
       {
-        const APL_Integer a = A->get_near_int();
-        const APL_Integer b =    get_near_int();
+        if (!A->is_near_int())   return E_DOMAIN_ERROR;
+        if (!is_near_int())      return E_DOMAIN_ERROR;
+
+        const APL_Integer a = A->get_checked_near_int();
+        const APL_Integer b =    get_checked_near_int();
         APL_Integer gcd;
         const ErrorCode err = int_gcd(gcd, a, b);
         if (err)   return err;
@@ -1528,8 +1533,10 @@ const APL_Float qct = Workspace::get_CT();
    //
    if (A->is_near_int() && is_near_int())
       {
-        const APL_Integer a = A->get_near_int();
-        const APL_Integer b =    get_near_int();
+        if (!A->is_near_int())   return E_DOMAIN_ERROR;
+        if (!is_near_int())      return E_DOMAIN_ERROR;
+        const APL_Integer a = A->get_checked_near_int();
+        const APL_Integer b =    get_checked_near_int();
         APL_Integer gcd;
         const ErrorCode err = int_gcd(gcd, a, b);
         if (err)   return err;
@@ -1580,8 +1587,8 @@ APL_Complex z;
 ErrorCode
 NumericCell::int_gcd(APL_Integer & z, APL_Integer a, APL_Integer b)
 {
-   if (a == 0x8000000000000000ULL)   return E_DOMAIN_ERROR;
-   if (b == 0x8000000000000000ULL)   return E_DOMAIN_ERROR;
+   if ((uint64_t)a == 0x8000000000000000ULL)   return E_DOMAIN_ERROR;
+   if ((uint64_t)b == 0x8000000000000000ULL)   return E_DOMAIN_ERROR;
 
    if (a < 0)   a = - a;
    if (b < 0)   b = - b;
