@@ -45,11 +45,20 @@ Parser::parse(const UCS_string & input, Token_string & tos) const
 Token_string tos1;
 
    {
-     Tokenizer tokenizer(pmode, LOC);
+     Tokenizer tokenizer(pmode, LOC, macro);
      ErrorCode ec = tokenizer.tokenize(input, tos1);
      if (ec != E_NO_ERROR)   return ec;
    }
 
+   // special case: single token (to speed up ⍎)
+   //
+   if (tos1.size() == 1)
+      {
+        const ErrorCode err = parse_statement(tos1);
+        if (err == E_NO_ERROR)   tos.append(tos1[0]);
+        return err;
+      }
+
    return parse(tos1, tos);
 }
 //-----------------------------------------------------------------------------
@@ -147,6 +156,14 @@ Parser::parse_statement(Token_string & tos)
    collect_constants(tos);
    remove_void_token(tos);
 
+   // special case: single APL value (to speed up ⍎)
+   //
+   if (tos.size() == 1)
+      {
+        Log(LOG_parse)   CERR << "parse 3a: single value " << tos[0] << endl;;
+        return E_NO_ERROR;
+      }
+
    Log(LOG_parse)
       {
         CERR << "parse 4 [" << tos.size() << "]: ";
@@ -162,16 +179,7 @@ Parser::parse_statement(Token_string & tos)
         tos.print(CERR);
       }
 
-   // 5. resolve ambiguous / ⌿ \ and ⍀ operators/functions
-   //
-   degrade_scan_reduce(tos);
-   Log(LOG_parse)
-      {
-        CERR << "parse 6 [" << tos.size() << "]: ";
-        tos.print(CERR);
-      }
-
-   // 6. update distances between (), [], and {}
+   // 5. update distances between (), [], and {}
    //
    {
      const ErrorCode ec = match_par_bra(tos, false);
@@ -467,35 +475,10 @@ Parser::remove_nongrouping_parantheses(Token_string & tos)
        }
 }
 //-----------------------------------------------------------------------------
-void
-Parser::degrade_scan_reduce(Token_string & tos)
-{
-   loop(src, tos.size())
-       {
-         if (tos[src].get_Id() == ID::OPER1_REDUCE  ||
-             tos[src].get_Id() == ID::OPER1_REDUCE1 ||
-             tos[src].get_Id() == ID::OPER1_SCAN    ||
-             tos[src].get_Id() == ID::OPER1_SCAN1)
-            {
-              // tos[src] is / ⌿ \ or ⍀
-              //
-              const bool is_function = src == 0 ||   // (will SYNTAX_ERROR)
-                         check_if_value(tos, src - 1);
-              if (is_function)
-                 {
-                   // the (back-) slash is a function, not an operator.
-                   //
-                   const int ftag = (tos[src].get_tag() & ~TC_MASK) | TC_FUN12;
-                   tos[src].ChangeTag((TokenTag)ftag);
-                 }
-            }
-       }
-}
-//-----------------------------------------------------------------------------
 bool
 Parser::check_if_value(const Token_string & tos, int pos)
 {
-   // figure if tos[pos] (the token left og /. ⌿. \. or ⍀ is the end of a
+   // figure if tos[pos] (the token left of /. ⌿. \. or ⍀) is the end of a
    // function (and then return false) or the end of a value (and then
    // return true).
    //
@@ -508,6 +491,7 @@ Parser::check_if_value(const Token_string & tos, int pos)
         case TC_L_PARENT:   // e.g. (/       (actually syntax error)
         case TC_VALUE:      // e.g. 5/
         case TC_RETURN:     // e.g.  /       (actually syntax error)
+        case TC_OPER2:      // e.g. ./
              return true;   // tos[pos] is at the end of a value
 
         case TC_R_BRACK:    // e.g. +[1]/2 or 2/[1]2
@@ -563,6 +547,7 @@ Simple_string<ShapeItem> stack;
          TokenClass tc_peer;
          switch(tos[t].get_Class())
            {
+             // for [ ( or { push the position onto stack
              case TC_L_BRACK:  if (tos[t].get_tag() != TOK_L_BRACK)   continue;
              case TC_L_PARENT:
              case TC_L_CURLY:  stack.append(t);
@@ -597,6 +582,16 @@ Simple_string<ShapeItem> stack;
           tos[t1].set_int_val2(diff);
        }
 
+   // if there are unmatched items: return syntax error of the outer token
+   //
+   if (stack.size())
+      {
+        const TokenClass outer = tos[stack[0]].get_Class();
+        if (outer == TC_L_BRACK)    return E_UNBALANCED_BRACKET;
+        if (outer == TC_L_PARENT)   return E_UNBALANCED_PARENT;
+        return E_UNBALANCED_CURLY;
+      }
+
    return E_NO_ERROR;
 }
 //-----------------------------------------------------------------------------
@@ -743,7 +738,7 @@ Parser::mark_lsymb(Token_string & tos)
 {
    loop(ass, tos.size())
       {
-        if (tos[ass].get_tag() != TOK_ASSIGN)   continue;
+        if (tos[ass].get_Class() != TC_ASSIGN)   continue;
 
         // found ←. move backwards. Before that we handle the special case of
         // vector specification, i.e. (SYM SYM ... SYM) ← value
