@@ -23,6 +23,8 @@
 #include "DerivedFunction.hh"
 #include "Executable.hh"
 #include "IndexExpr.hh"
+#include "LvalCell.hh"
+#include "PointerCell.hh"
 #include "Prefix.hh"
 #include "StateIndicator.hh"
 #include "Symbol.hh"
@@ -140,7 +142,7 @@ TokenClass next = body[pc].get_Class();
       {
         ++pc;
         if (!is_value_parent(pc))   return false;   // (fun)) XXX
-        const int offset = body[pc].get_int_val();
+        const int offset = body[pc].get_int_val2();
         pc += offset;
         if (pc >= body.size())   return true;   // syntax error
         next = body[pc].get_Class();
@@ -167,14 +169,52 @@ TokenClass next = body[pc].get_Class();
         return true;
       }
 
+   // dyadic operator with numeric function argument, for example:  ⍤ 0 
+   //
+   if (next == TC_VALUE       &&
+       pc < (body.size() - 1) &&
+       body[pc+1].get_Class() == TC_OPER2)   return false;
+
    return true;
 }
 //-----------------------------------------------------------------------------
 bool
+Prefix::is_fun_or_oper(int pc) const
+{
+   // this function is called when / ⌿ \ or ⍀ shall be resolved. pc points
+   // to the token left of / ⌿ \ or ⍀.
+   //
+TokenTag tag_LO = body[pc].get_tag();
+
+   if (tag_LO == TOK_R_BRACK)
+      {
+        // e.g. fun[...]/ or value[...]/ Skip over [...]
+        //
+        pc += body[pc].get_int_val2();
+        Assert1(body[pc++].get_Class() == TC_L_BRACK);   // opening [
+        tag_LO = body[pc].get_tag();
+      }
+
+   if (tag_LO == TOK_R_PARENT)   return !is_value_parent(pc);
+   if (body[pc].get_Class() == TC_OPER2)   return false;   // make / a function
+
+   if ((tag_LO & TV_MASK) == TV_FUN)   return true;
+
+   if (tag_LO == TOK_SYMBOL)
+      {
+        Symbol * sym = body[pc].get_sym_ptr();
+        if (sym == 0)   return false;
+        return sym->get_function() != 0;
+      }
+
+   return false;   // not a function or operator
+}
+//-----------------------------------------------------------------------------
+bool
 Prefix::is_value_bracket() const
 {
    Assert1(body[PC - 1].get_Class() == TC_R_BRACK);
-const int offset = body[PC - 1].get_int_val();
+const int offset = body[PC - 1].get_int_val2();
    Assert1(body[PC + offset - 1].get_Class() == TC_L_BRACK);   // opening [
 
 const Token & tok1 = body[PC + offset];
@@ -228,7 +268,7 @@ int count = 0;
         const Token & tok = at(s).tok;
         if (tok.get_ValueType() != TV_VAL)      continue;
 
-        if (tok.get_apl_val()->is_or_contains(value))
+        if (Value::is_or_contains(tok.get_apl_val().get(), value))
            {
              out << prefix << " Fifo [" << s << "]" << endl;
              ++count;
@@ -338,6 +378,7 @@ Prefix::unmark_all_values() const
 }
 //-----------------------------------------------------------------------------
 
+// a hash table with all prefixes that can be reduced...
 #define PH(name, idx, prio, misc, len, fun) \
    { #name, idx, prio, misc, len, &Prefix::reduce_ ## fun, #fun },
 
@@ -390,6 +431,14 @@ grow:
 
      if (tcl == TC_SYMBOL)   // resolve symbol if necessary
         {
+          // reset the PC back to the previous token, so that a failed
+          // resolve() (aka VALUE ERROR) will re-fetch the token
+          //
+          // But not if symbol is ⎕LC because that would make the first
+          // element of ⎕LC too low!
+          //
+          if (tl.tok.get_tag() != TOK_Quad_LC)   --PC;
+
           Symbol * sym = tl.tok.get_sym_ptr();
           if (tl.tok.get_tag() == TOK_LSYMB2)
              {
@@ -397,13 +446,41 @@ grow:
                // (A B ... C)←. We return C and let the caller do the rest
                //
                sym->resolve(tl.tok, true);
+               Log(LOG_prefix_parser)
+                  CERR << "TOK_LSYMB2 " << sym->get_name()
+                       << "resolved to " << tl.tok << endl;
              }
           else
              {
                const bool left_sym = get_assign_state() == ASS_arrow_seen;
-               sym->resolve(tl.tok, left_sym);
+               bool resolved = false;
+               if (size() > 0 && at(0).tok.get_Class() == TC_INDEX && 
+                   tl.tok.get_tag() == TOK_SYMBOL)   // user defined variable
+                  {
+                    // indexed reference, e.g. A[N]. Calling sym->resolve()
+                    // would copy the entire variable and then index it, which
+                    // is inefficient if the variable is big. We rather call
+                    // Symbol::get_value() directly in order to avoid that
+                    //
+                    Value_P val = sym->get_value();
+                    if (!!val && !left_sym)
+                       {
+                         Token tok(TOK_APL_VALUE1, val);
+                         move_1(tl.tok, tok, LOC);
+                         resolved = true;
+                       }
+                  }
+               if (!resolved)   sym->resolve(tl.tok, left_sym);
+
                if (left_sym)   set_assign_state(ASS_var_seen);
+               Log(LOG_prefix_parser)
+                  {
+                    if (left_sym)   CERR << "TOK_LSYMB ";
+                    else            CERR << "TOK_SYMBOL ";
+                    CERR << "resolved to " << tl.tok << endl;
+                  }
              }
+          PC = lookahead_high + 1;   // resolve() succeeded: restore PC
 
           Log(LOG_prefix_parser)
              {
@@ -599,7 +676,7 @@ Prefix::dont_reduce(TokenClass next) const
       {
         if (next == TC_OPER2)           // DOP B
            {
-             return best->prio < BS_OP_RO;
+             return true;
            }
         else if (next == TC_VALUE)      // A B
            {
@@ -621,7 +698,7 @@ Prefix::dont_reduce(TokenClass next) const
       {
         if (next == TC_OPER2)
            {
-             return best->prio < BS_OP_RO;
+             return true;
            }
       }
 
@@ -741,6 +818,7 @@ Prefix::reduce_LPAR_B_RPAR_()
 
 Token result = at1();   // B or F
    if (result.get_tag() == TOK_APL_VALUE3)   result.ChangeTag(TOK_APL_VALUE1);
+
    pop_args_push_result(result);
    set_action(result);
 }
@@ -805,12 +883,64 @@ Prefix::reduce_MISC_F_B_()
       }
 
 Token result = at0().get_function()->eval_B(at1().get_apl_val());
-   if (result.get_tag() == TOK_ERROR)
+   if (result.get_Class() == TC_SI_LEAVE)
       {
-        Token_loc tl(result, get_range_low());
-        push(tl);
-        action = RA_RETURN;
-        return;
+        if (result.get_tag() == TOK_EA_EXEC)
+           {
+             // the macro Quad_EA was called and has returned an action
+             // to be performed in the caller. We pop the )SI entry for
+             // Quad_EA continue in the calling prefix parser
+             //
+             action = RA_SI_PUSHED;
+             const Cell * info = &result.get_apl_val()->get_ravel(0);
+             Workspace::pop_SI(LOC);
+             UCS_string stat(*info[1].get_pointer_value());
+             if (stat.size())
+                {
+                  // the B side of A ⎕EA B has failed or returned → or →N
+                  // so that stat (which is A or → or →N) shall be executed
+                  //
+//                CERR << "⎕ES: ⍎" << stat << endl;
+                  Token exec = Bif_F1_EXECUTE::execute_statement(stat);
+                  if (exec.get_Class() == TC_VALUE)   // ⍎ literal
+                     {
+                       move_1(Workspace::SI_top()->get_prefix().at0(),
+                              exec, LOC);
+                       return;
+                     }
+
+                  const APL_Integer major = info[2].get_int_value();
+                  const APL_Integer minor = info[3].get_int_value();
+                  const ErrorCode ec = (ErrorCode)(major << 16 | minor);
+                  Workspace::SI_top()->get_parent()->get_error().init(ec, LOC);
+                }
+              else              // committed value
+                {
+                  Token & si_pushed = Workspace::SI_top()->get_prefix().at0();
+                  Assert(si_pushed.get_tag() == TOK_SI_PUSHED);
+                  if (info[3].is_pointer_cell())
+                     {
+                       Value_P val = info[3].get_pointer_value();
+                       new (&si_pushed)  Token(TOK_APL_VALUE2, val);
+                     }
+                  else
+                     {
+                       Value_P scalar(LOC);
+                       scalar->next_ravel()->init(info[3], scalar.getref(),LOC);
+                       scalar->check_value(LOC);
+                       new (&si_pushed)  Token(TOK_APL_VALUE2, scalar);
+                     }
+                }
+             return;
+           }
+
+        if (result.get_tag() == TOK_ERROR)
+           {
+             Token_loc tl(result, get_range_low());
+             push(tl);
+             action = RA_RETURN;
+             return;
+           }
       }
 
    pop_args_push_result(result);
@@ -881,9 +1011,12 @@ Token result = at1().get_function()->eval_AB(at0().get_apl_val(),
 void
 Prefix::reduce_A_M_B_()
 {
-   Assert1(prefix_len == 3);
+const TokenTag tag = at1().get_tag();
+   if (tag == TOK_OPER1_REDUCE  || tag == TOK_OPER1_SCAN ||
+       tag == TOK_OPER1_REDUCE1 || tag == TOK_OPER1_SCAN1)
+      return reduce_A_F_B_();
 
-   reduce_A_F_B_();
+   syntax_error(LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -912,12 +1045,19 @@ Token result = at1().get_function()->eval_AXB(at0().get_apl_val(),
 void
 Prefix::reduce_A_M_C_B()
 {
-   reduce_A_F_C_B();
+const TokenTag tag = at1().get_tag();
+   if (tag == TOK_OPER1_REDUCE  || tag == TOK_OPER1_SCAN ||
+       tag == TOK_OPER1_REDUCE1 || tag == TOK_OPER1_SCAN1)
+      return reduce_A_F_C_B();
+
+   syntax_error(LOC);
 }
 //-----------------------------------------------------------------------------
 void
 Prefix::reduce_F_M__()
 {
+   Assert1(prefix_len == 2);
+
 DerivedFunction * derived =
    Workspace::SI_top()->fun_oper_cache.get(LOC);
    new (derived) DerivedFunction(at0(), at1().get_function(), LOC);
@@ -927,8 +1067,27 @@ DerivedFunction * derived =
 }
 //-----------------------------------------------------------------------------
 void
+Prefix::reduce_M_M__()
+{
+   if (is_fun_or_oper(PC))
+      {
+         action = RA_PUSH_NEXT;
+        return;
+      }
+
+const TokenTag tag = at0().get_tag();
+   if (tag == TOK_OPER1_REDUCE  || tag == TOK_OPER1_SCAN ||
+       tag == TOK_OPER1_REDUCE1 || tag == TOK_OPER1_SCAN1)
+      return reduce_F_M__();
+
+   syntax_error(LOC);
+}
+//-----------------------------------------------------------------------------
+void
 Prefix::reduce_F_M_C_()
 {
+   Assert1(prefix_len == 3);
+
 DerivedFunction * derived =
    Workspace::SI_top()->fun_oper_cache.get(LOC);
    new (derived) DerivedFunction(at0(), at1().get_function(),
@@ -939,6 +1098,94 @@ DerivedFunction * derived =
 }
 //-----------------------------------------------------------------------------
 void
+Prefix::reduce_M_M_C_()
+{
+   if (is_fun_or_oper(PC))
+      {
+         action = RA_PUSH_NEXT;
+        return;
+      }
+
+const TokenTag tag = at0().get_tag();
+   if (tag == TOK_OPER1_REDUCE  || tag == TOK_OPER1_SCAN ||
+       tag == TOK_OPER1_REDUCE1 || tag == TOK_OPER1_SCAN1)
+      return reduce_F_M_C_();
+
+   syntax_error(LOC);
+}
+//-----------------------------------------------------------------------------
+void
+Prefix::reduce_F_C_M_()
+{
+   Assert1(prefix_len == 3);
+
+DerivedFunction * F_C =
+   Workspace::SI_top()->fun_oper_cache.get(LOC);
+   new (F_C) DerivedFunction(at0().get_function(), at1().get_axes(), LOC);
+
+Token tok_F_C(TOK_FUN2, F_C);
+DerivedFunction * derived =
+   Workspace::SI_top()->fun_oper_cache.get(LOC);
+   new (derived) DerivedFunction(tok_F_C, at2().get_function(), LOC);
+
+   pop_args_push_result(Token(TOK_FUN2, derived));
+   action = RA_CONTINUE;
+}
+//-----------------------------------------------------------------------------
+void
+Prefix::reduce_M_C_M_()
+{
+   if (is_fun_or_oper(PC))
+      {
+         action = RA_PUSH_NEXT;
+        return;
+      }
+
+const TokenTag tag = at0().get_tag();
+   if (tag == TOK_OPER1_REDUCE  || tag == TOK_OPER1_SCAN ||
+       tag == TOK_OPER1_REDUCE1 || tag == TOK_OPER1_SCAN1)
+      return reduce_F_C_M_();
+
+   syntax_error(LOC);
+}
+//-----------------------------------------------------------------------------
+void
+Prefix::reduce_F_C_M_C()
+{
+   Assert1(prefix_len == 4);
+
+DerivedFunction * F_C =
+   Workspace::SI_top()->fun_oper_cache.get(LOC);
+   new (F_C) DerivedFunction(at0().get_function(), at1().get_axes(), LOC);
+
+Token tok_F_C(TOK_FUN2, F_C);
+DerivedFunction * derived =
+   Workspace::SI_top()->fun_oper_cache.get(LOC);
+   new (derived) DerivedFunction(tok_F_C, at2().get_function(),
+                                          at3().get_axes(), LOC);
+
+   pop_args_push_result(Token(TOK_FUN2, derived));
+   action = RA_CONTINUE;
+}
+//-----------------------------------------------------------------------------
+void
+Prefix::reduce_M_C_M_C()
+{
+   if (is_fun_or_oper(PC))
+      {
+         action = RA_PUSH_NEXT;
+        return;
+      }
+
+const TokenTag tag = at0().get_tag();
+   if (tag == TOK_OPER1_REDUCE  || tag == TOK_OPER1_SCAN ||
+       tag == TOK_OPER1_REDUCE1 || tag == TOK_OPER1_SCAN1)
+      return reduce_F_C_M_C();
+
+   syntax_error(LOC);
+}
+//-----------------------------------------------------------------------------
+void
 Prefix::reduce_B_D_B_()
 {
    reduce_F_D_G_();   // same as F2
@@ -1002,27 +1249,6 @@ Token result = Token(TOK_FUN2, derived);
 void
 Prefix::reduce_F_D_G_()
 {
-   // reduce, except if another dyadic operator is coming. In that case
-   // F belongs to the other operator and we simply continue.
-   //
-   if (PC < body.size())
-        {
-          const Token & tok = body[PC];
-          TokenClass next =  tok.get_Class();
-          if (next == TC_SYMBOL)
-             {
-               Symbol * sym = tok.get_sym_ptr();
-               const bool left_sym = get_assign_state() == ASS_arrow_seen;
-               next = sym->resolve_class(left_sym);
-             }
-
-          if (next == TC_OPER2)
-             {
-               action = RA_PUSH_NEXT;
-               return;
-             }
-        }
-
 DerivedFunction * derived =
    Workspace::SI_top()->fun_oper_cache.get(LOC);
    new (derived) DerivedFunction(at0(), at1().get_function(), at2(), LOC);
@@ -1032,6 +1258,42 @@ DerivedFunction * derived =
 }
 //-----------------------------------------------------------------------------
 void
+Prefix::reduce_F_D_M_()
+{
+const TokenTag tag = at2().get_tag();
+   if (tag == TOK_OPER1_REDUCE  || tag == TOK_OPER1_SCAN ||
+       tag == TOK_OPER1_REDUCE1 || tag == TOK_OPER1_SCAN1)
+      return reduce_F_D_G_();
+
+   syntax_error(LOC);
+}
+//-----------------------------------------------------------------------------
+void
+Prefix::reduce_M_D_G_()
+{
+const TokenTag tag = at0().get_tag();
+   if (tag == TOK_OPER1_REDUCE  || tag == TOK_OPER1_SCAN ||
+       tag == TOK_OPER1_REDUCE1 || tag == TOK_OPER1_SCAN1)
+      return reduce_F_D_G_();
+
+   syntax_error(LOC);
+}
+//-----------------------------------------------------------------------------
+void
+Prefix::reduce_M_D_M_()
+{
+const TokenTag tag0 = at0().get_tag();
+const TokenTag tag2 = at2().get_tag();
+   if ((tag0 == TOK_OPER1_REDUCE  || tag0 == TOK_OPER1_SCAN ||
+        tag0 == TOK_OPER1_REDUCE1 || tag0 == TOK_OPER1_SCAN1) && 
+       (tag2 == TOK_OPER1_REDUCE  || tag2 == TOK_OPER1_SCAN ||
+        tag2 == TOK_OPER1_REDUCE1 || tag2 == TOK_OPER1_SCAN1))
+      return reduce_F_D_G_();
+
+   syntax_error(LOC);
+}
+//-----------------------------------------------------------------------------
+void
 Prefix::reduce_F_D_C_B()
 {
    // reduce, except if another dyadic operator is coming. In that case
@@ -1131,6 +1393,15 @@ Token result = Token(TOK_APL_VALUE1, Z);
 }
 //-----------------------------------------------------------------------------
 void
+Prefix::reduce_V_C__()
+{
+Symbol * V = at0().get_sym_ptr();
+   copy_1(at0(), V->resolve_lv(LOC), LOC);
+   set_assign_state(ASS_var_seen);
+   action = RA_CONTINUE;
+}
+//-----------------------------------------------------------------------------
+void
 Prefix::reduce_V_C_ASS_B()
 {
 Symbol * V = at0().get_sym_ptr();
@@ -1196,18 +1467,12 @@ Symbol * V = at1().get_sym_ptr();
 }
 //-----------------------------------------------------------------------------
 void
-Prefix::reduce_M_V__()
-{
-   reduce_F_V__();   // same as F2
-}
-//-----------------------------------------------------------------------------
-void
 Prefix::reduce_A_ASS_B_()
 {
 Value_P A = at0().get_apl_val();
 Value_P B = at2().get_apl_val();
 
-   A->assign_cellrefs(B);   // erases A !
+   A->assign_cellrefs(B);
 
 Token result = Token(TOK_APL_VALUE2, B);
    pop_args_push_result(result);
@@ -1220,14 +1485,19 @@ void
 Prefix::reduce_V_ASS_B_()
 {
 Value_P B = at2().get_apl_val();
-Symbol * V = at0().get_sym_ptr();
-   V->assign(B, LOC);
 
-Token result = Token(TOK_APL_VALUE2, B);
-   pop_args_push_result(result);
+   Assert1(B->get_owner_count() >= 2);   // owners are at least B and at2()
+const bool clone = B->get_owner_count() != 2 || at1().get_tag() != TOK_ASSIGN1;
+Symbol * V = at0().get_sym_ptr();
+   pop_and_discard();   // V
+   pop_and_discard();   // ←
+
+   at0().ChangeTag(TOK_APL_VALUE2);   // change value to committed value
 
    set_assign_state(ASS_none);
    action = RA_CONTINUE;
+
+   V->assign(B, clone, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -1425,7 +1695,7 @@ DynArray(Symbol *, symbols, count + 1);
       }
 
 Value_P B = at3().get_apl_val();
-   Symbol::vector_assignment(symbols, count + 1, B);
+   Symbol::vector_assignment(symbols.get_data(), count + 1, B);
 
    set_assign_state(ASS_none);
 
@@ -1455,11 +1725,13 @@ const bool trace = (at0().get_int_val() & 1) != 0;
 Token Void(TOK_VOID);
    si.statement_result(Void, trace);
    action = RA_PUSH_NEXT;
-   if (attention_raised && end_of_line)
+   if (attention_is_raised() && end_of_line)
       {
-        attention_raised = false;
-        interrupt_raised = false;
-        ATTENTION;
+        const bool int_raised = interrupt_is_raised();
+        clear_attention_raised(LOC);
+        clear_interrupt_raised(LOC);
+        if (int_raised)   INTERRUPT
+        else              ATTENTION
       }
 }
 //-----------------------------------------------------------------------------
@@ -1475,14 +1747,17 @@ const bool trace = (at0().get_int_val() & 1) != 0;
 
    pop_and_discard();   // pop END
 Token B = pop().tok;    // pop B
+   si.fun_oper_cache.reset();
    si.statement_result(B, trace);
 
    action = RA_PUSH_NEXT;
-   if (attention_raised && end_of_line)
+   if (attention_is_raised() && end_of_line)
       {
-        attention_raised = false;
-        interrupt_raised = false;
-        ATTENTION;
+        const bool int_raised = interrupt_is_raised();
+        clear_attention_raised(LOC);
+        clear_interrupt_raised(LOC);
+        if (int_raised)   INTERRUPT
+        else              ATTENTION
       }
 }
 //-----------------------------------------------------------------------------
@@ -1493,6 +1768,8 @@ Prefix::reduce_END_GOTO_B_()
 
    if (size() != 3)   syntax_error(LOC);
 
+   si.fun_oper_cache.reset();
+
    // at0() is either TOK_END or TOK_ENDL.
    //
 const bool end_of_line = at0().get_tag() == TOK_ENDL;
@@ -1536,11 +1813,13 @@ const Token result = si.jump(line);
            }
 
         action = RA_PUSH_NEXT;
-        if (attention_raised && end_of_line)
+        if (attention_is_raised() && end_of_line)
            {
-             attention_raised = false;
-             interrupt_raised = false;
-             ATTENTION;
+             const bool int_raised = interrupt_is_raised();
+             clear_attention_raised(LOC);
+             clear_interrupt_raised(LOC);
+             if (int_raised)   INTERRUPT
+             else              ATTENTION
            }
 
         return;
@@ -1549,11 +1828,13 @@ const Token result = si.jump(line);
    if (result.get_tag() == TOK_VOID)   // branch taken, e.g. →N
       {
         action = RA_PUSH_NEXT;
-        if (attention_raised && end_of_line)
+        if (attention_is_raised() && end_of_line)
            {
-             attention_raised = false;
-             interrupt_raised = false;
-             ATTENTION;
+             const bool int_raised = interrupt_is_raised();
+             clear_attention_raised(LOC);
+             clear_interrupt_raised(LOC);
+             if (int_raised)   INTERRUPT
+             else              ATTENTION
            }
 
         return;
@@ -1570,11 +1851,13 @@ const Function_PC new_pc = si.get_PC();
 
    Assert1(size() == 0);
    action = RA_PUSH_NEXT;
-   if (attention_raised && end_of_line)
+   if (attention_is_raised() && end_of_line)
       {
-        attention_raised = false;
-        interrupt_raised = false;
-        ATTENTION;
+        const bool int_raised = interrupt_is_raised();
+        clear_attention_raised(LOC);
+        clear_interrupt_raised(LOC);
+        if (int_raised)   INTERRUPT
+        else              ATTENTION
       }
 }
 //-----------------------------------------------------------------------------
@@ -1585,6 +1868,8 @@ Prefix::reduce_END_GOTO__()
 
    if (size() != 2)   syntax_error(LOC);
 
+   si.fun_oper_cache.reset();
+
 const bool trace = at0().get_Class() == TC_END &&
                   (at0().get_int_val() & 1) != 0;
    if (trace)
