@@ -71,10 +71,11 @@ const ShapeItem ec = value.element_count();
    if (value.is_scalar())
       {
         PERFORMANCE_START(start_1)
+        const Cell & cell = value.get_ravel(0);
         PrintContext pctx1(pctx);
-        if (value.get_ravel(0).need_scaling(pctx))   pctx1.set_scaled();
+        if (cell.need_scaling(pctx))   pctx1.set_scaled();
 
-        *this = value.get_ravel(0).character_representation(pctx1);
+        *this = cell.character_representation(pctx1);
 
         // pad the value unless it is framed
         if (value.compute_depth() > 1 && !framed)
@@ -95,6 +96,28 @@ const ShapeItem ec = value.element_count();
         return;
       }
 
+   if (pctx.get_style() & PST_QUOTE_CHARS)
+      {
+        if (value.is_char_vector())
+           {
+             UCS_string ucs;
+             ucs.append(UNI_ASCII_DOUBLE_QUOTE);
+             loop(v, ec)   ucs.append(value.get_ravel(v).get_char_value());
+             ucs.append(UNI_ASCII_DOUBLE_QUOTE);
+             append_ucs(ucs);
+             update_info();
+             complete = true;
+             return;
+           }
+
+        if (value.is_char_array())
+           {
+             pctx.set_style(PR_BOXED_GRAPHIC2);
+             new (this)   PrintBuffer(value, pctx, out);
+             return;
+           }
+      }
+
    if (ec == 0)   // empty value of any dimension
       {
         pb_empty(value, pctx, outer_style);
@@ -103,6 +126,7 @@ const ShapeItem ec = value.element_count();
              UCS_string ucs(*this, value.get_rank(), _pctx.get_PW());
              if (ucs.size())   *out << ucs << endl;
             }
+        update_info();
         complete = true;
         return;
       }
@@ -115,13 +139,15 @@ const ShapeItem ec = value.element_count();
              UCS_string ucs(*this, value.get_rank(), _pctx.get_PW());
              if (ucs.size())   *out << ucs << endl;
             }
+        update_info();
         complete = true;
         return;
       }
 
    // non-trivial PrintBuffer
    //
-   const ShapeItem cols = value.get_last_shape_item();
+const ShapeItem cols = value.get_last_shape_item();
+
    if (cols <= PB_MAX_COLS)   // few columns
       {
         DynArray(bool, scaling, cols);
@@ -130,13 +156,15 @@ const ShapeItem ec = value.element_count();
            {
              DynArray(PrintBuffer, item_matrix, ec);
              do_PrintBuffer(value, pctx, out, outer_style,
-                            scaling, pcols, item_matrix);
+                            scaling.get_data(), pcols.get_data(),
+                            item_matrix.get_data());
            }
         else
            {
              PrintBuffer * item_matrix = new PrintBuffer[ec];
              do_PrintBuffer(value, pctx, out, outer_style,
-                            scaling, pcols, item_matrix);
+                            scaling.get_data(), pcols.get_data(),
+                            item_matrix);
              delete [] item_matrix;
            }
       }
@@ -148,7 +176,7 @@ const ShapeItem ec = value.element_count();
            {
              DynArray(PrintBuffer, item_matrix, ec);
              do_PrintBuffer(value, pctx, out, outer_style,
-                            scaling, pcols, item_matrix);
+                            scaling, pcols, item_matrix.get_data());
            }
         else
            {
@@ -174,6 +202,7 @@ const bool framed = outer_style & (PST_CS_MASK | PST_CS_OUTER);
 const ShapeItem ec = value.element_count();
 const uint64_t ii_count = interrupt_count;
 const bool huge = out && ec > 10000;
+const bool nested = !value.is_simple();
 
    {
      // 1. create a vector with a bool per column that tells if the
@@ -202,9 +231,12 @@ const bool huge = out && ec > 10000;
      //    therefore we have (⍴,value) items. Items are rectangular.
      //
      PERFORMANCE_START(start_2)
+     vector<Rank> max_row_ranks;
+     max_row_ranks.reserve(rows);
      loop(y, rows)
         {
           ShapeItem max_row_height = 0;
+          max_row_ranks.push_back(0);
 
           loop(x, cols)
               {
@@ -216,6 +248,14 @@ const bool huge = out && ec > 10000;
                 const Cell & cell = value.get_ravel(x + y*cols);
                 item = cell.character_representation(pctx1);
 
+                if (cell.is_pointer_cell())
+                   {
+                     Value_P sub_val = cell.get_pointer_value();
+                     const Rank sub_rank = sub_val->get_rank();
+                     if (max_row_ranks.back() < sub_rank)
+                        max_row_ranks.back() = sub_rank;
+                   }
+
                 if (max_row_height < item.get_height())
                    max_row_height = item.get_height();
 
@@ -257,7 +297,7 @@ const bool huge = out && ec > 10000;
         }
      PERFORMANCE_END(fs_PrintBuffer3_B, start_3, ec)
 
-     // 4. combine colums, then rows.
+     // 4. combine columms, then rows.
      //
      PERFORMANCE_START(start_4)
 
@@ -270,8 +310,7 @@ const bool huge = out && ec > 10000;
           // PrintBuffer pcol. Insert separator rows as needed.
           //
           PrintBuffer & dest = pcols[x];
-          Assert(&dest);
-          new (&dest) PrintBuffer;
+          new (pcols + x) PrintBuffer;
 
           // compute the final height of dest and reserve enough rows as to
           // avoid unnecessary copies
@@ -280,7 +319,9 @@ const bool huge = out && ec > 10000;
             ShapeItem dest_height = 0;
             loop(y, rows)
                {
-                 dest_height += separator_rows(y, value.get_shape());
+                 dest_height += separator_rows(y, value, nested,
+                                               max_row_ranks[y],
+                                               y ? max_row_ranks[y - 1] : 0);
                  dest_height += item_matrix[y*cols + x].get_height();
                }
             dest.buffer.reserve(dest_height);
@@ -292,7 +333,9 @@ const bool huge = out && ec > 10000;
 
                 // insert separator row(s)
                 //
-                const ShapeItem srows = separator_rows(y, value.get_shape());
+                const ShapeItem srows = separator_rows(y, value, nested,
+                                                       max_row_ranks[y],
+                                                  y ? max_row_ranks[y - 1] : 0);
                 if (srows)   // unless first row
                    {
                     const UCS_string sepa_row(dest.get_width(0), UNI_iPAD_L0);
@@ -388,16 +431,17 @@ const bool huge = out && ec > 10000;
           UCS_string ucs(*this, value.get_rank(), pctx.get_PW());
           if (ucs.size())   *out << ucs << endl;
         }
-     complete = true;
 
      PERFORMANCE_END(fs_PrintBuffer5_B, start_5, ec)
    }
 
+   complete = true;
+   update_info();
    return;
 
 interrupted:
-   attention_raised = false;
-   interrupt_raised = false;
+   clear_attention_raised(LOC);
+   clear_interrupt_raised(LOC);
    *out << endl << "INTERRUPT" << endl;
 }
 //-----------------------------------------------------------------------------
@@ -449,11 +493,11 @@ ShapeItem bp_len = 0;
                   }
               col += chunk_len;
 
-              if (interrupt_raised)
+              if (interrupt_is_raised())
                  {
                    out << endl << "INTERRUPT" << endl;
-                   attention_raised = false;
-                   interrupt_raised = false;
+                   clear_attention_raised(LOC);
+                   clear_interrupt_raised(LOC);
                    delete del;
                    return;
                  }
@@ -549,10 +593,13 @@ ColInfo ci;
 }
 //-----------------------------------------------------------------------------
 ShapeItem
-PrintBuffer::separator_rows(ShapeItem y, const Shape & shape)
+PrintBuffer::separator_rows(ShapeItem y, const Value & value, bool nested,
+                            Rank rk1, Rank rk2)
 {
    if (y == 0)   return 0;
 
+const Shape & shape = value.get_shape();
+
 ShapeItem prod = 1;
 ShapeItem ret = 0;
    loop(r, shape.get_rank() - 2)
@@ -562,22 +609,28 @@ ShapeItem ret = 0;
          else                 break;
        }
 
+   if (nested)   // see lrm p. 138
+      {
+        const int max_rk = rk1 > rk2 ? rk1 : rk2;
+        if (max_rk > 1)   ret += max_rk - 1;
+      }
+
    return ret;
 }
 //-----------------------------------------------------------------------------
 Unicode
-PrintBuffer::get_char(uint32_t x, uint32_t y) const
+PrintBuffer::get_char(int x, int y) const
 { 
-   Assert(y < buffer.size());
-   Assert(x < buffer[y].size());
+   Assert(y < (int)buffer.size());
+   Assert(x < (int)buffer[y].size());
    return buffer[y][x];
 }
 //-----------------------------------------------------------------------------
 void
-PrintBuffer::set_char(uint32_t x, uint32_t y, Unicode uc)
+PrintBuffer::set_char(int x, int y, Unicode uc)
 {
-   Assert(y < buffer.size());
-   Assert(x < buffer[y].size());
+   Assert(y < (int)buffer.size());
+   Assert(x < (int)buffer[y].size());
    buffer[y][x] = uc;
 }
 //-----------------------------------------------------------------------------
@@ -605,6 +658,24 @@ PrintBuffer::pad_height(Unicode pad, ShapeItem height)
 }
 //-----------------------------------------------------------------------------
 void
+PrintBuffer::pad_to_spaces()
+{
+   loop(y, get_height())
+   loop(x, get_width(y))
+      if (is_iPAD_char(get_char(x, y)))   set_char(x, y, UNI_ASCII_SPACE);
+}
+//-----------------------------------------------------------------------------
+void
+PrintBuffer::pad_height_above(Unicode pad, ShapeItem height)
+{
+   if (height > get_height())
+      {
+        UCS_string ucs(get_width(0), pad);
+        while (height > get_height())   buffer.insert(buffer.begin(), ucs);
+      }
+}
+//-----------------------------------------------------------------------------
+void
 PrintBuffer::get_frame_chars(PrintStyle pst,
                              Unicode & HORI, Unicode & VERT,
                              Unicode & NW, Unicode & NE,
@@ -660,7 +731,7 @@ PrintBuffer::get_frame_chars(PrintStyle pst,
 }
 //-----------------------------------------------------------------------------
 void
-PrintBuffer::add_frame(PrintStyle style, const Shape & shape, uint32_t depth)
+PrintBuffer::add_frame(PrintStyle style, const Shape & shape, int depth)
 {
    Assert(is_rectangular());
 
