@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ Token
 Quad_FX::eval_B(Value_P B)
 {
 static const int eprops[] = { 0, 0, 0, 0 };
-   return do_quad_FX(eprops, B, UTF8_string("⎕FX"));
+   return do_quad_FX(eprops, B, UTF8_string("⎕FX"), false);
 }
 //-----------------------------------------------------------------------------
 Token
@@ -97,7 +97,7 @@ UTF8_string creator("⎕FX");
         default: LENGTH_ERROR;
       }
 
-   return do_quad_FX(eprops, B, creator);
+   return do_quad_FX(eprops, B, creator, false);
 }
 //-----------------------------------------------------------------------------
 Token
@@ -107,14 +107,13 @@ Quad_FX::eval_AXB(Value_P A, Value_P X, Value_P B)
    if (A->get_rank() > 1)                RANK_ERROR;
    if (!A->is_char_string())             DOMAIN_ERROR;
 
-const Axis axis = X->get_single_axis(10);
-
+const Axis axis = Value::get_single_axis(X.get(), 10);
    return do_native_FX(A, axis, B);
 }
 //-----------------------------------------------------------------------------
 Token
 Quad_FX::do_quad_FX(const int * exec_props, Value_P B,
-                    const UTF8_string & creator)
+                    const UTF8_string & creator, bool tolerant)
 {
    if (B->get_rank() > 2)   RANK_ERROR;
    if (B->get_rank() < 1)   RANK_ERROR;
@@ -167,7 +166,10 @@ UCS_string text;
                   if (!c1.is_character_cell())   DOMAIN_ERROR;
                   text.append(c1.get_char_value());
                 }
-             else DOMAIN_ERROR;
+             else
+                {
+                  DOMAIN_ERROR;
+                }
 
              text.append(UNI_ASCII_LF);
            }
@@ -185,19 +187,20 @@ UCS_string text;
            }
       }
 
-   return do_quad_FX(exec_props, text, creator);
+   return do_quad_FX(exec_props, text, creator, tolerant);
 }
 //-----------------------------------------------------------------------------
 Token
 Quad_FX::do_quad_FX(const int * exec_props, const UCS_string & text,
-                    const UTF8_string & creator)
+                    const UTF8_string & creator, bool tolerant)
 {
 int error_line = 0;
-UserFunction * fun = UserFunction::fix(text, error_line, false, LOC, creator);
+UserFunction * fun = UserFunction::fix(text, error_line, false, LOC, creator,
+                                       tolerant);
    if (fun == 0)
       {
         Value_P Z(LOC);
-        new (&Z->get_ravel(0))   IntCell(error_line + Workspace::get_IO());
+        new (Z->next_ravel())   IntCell(error_line + Workspace::get_IO());
         Z->check_value(LOC);
         return Token(TOK_APL_VALUE1, Z);
       }
