@@ -33,6 +33,8 @@
 Quad_TF   Quad_TF::_fun;
 Quad_TF * Quad_TF::fun = &Quad_TF::_fun;
 
+enum { Quad_PP_TF = 17 };
+
 //-----------------------------------------------------------------------------
 Token
 Quad_TF::eval_AB(Value_P A, Value_P B)
@@ -68,6 +70,8 @@ Value_P Z;
         DOMAIN_ERROR;
       }
 
+   if (!Z)   DOMAIN_ERROR;
+
    Z->check_value(LOC);
    return Token(TOK_APL_VALUE1, Z);
 }
@@ -160,16 +164,35 @@ const ShapeItem ec = val->element_count();
              const Cell & cell = val->get_ravel(e);
              if (cell.is_integer_cell())
                 {
+                  const int sign = ucs.size();
                   ucs.append_number(cell.get_int_value());
+                  if (ucs[sign] == '-')   ucs[sign] = UNI_OVERBAR;
                 }
-             else // if (cell.is_near_real())
+             else if (cell.is_near_real())
                 {
-                  PrintContext pctx(PR_APL_MIN, MAX_Quad_PP, MAX_Quad_PW);
-                  const APL_Float value = cell.get_real_value();
+                  PrintContext pctx(PR_APL_MIN, Quad_PP_TF, MAX_Quad_PW);
                   bool scaled = true;
-                  UCS_string ucs1(value, scaled, pctx);
+                  UCS_string ucs1(cell.get_real_value(), scaled, pctx);
                   ucs.append(ucs1);
                 }
+             else if (cell.is_numeric())
+                {
+                  PrintContext pctx(PR_APL_MIN, Quad_PP_TF, MAX_Quad_PW);
+                  bool scaled = true;
+                  UCS_string ucs1(cell.get_real_value(), scaled, pctx);
+                  ucs.append(ucs1);
+
+                  ucs.append(UNI_ASCII_J);
+
+                  ucs1 = UCS_string(cell.get_imag_value(), scaled, pctx);
+                  ucs.append(ucs1);
+                }
+             else
+                {
+                  Workspace::more_error() =
+                             UCS_string("Non-number in 1 ⎕TF N record");
+                  return Value_P();
+                }
            }
       }
 
@@ -206,31 +229,68 @@ UCS_string ucs;
 Value_P
 Quad_TF::tf1_inv(const UCS_string & ravel)
 {
+UCS_string & more = Workspace::more_error();
 const size_t len = ravel.size();
-   if (len < 2)   return Value_P();
+
+   if (len < 2)
+      {
+        more = UCS_string("1 ⎕TF record too short");
+        return Value_P();
+      }
 
    // mode should be 'F', 'N', or 'C'.
 const Unicode mode = ravel[0];
-   if (mode != 'C' && mode != 'F' && mode != 'N')   return Value_P();
+   if (mode != 'C' && mode != 'F' && mode != 'N')
+      {
+        more = UCS_string("1 ⎕TF record type not F, N, or C");
+        return Value_P();
+      }
 
 UCS_string name(ravel[1]);
    if (!Avec::is_quad(name[0]) && !Avec::is_first_symbol_char(name[0]))
-      return Value_P();
+      {
+        more = UCS_string("Bad variable name in 1 ⎕TF record");
+        return Value_P();
+      }
 
 ShapeItem idx = 2;
-   while (idx < len && Avec::is_symbol_char(ravel[idx]))   name.append(ravel[idx++]);
+   while (idx < len && Avec::is_symbol_char(ravel[idx]))
+         name.append(ravel[idx++]);
 
-   if (Avec::is_quad(name[0]) && name.size() == 1)   return Value_P();
-   if (ravel[idx++] != UNI_ASCII_SPACE)   return Value_P();
+   if (Avec::is_quad(name[0]) && name.size() == 1)
+      {
+        more = UCS_string("Bad variable name in 1 ⎕TF record");
+        return Value_P();
+      }
+
+   if (ravel[idx++] != UNI_ASCII_SPACE)
+      {
+        more = UCS_string("missing space (before rank) in 1 ⎕TF record");
+        return Value_P();
+      }
 
 ShapeItem idx0 = idx;
 Rank rank = 0;
    while (idx < len && Avec::is_digit(ravel[idx]))
       rank = 10 * rank + ravel[idx++] - UNI_ASCII_0;
 
-   if (rank == 0 && idx0 == idx)   return Value_P();   // no rank
-   if (ravel[idx++] != UNI_ASCII_SPACE)   return Value_P();
-   if (rank > MAX_RANK)    return Value_P();
+   if (rank == 0 && idx0 == idx)
+      {
+        more = UCS_string("missing rank in 1 ⎕TF record");
+        return Value_P();
+      }
+
+   if (ravel[idx++] != UNI_ASCII_SPACE)
+      {
+        more = UCS_string("missing space (after rank) in 1 ⎕TF record");
+        return Value_P();
+      }
+
+   if (rank > MAX_RANK)
+      {
+        more = UCS_string("max. rank exceeded in 1 ⎕TF record");
+        return Value_P();
+      }
 
 Shape shape;
    loop(r, rank)
@@ -239,8 +299,17 @@ Shape shape;
         ShapeItem sh = 0;
         while (idx < len && Avec::is_digit(ravel[idx]))
            sh = 10 * sh + ravel[idx++] - UNI_ASCII_0;
-        if (sh == 0 && idx0 == idx)   return Value_P();   // shape
-        if (ravel[idx++]!= UNI_ASCII_SPACE)  return Value_P();
+        if (sh == 0 && idx0 == idx)   // no shape
+           {
+             more = UCS_string("too few shape items in 1 ⎕TF record");
+             return Value_P();
+           }
+
+        if (ravel[idx++] != UNI_ASCII_SPACE)
+           {
+             more = UCS_string("missing space (in shape) in 1 ⎕TF record");
+             return Value_P();
+           }
         shape.add_shape_item(sh);
       }
 
@@ -251,19 +320,35 @@ NamedObject * sym_or_fun = Workspace::lookup_existing_name(name);
       {
         symbol = sym_or_fun->get_symbol();
         nc = sym_or_fun->get_nc();
-        if (symbol && symbol->is_readonly())   return Value_P();
+        if (symbol && symbol->is_readonly())
+           {
+             more = UCS_string("symbol cannot be modified in 1 ⎕TF record");
+             return Value_P();
+           }
      }
 
 const size_t data_chars = len - idx;
 
    if (mode == UNI_ASCII_F)   // function
       {
-        if (rank != 2)   return Value_P();
+        if (rank != 2)
+           {
+             more = UCS_string("function text is not a matrix in 1 ⎕TF record");
+             return Value_P();
+           }
 
         if (nc != NC_UNUSED_USER_NAME && nc != NC_FUNCTION && nc != NC_OPERATOR)
-           return Value_P();
+           {
+             more = UCS_string(
+                    "symbol is an existing variable in 1 ⎕TF F record");
+             return Value_P();
+           }
 
-        if (data_chars != shape.get_volume())   return Value_P();
+        if (data_chars != shape.get_volume())
+           {
+             more = UCS_string("item count mismatch in 1 ⎕TF N record");
+             return Value_P();
+           }
 
         Value_P new_val(shape, LOC);
         loop(d, data_chars)
@@ -274,8 +359,17 @@ const size_t data_chars = len - idx;
       }
    else if (mode == UNI_ASCII_C)   // char array
       {
-        if (data_chars != shape.get_volume())   return Value_P();
-        if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)   return Value_P();
+        if (data_chars != shape.get_volume())
+           {
+             more = UCS_string("item count mismatch in 1 ⎕TF C record");
+             return Value_P();
+           }
+        if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)
+           {
+             more = UCS_string(
+                    "symbol is existing and not a variable in 1 ⎕TF C record");
+             return Value_P();
+           }
 
         Value_P new_val(shape, LOC);
         loop(d, data_chars)
@@ -283,20 +377,30 @@ const size_t data_chars = len - idx;
 
         new_val->check_value(LOC);
 
-        if (symbol == 0)
-           symbol = Workspace::lookup_symbol(name);
-
-        symbol->assign(new_val, LOC);
+        if (symbol == 0)   symbol = Workspace::lookup_symbol(name);
+        symbol->assign(new_val, false, LOC);
       }
    else if (mode == UNI_ASCII_N)   // numeric array
       {
-        if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)   return Value_P();
+        if (nc != NC_UNUSED_USER_NAME && nc != NC_VARIABLE)
+           {
+             more = UCS_string("symbol cannot be assigned in 1 ⎕TF N record");
+             return Value_P();
+           }
 
         UCS_string data(ravel, idx, len - idx);
-        Tokenizer tokenizer(PM_EXECUTE, LOC);
+        Tokenizer tokenizer(PM_EXECUTE, LOC, false);
         Token_string tos;
-        if (tokenizer.tokenize(data, tos) != E_NO_ERROR)   return Value_P();
-        if (tos.size() != shape.get_volume())           return Value_P();
+        if (tokenizer.tokenize(data, tos) != E_NO_ERROR)
+           {
+             more = UCS_string("tokenization failed in 1 ⎕TF N record");
+             return Value_P();
+           }
+        if (tos.size() != shape.get_volume())
+           {
+             more = UCS_string("item count mismatch in 1 ⎕TF N record");
+             return Value_P();
+           }
 
         // check that all token are numeric...
         //
@@ -306,6 +410,7 @@ const size_t data_chars = len - idx;
              if (tag == TOK_INTEGER)   continue;
              if (tag == TOK_REAL)      continue;
              if (tag == TOK_COMPLEX)   continue;
+             more = UCS_string( "Non-number in 1 ⎕TF N record");
              return Value_P();
            }
 
@@ -330,7 +435,7 @@ const size_t data_chars = len - idx;
 
         if (!symbol)   symbol = Workspace::lookup_symbol(name);
 
-        symbol->assign(new_val, LOC);
+        symbol->assign(new_val, false, LOC);
       }
    else Assert(0);   // since checked above
    
@@ -396,7 +501,7 @@ Token_string tos;
    try
       {
         UCS_string ucs1 = no_UCS(ravel);
-        const Parser parser(PM_EXECUTE, LOC);
+        const Parser parser(PM_EXECUTE, LOC, false);
         parser.parse(ucs1, tos);
       }
    catch(...)
@@ -409,7 +514,6 @@ Token_string tos;
    //
    tf2_simplify(tos);
 
-
    if (tos.size() < 2)   return UCS_string();   // too short for an inverse 2⎕TF
 
    // it could happen that some system variable ⎕XX which is not known by
@@ -427,25 +531,24 @@ Token_string tos;
 
    // we expect either VAR ← VALUE ... or ⎕FX fun-text. Try VAR ← VALUE.
    //
-   if (tos[0].get_Class() == TC_SYMBOL && tos[1].get_tag() == TOK_ASSIGN)
+   if (tos[0].get_Class() == TC_SYMBOL && tos[1].get_Class() == TC_ASSIGN)
       {
         // at this point, we expect SYM ← VALUE.
         //
         if (tos.size() != 3)                  return UCS_string();
         if (tos[2].get_Class() != TC_VALUE)   return UCS_string();
 
-         tos[0].get_sym_ptr()->assign(tos[2].get_apl_val(), LOC);
+         tos[0].get_sym_ptr()->assign(tos[2].get_apl_val(), true, LOC);
          return tos[0].get_sym_ptr()->get_name();   // valid 2⎕TF
       }
 
-   // Try ⎕FX fun-text
+   // try dyadic ⎕FX (native function)
    //
-   if (tos.size() == 3)   // native function
+   if (tos.size() == 3                   &&
+       tos[0].get_Class() == TC_VALUE    &&
+       tos[1].get_tag()   == TOK_Quad_FX &&
+       tos[2].get_Class() == TC_VALUE)
       {
-        if (tos[0].get_Class() != TC_VALUE)    return UCS_string();
-        if (tos[1].get_tag() != TOK_Quad_FX)   return UCS_string();
-        if (tos[2].get_Class() != TC_VALUE)    return UCS_string();
-
         Value_P fname   =  tos[0].get_apl_val();
         Value_P so_path =  tos[2].get_apl_val();
 
@@ -459,22 +562,21 @@ Token_string tos;
         return UCS_string();
       }
 
+   // monadic ⎕FX. at this point we should have ⎕FX Value ...
+   //
    if (tos.size() != 2)                   return UCS_string();
    if (tos[0].get_tag() != TOK_Quad_FX)   return UCS_string();
    if (tos[1].get_Class() != TC_VALUE)    return UCS_string();
 
-   {
-     Value_P function_text =  tos[1].get_apl_val();
-     const Token tok = Quad_FX::fun->eval_B(function_text);
+static const int eprops[] = { 0, 0, 0, 0 };
+const Token tok = Quad_FX::do_quad_FX(eprops, tos[1].get_apl_val(),
+                                      UTF8_string("2 ⎕TF"), true);
 
-   if (tok.get_Class() == TC_VALUE)   // ⎕FX successful
-      {
-        Value_P val = tok.get_apl_val();
-        return UCS_string(*val.get());
-      }
-   }
+   if (tok.get_Class() != TC_VALUE)   return UCS_string();   // error in ⎕FX
 
-   return UCS_string();
+   // ⎕FX succeeded (and the token returned is the name of the function)
+   //
+   return UCS_string(*tok.get_apl_val().get());
 }
 //-----------------------------------------------------------------------------
 bool
@@ -482,7 +584,19 @@ Quad_TF::tf2_shape(UCS_string & ucs, const Shape & shape)
 {
    // dont print anything if shape is a scalar or non-empty vector
    //
-   if (shape.get_volume() != 0 && shape.get_rank() <= 1)   return false;
+   if (shape.get_rank() == 0)   return false;   // scalar
+
+   if (shape.get_rank() == 1)   // true vector
+      {
+        if (shape.get_volume() == 1)
+           {
+             ucs.append(UNI_ASCII_L_PARENT);
+             ucs.append(UNI_ASCII_COMMA);
+             return true;
+           }
+
+        if (shape.get_volume() != 0)   return false;
+      }
 
    ucs.append(UNI_ASCII_L_PARENT);
    loop(r, shape.get_rank())
@@ -612,7 +726,7 @@ bool have_parenth = tf2_shape(ucs, value->get_shape());
            }
         else if (cell.is_complex_cell())
            {
-             PrintContext pctx(PR_APL_MIN, MAX_Quad_PP, MAX_Quad_PW);
+             PrintContext pctx(PR_APL_MIN, Quad_PP_TF, MAX_Quad_PW);
              bool scaled = true;
              UCS_string ucs1(cell.get_real_value(), scaled, pctx);
              UCS_string ucs2(cell.get_imag_value(), scaled, pctx);
@@ -622,7 +736,7 @@ bool have_parenth = tf2_shape(ucs, value->get_shape());
            }
         else if (cell.is_float_cell())
            {
-             PrintContext pctx(PR_APL_MIN, MAX_Quad_PP, MAX_Quad_PW);
+             PrintContext pctx(PR_APL_MIN, Quad_PP_TF, MAX_Quad_PW);
              bool scaled = true;
              UCS_string ucs1(cell.get_real_value(), scaled, pctx);
              ucs.append(ucs1);
@@ -641,153 +755,441 @@ bool have_parenth = tf2_shape(ucs, value->get_shape());
    return false;
 }
 //-----------------------------------------------------------------------------
-int
+void
 Quad_TF::tf2_simplify(Token_string & tos)
 {
-int errors = 0;
-
-   for (bool progress = true; progress && !errors;)
+   for (int progress = 1; progress;)
        {
-         progress = false;
-         errors += tf2_remove_UCS(tos);
-         errors += tf2_remove_RHO(tos, progress);
-         errors += tf2_remove_parentheses(tos, progress);
-         errors += tf2_glue(tos, progress);
+         progress = 0;
+         tf2_remove_UCS(tos);
+         tf2_remove_RHO(tos, progress);
+         tf2_remove_COMMA(tos, progress);
+         tf2_remove_ENCLOSE_ENCLOSE(tos, progress);
+         tf2_remove_ENCLOSE(tos, progress);
+         tf2_remove_ENCLOSE1(tos, progress);
+         tf2_remove_sequence(tos, progress);
+         tf2_remove_sequence1(tos, progress);
+         tf2_remove_parentheses(tos, progress);
+         tf2_glue(tos, progress);
        }
-
-   return errors;
 }
 //-----------------------------------------------------------------------------
-int
+void
 Quad_TF::tf2_remove_UCS(Token_string & tos)
 {
-int d = 0;
-bool ucs_active = false;
+ShapeItem skipped = 0;
 
    loop(s, tos.size())
       {
         if (s < (tos.size() - 1) && tos[s].get_tag() == TOK_Quad_UCS)
            {
-              ucs_active = true;
-              continue;
+             s += 1;     skipped += 1;    // skip ⎕UCS
+             for (; s < tos.size() && tos[s].get_Class() == TC_VALUE; ++s)
+                {
+                  tos[s].get_apl_val()->toggle_UCS();
+                  move_1(tos[s - skipped], tos[s], LOC);
+                }
            }
 
-        if (d != s)   move_1(tos[d], tos[s], LOC);   // dont copy to itself
+        if (skipped)   move_1(tos[s - skipped], tos[s], LOC);
 
-        if (ucs_active)   // translate
-           {
-              Assert(tos[d].get_Class() == TC_VALUE);
-              tos[d].get_apl_val()->toggle_UCS();
-              ucs_active = false;
-           }
-
-        ++d;
       }
 
-   tos.shrink(d);
-   return 0;   // no error(s)
+   if (skipped)   tos.shrink(tos.size() - skipped);
 }
 //-----------------------------------------------------------------------------
-int
-Quad_TF::tf2_remove_RHO(Token_string & tos, bool & progress)
+void
+Quad_TF::tf2_remove_RHO(Token_string & tos, int & progress)
 {
-int d = 0;
+ShapeItem skipped = 0;
 
    loop(s, tos.size())
       {
         // we replace A⍴B by B reshaped to A. But only if the element count in
         // B agrees with A (otherwise we need to glue first).
         //
-        if (s < (tos.size() - 2)                  &&
-            tos[s    ].get_Class() == TC_VALUE    &&                // A
-            tos[s + 1].get_tag()   == TOK_F12_RHO &&                // ⍴
-            tos[s + 2].get_Class() == TC_VALUE)                     // B 
+        if (s >= (tos.size() - 2)                 ||
+            tos[s    ].get_Class() != TC_VALUE    ||                // A
+            tos[s + 1].get_tag()   != TOK_F12_RHO ||                // ⍴
+            tos[s + 2].get_Class() != TC_VALUE)                     // B 
            {
-             Value_P aval = tos[s].get_apl_val();
-             Value_P bval = tos[s + 2].get_apl_val();
-             Shape sh(aval, 0, 0);
-             if (sh.get_volume() == bval->element_count())   // same shape
-                {
-                  Token t(TOK_APL_VALUE1, bval);   // grouped value
-                  move_1(tos[d], t, LOC);
-                  bval->set_shape(sh);
-                  progress = true;
-                  ++d;
-                  s += 2;
-                 continue;
-                }
-             else
-                {
-                  move_2(tos[d], Bif_F12_RHO::do_reshape(sh, *bval), LOC);
-                  progress = true;
-                  ++d;
-                  s += 2;
-                 continue;
-                }
+             if (skipped)   // dont copy to itself
+                move_1(tos[s - skipped], tos[s], LOC);
+             continue;
            }
 
-        if (d != s)   move_1(tos[d], tos[s], LOC);   // dont copy to itself
-        ++d;
+        Shape sh;
+        {
+          Value_P aval = tos[s].get_apl_val();
+          sh = Shape(aval, /* ⎕IO */ 0);
+          tos[s].extract_apl_val(LOC);
+        }
+        s += 2;     skipped += 2;    // skip A ⍴
+
+        Value_P bval = tos[s].get_apl_val();
+        if (sh.get_volume() == bval->element_count())   // same volume
+           {
+             move_1(tos[s - skipped], tos[s], LOC);
+             tos[s - skipped].ChangeTag(TOK_APL_VALUE1);
+             bval->set_shape(sh);
+           }
+        else
+           {
+             Token t = Bif_F12_RHO::do_reshape(sh, *bval);
+             move_1(tos[s - skipped], t, LOC);
+           }
       }
 
-   tos.shrink(d);
-   return 0;   // no error(s)
+   if (skipped)
+      {
+        tos.shrink(tos.size() - skipped);
+        ++progress;
+      }
 }
 //-----------------------------------------------------------------------------
-int
-Quad_TF::tf2_remove_parentheses(Token_string & tos, bool & progress)
+void
+Quad_TF::tf2_remove_COMMA(Token_string & tos, int & progress)
 {
-int d = 0;
+ShapeItem skipped = 0;
 
-   loop(s, tos.size())   // ⍴ must not be first or last
+   loop(s, tos.size())
       {
-        // we replace A⍴B by B reshaped to A. But only if the element count in
-        // B agrees with A (otherwise we need to glue first).
+        // we replace , B by B reshaped or A , B by AB
         //
-        if (s < (tos.size() - 2)                 &&
+        if (s < (tos.size() - 1)                    &&
+            tos[s    ].get_tag()   == TOK_F12_COMMA &&   // ,
+            tos[s + 1].get_Class() == TC_VALUE)          // B 
+           {
+             const ShapeItem d_1 = s - skipped - 1;
+             if (s > 0 && tos[d_1].get_tag() == TOK_R_PARENT)   // value to be
+                {
+                  // TOK_R_PARENT will become a value: just copy it for now
+                  //
+                  if (skipped)   move_1(tos[s - skipped], tos[s], LOC);
+                  continue;
+                }
+
+             if (s > 0 && tos[d_1].get_Class() == TC_VALUE)   // value
+                {
+                  Value_P A = tos[d_1].get_apl_val();
+                  Value_P B = tos[s + 1].get_apl_val();
+                  Assert(A->get_rank() <= 1);
+                  Assert(B->get_rank() <= 1);
+                  Value_P Z(A->element_count() + B->element_count(), LOC);
+                  loop(a, A->element_count())
+                     Z->next_ravel()->init(A->get_ravel(a), Z.getref(), LOC);
+                  loop(b, B->element_count())
+                     Z->next_ravel()->init(B->get_ravel(b), Z.getref(), LOC);
+
+                  tos[s + 1].clear(LOC);
+                  Token tok_AB(TOK_APL_VALUE1, Z);
+                  move_2(tos[d_1], tok_AB, LOC);
+                  s += 1;   skipped += 2;   // skip , (of A , B) but not B
+                  continue;   // don't move_1() below
+                }
+
+             // monadic , B
+             s += 1;   skipped += 1;    // skip ,
+
+             Value_P bval = tos[s].get_apl_val();
+             const Shape sh(bval->element_count());
+             bval->set_shape(sh);
+
+             tos[s].ChangeTag(TOK_APL_VALUE1);
+             move_1(tos[s - skipped], tos[s], LOC);
+             continue;
+           }
+
+        if (skipped)   move_1(tos[s - skipped], tos[s], LOC);
+      }
+
+   if (skipped)
+      {
+        tos.shrink(tos.size() - skipped);
+        ++progress;
+      }
+}
+//-----------------------------------------------------------------------------
+void
+Quad_TF::tf2_remove_ENCLOSE_ENCLOSE(Token_string & tos, int & progress)
+{
+ShapeItem skipped = 0;
+
+   loop(s, tos.size())
+      {
+        if (s >= (tos.size() - 2)                       ||
+            (s && tos[s - 1].get_Class() == TC_VALUE)   ||   // dyadic ⊂
+            tos[s    ].get_tag()   != TOK_F12_PARTITION ||   // not ⊂
+            tos[s + 1].get_tag()   != TOK_F12_PARTITION ||   // not ⊂
+            tos[s + 2].get_Class() != TC_VALUE)              // not B 
+           {
+             if (skipped)   // dont copy to itself
+                move_1(tos[s - skipped], tos[s], LOC);
+             continue;
+           }
+
+        skipped++;   // ignore ⊂ at tos[s]
+      }
+
+   if (skipped)
+      {
+        tos.shrink(tos.size() - skipped);
+        ++progress;
+      }
+}
+//-----------------------------------------------------------------------------
+void
+Quad_TF::tf2_remove_sequence(Token_string & tos, int & progress)
+{
+ShapeItem skipped = 0;
+
+//   tos.print(CERR);
+
+   // replace N - ⎕IO - ⍳ K  by  N N+1 ... N+K-1
+   //         0 1 2   3 4 5
+   //
+   loop(s, tos.size())
+      {
+        if ((s >= (tos.size() - 5))                    ||   // too short
+            tos[s    ].get_Class() != TC_VALUE         ||   // not N
+            tos[s + 1].get_tag()   != TOK_F12_MINUS    ||   // not -
+            tos[s + 2].get_tag()   != TOK_Quad_IO      ||   // not ⎕IO
+            tos[s + 3].get_tag()   != TOK_F12_MINUS    ||   // not -
+            tos[s + 4].get_tag()   != TOK_F12_INDEX_OF ||   // not ⍳
+            tos[s + 5].get_Class() != TC_VALUE         ||   // not K
+           !tos[s    ].get_apl_val()->is_int_scalar()  ||   // N not integer
+           !tos[s + 5].get_apl_val()->is_int_scalar())      // K not integer
+           {
+             if (skipped)   // dont copy to itself
+                move_1(tos[s - skipped], tos[s], LOC);
+             continue;
+           }
+
+        const APL_Integer N = tos[s].get_apl_val()->get_ravel(0)
+                                    .get_int_value();
+        const APL_Integer K = tos[s + 5].get_apl_val()->get_ravel(0)
+                                        .get_int_value();
+
+        loop(j, 6)   tos[s + j].clear(LOC);
+
+        Value_P sequence(K, LOC);
+        loop(k, K)   new (sequence->next_ravel())   IntCell(N + k);
+        sequence->check_value(LOC);
+        Token tok(TOK_APL_VALUE1, sequence);
+        move_2(tos[s - skipped], tok, LOC);
+        s += 5;   skipped += 5;
+      }
+
+   if (skipped)
+      {
+        tos.shrink(tos.size() - skipped);
+        ++progress;
+      }
+}
+//-----------------------------------------------------------------------------
+void
+Quad_TF::tf2_remove_sequence1(Token_string & tos, int & progress)
+{
+ShapeItem skipped = 0;
+
+//   tos.print(CERR);
+
+   // replace N - M × ⎕IO - ⍳ K by M(N N+1 ... N+K-1)
+   //         0 1 2 3 4   5 6 7
+   //
+   loop(s, tos.size())
+      {
+        if ((s >= (tos.size() - 7))                    ||   // too short
+            tos[s    ].get_Class() != TC_VALUE         ||   // not N
+            tos[s + 1].get_tag()   != TOK_F12_MINUS    ||   // not -
+            tos[s + 2].get_Class() != TC_VALUE         ||   // not K
+            tos[s + 3].get_tag()   != TOK_F12_TIMES    ||   // not ×
+            tos[s + 4].get_tag()   != TOK_Quad_IO      ||   // not ⎕IO
+            tos[s + 5].get_tag()   != TOK_F12_MINUS    ||   // not -
+            tos[s + 6].get_tag()   != TOK_F12_INDEX_OF ||   // not ⍳
+            tos[s + 7].get_Class() != TC_VALUE         ||   // not K
+           !tos[s    ].get_apl_val()->is_int_scalar()  ||   // N not integer
+           !tos[s + 2].get_apl_val()->is_int_scalar()  ||   // M not integer
+           !tos[s + 7].get_apl_val()->is_int_scalar())      // K not integer
+           {
+             if (skipped)   // dont copy to itself
+                move_1(tos[s - skipped], tos[s], LOC);
+             continue;
+           }
+
+        const APL_Integer N = tos[s].get_apl_val()->get_ravel(0)
+                                    .get_int_value();
+        const APL_Integer M = tos[s + 2].get_apl_val()->get_ravel(0)
+                                        .get_int_value();
+        const APL_Integer K = tos[s + 7].get_apl_val()->get_ravel(0)
+                                        .get_int_value();
+
+        loop(j, 6)   tos[s + j].clear(LOC);
+
+        Value_P sequence(K, LOC);
+        loop(k, K)   new (sequence->next_ravel())   IntCell(M * (N + k));
+        sequence->check_value(LOC);
+        Token tok(TOK_APL_VALUE1, sequence);
+        move_2(tos[s - skipped], tok, LOC);
+        s += 7;   skipped += 7;
+      }
+
+   if (skipped)
+      {
+        tos.shrink(tos.size() - skipped);
+        ++progress;
+      }
+}
+//-----------------------------------------------------------------------------
+void
+Quad_TF::tf2_remove_ENCLOSE(Token_string & tos, int & progress)
+{
+ShapeItem skipped = 0;
+
+   // replace  ( ⊂ B )  by an enclosed B  
+   //
+   loop(s, tos.size())
+      {
+        if (s >= (tos.size() - 3)                       ||
+            tos[s    ].get_tag()   != TOK_L_PARENT      ||   // not (
+            tos[s + 1].get_tag()   != TOK_F12_PARTITION ||   // not ⊂
+            tos[s + 2].get_Class() != TC_VALUE          ||   // not B 
+            tos[s + 3].get_tag()   != TOK_R_PARENT)          // not )
+           {
+             if (skipped)   // dont copy to itself
+                move_1(tos[s - skipped], tos[s], LOC);
+             continue;
+           }
+
+        Value_P B = tos[s + 2].get_apl_val();
+        if (B->is_scalar())
+           {
+             move_1(tos[s - skipped], tos[s + 2], LOC);
+           }
+        else
+           {
+             Value_P enc_B(LOC);
+             new (enc_B->next_ravel()) PointerCell(B, enc_B.getref());
+             Token tok(TOK_APL_VALUE1, enc_B);
+             move_2(tos[s - skipped], tok, LOC);
+             tos[s + 2].clear(LOC);   // B
+           }
+        s += 3;   skipped += 3;
+      }
+
+   if (skipped)
+      {
+        tos.shrink(tos.size() - skipped);
+        ++progress;
+      }
+}
+//-----------------------------------------------------------------------------
+void
+Quad_TF::tf2_remove_ENCLOSE1(Token_string & tos, int & progress)
+{
+ShapeItem skipped = 0;
+
+   // replace  ⍴ ⊂ B  by ⍴ enclosed B  
+   //
+   loop(s, tos.size())
+      {
+        if (s >= (tos.size() - 2)                       ||
+            tos[s    ].get_tag()   != TOK_F12_RHO       ||   // not ⍴
+            tos[s + 1].get_tag()   != TOK_F12_PARTITION ||   // not ⊂
+            tos[s + 2].get_Class() != TC_VALUE)              // not B 
+           {
+             if (skipped)   // dont copy to itself
+                move_1(tos[s - skipped], tos[s], LOC);
+             continue;
+           }
+
+        if (skipped)   // dont copy to itself
+           move_1(tos[s - skipped], tos[s], LOC);   // move ⍴
+
+        Value_P B = tos[s + 2].get_apl_val();
+        if (B->is_scalar())
+           {
+             move_1(tos[s - skipped + 1], tos[s + 2], LOC);
+           }
+        else
+           {
+             Value_P enc_B(LOC);
+             new (enc_B->next_ravel()) PointerCell(B, enc_B.getref());
+             Token tok(TOK_APL_VALUE1, enc_B);
+             move_2(tos[s - skipped + 1], tok, LOC);
+             tos[s + 2].clear(LOC);   // B
+           }
+        s += 2;   ++skipped;
+      }
+
+   if (skipped)
+      {
+        tos.shrink(tos.size() - skipped);
+        ++progress;
+      }
+}
+//-----------------------------------------------------------------------------
+void
+Quad_TF::tf2_remove_parentheses(Token_string & tos, int & progress)
+{
+ShapeItem skipped = 0;
+
+   loop(s, tos.size())
+      {
+        // we replace ( B ) by B 
+        //
+        if (s < (tos.size() - 2)                   &&
             tos[s].get_tag()       == TOK_L_PARENT &&
             tos[s + 1].get_Class() == TC_VALUE     &&
             tos[s + 2].get_tag()   == TOK_R_PARENT)   // ( B )
            {
-             move_1(tos[d], tos[s + 1], LOC);   // override A of A⍴B
-             progress = true;
-             ++d;
-             s += 2;
+             move_1(tos[s - skipped], tos[s + 1], LOC);  // override ( of (B)
+             s += 2;   skipped += 2;
              continue;
            }
 
-        if (d != s)   move_1(tos[d], tos[s], LOC);   // dont copy to itself
-        ++d;
+        if (skipped)   // dont copy to itself
+           move_1(tos[s - skipped], tos[s], LOC);
       }
 
-   tos.shrink(d);
-   return 0;   // no error(s)
+   if (skipped)
+      {
+        tos.shrink(tos.size() - skipped);
+        ++progress;
+      }
 }
 //-----------------------------------------------------------------------------
-int
-Quad_TF::tf2_glue(Token_string & tos, bool & progress)
+void
+Quad_TF::tf2_glue(Token_string & tos, int & progress)
 {
-int d = 0;
+ShapeItem skipped = 0;
 
    loop(s, tos.size())   // ⍴ must not be first or last
       {
-        if (d != s)   move_1(tos[d], tos[s], LOC);   // dont copy to itself
+        // the first value (if any) survives
+        //
+        if (skipped)   // dont copy to itself
+           move_1(tos[s - skipped], tos[s], LOC);
 
-        if (tos[d].get_Class() == TC_VALUE)
-           {
-             while ((s + 1) < tos.size() && tos[s + 1].get_Class() == TC_VALUE)
-                {
-                   Value::glue(tos[d], tos[d], tos[s + 1], LOC);
-                   progress = true;
-                  ++s;
-                }
-           }
+        if (tos[s - skipped].get_Class() != TC_VALUE)   continue;
 
-        ++d;
+         // subsequent values (if any) are glued to the first value which
+         // is now tos[s - skipped]
+         //
+         while ((s + 1) < tos.size() && tos[s + 1].get_Class() == TC_VALUE)
+            {
+               Token t;
+               move_1(t, tos[s - skipped], LOC);
+               Value::glue(tos[s - skipped], t, tos[s + 1], LOC);
+               s += 1;   skipped += 1;
+            }
       }
 
-   tos.shrink(d);
-   return 0;   // no error(s)
+   if (skipped)
+      {
+        tos.shrink(tos.size() - skipped);
+        ++progress;
+      }
 }
 //-----------------------------------------------------------------------------
 UCS_string
@@ -919,11 +1321,11 @@ NamedObject * obj = Workspace::lookup_existing_name(name);
 Symbol * symbol = obj->get_symbol();
    if (symbol)
       {
-        const Value & value = *symbol->get_apl_value();
-        if (&value)
+        const Value * value = &*symbol->get_apl_value();
+        if (value)
            {
              CDR_string cdr;
-             CDR::to_CDR(cdr, value);
+             CDR::to_CDR(cdr, *value);
 
              return Value_P(cdr, LOC);
            }
