@@ -237,6 +237,7 @@ Value_P Z(B->get_shape(), LOC);
    for (PJob_scalar_B * job = joblist_B.next_job();
         job; job = joblist_B.next_job())
       {
+#if PARALLEL_ENABLED
         if (  Parallel::run_parallel
            && Thread_context::get_active_core_count() > 1
            && job->len_Z > get_monadic_threshold())
@@ -255,6 +256,8 @@ Value_P Z(B->get_shape(), LOC);
               }
            }
         else   // sequential
+#endif // PARALLEL_ENABLED
+
            {
              loop(z, job->len_Z)
                 {
@@ -396,8 +399,8 @@ ScalarFunction::eval_scalar_AB(Value_P A, Value_P B, prim_f2 fun)
 {
 PERFORMANCE_START(start_1)
 
-const int inc_A = A->is_scalar_or_len1_vector() ? 0 : 1;
-const int inc_B = B->is_scalar_or_len1_vector() ? 0 : 1;
+const int inc_A = A->is_scalar_extensible() ? 0 : 1;
+const int inc_B = B->is_scalar_extensible() ? 0 : 1;
 const Shape * shape_Z = 0;
    if      (A->is_scalar())      shape_Z = &B->get_shape();
    else if (B->is_scalar())      shape_Z = &A->get_shape();
@@ -429,6 +432,7 @@ Value_P Z(*shape_Z, LOC);
    for (PJob_scalar_AB * job = joblist_AB.next_job();
         job; job = joblist_AB.next_job())
       {
+#if PARALLEL_ENABLED
         if (  Parallel::run_parallel
            && Thread_context::get_active_core_count() > 1
            && job->len_Z > get_dyadic_threshold())
@@ -447,6 +451,7 @@ Value_P Z(*shape_Z, LOC);
                }
            }
         else
+#endif // PARALLEL_ENABLED
            {
              loop(z, job->len_Z)
                 {
@@ -462,9 +467,9 @@ Value_P Z(*shape_Z, LOC);
                           Value_P A1 = cell_A.get_pointer_value();
                           Value_P B1 = cell_B.get_pointer_value();
                           const int inc_A1 =
-                                    A1->is_scalar_or_len1_vector() ? 0 : 1;
+                                    A1->is_scalar_extensible() ? 0 : 1;
                           const int inc_B1 =
-                                    B1->is_scalar_or_len1_vector() ? 0 : 1;
+                                    B1->is_scalar_extensible() ? 0 : 1;
                           const Shape * sh_Z1 = &B1->get_shape();
                           if      (A1->is_scalar())   sh_Z1 = &B1->get_shape();
                           else if (B1->is_scalar())   sh_Z1 = &A1->get_shape();
@@ -485,6 +490,7 @@ Value_P Z(*shape_Z, LOC);
 
                           POOL_LOCK(joblist_AB.parallel_jobs_lock,
                              Value_P Z1(*sh_Z1, LOC);
+                             Z1->set_complete();
                              new (&cell_Z) PointerCell(Z1, job->value_Z);
 
                              PJob_scalar_AB j1(Z1.getref(),
@@ -498,7 +504,7 @@ Value_P Z(*shape_Z, LOC);
                           //
                           Value_P A1 = cell_A.get_pointer_value();
                           const int inc_A1 =
-                                    A1->is_scalar_or_len1_vector() ? 0 : 1;
+                                    A1->is_scalar_extensible() ? 0 : 1;
 
                           const ShapeItem len_Z1 = A1->get_shape().get_volume();
                           if (len_Z1 == 0)
@@ -512,6 +518,7 @@ Value_P Z(*shape_Z, LOC);
                              {
                                 POOL_LOCK(joblist_AB.parallel_jobs_lock,
                                   Value_P Z1(A1->get_shape(), LOC);
+                                  Z1->set_complete();
                                   new (&cell_Z) PointerCell(Z1, job->value_Z);
 
                                   PJob_scalar_AB j1(Z1.getref(),
@@ -527,7 +534,7 @@ Value_P Z(*shape_Z, LOC);
                           //
                           Value_P B1 = cell_B.get_pointer_value();
                           const int inc_B1 =
-                                    B1->is_scalar_or_len1_vector() ? 0 : 1;
+                                    B1->is_scalar_extensible() ? 0 : 1;
 
                           const ShapeItem len_Z1 = B1->get_shape().get_volume();
                           if (len_Z1 == 0)
@@ -542,6 +549,7 @@ Value_P Z(*shape_Z, LOC);
                                POOL_LOCK(joblist_AB.parallel_jobs_lock,
                                   Value_P Z1(B1->get_shape(), LOC);
                                   new (&cell_Z) PointerCell(Z1, job->value_Z);
+                                  Z1->set_complete();
 
                                   PJob_scalar_AB j1(Z1.getref(),
                                                     &cell_A, 0,
@@ -569,7 +577,7 @@ CELL_PERFORMANCE_END(get_statistics_AB(), start_2, z)
       }
 
    joblist_AB.cancel_jobs();
-   Z->set_default(*B.get());
+   Z->set_default(*B.get(), LOC);
    Z->check_value(LOC);
 
 PERFORMANCE_END(fs_SCALAR_AB, start_1, len_Z)
@@ -601,8 +609,8 @@ ShapeItem end_z = z + slice_len;
                      //
                      Value_P A1 = cell_A.get_pointer_value();
                      Value_P B1 = cell_B.get_pointer_value();
-                     const int inc_A1 = A1->is_scalar_or_len1_vector() ? 0 : 1;
-                     const int inc_B1 = B1->is_scalar_or_len1_vector() ? 0 : 1;
+                     const int inc_A1 = A1->is_scalar_extensible() ? 0 : 1;
+                     const int inc_B1 = B1->is_scalar_extensible() ? 0 : 1;
                      const Shape * sh_Z1 = &B1->get_shape();
                      if      (A1->is_scalar())   sh_Z1 = &B1->get_shape();
                      else if (B1->is_scalar())   sh_Z1 = &A1->get_shape();
@@ -628,6 +636,7 @@ ShapeItem end_z = z + slice_len;
 
                      POOL_LOCK(joblist_AB.parallel_jobs_lock,
                         Value_P Z1(*sh_Z1, LOC);
+                        Z1->set_complete();
                         new (&cell_Z) PointerCell(Z1, job.value_Z);
 
                         PJob_scalar_AB j1(Z1.getref(),
@@ -640,7 +649,7 @@ ShapeItem end_z = z + slice_len;
                      // A is nested, B is not
                      //
                      Value_P A1 = cell_A.get_pointer_value();
-                     const int inc_A1 = A1->is_scalar_or_len1_vector() ? 0 : 1;
+                     const int inc_A1 = A1->is_scalar_extensible() ? 0 : 1;
 
                      const ShapeItem len_Z1 = A1->get_shape().get_volume();
                      if (len_Z1 == 0)
@@ -655,6 +664,7 @@ ShapeItem end_z = z + slice_len;
                           POOL_LOCK(joblist_AB.parallel_jobs_lock,
                              Value_P Z1(A1->get_shape(), LOC);
                              new (&cell_Z) PointerCell(Z1, job.value_Z);
+                             Z1->set_complete();
 
                              PJob_scalar_AB j1(Z1.getref(),
                                                &A1->get_ravel(0), inc_A1,
@@ -668,7 +678,7 @@ ShapeItem end_z = z + slice_len;
                      // A is not nested, B is nested
                      //
                      Value_P B1 = cell_B.get_pointer_value();
-                     const int inc_B1 = B1->is_scalar_or_len1_vector() ? 0 : 1;
+                     const int inc_B1 = B1->is_scalar_extensible() ? 0 : 1;
 
                      const ShapeItem len_Z1 = B1->get_shape().get_volume();
                      if (len_Z1 == 0)
@@ -683,6 +693,7 @@ ShapeItem end_z = z + slice_len;
                           POOL_LOCK(joblist_AB.parallel_jobs_lock,
                              Value_P Z1(B1->get_shape(), LOC);
                              new (&cell_Z) PointerCell(Z1, job.value_Z);
+                             Z1->set_complete();
 
                              PJob_scalar_AB j1(Z1.getref(),
                                                &cell_A, 0,
@@ -709,7 +720,7 @@ ScalarFunction::eval_fill_AB(Value_P A, Value_P B)
 {
    // eval_fill_AB() is called when A or B or both are empty
    //
-   if (A->is_scalar_or_len1_vector())   // then B is empty
+   if (A->is_scalar_extensible())   // then B is empty
       {
         Value_P Z = B->clone(LOC);
         Z->to_proto();
@@ -717,7 +728,7 @@ ScalarFunction::eval_fill_AB(Value_P A, Value_P B)
         return Token(TOK_APL_VALUE1, Z);
       }
 
-   if (B->is_scalar_or_len1_vector())   // then A is empty
+   if (B->is_scalar_extensible())   // then A is empty
       {
         Value_P Z = A->clone(LOC);
         Z->to_proto();
@@ -779,15 +790,16 @@ const Cell & cell_FI0 = FI0->get_ravel(0);
            Value_P sub(proto_B.get_pointer_value()->get_shape(),LOC);
         const ShapeItem len_sub = sub->nz_element_count();
         Cell * csub = &sub->get_ravel(0);
-        loop(s, len_sub)   csub++->init(cell_FI0, sub.getref());
+        loop(s, len_sub)   csub++->init(cell_FI0, sub.getref(), LOC);
+        sub->check_value(LOC);
 
         while (Z->more())
            new (Z->next_ravel()) PointerCell(sub->clone(LOC), Z.getref()))
       }
    else
       {
-        while (Z->more())   Z->next_ravel()->init(cell_FI0, Z.getref());
-        if (Z->is_empty())  Z->get_ravel(0).init(cell_FI0, Z.getref());
+        while (Z->more())   Z->next_ravel()->init(cell_FI0, Z.getref(), LOC);
+        if (Z->is_empty())  Z->get_ravel(0).init(cell_FI0, Z.getref(), LOC);
       }
 
    Z->check_value(LOC);
@@ -799,7 +811,7 @@ ScalarFunction::eval_scalar_AXB(Value_P A, Value_P X, Value_P B, prim_f2 fun)
 {
 PERFORMANCE_START(start_1)
 
-   if (A->is_scalar_or_len1_vector() || B->is_scalar_or_len1_vector() || !X)
+   if (!X || A->is_scalar_extensible() || B->is_scalar_extensible())
       return eval_scalar_AB(A, B, fun);
 
    if (X->get_rank() > 1)   AXIS_ERROR;
@@ -811,6 +823,7 @@ bool axis_in_X[MAX_RANK];
    loop(r, MAX_RANK)   axis_in_X[r] = false;
 
 const ShapeItem len_X = X->element_count();
+
    loop(iX, len_X)
        {
          APL_Integer i = X->get_ravel(iX).get_near_int() - qio;
@@ -820,12 +833,28 @@ const ShapeItem len_X = X->element_count();
          axis_in_X[i] = true;
        }
 
-Value_P Z = (rank_A < rank_B) ? eval_scalar_AXB(A, axis_in_X, B, fun, false)
-                              : eval_scalar_AXB(B, axis_in_X, A, fun, true);
+   if (rank_A == rank_B)
+      {
+        if (rank_A != len_X)   AXIS_ERROR;
+        return eval_scalar_AB(A, B, fun);
+      }
 
+   if (rank_A < rank_B)
+      {
+        if (rank_A != len_X)   AXIS_ERROR;
+
+        Value_P Z = eval_scalar_AXB(A, axis_in_X, B, fun, false);
 PERFORMANCE_END(fs_SCALAR_AB, start_1, Z->nz_element_count())
+        return Token(TOK_APL_VALUE1, Z);
+      }
+    else
+      {
+        if (rank_B != len_X)   AXIS_ERROR;
 
-   return Token(TOK_APL_VALUE1, Z);
+        Value_P Z = eval_scalar_AXB(B, axis_in_X, A, fun, true);
+PERFORMANCE_END(fs_SCALAR_AB, start_1, Z->nz_element_count())
+        return Token(TOK_APL_VALUE1, Z);
+      }
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -881,7 +910,7 @@ const Cell * cB = &B->get_ravel(0);
             expand_pointers(Z->next_ravel(), Z.getref(), cA, cB++, fun);
        }
 
-   Z->set_default(*B.get());
+   Z->set_default(*B.get(), LOC);
 
    Z->check_value(LOC);
    return Z;
@@ -900,7 +929,7 @@ Shape shape_A;
 
    if (A->get_rank() > B->get_rank())   // then Z is all zeros.
       {
-        loop(z, len_Z)   new (&Z->get_ravel(z))   IntCell(0);
+        loop(z, len_Z)   new (Z->next_ravel())   IntCell(0);
         goto done;
       }
 
@@ -918,7 +947,7 @@ Shape shape_A;
        {
          if (shape_A.get_shape_item(r) > B->get_shape_item(r))
             {
-              loop(z, len_Z)   new (&Z->get_ravel(z))   IntCell(0);
+              loop(z, len_Z)   new (Z->next_ravel())   IntCell(0);
               goto done;
             }
        }
@@ -935,7 +964,6 @@ CELL_PERFORMANCE_END(get_statistics_AB(), start_2, zi.get_total())
        }
 
 done:
-   Z->set_default_Zero();
    Z->check_value(LOC);
 
 PERFORMANCE_END(fs_SCALAR_AB, start_1, Z->nz_element_count());
@@ -973,40 +1001,39 @@ const Shape weight = B->get_shape().reverse_scan();
 Token
 Bif_F12_ROLL::eval_AB(Value_P A, Value_P B)
 {
-const APL_Integer qio = Workspace::get_IO();
-
    // draw A items  from the set [quad-IO ... B]
    //
-   if (!A->is_scalar_or_len1_vector())   RANK_ERROR;
-   if (!B->is_scalar_or_len1_vector())   RANK_ERROR;
+   if (!A->is_scalar_extensible())   RANK_ERROR;
+   if (!B->is_scalar_extensible())   RANK_ERROR;
 
-const uint32_t aa = A->get_ravel(0).get_near_int();
+const ShapeItem zlen = A->get_ravel(0).get_near_int();
 APL_Integer set_size = B->get_ravel(0).get_near_int();
-   if (aa > set_size)           DOMAIN_ERROR;
+   if (zlen > set_size)         DOMAIN_ERROR;
    if (set_size <= 0)           DOMAIN_ERROR;
    if (set_size > 0x7FFFFFFF)   DOMAIN_ERROR;
 
-
-Value_P Z(aa, LOC);
+Value_P Z(zlen, LOC);
 
    // set_size can be rather big, so we new/delete it
    //
-uint32_t * idx_B = new uint32_t[set_size];
-   if (idx_B == 0)   DOMAIN_ERROR;
-   
-   loop(c, set_size)   idx_B[c] = c + qio;
+uint8_t * used = new uint8_t[(set_size + 7)/8];
+   if (used == 0)   throw_apl_error(E_WS_FULL, LOC);
+   memset(used, 0, (set_size + 7)/8);
 
-   loop(z, aa)
+   loop(z, zlen)
        {
-         const uint64_t rnd = Workspace::get_RL(set_size) % set_size;
-         new (&Z->get_ravel(z)) IntCell(idx_B[rnd]);
-         idx_B[rnd] = idx_B[set_size - 1];   // move last item in.
-         --set_size;
+         const uint64_t rnd = Workspace::get_RL(set_size);
+
+         if (used[rnd >> 3] & 1 << (rnd & 7))   // already drawn
+            {
+              --z;
+              continue;
+            }
+         used[rnd >> 3] |= 1 << (rnd & 7);
+         new (Z->next_ravel()) IntCell(rnd + Workspace::get_IO());
        }
 
-   delete [] idx_B;
-
-   Z->set_default_Zero();
+   delete [] used;
 
    Z->check_value(LOC);
    return Token(TOK_APL_VALUE1, Z);
@@ -1038,8 +1065,8 @@ const Cell * C = &B.get_ravel(0);
             continue;
           }
 
-       if (!C->is_near_int())       return true;
-       if (C->get_near_int() < 0)   return true;
+       if (!C->is_near_int())               return true;
+       if (C->get_checked_near_int() < 0)   return true;
 
         ++C;
       }
@@ -1057,7 +1084,7 @@ const uint32_t len_B = B->element_count();
 const APL_Float qct = Workspace::get_CT();
 Value_P Z(len_A, LOC);
 
-uint32_t len_Z = 0;
+ShapeItem len_Z = 0;
 
    loop(a, len_A)
       {
@@ -1074,14 +1101,14 @@ uint32_t len_Z = 0;
 
         if (!found)
            {
-             Z->get_ravel(len_Z++).init(cell_A, Z.getref());
+             Z->next_ravel()->init(cell_A, Z.getref(), LOC);
+             ++len_Z;
            }
       }
 
    Z->set_shape_item(0, len_Z);
 
-   Z->set_default(*B.get());
-
+   Z->set_default(*A.get(), LOC);
    Z->check_value(LOC);
    return Token(TOK_APL_VALUE1, Z);
 }
