@@ -22,7 +22,7 @@
 #include "Value.icc"
 
 //-----------------------------------------------------------------------------
-Shape::Shape(Value_P A, APL_Float qct, int qio_A)
+Shape::Shape(Value_P A, int qio_A)
    : rho_rho(0),
      volume(1)
 {
@@ -120,6 +120,28 @@ ShapeItem w = 1;
    return p;
 }
 //-----------------------------------------------------------------------------
+Shape
+Shape::offset_to_index(ShapeItem offset) const
+{
+Shape ret;
+   ret.rho_rho = rho_rho;
+   for (int r = rho_rho; r > 0;)
+       {
+         --r;
+         if (rho[r] == 0)
+            {
+              Assert(offset == 0);
+              ret.rho[r] = 0;
+              continue;
+            }
+
+         ret.rho[r] = offset % rho[r];
+         offset /= rho[r];
+       }
+
+   return ret;
+}
+//-----------------------------------------------------------------------------
 void
 Shape::check_same(const Shape & B, ErrorCode rank_err, ErrorCode len_err,
                   const char * loc) const
