@@ -25,6 +25,7 @@
 #include "IndexExpr.hh"
 #include "Output.hh"
 #include "Prefix.hh"
+#include "SharedValuePointer.hh"
 #include "StateIndicator.hh"
 #include "SystemLimits.hh"
 #include "UserFunction.hh"
@@ -33,8 +34,7 @@
 //-----------------------------------------------------------------------------
 StateIndicator::StateIndicator(Executable * exec, StateIndicator * _par)
    : executable(exec),
-     safe_execution(false),
-     eoc_handlers(0),
+     safe_execution_count(_par ? _par->safe_execution_count : 0),
      level(_par ? 1 + _par->get_level() : 0),
      error(E_NO_ERROR, LOC),
      current_stack(*this, exec->get_body()),
@@ -52,7 +52,7 @@ StateIndicator::~StateIndicator()
    // if executable is a user defined function then pop its local vars.
    // otherwise delete the body token
    //
-   if (executable->get_parse_mode() == PM_FUNCTION)
+   if (get_parse_mode() == PM_FUNCTION)
       {
         const UserFunction * ufun = get_executable()->get_ufun();
         if (ufun)   ufun->pop_local_vars();
@@ -63,20 +63,6 @@ StateIndicator::~StateIndicator()
          delete executable;
          executable = 0;
       }
-
-   // clean up EOC handlers
-   //
-   if (eoc_handlers)
-      {
-        Log(LOG_EOC_handlers)
-           CERR << "StateIndicator::~StateIndicator() cleans up eoc_handlers"
-                << endl << "    deleting EOC handler "
-                << (const void *)eoc_handlers << endl;
-
-        eoc_handlers->loc = "(deleted)";
-        delete eoc_handlers;
-        eoc_handlers = 0;
-      }
 }
 //-----------------------------------------------------------------------------
 void
@@ -117,7 +103,7 @@ StateIndicator::function_name() const
 {
    Assert(executable);
 
-   switch(executable->get_parse_mode())
+   switch(get_parse_mode())
       {
         case PM_FUNCTION:
              return executable->get_name();
@@ -143,13 +129,13 @@ StateIndicator::function_name() const
 void
 StateIndicator::print(ostream & out) const
 {
-   out << "Depth:    " << level << endl;
-   out << "Exec:     " << executable << endl;
-   out << "Safe ex:  " << (safe_execution ? "yes" : "no") << endl;
+   out << "Depth:      " << level                << endl;
+   out << "Exec:       " << executable           << endl;
+   out << "Safe exec:  " << safe_execution_count << endl;
 
    Assert(executable);
 
-   switch(get_executable()->get_parse_mode())
+   switch(get_parse_mode())
       {
         case PM_FUNCTION:
              out << "Pmode:    ∇ "
@@ -165,7 +151,7 @@ StateIndicator::print(ostream & out) const
              break;
 
         default:
-             out << "??? Bad pmode " << executable->get_parse_mode();
+             out << "??? Bad pmode " << get_parse_mode();
       }
    out << endl;
 
@@ -196,7 +182,7 @@ StateIndicator::list(ostream & out, SI_mode mode) const
 
    // pmode column
    //
-   switch(get_executable()->get_parse_mode())
+   switch(get_parse_mode())
       {
         case PM_FUNCTION:
              Assert(executable);
@@ -365,7 +351,7 @@ StateIndicator::jump(Value_P value)
         // →⍬ in immediate execution means resume (retry) suspended function
         // →⍬ on ⍎ or defined functions means do nothing
         //
-        if (get_executable()->get_parse_mode() == PM_STATEMENT_LIST)
+        if (get_parse_mode() == PM_STATEMENT_LIST)
            return Token(TOK_BRANCH, int64_t(Function_Retry));
 
         return Token(TOK_NOBRANCH);           // stay in context
@@ -389,22 +375,6 @@ const UserFunction * ufun = get_executable()->get_ufun();
 void
 StateIndicator::escape()  
 {
-EOC_arg * e;
-   while ((e = eoc_handlers))
-      {
-        Log(LOG_EOC_handlers)
-           CERR << "StateIndicator::escape() cleans up eoc_handler from "
-                << e->loc << endl;
-
-        Token tok(TOK_ESCAPE);
-        e->handler(tok);
-        Log(LOG_EOC_handlers)
-           CERR << "StateIndicator::~StateIndicator() deletes "
-                << (const void *)e << endl;
-         
-        eoc_handlers = e->next;
-        delete e;
-      }
 }
 //-----------------------------------------------------------------------------
 Token
@@ -503,80 +473,6 @@ Value_P * X = current_stack.locate_X();
    if (X)   *X = new_value;
 }
 //-----------------------------------------------------------------------------
-void
-StateIndicator::add_eoc_handler(EOC_HANDLER handler, EOC_arg & arg,
-                                const char * loc)
-{
-EOC_arg * new_e =  new EOC_arg(handler, arg, loc);
-
-   if (EOC_arg * e = eoc_handlers)
-      {
-        while (e->next)   e = e->next;
-        e->next = new_e;
-      }
-   else   // first EOC handler
-      {
-        eoc_handlers = new_e;
-      }
-
-   Log(LOG_EOC_handlers)
-      {
-        int count = 0;
-        for (EOC_arg * e = eoc_handlers; e; e = e->next)   ++count;
-        CERR << "SI[" << level << "] added new EOC handler(" << new_e
-             << " from " << loc << " len=" << count << endl;
-      }
-}
-//-----------------------------------------------------------------------------
-EOC_arg *
-StateIndicator::remove_eoc_handlers()
-{
-EOC_arg * ret = eoc_handlers;
-
-   Log(LOG_EOC_handlers)
-      {
-         CERR << "SI[" << level << "] remove_eoc_handler(" << eoc_handlers
-              << endl;
-      }
-
-   eoc_handlers = 0;
-   return ret;
-}
-//-----------------------------------------------------------------------------
-void
-StateIndicator::move_eoc_handler(EOC_HANDLER handler, EOC_arg * old_arg,
-                                 const char * loc)
-{
-   eoc_handlers = old_arg;
-   Assert(eoc_handlers);
-
-   Log(LOG_EOC_handlers)
-      {
-         CERR << "SI[" << level << "] move_eoc_handler(" << eoc_handlers
-              << " from " << eoc_handlers->loc
-              << " to " << loc << ")" << endl;
-      }
-
-   eoc_handlers->handler = handler;
-   eoc_handlers->loc = loc;
-}
-
-//-----------------------------------------------------------------------------
-bool
-StateIndicator::call_eoc_handler(Token & token)
-{
-   if (!eoc_handlers)     return false;   // no eoc_handler
-
-   Log(LOG_EOC_handlers)
-       CERR << "SI[" << level << "] call_eoc_handler(" << eoc_handlers
-            << " from " << eoc_handlers->loc << ")" << endl;
-
-
-const bool goon = eoc_handlers->handler(token);
-   return goon;   // false: success (done), true: goon
-
-}
-//-----------------------------------------------------------------------------
 Function_Line
 StateIndicator::get_line() const
 {
@@ -610,7 +506,7 @@ StateIndicator::statement_result(Token & result, bool trace)
 
    fun_oper_cache.reset();
 
-// if (get_executable()->get_parse_mode() == PM_EXECUTE)   return;
+// if (get_parse_mode() == PM_EXECUTE)   return;
 
    // if result is a value then print it, unless it is a committed value
    // (i.e. TOK_APL_VALUE2)
@@ -677,7 +573,7 @@ const int boxing_format = Command::get_boxing_format();
    else
       {
         const PrintContext pctx = Workspace::get_PrintContext(PST_NONE);
-        Value_P B1 = Quad_CR::do_CR(boxing_format, *B.get(), pctx);
+        Value_P B1 = Quad_CR::do_CR(boxing_format, B.get(), pctx);
         B1->print(COUT);
       }
 }
@@ -685,14 +581,14 @@ const int boxing_format = Command::get_boxing_format();
 Unicode
 StateIndicator::get_parse_mode_name() const
 {
-   switch(get_executable()->get_parse_mode())
+   switch(get_parse_mode())
       {
         case PM_FUNCTION:       return UNI_NABLA;
         case PM_STATEMENT_LIST: return UNI_DIAMOND;
         case PM_EXECUTE:        return UNI_EXECUTE;
      }
 
-   CERR << "pmode = " << get_executable()->get_parse_mode() << endl;
+   CERR << "pmode = " << get_parse_mode() << endl;
    Assert(0 && "bad pmode");
 }
 //-----------------------------------------------------------------------------
