@@ -123,25 +123,24 @@ Symbol::print_verbose(ostream & out) const
 }
 //-----------------------------------------------------------------------------
 void
-Symbol::assign(Value_P new_value, const char * loc)
+Symbol::assign(Value_P new_value, bool clone, const char * loc)
 {
    Assert(value_stack.size());
 
-#if 0 // TODO: make this work
    if (!new_value->is_complete())
       {
         CERR << "Incomplete value at " LOC << endl;
         new_value->print_properties(CERR, 0);
+        print_history(CERR, new_value.get(), LOC);
         Assert(0);
       }
-#endif
 
 ValueStackItem & vs = value_stack.back();
 
    switch(vs.name_class)
       {
         case NC_UNUSED_USER_NAME:
-             new_value = new_value->clone(loc);
+             if (clone)  new_value = new_value->clone(loc);
 
              vs.name_class = NC_VARIABLE;
              vs.apl_val = new_value;
@@ -151,7 +150,7 @@ ValueStackItem & vs = value_stack.back();
         case NC_VARIABLE:
              if (vs.apl_val == new_value)   return;   // X←X
 
-             new_value = new_value->clone(loc);
+             if (clone)  new_value = new_value->clone(loc);
 
              vs.apl_val = new_value;
              if (monitor_callback)   monitor_callback(*this, SEV_ASSIGNED);
@@ -173,10 +172,24 @@ Symbol::assign_indexed(Value_P X, Value_P B)   // A[X] ← B
    // an index with no semicolons. If X contains semicolons, then
    // assign_indexed(IndexExpr IX, ...) is called instead.
    // 
+const APL_Integer qio = Workspace::get_IO();
+
 Value_P A = get_apl_value();
+const ShapeItem max_idx = A->element_count();
+   if (!!X && X->is_scalar() && B->is_scalar() && A->get_rank() == 1)
+      {
+        const APL_Integer idx = X->get_ravel(0).get_near_int() - qio;
+        if (idx >= 0 && idx < max_idx)
+           {
+             Cell & cell = A->get_ravel(idx);
+             cell.release(LOC);
+             cell.init(B->get_ravel(0), A.getref(), LOC);
+             return;
+           }
+      }
+
    if (A->get_rank() != 1)   RANK_ERROR;
 
-const ShapeItem max_idx = A->element_count();
    if (!X)   // X[] ← B
       {
         const Cell & src = B->get_ravel(0);
@@ -184,7 +197,7 @@ const ShapeItem max_idx = A->element_count();
             {
               Cell & dest = A->get_ravel(a);
               dest.release(LOC);   // free sub-values etc (if any)
-              dest.init(src, A.getref());
+              dest.init(src, A.getref(), LOC);
             }
         if (monitor_callback)   monitor_callback(*this, SEV_ASSIGNED);
         return;
@@ -192,7 +205,6 @@ const ShapeItem max_idx = A->element_count();
 
 const ShapeItem ec_B = B->element_count();
 const ShapeItem ec_X = X->element_count();
-const APL_Integer qio = Workspace::get_IO();
 const int incr_B = (ec_B == 1) ? 0 : 1;   // maybe scalar extend B
 const Cell * cX = &X->get_ravel(0);
 const Cell * cB = &B->get_ravel(0);
@@ -206,7 +218,7 @@ const Cell * cB = &B->get_ravel(0);
         if (idx >= max_idx)   INDEX_ERROR;
         Cell & dest = A->get_ravel(idx);
         dest.release(LOC);   // free sub-values etc (if any)
-        dest.init(*cB, A.getref());
+        dest.init(*cB, A.getref(), LOC);
 
          cB += incr_B;
       }
@@ -286,9 +298,11 @@ const int incr_B = (ec_B == 1) ? 0 : 1;
    while (!mult.done())
       {
         const ShapeItem offset_A = mult.next();
+        if (offset_A < 0)                     INDEX_ERROR;
+        if (offset_A >= A->element_count())   INDEX_ERROR;
         Cell & dest = A->get_ravel(offset_A);
         dest.release(LOC);   // free sub-values etc (if any)
-        dest.init(*cB, A.getref());
+        dest.init(*cB, A.getref(), LOC);
         cB += incr_B;
      }
 
@@ -299,15 +313,25 @@ void
 Symbol::assign_named_lambda(Function * lambda, const char * loc)
 {
 ValueStackItem & vs = value_stack.back();
+UserFunction * ufun = lambda->get_ufun1();
+   Assert(ufun);
 
    switch(vs.name_class)
       {
-        case NC_UNUSED_USER_NAME:
         case NC_FUNCTION:
         case NC_OPERATOR:
+             if (!vs.sym_val.function->is_lambda())   SYNTAX_ERROR;
+             Assert(vs.sym_val.function->get_ufun1());
+             vs.sym_val.function->get_ufun1()->decrement_refcount(LOC);
 
-             vs.name_class = NC_FUNCTION;
-             vs.sym_val.function = lambda;
+             /* fall through */
+
+        case NC_UNUSED_USER_NAME:
+             if (lambda->is_operator())   vs.name_class = NC_OPERATOR;
+             else                         vs.name_class = NC_FUNCTION;
+
+             vs.sym_val.function = ufun;
+             ufun->increment_refcount(LOC);
              if (monitor_callback)   monitor_callback(*this, SEV_ASSIGNED);
              return;
 
@@ -320,7 +344,7 @@ Symbol::pop()
 {
    if (value_stack.size() == 0)
       {
-       CERR << "Symbol is: '" << get_name() << "'" << endl;
+       CERR << "Symbol is: '" << get_name() << "' at " << LOC << endl;
         Assert(0 && LOC);
       }
 
@@ -407,7 +431,7 @@ Symbol::push_value(Value_P value)
 ValueStackItem vs;
    value_stack.push_back(vs);
    if (monitor_callback)   monitor_callback(*this, SEV_PUSHED);
-   assign(value, LOC);
+   assign(value, true, LOC);
 
    Log(LOG_SYMBOL_push_pop)
       {
@@ -631,7 +655,8 @@ Symbol::resolve(Token & tok, bool left_sym)
 
    Assert1(value_stack.size());
 
-   switch(value_stack.back().name_class)
+const ValueStackItem & vs = value_stack.back();
+   switch(vs.name_class)
       {
         case NC_UNUSED_USER_NAME:
              if (!left_sym)   throw_symbol_error(get_name(), LOC);
@@ -641,7 +666,7 @@ Symbol::resolve(Token & tok, bool left_sym)
              if (left_sym)   SYNTAX_ERROR;   // assignment to (read-only) label
 
              {
-               IntCell lab(value_stack.back().sym_val.label);
+               IntCell lab(vs.sym_val.label);
                Value_P value(lab, LOC);
                Token t(TOK_APL_VALUE1, value);
                move_1(tok, t, LOC);
@@ -660,8 +685,13 @@ Symbol::resolve(Token & tok, bool left_sym)
 
         case NC_FUNCTION:
         case NC_OPERATOR:
-             move_2(tok, value_stack.back().sym_val.function
-                                                    ->get_token(), LOC);
+             if (left_sym && vs.sym_val.function->is_lambda())
+                {
+                  // lambda re-assign, e.g. SYM←{ ... }
+                  //
+                  return;
+                }
+             move_2(tok, vs.sym_val.function->get_token(), LOC);
              return;
 
         case NC_SHARED_VAR:
@@ -670,7 +700,7 @@ Symbol::resolve(Token & tok, bool left_sym)
              return;
 
         default:
-             CERR << "Symbol is '" << get_name() << "'" << endl;
+             CERR << "Symbol is '" << get_name() << "' at " << LOC << endl;
              SYNTAX_ERROR;
       }
 }
@@ -742,7 +772,17 @@ ValueStackItem & vs = value_stack.back();
       }
    else if (vs.name_class == NC_FUNCTION || vs.name_class == NC_OPERATOR)
       {
-        if (!vs.sym_val.function->is_native())
+        if (vs.sym_val.function->is_native())
+           {
+             // do not delete native functions
+           }
+        else if (vs.sym_val.function->is_lambda())
+           {
+             UserFunction * ufun = vs.sym_val.function->get_ufun1();
+             Assert(ufun);
+             ufun->decrement_refcount(LOC);
+           }
+        else
            {
              const UserFunction * ufun = vs.sym_val.function->get_ufun1();
              const Executable * exec = ufun;
@@ -761,10 +801,11 @@ ValueStackItem & vs = value_stack.back();
                   delete ufun;
                 }
            }
+        vs.name_class = NC_UNUSED_USER_NAME;
       }
 
    vs.clear();
-   if (monitor_callback)   monitor_callback(*this, SEV_ERASED);
+   call_monitor_callback(SEV_ERASED);
    return 1;
 }
 //-----------------------------------------------------------------------------
@@ -806,7 +847,7 @@ ValueStackItem & vs = value_stack.back();
         set_SV_key(key);
 
         // assign old value to shared variable
-        assign(old_value, LOC);
+        assign(old_value, true, LOC);
 
         return;
       }
@@ -979,7 +1020,7 @@ int count = 0;
          switch(item.name_class)
             {
               case NC_VARIABLE:
-                   if (item.apl_val->is_or_contains(value))
+                   if (Value::is_or_contains(item.apl_val.get(), value))
                       {
                          out << "    Variable[vs=" << v << "] "
                             << get_name() << endl;
@@ -1024,13 +1065,14 @@ const Cell * cV = &values->get_ravel(0);
         Symbol * sym = symbols[sym_count - s - 1];
         if (cV->is_pointer_cell())
            {
-             sym->assign(cV->get_pointer_value(), LOC);
+             sym->assign(cV->get_pointer_value(), true, LOC);
            }
         else
            {
              Value_P val(LOC);
-             val->get_ravel(0).init(*cV, val.getref());
-             sym->assign(val, LOC);
+             val->next_ravel()->init(*cV, val.getref(), LOC);
+             val->check_value(LOC);
+             sym->assign(val, true, LOC);
            }
 
         cV += incr;   // scalar extend values
@@ -1044,8 +1086,7 @@ const ValueStackItem & vs = (*this)[0];
    if (vs.name_class == NC_VARIABLE)
       {
         vector<UCS_string> CR10;
-        const UCS_string pick;
-        Quad_CR::do_CR10_var(CR10, get_name(), pick, *vs.apl_val.get());
+        Quad_CR::do_CR10_var(CR10, get_name(), *vs.apl_val.get());
 
         loop(l, CR10.size())
            {
@@ -1073,16 +1114,54 @@ const ValueStackItem & vs = (*this)[0];
              return;
            }
 
-        out << "∇";
         const UCS_string text = ufun->canonical(false);
-        loop(u, text.size())
+        if (ufun->is_lambda())
            {
-              out << text[u];
-              if (text[u] == '\n' && u < text.size() - 1)   out << " ";
+             out << get_name();
+             out << "←{";
+             int t = 0;
+             while (t < text.size())   // skip λ header
+                {
+                  const Unicode uni = text[t++];
+                  if (uni == UNI_ASCII_LF)   break;
+                }
+
+             // skip λ← and spaces
+             while (t < text.size() && text[t] <= ' ')   ++t;
+             if    (t < text.size() && text[t] == UNI_LAMBDA)   ++t;
+             while (t < text.size() && text[t] <= ' ')   ++t;
+             if    (t < text.size() && text[t] == UNI_LEFT_ARROW)   ++t;
+             while (t < text.size() && text[t] <= ' ')   ++t;
+
+             // copy body
+             //
+             while (t < text.size())
+                {
+                   const Unicode uni = text[t++];
+                   if (uni == UNI_ASCII_LF)   break;
+                   out << uni;
+                }
+
+             // append local variables
+             //
+             loop(l, ufun->local_var_count())
+                 {
+                   out << ";" << ufun->get_local_var(l).get_name();
+                 }
+             out << UNI_ASCII_R_CURLY << endl;
+           }
+        else
+           {
+             out << "∇";
+             loop(u, text.size())
+                {
+                   out << text[u];
+                   if (text[u] == '\n' && u < text.size() - 1)   out << " ";
+                }
+             if (ufun->get_exec_properties()[0])   out << "⍫";
+             else                                  out << "∇";
+             out << endl << endl;
            }
-        if (ufun->get_exec_properties()[0])   out << "⍫";
-        else                                  out << "∇";
-        out << endl << endl;
       }
 }
 //-----------------------------------------------------------------------------
@@ -1099,7 +1178,8 @@ ValueStackItem & tos = value_stack[0];
    switch(tos.name_class)
       {
         case NC_LABEL:
-             Assert(0 && "should not happen since stack height == 1");
+             // should not happen since stack height == 1"
+             FIXME;
              break;
 
         case NC_VARIABLE:
@@ -1158,7 +1238,7 @@ const bool ws_to_ws = Svar_DB::is_ws_to_ws(get_SV_key());
                  {
                    CERR << "Shared variable ";
                    for (const uint32_t * varname =
-                                         Svar_DB::get_varname(get_SV_key());
+                                         Svar_DB::get_svar_name(get_SV_key());
                         varname && *varname; ++varname)
                        CERR << (Unicode)(*varname++);
                    CERR << " is blocked on set. Waiting ...";
@@ -1249,7 +1329,7 @@ const bool ws_to_ws = Svar_DB::is_ws_to_ws(get_SV_key());
                    CERR << "apl" << ProcessorID::get_id().proc
                                     << ": Shared variable ";
                    for (const uint32_t * varname =
-                                         Svar_DB::get_varname(get_SV_key());
+                                         Svar_DB::get_svar_name(get_SV_key());
                         varname && *varname; ++varname)
                        CERR << (Unicode)(*varname++);
                    CERR << " is blocked on use. Waiting ...";
@@ -1328,9 +1408,10 @@ const ErrorCode err(ErrorCode(response->get__VALUE_IS__error()));
                   << response->get__VALUE_IS__error_loc() << endl;
            }
 
+        string eloc = response->get__VALUE_IS__error_loc();
         delete response;
         if (del)   delete del;
-        throw_apl_error(err, response->get__VALUE_IS__error_loc().c_str());
+        throw_apl_error(err, eloc.c_str());
       }
 
 const string & data = response->get__VALUE_IS__cdr_value();
