@@ -31,6 +31,7 @@
 #include "Bif_F12_FORMAT.hh"
 #include "CDR.hh"
 #include "CharCell.hh"
+#include "Command.hh"
 #include "Common.hh"
 #include "FloatCell.hh"
 #include "IndexExpr.hh"
@@ -59,7 +60,7 @@ Unicode Quad_AV::qav[MAX_AV];
 
 //=============================================================================
 void
-SystemVariable::assign(Value_P value, const char * loc)
+SystemVariable::assign(Value_P value, bool clone, const char * loc)
 {
    CERR << "SystemVariable::assign() not (yet) implemented for "
         << get_Id() << endl;
@@ -127,6 +128,7 @@ SystemVariable::get_attributes(int mode, Cell * dest) const
 Quad_AV::Quad_AV()
    : RO_SystemVariable(ID::Quad_AV)
 {
+   Assert1(MAX_AV == 256);
 Value_P AV(MAX_AV, LOC);
    loop(cti, MAX_AV)
        {
@@ -138,7 +140,7 @@ Value_P AV(MAX_AV, LOC);
        }
    AV->check_value(LOC);
 
-   Symbol::assign(AV, LOC);
+   Symbol::assign(AV, false, LOC);
 }
 //-----------------------------------------------------------------------------
 Unicode
@@ -153,7 +155,7 @@ Quad_AI::Quad_AI()
      session_start(now()),
      user_wait(0)
 {
-   Symbol::assign(get_apl_value(), LOC);
+   Symbol::assign(get_apl_value(), false, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -162,11 +164,12 @@ Quad_AI::get_apl_value() const
 const int total_ms = (now() - session_start)/1000;
 const int user_ms  = user_wait/1000;
 
-Value_P ret(4, LOC);
-   new (&ret->get_ravel(0))   IntCell(ProcessorID::get_own_ID());
-   new (&ret->get_ravel(1))   IntCell(total_ms - user_ms);
-   new (&ret->get_ravel(2))   IntCell(total_ms);
-   new (&ret->get_ravel(3))   IntCell(user_ms);
+Value_P ret(5, LOC);
+   new (ret->next_ravel())   IntCell(ProcessorID::get_own_ID());
+   new (ret->next_ravel())   IntCell(total_ms - user_ms);
+   new (ret->next_ravel())   IntCell(total_ms);
+   new (ret->next_ravel())   IntCell(user_ms);
+   new (ret->next_ravel())   IntCell(Command::get_APL_expression_count());
 
    ret->check_value(LOC);
    return ret;
@@ -183,16 +186,13 @@ Quad_ARG::get_apl_value() const
 const int argc = uprefs.expanded_argv.size();
 
 Value_P Z(argc, LOC);
-Cell * C = &Z->get_ravel(0);
-
    loop(a, argc)
       {
         const char * arg = uprefs.expanded_argv[a];
-        const int len = strlen(arg);
-        Value_P val(len, LOC);
-        loop(l, len)   new (val->next_ravel())   CharCell(Unicode(arg[l]));
-
-        new (C++)   PointerCell(val, Z.getref());
+        UTF8_string utf(arg);
+        UCS_string ucs(utf);
+        Value_P sub(ucs, LOC);
+        new (Z->next_ravel())   PointerCell(sub, Z.getref());
       }
 
    Z->check_value(LOC);
@@ -202,11 +202,11 @@ Cell * C = &Z->get_ravel(0);
 Quad_CT::Quad_CT()
    : SystemVariable(ID::Quad_CT)
 {
-   Symbol::assign(FloatScalar(DEFAULT_Quad_CT, LOC), LOC);
+   Symbol::assign(FloatScalar(DEFAULT_Quad_CT, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_CT::assign(Value_P value, const char * loc)
+Quad_CT::assign(Value_P value, bool clone, const char * loc)
 {
    if (!value->is_scalar_or_len1_vector())
       {
@@ -226,11 +226,11 @@ APL_Float val = cell.get_real_value();
    //
    if (val > MAX_Quad_CT)
       {
-        Symbol::assign(FloatScalar(MAX_Quad_CT, LOC), LOC);
+        Symbol::assign(FloatScalar(MAX_Quad_CT, LOC), false, LOC);
       }
    else
       {
-        Symbol::assign(value, LOC);
+        Symbol::assign(value, clone, LOC);
       }
 }
 //=============================================================================
@@ -246,7 +246,7 @@ const Error * err = 0;
               err = &si->get_error();
               break;
             }
-         if (si->get_executable()->get_parse_mode() == PM_FUNCTION)   break;
+         if (si->get_parse_mode() == PM_FUNCTION)   break;
        }
 
    if (err == 0)   // no SI entry with non-zero error code
@@ -255,43 +255,17 @@ const Error * err = 0;
         //
         Shape sh((ShapeItem)3, (ShapeItem)0);
         Value_P Z(sh, LOC);
-        new (&Z->get_ravel(0))   CharCell(UNI_ASCII_SPACE);
+        Z->set_proto_Spc();
         Z->check_value(LOC);
         return Z;
       }
 
-const UCS_string msg_1 = err->get_error_line_1();
-const UCS_string msg_2 = err->get_error_line_2();
-const UCS_string msg_3 = err->get_error_line_3();
-
-   // compute max line length of the 3 error lines,
-   //
-const ShapeItem len_1 = msg_1.size();
-const ShapeItem len_2 = msg_2.size();
-const ShapeItem len_3 = msg_3.size();
-
-ShapeItem max_len = len_1;
-   if (max_len < len_2)   max_len = len_2;
-   if (max_len < len_3)   max_len = len_3;
-
-const Shape sh(3, max_len);
-Value_P Z(sh, LOC);
-   loop(l, max_len)
-      {
-        new (&Z->get_ravel(l))
-         CharCell((l < len_1) ? msg_1[l] : UNI_ASCII_SPACE);
-      }
-   loop(l, max_len)
-      {
-        new (&Z->get_ravel(l + max_len))
-         CharCell((l < len_2) ? msg_2[l] : UNI_ASCII_SPACE);
-      }
-   loop(l, max_len)
-      {
-        new (&Z->get_ravel(l + 2*max_len))
-         CharCell((l < len_3) ? msg_3[l] : UNI_ASCII_SPACE);
-      }
-
+PrintBuffer pb;
+   pb.append_ucs(err->get_error_line_1());
+   pb.append_ucs(err->get_error_line_2());
+   pb.append_ucs(err->get_error_line_3());
+   pb.pad_to_spaces();   // replace pad char by spaces
+Value_P Z(pb, LOC);
    Z->check_value(LOC);
    return Z;
 }
@@ -310,13 +284,13 @@ ErrorCode ec = E_NO_ERROR;
               break;
             }
 
-         if (si->get_executable()->get_parse_mode() == PM_FUNCTION)   break;
+         if (si->get_parse_mode() == PM_FUNCTION)   break;
        }
 
 Value_P Z(2, LOC);
 
-   new (&Z->get_ravel(0)) IntCell(Error::error_major(ec));
-   new (&Z->get_ravel(1)) IntCell(Error::error_minor(ec));
+   new (Z->next_ravel()) IntCell(Error::error_major(ec));
+   new (Z->next_ravel()) IntCell(Error::error_minor(ec));
 
    Z->check_value(LOC);
    return Z;
@@ -333,7 +307,7 @@ Value_P QFC(6, LOC);
    new (QFC->next_ravel()) CharCell(UNI_OVERBAR);
    QFC->check_value(LOC);
 
-   Symbol::assign(QFC, LOC);
+   Symbol::assign(QFC, false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -350,11 +324,11 @@ Value_P QFC(6, LOC);
    new (QFC->next_ravel()) CharCell(UNI_OVERBAR);
    QFC->check_value(LOC);
 
-   Symbol::assign(QFC, LOC);
+   Symbol::assign(QFC, false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_FC::assign(Value_P value, const char * loc)
+Quad_FC::assign(Value_P value, bool clone, const char * loc)
 {
    if (!value->is_scalar_or_vector())   RANK_ERROR;
 
@@ -379,7 +353,7 @@ Unicode fc[6] = { UNI_ASCII_FULLSTOP, UNI_ASCII_COMMA,      UNI_STAR_OPERATOR,
 
 UCS_string ucs(fc, 6);
 Value_P new_val(ucs, LOC);
-   Symbol::assign(new_val, LOC);
+   Symbol::assign(new_val, false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -392,7 +366,7 @@ Quad_FC::assign_indexed(IndexExpr & IX, Value_P value)
    // IX must be an elided index (like in ⎕FC[] ← value)
    //
    Assert1(!IX.values[0]);
-   assign(value, LOC);   // ⎕FC[]←value
+   assign(value, true, LOC);   // ⎕FC[]←value
 }
 //-----------------------------------------------------------------------------
 void
@@ -416,7 +390,7 @@ Unicode fc[6];
 
    loop(e, ec)
       {
-        const APL_Integer idx = X->get_ravel(e).get_int_value() - qio;
+        const APL_Integer idx = X->get_ravel(e).get_near_int() - qio;
         if (idx < 0)   continue;
         if (idx > 5)   continue;
 
@@ -430,17 +404,17 @@ Unicode fc[6];
 
 UCS_string ucs(fc, 6);
 Value_P new_val(ucs, LOC);
-   Symbol::assign(new_val, LOC);
+   Symbol::assign(new_val, false, LOC);
 }
 //=============================================================================
 Quad_IO::Quad_IO()
    : SystemVariable(ID::Quad_IO)
 {
-   Symbol::assign(IntScalar(1, LOC), LOC);
+   Symbol::assign(IntScalar(1, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_IO::assign(Value_P value, const char * loc)
+Quad_IO::assign(Value_P value, bool clone, const char * loc)
 {
    if (!value->is_scalar_or_len1_vector())
       {
@@ -449,19 +423,19 @@ Quad_IO::assign(Value_P value, const char * loc)
       }
 
    if (value->get_ravel(0).get_near_bool())
-      Symbol::assign(IntScalar(1, LOC), LOC);
+      Symbol::assign(IntScalar(1, LOC), false, LOC);
    else
-      Symbol::assign(IntScalar(0, LOC), LOC);
+      Symbol::assign(IntScalar(0, LOC), false, LOC);
 }
 //=============================================================================
 Quad_L::Quad_L()
  : NL_SystemVariable(ID::Quad_L)
 {
-   Symbol::assign(IntScalar(0, LOC), LOC);
+   Symbol::assign(IntScalar(0, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_L::assign(Value_P value, const char * loc)
+Quad_L::assign(Value_P value, bool clone, const char * loc)
 {
 StateIndicator * si = Workspace::SI_top_fun();
    if (si == 0)   return;
@@ -489,7 +463,7 @@ StateIndicator * si = Workspace::SI_top_error();
 Quad_LC::Quad_LC()
    : RO_SystemVariable(ID::Quad_LC)
 {
-   Symbol::assign(IntScalar(0, LOC), LOC);
+   Symbol::assign(IntScalar(0, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -502,7 +476,7 @@ int len = 0;
    for (StateIndicator * si = Workspace::SI_top();
         si; si = si->get_parent())
        {
-         if (si->get_executable()->get_parse_mode() == PM_FUNCTION)   ++len;
+         if (si->get_parse_mode() == PM_FUNCTION)   ++len;
        }
 
 Value_P Z(len, LOC);
@@ -510,11 +484,10 @@ Value_P Z(len, LOC);
    for (StateIndicator * si = Workspace::SI_top();
         si; si = si->get_parent())
        {
-         if (si->get_executable()->get_parse_mode() == PM_FUNCTION)
+         if (si->get_parse_mode() == PM_FUNCTION)
             new (Z->next_ravel())   IntCell(si->get_line());
        }
 
-   Z->set_default_Zero();
    Z->check_value(LOC);
    return Z;
 }
@@ -522,16 +495,16 @@ Value_P Z(len, LOC);
 Quad_LX::Quad_LX()
    : NL_SystemVariable(ID::Quad_LX)
 {
-   Symbol::assign(Str0(LOC), LOC);
+   Symbol::assign(Str0(LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_LX::assign(Value_P value, const char * loc)
+Quad_LX::assign(Value_P value, bool clone, const char * loc)
 {
    if (value->get_rank() > 1)      RANK_ERROR;
    if (!value->is_char_string())   DOMAIN_ERROR;
 
-   Symbol::assign(value, LOC);
+   Symbol::assign(value, clone, LOC);
 }
 //=============================================================================
 Quad_PP::Quad_PP()
@@ -542,66 +515,51 @@ Value_P value(LOC);
    new (&value->get_ravel(0)) IntCell(DEFAULT_Quad_PP);
 
    value->check_value(LOC);
-   Symbol::assign(value, LOC);
+   Symbol::assign(value, false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_PP::assign(Value_P value, const char * loc)
+Quad_PP::assign(Value_P value, bool clone, const char * loc)
 {
-   if (!value->is_scalar_or_len1_vector())
-      {
-        if (value->get_rank() > 1)   RANK_ERROR;
-        else                         LENGTH_ERROR;
-      }
+APL_Integer pp = value->get_sole_integer();
+   if (pp < MIN_Quad_PP)   DOMAIN_ERROR;
+   if (pp > MAX_Quad_PP)   pp = MAX_Quad_PP;
 
-const Cell & cell = value->get_ravel(0);
-APL_Integer val = cell.get_near_int();
-   if (val < MIN_Quad_PP)   DOMAIN_ERROR;
-
-   if (val > MAX_Quad_PP)   val = MAX_Quad_PP;
-
-   Symbol::assign(IntScalar(val, LOC), LOC);
+   Symbol::assign(IntScalar(pp, LOC), false, LOC);
 }
 //=============================================================================
 Quad_PR::Quad_PR()
    : SystemVariable(ID::Quad_PR)
 {
-   Symbol::assign(CharScalar(UNI_ASCII_SPACE, LOC), LOC);
+   Symbol::assign(CharScalar(UNI_ASCII_SPACE, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_PR::assign(Value_P value, const char * loc)
+Quad_PR::assign(Value_P value, bool clone, const char * loc)
 {
 UCS_string ucs = value->get_UCS_ravel();
 
    if (ucs.size() > 1)   LENGTH_ERROR;
 
-   Symbol::assign(value, LOC);
+   Symbol::assign(value, clone, LOC);
 }
 //=============================================================================
 Quad_PS::Quad_PS()
    : SystemVariable(ID::Quad_PS)
 {
-   Symbol::assign(IntScalar(0, LOC), LOC);
+   Symbol::assign(IntScalar(0, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_PS::assign(Value_P value, const char * loc)
+Quad_PS::assign(Value_P value, bool clone, const char * loc)
 {
-   if (!value->is_scalar_or_len1_vector())
-      {
-        if (value->get_rank() > 1)   RANK_ERROR;
-        else                         LENGTH_ERROR;
-      }
-
-const Cell & cell = value->get_ravel(0);
-const APL_Integer val = cell.get_near_int();
-   switch(val)
+const APL_Integer ps = value->get_sole_integer();
+   switch(ps)
       {
         case 0:
         case 1:
         case 2:
-        case 3:  Symbol::assign(IntScalar(val, LOC), LOC);   return;
+        case 3:  Symbol::assign(IntScalar(ps, LOC), false, LOC);   return;
         default: DOMAIN_ERROR;
       }
 }
@@ -609,28 +567,21 @@ const APL_Integer val = cell.get_near_int();
 Quad_PW::Quad_PW()
    : SystemVariable(ID::Quad_PW)
 {
-   Symbol::assign(IntScalar(DEFAULT_Quad_PW, LOC), LOC);
+   Symbol::assign(IntScalar(DEFAULT_Quad_PW, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_PW::assign(Value_P value, const char * loc)
+Quad_PW::assign(Value_P value, bool clone, const char * loc)
 {
-   if (!value->is_scalar_or_len1_vector())
-      {
-        if (value->get_rank() > 1)   RANK_ERROR;
-        else                         LENGTH_ERROR;
-      }
+const APL_Integer pw = value->get_sole_integer();
 
-const Cell & cell = value->get_ravel(0);
-const APL_Integer val = cell.get_near_int();
-
-   // min val is 30. Ignore smaller values.
-   if (val < 30)   return;
+   // min. ⎕PW is 30. Ignore smaller values.
+   if (pw < MIN_Quad_PW)   return;
 
    // max val is system specific. Ignore larger values.
-   if (val > MAX_Quad_PW)   return;
+   if (pw > MAX_Quad_PW)   return;
 
-   Symbol::assign(value, LOC);
+   Symbol::assign(IntScalar(pw, LOC), false, LOC);
 }
 //=============================================================================
 Quad_Quad::Quad_Quad()
@@ -639,7 +590,7 @@ Quad_Quad::Quad_Quad()
 }
 //-----------------------------------------------------------------------------
 void
-Quad_Quad::assign(Value_P value, const char * loc)
+Quad_Quad::assign(Value_P value, bool clone, const char * loc)
 {
    // write pending LF from  ⍞ (if any)
    Quad_QUOTE::done(true, LOC);
@@ -674,7 +625,7 @@ Quad_QUOTE::Quad_QUOTE()
    //
 Value_P dummy(UCS_string(LOC), LOC);
    dummy->set_complete();
-   Symbol::assign(dummy, LOC);
+   Symbol::assign(dummy, false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -692,7 +643,7 @@ Quad_QUOTE::done(bool with_LF, const char * loc)
 }
 //-----------------------------------------------------------------------------
 void
-Quad_QUOTE::assign(Value_P value, const char * loc)
+Quad_QUOTE::assign(Value_P value, bool clone, const char * loc)
 {
    Log(LOG_cork)
       CERR << "Quad_QUOTE::assign() called, buffer = ["
@@ -743,7 +694,7 @@ UCS_string line;
                       LineHistory::quote_quad_history);
    done(false, LOC);   // if get_quad_cr_line() has not called it
 
-   if (interrupt_raised)   INTERRUPT;
+   if (interrupt_is_raised())   INTERRUPT
 
 const UCS_string qpr = Workspace::get_PR();
 
@@ -765,11 +716,11 @@ Value_P Z(line, LOC);
 Quad_R::Quad_R()
  : NL_SystemVariable(ID::Quad_R)
 {
-   Symbol::assign(IntScalar(0, LOC), LOC);
+   Symbol::assign(IntScalar(0, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_R::assign(Value_P value, const char * loc)
+Quad_R::assign(Value_P value, bool clone, const char * loc)
 {
 StateIndicator * si = Workspace::SI_top_fun();
    if (si == 0)   return;
@@ -795,7 +746,7 @@ StateIndicator * si = Workspace::SI_top_error();
 }
 //=============================================================================
 void
-Quad_SYL::assign(Value_P value, const char * loc)
+Quad_SYL::assign(Value_P value, bool clone, const char * loc)
 {
    // Quad_SYL is mostly read-only, so we only allow assign_indexed() with
    // certain values.
@@ -805,7 +756,7 @@ Quad_SYL::assign(Value_P value, const char * loc)
    // this assign is called from the constructor in order to trigger the
    // creation of a symbol for Quad_SYL.
    //
-   Symbol::assign(IntScalar(0, LOC), LOC);
+   Symbol::assign(IntScalar(0, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
@@ -834,11 +785,9 @@ Value_P X1 = IDX.extract_value(1);
 void
 Quad_SYL::assign_indexed(Value_P X, Value_P B)
 {
-const APL_Float qct = Workspace::get_CT();
-
-   if (!(X->is_int_scalar(qct) || X->is_int_vector(qct)))   INDEX_ERROR;
-   if (!(B->is_int_scalar(qct) || B->is_int_vector(qct)))   DOMAIN_ERROR;
-   if (X->element_count() != B->element_count())            LENGTH_ERROR;
+   if (!(X->is_int_scalar() || X->is_int_vector()))   INDEX_ERROR;
+   if (!(B->is_int_scalar() || B->is_int_vector()))   DOMAIN_ERROR;
+   if (X->element_count() != B->element_count())      LENGTH_ERROR;
 
 const ShapeItem ec = X->element_count();
 const APL_Integer qio = Workspace::get_IO();
@@ -870,7 +819,11 @@ const APL_Integer qio = Workspace::get_IO();
            }
         else if (x == SYL_CURRENT_CORES)   // number of cores
            {
+#if PARALLEL_ENABLED
              if (Parallel::set_core_count((CoreCount)b, false))   DOMAIN_ERROR;
+#else
+             DOMAIN_ERROR;
+#endif
            }
         else if (x == SYL_PRINT_LIMIT)   // print length limit
            {
@@ -910,13 +863,13 @@ Value_P QCT(3, LOC);
    new (QCT->next_ravel()) CharCell(UNI_ASCII_LF);
    QCT->check_value(LOC);
 
-   Symbol::assign(QCT, LOC);
+   Symbol::assign(QCT, false, LOC);
 }
 //=============================================================================
 Quad_TS::Quad_TS()
    : RO_SystemVariable(ID::Quad_TS)
 {
-   Symbol::assign(IntScalar(0, LOC), LOC);
+   Symbol::assign(IntScalar(0, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -926,13 +879,13 @@ const APL_time_us offset_us = 1000000 * Workspace::get_v_Quad_TZ().get_offset();
 const YMDhmsu time(now() + offset_us);
 
 Value_P Z(7, LOC);
-   new (&Z->get_ravel(0)) IntCell(time.year);
-   new (&Z->get_ravel(1)) IntCell(time.month);
-   new (&Z->get_ravel(2)) IntCell(time.day);
-   new (&Z->get_ravel(3)) IntCell(time.hour);
-   new (&Z->get_ravel(4)) IntCell(time.minute);
-   new (&Z->get_ravel(5)) IntCell(time.second);
-   new (&Z->get_ravel(6)) IntCell(time.micro / 1000);
+   new (Z->next_ravel()) IntCell(time.year);
+   new (Z->next_ravel()) IntCell(time.month);
+   new (Z->next_ravel()) IntCell(time.day);
+   new (Z->next_ravel()) IntCell(time.hour);
+   new (Z->next_ravel()) IntCell(time.minute);
+   new (Z->next_ravel()) IntCell(time.second);
+   new (Z->next_ravel()) IntCell(time.micro / 1000);
 
    Z->check_value(LOC);
    return Z;
@@ -946,9 +899,9 @@ Quad_TZ::Quad_TZ()
    // ⎕TZ is the offset in hours between GMT and local time
    //
    if (offset_seconds % 3600 == 0)   // full hour
-      Symbol::assign(IntScalar(offset_seconds/3600, LOC), LOC);
+      Symbol::assign(IntScalar(offset_seconds/3600, LOC), false, LOC);
    else
-      Symbol::assign(FloatScalar(offset_seconds/3600, LOC), LOC);
+      Symbol::assign(FloatScalar(offset_seconds/3600, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 int
@@ -982,7 +935,7 @@ int diff_minutes = local_minutes - gm_minutes;
 }
 //-----------------------------------------------------------------------------
 void
-Quad_TZ::assign(Value_P value, const char * loc)
+Quad_TZ::assign(Value_P value, bool clone, const char * loc)
 {
    if (!value->is_scalar())   RANK_ERROR;
 
@@ -991,11 +944,11 @@ Quad_TZ::assign(Value_P value, const char * loc)
 Cell & cell = value->get_ravel(0);
    if (cell.is_integer_cell())
       {
-        const APL_Integer ival = cell.get_int_value();
+        const APL_Integer ival = cell.get_near_int();
         if (ival < -12)   return;
         if (ival > 14)    return;
         offset_seconds = ival*3600;
-        Symbol::assign(value, LOC);
+        Symbol::assign(value, clone, LOC);
         return;
       }
 
@@ -1005,7 +958,7 @@ Cell & cell = value->get_ravel(0);
         if (fval < -12.1)   return;
         if (fval > 14.1)    return;
         offset_seconds = int(0.5 + fval*3600);
-        Symbol::assign(value, LOC);
+        Symbol::assign(value, clone, LOC);
         return;
       }
 
@@ -1015,7 +968,7 @@ Cell & cell = value->get_ravel(0);
 Quad_UL::Quad_UL()
    : RO_SystemVariable(ID::Quad_UL)
 {
-   Symbol::assign(get_apl_value(), LOC);
+   Symbol::assign(get_apl_value(), false, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -1040,7 +993,7 @@ int user_count = 0;
 #endif
 
 Value_P Z(LOC);
-   new (&Z->get_ravel(0))   IntCell(user_count);
+   new (Z->next_ravel())   IntCell(user_count);
    Z->check_value(LOC);
    return Z;
 }
@@ -1048,7 +1001,7 @@ Value_P Z(LOC);
 Quad_WA::Quad_WA()
    : RO_SystemVariable(ID::Quad_WA)
 {
-   Symbol::assign(IntScalar(0, LOC), LOC);
+   Symbol::assign(IntScalar(0, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 Value_P
@@ -1109,8 +1062,7 @@ uint64_t proc_mem = 0;            // memory as reported proc/mem_info
            }
       }
 
-   new (&Z->get_ravel(0))   IntCell(total);
-
+   new (Z->next_ravel())   IntCell(total);
    Z->check_value(LOC);
    return Z;
 }
@@ -1118,11 +1070,11 @@ uint64_t proc_mem = 0;            // memory as reported proc/mem_info
 Quad_X::Quad_X()
  : NL_SystemVariable(ID::Quad_X)
 {
-   Symbol::assign(IntScalar(0, LOC), LOC);
+   Symbol::assign(IntScalar(0, LOC), false, LOC);
 }
 //-----------------------------------------------------------------------------
 void
-Quad_X::assign(Value_P value, const char * loc)
+Quad_X::assign(Value_P value, bool clone, const char * loc)
 {
 StateIndicator * si = Workspace::SI_top_fun();
    if (si == 0)   return;
