@@ -205,21 +205,23 @@ operator << (ostream & out, const Token & token)
              else
                 FIXME;
 
-        case TC_SYMBOL:   if (token.get_tag() == TOK_LSYMB)
-                             {
-                               token.get_sym_ptr()->print(out << "'");
-                               out << UNI_LEFT_ARROW;
-                             }
-                          else if (token.get_tag() == TOK_LSYMB2)
-                             {
-                               token.get_sym_ptr()->print(out << "'(... ");
-                               out << ")←";
-                             }
-                          else
-                             {
-                               token.get_sym_ptr()->print(out << "'");
-                             }
-                          return out;
+        case TC_SYMBOL:
+             if (token.get_tag() == TOK_LSYMB)
+                {
+                  token.get_sym_ptr()->print(out << "'");
+                  out << UNI_LEFT_ARROW;
+                }
+             else if (token.get_tag() == TOK_LSYMB2)
+                {
+                  token.get_sym_ptr()->print(out << "'(... ");
+                  out << ")←";
+                }
+             else
+                {
+                  token.get_sym_ptr()->print(out << "'");
+                }
+             return out;
+
         case TC_FUN0:
         case TC_FUN12:
         case TC_OPER1:
@@ -250,7 +252,8 @@ Token::value_use_count() const
    return value._apl_val()->get_owner_count();
 }
 //-----------------------------------------------------------------------------
-void Token::extract_apl_val(const char * loc) const
+void
+Token::extract_apl_val(const char * loc) const
 {
    if (is_apl_val())   ptr_clear(value._apl_val(), loc);
 }
@@ -275,6 +278,7 @@ Token::print_function(ostream & out) const
         case TOK_Quad_UCS:
         case TOK_Quad_AT:
         case TOK_Quad_EA:
+        case TOK_Quad_EB:
         case TOK_Quad_SVC:
         case TOK_Quad_TF:
         case TOK_Quad_ES:
@@ -290,7 +294,6 @@ Token::print_function(ostream & out) const
         case TOK_F2_FIND:       return out << UNI_EPSILON_UBAR;
         case TOK_F2_EQUAL:      return out << UNI_ASCII_EQUAL;
         case TOK_F2_GREATER:    return out << UNI_ASCII_GREATER;
-        case TOK_ASSIGN:        return out << UNI_LEFT_ARROW;
         case TOK_F2_AND:        return out << UNI_AND;
         case TOK_F2_OR:         return out << UNI_OR;
         case TOK_F2_INDEX:      return out << UNI_SQUISH_Quad;
@@ -341,14 +344,22 @@ Token::print_function(ostream & out) const
         case TOK_OPER1_SCAN:
         case TOK_OPER1_SCAN1:
         case TOK_OPER2_OUTER:
-        case TOK_OPER2_INNER: return out << get_Id();
+        case TOK_OPER2_INNER:   return out << get_Id();
 
-        case TOK_FUN0:      return get_function()->print(out << "USER-F0 ");
-        case TOK_FUN2:      return get_function()->print(out << "USER-F2 ");
-        case TOK_OPER1:     return get_function()->print(out << "USER-OP1 ");
-        case TOK_OPER2:     return get_function()->print(out << "USER-OP2 ");
+        case TOK_FUN0:          out << "USER-F0 ";
+                                get_function()->print(out);
+                                return out;
+        case TOK_FUN2:          out << "USER-F2 ";
+                                get_function()->print(out);
+                                return out;
+        case TOK_OPER1:         out << "USER-OP1 ";
+                                get_function()->print(out);
+                                return out;
+        case TOK_OPER2:         out << "USER-OP2 ";
+                                get_function()->print(out);
+                                return out;
 
-        default: break;
+        default:                break;
       }
 
    // unknown tag.
@@ -479,7 +490,7 @@ UCS_string fn = fun_name;
 
    // print a value
 
-PrintContext pctx = Workspace::get_PrintContext();
+PrintContext pctx = Workspace::get_PrintContext(PR_APL);
 const Value & val = *get_apl_val();
    if (val.get_rank() == 0)   // scalar
       {
@@ -488,8 +499,7 @@ const Value & val = *get_apl_val();
    else if (val.get_rank() == 1)   // vector
       {
         if (val.element_count() == 0 &&   // empty vector
-            (val.get_ravel(0).is_character_cell() ||
-             val.get_ravel(0).is_numeric()))
+            val.get_ravel(0).is_simple_cell())
            {
              out << endl;
              return;
@@ -558,7 +568,7 @@ UCS_string ucs;
 
         case TC_VALUE:
              {
-               PrintContext pctx(style, 16, 80);
+               PrintContext pctx(style, DEFAULT_Quad_PP, DEFAULT_Quad_PW);
                PrintBuffer pbuf(*get_apl_val(), pctx, 0);
                if (pbuf.get_height() == 0)   return ucs;
                return pbuf.l1();
@@ -804,11 +814,29 @@ move_2(Token & dst, const Token & src, const char * loc)
 }
 //-----------------------------------------------------------------------------
 void
-Token_string::print(ostream & out, int from) const
+Token_string::reverse_from_to(ShapeItem from, ShapeItem to)
 {
-   loop(t, size() - from)
+Token * t1 = items + from;
+Token * t2 = items + to;
+   Assert(0 <= from);
+   Assert(from <= to);
+   Assert(to <= size());
+
+   for (;t1 < t2; ++t1, --t2)
        {
-         const Token & tok = (*this)[from + t];
+         char tt[sizeof(Token)];
+         memcpy(tt, t1, sizeof(Token));
+         memcpy(t1, t2,   sizeof(Token));
+         memcpy(t2, tt, sizeof(Token));
+       }
+}
+//-----------------------------------------------------------------------------
+void
+Token_string::print(ostream & out) const
+{
+   loop(t, size())
+       {
+         const Token & tok = (*this)[t];
          out << "`" << tok << "  ";
        }
 
