@@ -79,7 +79,8 @@ Source<Unicode> src(input);
    while ((rest_1 = rest_2 = src.rest()) != 0)
       {
         const Unicode uni = *src;
-        if (uni == UNI_COMMENT)   break;
+        if (uni == UNI_COMMENT)             break;   // ⍝ comment
+        if (uni == UNI_ASCII_NUMBER_SIGN)   break;   // # comment
 
         const Token tok = Avec::uni_to_token(uni, LOC);
 
@@ -155,9 +156,19 @@ Source<Unicode> src(input);
                       }
                    else if (uni == UNI_LAMBDA)
                       {
-                        ++src;
-                        tos.append(Token(TOK_LAMBDA,
-                                   &Workspace::get_v_LAMBDA()), LOC);
+                        // this could be λ like in λ← ...
+                        // or λ1 or λ2 or ... as in ... ⍺ λ1 ⍵
+                        //
+                        if (src.rest() > 1 && Avec::is_digit(src[1]))   // λn
+                           {
+                             tokenize_symbol(src, tos);
+                           }
+                        else   // λ
+                           {
+                             ++src;
+                             tos.append(Token(TOK_LAMBDA,
+                                        &Workspace::get_v_LAMBDA()), LOC);
+                           }
                       }
                    else if (uni == UNI_OMEGA)
                       {
@@ -223,6 +234,28 @@ Source<Unicode> src(input);
                    break;
 
               case TC_ASSIGN:
+                   ++src;
+                   {
+                     const bool sym = tos.size() >= 1 &&
+                                tos[tos.size() - 1].get_tag() == TOK_SYMBOL;
+                     const bool dia = tos.size() > 1 &&
+                                tos[tos.size() - 2].get_tag() == TOK_DIAMOND;
+                     const bool col = tos.size() > 1 &&
+                                tos[tos.size() - 2].get_tag() == TOK_COLON;
+
+                   tos.append(tok, LOC);
+
+                   // change token tag of ← if:
+                   //
+                   //   SYM ←   (at the start of line),        or
+                   // ◊ SYM ←   (at the start of statement),   or
+                   // : SYM ←   (at the start of statement after label)
+                   //
+                   if (sym && ((tos.size() == 2) || dia || col))
+                      tos.last().ChangeTag(TOK_ASSIGN1);
+                   }
+                   break;
+
               case TC_L_PARENT:
               case TC_R_PARENT:
               case TC_L_BRACK:
@@ -415,7 +448,7 @@ Tokenizer::tokenize_string1(Source<Unicode> & src, Token_string & tos)
    Log(LOG_tokenize)   CERR << "tokenize_string1(" << src << ")" << endl;
 
 const Unicode uni = src.get();
-   Assert(uni == UNI_SINGLE_QUOTE);
+   Assert(Avec::is_single_quote(uni));
 
 UCS_string string_value;
 bool got_end = false;
@@ -424,12 +457,12 @@ bool got_end = false;
        {
          const Unicode uni = src.get();
 
-         if (uni == UNI_SINGLE_QUOTE)
+         if (Avec::is_single_quote(uni))
             {
               // a single ' is the end of the string, while a double '
               // (i.e. '') is a single '. 
               //
-              if ((src.rest() == 0) || (*src != UNI_SINGLE_QUOTE))
+              if ((src.rest() == 0) || !Avec::is_single_quote(*src))
                  {
                    got_end = true;
                    break;
@@ -546,30 +579,32 @@ Tokenizer::tokenize_number(Source<Unicode> & src, Token_string & tos)
    // real 'D' real   // magnitude + angle in degrees
    // real 'R' real   // magnitude + angle in radian
 
-APL_Float real_flt   = 0.0;   // always valid
+APL_Float   real_flt = 0.0;   // always valid
 APL_Integer real_int = 0;     // valid if need_float is false
-bool need_float = false;
-const bool real_valid = tokenize_real(src, need_float, real_flt, real_int);
+bool        real_need_float = false;
+const bool real_valid = tokenize_real(src, real_need_float, real_flt, real_int);
+   if (!real_need_float)   real_flt = real_int;
    if (!real_valid)
       {
         rest_2 = src.rest();
         throw_parse_error(E_BAD_NUMBER, LOC, loc);
       }
 
-   if (src.rest() && *src == UNI_ASCII_J)
+   if (src.rest() && (*src == UNI_ASCII_J || *src == UNI_ASCII_j))
       {
         ++src;   // skip 'J'
 
         APL_Float   imag_flt = 0.0;   // always valid
-        APL_Integer imag_int = 0;     // valid if need_float is false
-        bool need_float = false;
-        const bool imag_valid = tokenize_real(src, need_float,
+        APL_Integer imag_int = 0;     // valid if imag_need_float is false
+        bool imag_need_float = false;
+        const bool imag_valid = tokenize_real(src, imag_need_float,
                                               imag_flt, imag_int);
+        if (!imag_need_float)   imag_flt = imag_flt;
 
         if (!imag_valid)
            {
              --src;   // undo skip 'J'
-             if (need_float)
+             if (real_need_float)
                 {
                   tos.append(Token(TOK_REAL,    real_flt));
                   Log(LOG_tokenize)
@@ -581,27 +616,28 @@ const bool real_valid = tokenize_real(src, need_float, real_flt, real_int);
                   Log(LOG_tokenize)
                      CERR << "  tokenize_number: integer " << real_int << endl;
                 }
-             return;
+             goto done;;
            }
 
         tos.append(Token(TOK_COMPLEX, real_flt, imag_flt));
         Log(LOG_tokenize)   CERR << "  tokenize_number: complex "
                                  << real_flt << "J" << imag_flt << endl;
       }
-   else if (src.rest() && *src == UNI_ASCII_D)
+   else if (src.rest() && (*src == UNI_ASCII_D || *src == UNI_ASCII_d))
       {
         ++src;   // skip 'D'
 
         APL_Float   degrees_flt = 0;  // always valid
         APL_Integer degrees_int = 0;  // valid if imag_floating is true
-        bool need_float = false;
-        const bool imag_valid = tokenize_real(src, need_float,
+        bool imag_need_float = false;
+        const bool imag_valid = tokenize_real(src, imag_need_float,
                                               degrees_flt, degrees_int);
+        if (!imag_need_float)   degrees_flt = degrees_flt;
 
         if (!imag_valid)
            {
              --src;   // undo skip 'D'
-             if (need_float)
+             if (real_need_float)
                 {
                   tos.append(Token(TOK_REAL,    real_flt));
                   Log(LOG_tokenize)
@@ -613,7 +649,7 @@ const bool real_valid = tokenize_real(src, need_float, real_flt, real_int);
                   Log(LOG_tokenize)
                      CERR << "  tokenize_number: integer " << real_int << endl;
                 }
-             return;
+             goto done;;
            }
 
         // real_flt is the magnitude and the angle is in degrees.
@@ -624,20 +660,21 @@ const bool real_valid = tokenize_real(src, need_float, real_flt, real_int);
         Log(LOG_tokenize)   CERR << "  tokenize_number: complex " << real
                                  << "J" << imag << endl;
       }
-   else if (src.rest() && *src == UNI_ASCII_R)
+   else if (src.rest() && (*src == UNI_ASCII_R || *src == UNI_ASCII_r))
       {
         ++src;   // skip 'R'
 
         APL_Float radian_flt;     // always valid
         APL_Integer radian_int;   // valid if imag_floating is true
-        bool need_float = false;
-        const bool imag_valid = tokenize_real(src, need_float,
+        bool radian_need_float = false;
+        const bool imag_valid = tokenize_real(src, radian_need_float,
                                               radian_flt, radian_int);
+        if (!radian_need_float)   radian_flt = radian_int;
 
         if (!imag_valid)
            {
              --src;   // undo skip 'R'
-             if (need_float)
+             if (real_need_float)
                 {
                   tos.append(Token(TOK_REAL,    real_flt));
                   Log(LOG_tokenize)
@@ -649,7 +686,7 @@ const bool real_valid = tokenize_real(src, need_float, real_flt, real_int);
                   Log(LOG_tokenize)
                      CERR << "  tokenize_number: integer " << real_int << endl;
                 }
-             return;
+             goto done;;
            }
 
         // real_flt is the magnitude and the angle is in radian.
@@ -661,19 +698,37 @@ const bool real_valid = tokenize_real(src, need_float, real_flt, real_int);
                                  << "J" << imag << endl;
       }
    else 
-     {
-       if (need_float)
-          {
-            tos.append(Token(TOK_REAL,    real_flt));
-            Log(LOG_tokenize)
-               CERR << "  tokenize_number: real " << real_flt << endl;
-          }
-       else
-          {
-            tos.append(Token(TOK_INTEGER, real_int));
-            Log(LOG_tokenize)
-               CERR << "  tokenize_number: integer " << real_int << endl;
-          }
+      {
+        if (real_need_float)
+           {
+             tos.append(Token(TOK_REAL,    real_flt));
+             Log(LOG_tokenize)
+                CERR << "  tokenize_number: real " << real_flt << endl;
+           }
+        else
+           {
+             tos.append(Token(TOK_INTEGER, real_int));
+             Log(LOG_tokenize)
+                CERR << "  tokenize_number: integer " << real_int << endl;
+           }
+      }
+
+done:
+   // ISO 13751 requires a space between two numeric scalar literals (page 42),
+   // but not between a numeric scalar literal and an identifier.
+   //
+   // IBM APL2 requires a space in both cases. For example, 10Q10 is tokenized
+   // as  10Q 10
+   //
+   // We follow ISO. The second numeric literal cannot start with 0-9 because
+   // that would have been eaten by the first literal. Therefor the only cases
+   // remaining to be checked are a numeric scalar literal followed by ¯
+   // or by . (page 42)
+   //
+   if (src.rest())
+      {
+        if (*src == UNI_OVERBAR || *src == '.')
+                   throw_parse_error(E_BAD_NUMBER, LOC, loc);
       }
 }
 //-----------------------------------------------------------------------------
@@ -681,7 +736,6 @@ bool
 Tokenizer::tokenize_real(Source<Unicode> & src, bool & need_float,
                          APL_Float & flt_val, APL_Integer & int_val)
 {
-   flt_val = 0.0;
    int_val = 0;
    need_float = false;
    
@@ -691,6 +745,7 @@ UTF8_string expo_part;
 bool negative = false;
 bool expo_negative = false;
 bool skipped_0 = false;
+bool dot_seen = false;
 
    // leading sign
    //
@@ -706,17 +761,15 @@ bool skipped_0 = false;
 
    // integer part
    //
-   while (src.rest() && *src >= '0' && *src <= '9')
-      {
-        digits.append(src.get());
-      }
+   while (src.rest() && Avec::is_digit(*src))   digits.append(src.get());
 
    // fractional part
    //
    if (src.rest() &&  *src == UNI_ASCII_FULLSTOP)   // fract part present
       {
-        ++src;
-        while (src.rest() && *src >= '0' && *src <= '9')
+        ++src;   // skip '.'
+        dot_seen = true;
+        while (src.rest() && Avec::is_digit(*src))
            {
              fract_part.append(src.get());
            }
@@ -730,27 +783,27 @@ bool skipped_0 = false;
 
    // exponent part
    //
-   if (src.rest() > 1 &&
+   if (src.rest() >= 2 &&   // at least E and a digit or ¯
        (*src == UNI_ASCII_E || *src == UNI_ASCII_e))   // maybe exponent
       {
-        if ((src[1] == UNI_OVERBAR && src[2] >= '0' && src[2] <= '9') ||
-            (src[1] >= '0' && src[1] <= '9'))
+        expo_negative = src[1] == UNI_OVERBAR;
+        if (Avec::is_digit(src[1]) ||
+            (expo_negative && src.rest() > 2 && Avec::is_digit(src[2])))
            {
              need_float = true;
-             ++src;   // skip e/E
-             if (src.rest() && *src == UNI_OVERBAR)
-           {
-                  ++src;   // skip ¯
-                  expo_negative = true;
-                }
+             ++src;                        // skip e/E
+             if (expo_negative)   ++src;   // skip ¯
 
-             while (src.rest() && *src >= '0' && *src <= '9')
+             while (src.rest() && Avec::is_digit(*src))
                 {
                   expo_part.append(src.get());
                 }
            }
       }
 
+   // second dot ?
+   if (dot_seen && src.rest() && *src == UNI_ASCII_FULLSTOP)   return false;
+
 int expo = 0;
    loop(d, expo_part.size())   expo = 10*expo + (expo_part[d] - '0');
    if (expo_negative)   expo = - expo;
@@ -849,32 +902,35 @@ int expo = 0;
             --expo;
           }
 
-        flt_val = v;
+        // perfom the scaling in higher precision to avoid rounding errors
+        //
+        long double dval = v;
         if (expo > 0)
            {
-             if (expo >= 256)   { expo -= 256;   flt_val *= 1E256; }
-             if (expo >= 128)   { expo -= 128;   flt_val *= 1E128; }
-             if (expo >=  64)   { expo -=  64;   flt_val *= 1E64;  }
-             if (expo >=  32)   { expo -=  32;   flt_val *= 1E32;  }
-             if (expo >=  16)   { expo -=  16;   flt_val *= 1E16;  }
-             if (expo >=   8)   { expo -=   8;   flt_val *= 1E8;   }
-             if (expo >=   4)   { expo -=   4;   flt_val *= 1E4;   }
-             if (expo >=   2)   { expo -=   2;   flt_val *= 1E2;   }
-             if (expo >=   1)   { expo -=   1;   flt_val *= 1E1;   }
+             if (expo >= 256)   { expo -= 256;   dval *= 1E256; }
+             if (expo >= 128)   { expo -= 128;   dval *= 1E128; }
+             if (expo >=  64)   { expo -=  64;   dval *= 1E64;  }
+             if (expo >=  32)   { expo -=  32;   dval *= 1E32;  }
+             if (expo >=  16)   { expo -=  16;   dval *= 1E16;  }
+             if (expo >=   8)   { expo -=   8;   dval *= 1E8;   }
+             if (expo >=   4)   { expo -=   4;   dval *= 1E4;   }
+             if (expo >=   2)   { expo -=   2;   dval *= 1E2;   }
+             if (expo >=   1)   { expo -=   1;   dval *= 1E1;   }
            }
         else if (expo < 0)
            {
-             if (expo <= -256)   { expo += 256;   flt_val /= 1E256; }
-             if (expo <= -128)   { expo += 128;   flt_val /= 1E128; }
-             if (expo <=  -64)   { expo +=  64;   flt_val /= 1E64;  }
-             if (expo <=  -32)   { expo +=  32;   flt_val /= 1E32;  }
-             if (expo <=  -16)   { expo +=  16;   flt_val /= 1E16;  }
-             if (expo <=   -8)   { expo +=   8;   flt_val /= 1E8;   }
-             if (expo <=   -4)   { expo +=   4;   flt_val /= 1E4;   }
-             if (expo <=   -2)   { expo +=   2;   flt_val /= 1E2;   }
-             if (expo <=   -1)   { expo +=   1;   flt_val /= 1E1;   }
+             if (expo <= -256)   { expo += 256;   dval /= 1E256; }
+             if (expo <= -128)   { expo += 128;   dval /= 1E128; }
+             if (expo <=  -64)   { expo +=  64;   dval /= 1E64;  }
+             if (expo <=  -32)   { expo +=  32;   dval /= 1E32;  }
+             if (expo <=  -16)   { expo +=  16;   dval /= 1E16;  }
+             if (expo <=   -8)   { expo +=   8;   dval /= 1E8;   }
+             if (expo <=   -4)   { expo +=   4;   dval /= 1E4;   }
+             if (expo <=   -2)   { expo +=   2;   dval /= 1E2;   }
+             if (expo <=   -1)   { expo +=   1;   dval /= 1E1;   }
            }
-        if (negative)   flt_val = - flt_val;
+        if (negative)   dval = - dval;
+        flt_val = dval;
       }
    else
       {
@@ -893,6 +949,11 @@ Tokenizer::tokenize_symbol(Source<Unicode> & src, Token_string & tos)
    Log(LOG_tokenize)   CERR << "tokenize_symbol() : " << src.rest() << endl;
 
 UCS_string symbol;
+   if (macro)
+      {
+        symbol.append(UNI_MUE);
+        symbol.append(UNI_ASCII_MINUS);
+      }
    symbol.append(src.get());
 
    while (src.rest())
@@ -936,6 +997,7 @@ UCS_string symbol;
       }
 
 Symbol * sym = Workspace::lookup_symbol(symbol);
+   Assert(sym);
    tos.append(Token(TOK_SYMBOL, sym));
 }
 //-----------------------------------------------------------------------------
