@@ -23,6 +23,7 @@
 
 #include "Common.hh"
 #include "FloatCell.hh"
+#include "Heapsort.hh"
 #include "Output.hh"
 #include "PrintBuffer.hh"
 #include "PrintOperator.hh"
@@ -34,7 +35,7 @@
 UCS_string::UCS_string(const char * cstring)
    : Simple_string<Unicode>(0, 0)
 {
-const size_t len = strlen(cstring);
+const int len = strlen(cstring);
    extend(len + 1);
 
    while (items_valid < len)   items[items_valid++] = Unicode(*cstring++);
@@ -47,7 +48,7 @@ UCS_string::UCS_string(const UTF8_string & utf)
    Log(LOG_char_conversion)
       CERR << "UCS_string::UCS_string(): utf = " << utf << endl;
 
-   for (uint32_t i = 0; i < utf.size();)
+   for (int i = 0; i < utf.size();)
       {
         const uint32_t b0 = utf[i++];
         uint32_t bx = b0;
@@ -472,6 +473,14 @@ UCS_string::copy_black_list(vector<UCS_string> & dest) const
       }
 }
 //-----------------------------------------------------------------------------
+ShapeItem
+UCS_string::LF_count() const
+{
+ShapeItem count = 0;
+   loop(u, size())   if ((*this)[u] == UNI_ASCII_LF)   ++count;
+   return count;
+}
+//-----------------------------------------------------------------------------
 bool
 UCS_string::contained_in(const vector<UCS_string> & list) const
 {
@@ -483,10 +492,10 @@ UCS_string::contained_in(const vector<UCS_string> & list) const
    return false;
 }
 //-----------------------------------------------------------------------------
-int
+ShapeItem
 UCS_string::substr_pos(const UCS_string & sub) const
 {
-const int start_positions = 1 + size() - sub.size();
+const ShapeItem start_positions = 1 + size() - sub.size();
    loop(start, start_positions)
       {
         bool mismatch = false;
@@ -709,6 +718,22 @@ char cc[40];
 }
 //-----------------------------------------------------------------------------
 void
+UCS_string::append_shape(const Shape & shape)
+{
+   loop(r, shape.get_rank())
+       {
+         if (r)   append(UNI_ASCII_SPACE);
+         ShapeItem s = shape.get_shape_item(r);
+         if (s < 0)
+            {
+              s = -s;
+              append(UNI_OVERBAR);
+            }
+         append_number(s);
+       }
+}
+//-----------------------------------------------------------------------------
+void
 UCS_string::append_float(double num)
 {
 char cc[60];
@@ -809,7 +834,7 @@ operator << (ostream & os, Unicode uni)
 ostream &
 operator << (ostream & os, const UCS_string & ucs)
 {
-const int fill_len = os.width() - ucs.size();;
+const int fill_len = os.width() - ucs.size();
 
    if (fill_len > 0)
       {
@@ -909,7 +934,7 @@ UCS_string ret;
 }
 //-----------------------------------------------------------------------------
 UCS_string
-UCS_string::from_double_expo_prec(double v, unsigned int fract_digits)
+UCS_string::from_double_expo_prec(double v, int fract_digits)
 {
 UCS_string ret;
 
@@ -1002,7 +1027,18 @@ int expo = 0;
 
    // print mantissa in fixed format
    //
-   ret.append(from_double_fixed_prec(v, fract_digits));
+UCS_string mantissa = from_double_fixed_prec(v, fract_digits);
+   if (mantissa.size() > 2 &&
+       mantissa[0] == UNI_ASCII_1 &&
+       mantissa[1] == UNI_ASCII_0 &&
+       mantissa[2] == UNI_ASCII_FULLSTOP)   // 9.xxx rounded up to 10.xxx
+      {
+        mantissa[1] = UNI_ASCII_FULLSTOP;
+        mantissa[2] = UNI_ASCII_0;
+       ++expo;
+      }
+       
+   ret.append(mantissa);
    ret.append(UNI_ASCII_E);
    ret.append(from_int(expo));
 
@@ -1010,12 +1046,18 @@ int expo = 0;
 }
 //-----------------------------------------------------------------------------
 UCS_string
-UCS_string::from_double_fixed_prec(double v, unsigned int fract_digits)
+UCS_string::from_double_fixed_prec(double v, int fract_digits)
 {
 UCS_string ret;
 
    if (v < 0)   { ret.append(UNI_OVERBAR);   v = - v; }
 
+   // in the loop below, there could be rounding errors when casting float
+   // to int. We therefore increase v slighly (by 0.3 of the rounded digit)
+   // to avoid that.
+   //
+   v += 0.03 * pow(10.0, -fract_digits);
+
    ret.append(from_big(v));   // leaves fractional part of v in v
 
    ret.append(UNI_ASCII_FULLSTOP);
@@ -1023,7 +1065,7 @@ UCS_string ret;
    loop(f, fract_digits + 1)
       {
         v *= 10;
-        const int vv = v;
+        const int vv = v;   // subject to rounding errors!
         ret.append(Unicode(UNI_ASCII_0 + vv));
         v -= vv;
       }
@@ -1041,12 +1083,20 @@ UCS_string::round_last_digit()
         for (int q = size() - 2; q >= 0; --q)
             {
               const Unicode cc = items[q];
-              if (cc >= UNI_ASCII_0 && cc <= UNI_ASCII_9)
-                 {
-                   items[q] = Unicode(cc + 1);
-                   if (cc != UNI_ASCII_9)   break;    // 0-8 rounded up: stop
-                   items[q] = UNI_ASCII_0;    // 9 rounded up: say 0 and repeat
-                 }
+              if (cc < UNI_ASCII_0)   continue;   // not a digit
+              if (cc > UNI_ASCII_9)   continue;   // not a digit
+
+              items[q] = Unicode(cc + 1);   // round up
+              if (cc != UNI_ASCII_9)   break;    // 0-8 rounded up: stop
+
+              items[q] = UNI_ASCII_0;    // 9 rounded up: say 0 and repeat
+              if (q)   continue;   // not first difit
+
+              // something like 9.xxx has been rounded up to, say, 0.xxx
+              // but should be 10.xxx Fix it.
+              //
+              for (int d = size() - 1; d > 0; --d)   items[d] = items[d - 1];
+              items[0] = UNI_ASCII_1;
             }
       }
 
@@ -1065,22 +1115,7 @@ void
 UCS_string::sort_names(const UCS_string ** names, int count)
 {
    if (count < 2)   return;
-
-   for (int h = 0; h < (count - 1); ++h)
-       {
-         // find smallest above (including) h
-         //
-         int smallest = h;
-         for (int j = h + 1; j < count; ++j)
-             {
-               if (names[smallest]->compare(*names[j]) == COMP_GT)
-                  {
-                    const UCS_string * tmp = names[smallest];
-                    names[smallest] = names[j];
-                    names[j] = tmp;
-                  }
-             }
-       }
+   Heapsort<const UCS_string *>::sort(names, count, 0, compare_names);
 }
 //----------------------------------------------------------------------------
 void
@@ -1088,12 +1123,12 @@ UCS_string::sort_names(vector<UCS_string> names)
 {
    if (names.size() < 2)   return;
 
-   for (int h = 0; h < (names.size() - 1); ++h)
+   for (size_t h = 0; h < (names.size() - 1); ++h)
        {
          // find smallest above (including) h
          //
          int smallest = h;
-         for (int j = h + 1; j < names.size(); ++j)
+         for (size_t j = h + 1; j < names.size(); ++j)
              {
                if (names[smallest].compare(names[j]) == COMP_GT)
                   {
