@@ -123,7 +123,7 @@ UTF8_string::UTF8_string(const UCS_string & ucs)
              loop(ii, ucs.size()) CERR << " " << HEX(ucs[ii]);
              CERR << endl;
 
-             Backtrace::show(__FILE__, __LINE__);
+             BACKTRACE
              Assert(0 && "Error in UTF8_string::UTF8_string(ucs)");
            }
       }
@@ -154,18 +154,18 @@ UTF8_string::dump_hex(ostream & out, int max_bytes) const
 }
 //-----------------------------------------------------------------------------
 Unicode
-UTF8_string::toUni(const UTF8 * string, int & len)
+UTF8_string::toUni(const UTF8 * string, int & len, bool verbose)
 {
 const uint32_t b0 = *string++;
    if (b0 < 0x80)                  { len = 1;   return Unicode(b0); }
 
 uint32_t bx = b0;   // the "significant" bits in b0
-   if ((b0 & 0xE0) == 0xC0)        { len = 2;   bx &= 0x1F; }
+   if      ((b0 & 0xE0) == 0xC0)   { len = 2;   bx &= 0x1F; }
    else if ((b0 & 0xF0) == 0xE0)   { len = 3;   bx &= 0x0F; }
    else if ((b0 & 0xF8) == 0xF0)   { len = 4;   bx &= 0x0E; }
    else if ((b0 & 0xFC) == 0xF8)   { len = 5;   bx &= 0x0E; }
    else if ((b0 & 0xFE) == 0xFC)   { len = 6;   bx &= 0x0E; }
-   else
+   else if (verbose)
       {
         CERR << "Bad UTF8 sequence: " << HEX(b0);
         loop(j, 6)
@@ -177,9 +177,14 @@ uint32_t bx = b0;   // the "significant" bits in b0
 
         CERR <<  " at " LOC << endl;
 
-        Backtrace::show(__FILE__, __LINE__);
+        BACKTRACE
         Assert(0 && "Internal error in UTF8_string::toUni()");
       }
+   else
+      {
+        len = 0;
+        return Invalid_Unicode;
+      }
 
 uint32_t uni = 0;
    loop(l, len - 1)
@@ -215,7 +220,7 @@ int len;
    else
       {
         CERR << "Bad UTF8 sequence: " << HEX(b0) << "... at " LOC << endl;
-        Backtrace::show(__FILE__, __LINE__);
+        BACKTRACE
         return Invalid_Unicode;
       }
 
@@ -269,6 +274,65 @@ const int ext_len = strlen(ext);
    return true;
 }
 //-----------------------------------------------------------------------------
+int
+UTF8_string::un_HTML(int in_HTML)
+{
+int dest = 0;
+bool got_tag = false;
+   loop(src, size())
+      {
+        const char cc = (char)(items[src]);
+        if (in_HTML == 2)   // in HTML header, i.e. < seen
+           {
+             if (cc == '>')   in_HTML = 1;   // in HTML but not in HTML tag
+             continue;
+           }
+
+        if (cc == '<')   // start of HTML tag
+           {
+             in_HTML = 2;   // in HTML tag
+             got_tag = true;
+             continue;
+           }
+
+        if (cc != '&')   // unless HTML-escaped char
+           {
+             items[dest++] = cc;
+             continue;
+           }
+
+        const int rest = size() - src;
+        if (rest > 5 && items[src + 1] == '#' && items[src + 4] == ';')
+           {
+             const long val = strtol((const char *)(items + src + 2), 0, 10);
+             items[dest++] = (char)val;
+             src += 4;
+           }
+        else if (rest > 4 && items[src + 1] == 'g' &&
+                             items[src + 2] == 't' &&
+                             items[src + 3] == ';')
+           {
+             items[dest++] = '>';
+             src += 3;   // skip gt;
+           }
+        else if (rest > 4 && items[src + 1] == 'l' &&
+                             items[src + 2] == 't' &&
+                             items[src + 3] == ';')
+           {
+             items[dest++] = '<';
+             src += 3;   // skip gt;
+           }
+        else
+           {
+             items[dest++] = cc;
+           }
+      }
+
+   if (got_tag)   dest = 0;
+   shrink(dest);
+   return in_HTML;
+}
+//-----------------------------------------------------------------------------
 ostream &
 operator<<(ostream & os, const UTF8_string & utf)
 {
