@@ -35,453 +35,23 @@
 #include "Value.icc"
 #include "Workspace.hh"
 
-//=============================================================================
-/// a user-define function signature and its properties
-const struct _header_pattern
-{
-   Fun_signature signature;      ///< a bitmap for header items
-   int           symbol_count;   ///< number of header items (excl local vars)
-   int           token_count;    ///< number of header token (excl local vars)
-   TokenTag      tags[11];       ///< tags of the header token (excl local vars)
-} header_patterns[] =            ///< all valid function headers
-{
-/// function result
-#define __Z  TOK_LSYMB, TOK_ASSIGN
-
-/// lambda result (λ)
-#define __z  TOK_LAMBDA, TOK_ASSIGN
-
-/// left function argument
-#define __A  TOK_SYMBOL
-
-/// left lambda argument
-#define __a  TOK_ALPHA
-
-/// left lambda operator argument
-#define __au  TOK_ALPHA_U
-
-/// a niladic function
-#define __F0 TOK_SYMBOL
-
-/// a monadic function
-#define __F1 TOK_SYMBOL
-
-/// a dyadic function
-#define __F2 TOK_SYMBOL
-
-/// a monadic operator
-#define __OP1 TOK_L_PARENT, TOK_SYMBOL, TOK_SYMBOL, TOK_R_PARENT
-
-/// a monadic operator in lambda
-#define __op1 TOK_L_PARENT, TOK_ALPHA_U, TOK_SYMBOL, TOK_R_PARENT
-
-/// a dyadic operator
-#define __OP2 TOK_L_PARENT, TOK_SYMBOL, TOK_SYMBOL, TOK_SYMBOL, TOK_R_PARENT
-
-/// a dyadic operator in lambda
-#define __op2 TOK_L_PARENT, TOK_ALPHA_U, TOK_SYMBOL, TOK_OMEGA_U, TOK_R_PARENT
-
-/// an axis
-#define __x   TOK_L_BRACK,  TOK_CHI, TOK_R_BRACK
-
-/// an axis
-#define __X   TOK_L_BRACK,  TOK_SYMBOL, TOK_R_BRACK
-
-/// right function argument
-#define __B  TOK_SYMBOL
-
-/// right lambda argument
-#define __b  TOK_OMEGA
-
-/// left lambda operator argument
-#define __ou  TOK_OMEGA_U
-
-   // niladic
-   //
- { SIG_F0             , 1,  1, {           __F0,            } },
-
- { SIG_Z_F0           , 2,  3, { __Z,      __F0,            } },
-
-   // monadic
-   //
- { SIG_F1_B           , 2,  2, {           __F1,       __B, } },
- { SIG_F1_X_B         , 3,  5, {           __F1,  __X, __B, } },
- { SIG_LO_OP1_B       , 3,  5, {           __OP1,      __B, } },
- { SIG_LO_OP1_X_B     , 4,  8, {           __OP1, __X, __B, } },
- { SIG_LO_OP2_RO_B    , 4,  6, {           __OP2,      __B, } },
-
- { SIG_Z_F1_B         , 3,  4, { __Z,      __F1,       __B, } },
- { SIG_Z_F1_X_B       , 4,  7, { __Z,      __F1,  __X, __B, } },
- { SIG_Z_LO_OP1_B     , 4,  7, { __Z,      __OP1,      __B, } },
- { SIG_Z_LO_OP1_X_B   , 5, 10, { __Z,      __OP1, __X, __B, } },
- { SIG_Z_LO_OP2_RO_B  , 5,  8, { __Z,      __OP2,      __B, } },
-
- { SIG_Z_F1_B         , 3,  4, { __z,      __F1,       __b, } },
- { SIG_Z_F1_X_B       , 4,  7, { __z,      __F1,  __x, __b, } },
- { SIG_Z_LO_OP1_B     , 4,  7, { __z,      __op1,      __b, } },
- { SIG_Z_LO_OP1_X_B   , 5, 10, { __z,      __op1, __x, __b, } },
- { SIG_Z_LO_OP2_RO_B  , 5,  8, { __z,      __op2,      __b, } },
-
-   // dyadic
-   //
- { SIG_A_F2_B         , 3,  3, {      __A, __F2,       __B } },
- { SIG_A_F2_X_B       , 4,  6, {      __A, __F2,  __X, __B } },
- { SIG_A_LO_OP1_B     , 4,  6, {      __A, __OP1,      __B } },
- { SIG_A_LO_OP1_X_B   , 5,  9, {      __A, __OP1, __X, __B } },
- { SIG_A_LO_OP2_RO_B  , 5,  7, {      __A, __OP2,      __B } },
-
- { SIG_Z_A_F2_B       , 4,  5, { __Z, __A, __F2,       __B } },
- { SIG_Z_A_F2_X_B     , 5,  8, { __Z, __A, __F2,  __X, __B } },
- { SIG_Z_A_LO_OP1_B   , 5,  8, { __Z, __A, __OP1,      __B } },
- { SIG_Z_A_LO_OP1_X_B , 6, 11, { __Z, __A, __OP1, __X, __B } },
- { SIG_Z_A_LO_OP2_RO_B, 6,  9, { __Z, __A, __OP2,      __B } },
-
- { SIG_Z_A_F2_B       , 4,  5, { __z, __a, __F2,       __b } },
- { SIG_Z_A_F2_X_B     , 5,  8, { __z, __a, __F2,  __x, __b } },
- { SIG_Z_A_LO_OP1_B   , 5,  8, { __z, __a, __op1,      __b } },
- { SIG_Z_A_LO_OP1_X_B , 6, 11, { __z, __a, __op1, __x, __b } },
- { SIG_Z_A_LO_OP2_RO_B, 6,  9, { __z, __a, __op2,      __b } },
-};
-
-/// the number of signatures
-enum { PATTERN_COUNT = sizeof(header_patterns) / sizeof(*header_patterns) };
-
 //-----------------------------------------------------------------------------
-UserFunction_header::UserFunction_header(const UCS_string text)
-  : from_lambda(false),
-    error(E_SYNTAX_ERROR),
-    error_cause(0),
-    sym_Z(0),
-    sym_A(0),
-    sym_LO(0),
-    sym_FUN(0),
-    sym_RO(0),
-    sym_X(0),
-    sym_B(0)
-{
-UCS_string header_line;
-
-   loop(t, text.size())
-       {
-         const Unicode uni = text[t];
-         if (uni == UNI_ASCII_CR)        ;        // ignore CR
-         else if (uni == UNI_ASCII_LF)   break;   // stop at LF
-         else                            header_line.append(uni);
-       }
-
-   if (header_line.size() == 0)   { error_cause = LOC;   return; }
-
-   Log(LOG_UserFunction__set_line)
-      {
-        CERR << "[0] " << header_line << endl;
-        // show_backtrace(__FILE__, __LINE__);
-      }
-
-   // add a semicolon as a guaranteed end marker.
-   // This to avoids checks of the header token count
-   // 
-   header_line.append(Unicode(';'));
-
-Token_string tos;
-   {
-     const Parser parser(PM_FUNCTION, LOC);
-     const ErrorCode err = parser.parse(header_line, tos);
-
-     if (err != E_NO_ERROR)   { error = err;   error_cause = LOC;  return; }
-   }
-
-   // count symbols before first semicolon, allow one symbol too much.
-   //
-int sym_count = 0;
-int tos_idx = 0;
-Symbol * symbols[12];
-   for (; tos_idx < 12; ++tos_idx)
-      {
-         if (tos_idx >= tos.size())                   break;
-         if (tos[tos_idx].get_tag() == TOK_SEMICOL)   break;
-         if (tos[tos_idx].get_Class() == TC_SYMBOL)
-            symbols[sym_count++] = tos[tos_idx].get_sym_ptr();
-      }
-
-   // find matching signature. If sym_count or tos_idx were too high above,
-   // then we will not find them in header_patterns and signal syntax error.
-   //
-Fun_signature signature = SIG_NONE;
-   loop(s, PATTERN_COUNT)
-      {
-        if (header_patterns[s].symbol_count != sym_count)   continue;
-        if (header_patterns[s].token_count != tos_idx)      continue;
-        bool match = true;
-        loop(t, tos_idx)
-           {
-             if (tos[t].get_tag() != header_patterns[s].tags[t])    // mismatch
-                {
-                   match = false;
-                   break;
-                }
-           }
-
-        if (match)
-           {
-             signature = header_patterns[s].signature;
-             break;   // found signature
-           }
-      }
-
-   if (signature == SIG_NONE)   { error_cause = LOC;   return; }
-
-   // note: constructor has set all symbol pointers to 0!
-   // store symbol pointers according to signature.
-   {
-     int sidx = 0;
-     if (signature & SIG_Z)    sym_Z   = symbols[sidx++];
-     if (signature & SIG_A)    sym_A   = symbols[sidx++];
-     if (signature & SIG_LO)   sym_LO  = symbols[sidx++];
-     if (signature & SIG_FUN)  sym_FUN = symbols[sidx++];
-     if (signature & SIG_RO)   sym_RO  = symbols[sidx++];
-     if (signature & SIG_X)    sym_X   = symbols[sidx++];
-     if (signature & SIG_B)    sym_B   = symbols[sidx++];
-
-     Assert1(sidx == sym_count);   // otherwise header_patterns is faulty
-     Assert1(sym_FUN);
-
-     function_name = sym_FUN->get_name();
-   }
-
-   while (tos_idx < (tos.size() - 1))
-      {
-        if (tos[tos_idx++].get_tag() != TOK_SEMICOL)
-           {
-             error_cause = LOC;
-             return;
-           }
-
-
-        if (tos_idx == tos.size())
-           {
-             error_cause = "trailing ; in function header";
-             return;
-           }
-
-        const TokenTag tag = tos[tos_idx].get_tag();
-        if (tag != TOK_SYMBOL && tag != TOK_Quad_CT
-                              && tag != TOK_Quad_FC
-                              && tag != TOK_Quad_IO
-                              && tag != TOK_Quad_PP
-                              && tag != TOK_Quad_PR
-                              && tag != TOK_Quad_PW
-                              && tag != TOK_Quad_RL)
-           {
-             CERR << "Offending token at " LOC " is: " << tos[tos_idx] << endl;
-             return;
-           }
-
-        local_vars.push_back(tos[tos_idx++].get_sym_ptr());
-      }
-
-   error = E_NO_ERROR;
-}
-//-----------------------------------------------------------------------------
-UserFunction_header::UserFunction_header(Fun_signature sig,
-                                         const UCS_string fname)
-  : from_lambda(true),
-    error(E_SYNTAX_ERROR),
-    function_name(fname),
-    sym_Z(0),
-    sym_A(0),
-    sym_LO(0),
-    sym_FUN(0),
-    sym_RO(0),
-    sym_X(0),
-    sym_B(0)
-{
-   // make sure that sig is valid
-   //
-Fun_signature sig1 = (Fun_signature)(sig | SIG_FUN);
-bool valid_signature = false;
-   loop(p, PATTERN_COUNT)
-      {
-        if (header_patterns[p].signature == sig1)
-          {
-            valid_signature = true;
-            break;
-          }
-      }
-
-   if (!valid_signature)   
-      {
-         error = E_SYNTAX_ERROR;
-         return;
-      }
-
-                       sym_Z  = &Workspace::get_v_LAMBDA();
-   if (sig & SIG_A)    sym_A  = &Workspace::get_v_ALPHA();
-   if (sig & SIG_LO)   sym_LO = &Workspace::get_v_ALPHA_U();
-   if (sig & SIG_RO)   sym_RO = &Workspace::get_v_OMEGA_U();
-   if (sig & SIG_B)    sym_B  = &Workspace::get_v_OMEGA();
-   if (sig & SIG_X)    sym_X  = &Workspace::get_v_CHI();
-
-   if (sig & SIG_FUN)   // named lambda
-      {
-         sym_FUN = Workspace::lookup_symbol(fname);
-      }
-
-   error = E_NO_ERROR;
-}
-//-----------------------------------------------------------------------------
-void
-UserFunction_header::pop_local_vars() const
-{
-   loop(l, label_values.size())   label_values[l].sym->pop();
-
-   loop(l, local_vars.size())   local_vars[l]->pop();
-
-   if (sym_B)    sym_B ->pop();
-   if (sym_X)    sym_X ->pop();
-   if (sym_A)    sym_A ->pop();
-   if (sym_LO)   sym_LO->pop();
-   if (sym_RO)   sym_RO->pop();
-   if (sym_Z)    sym_Z ->pop();
-}
-//-----------------------------------------------------------------------------
-void
-UserFunction_header::print_local_vars(ostream & out) const
-{
-   if (sym_Z)     out << " " << *sym_Z;
-   if (sym_A)     out << " " << *sym_A;
-   if (sym_LO)    out << " " << *sym_LO;
-   if (sym_RO)    out << " " << *sym_RO;
-   if (sym_B)     out << " " << *sym_B;
-
-   loop(l, local_vars.size())   out << " " << *local_vars[l];
-}
-//-----------------------------------------------------------------------------
-void
-UserFunction_header::check_duplicate_symbols()
-{
-   check_duplicate_symbol(sym_Z);
-   check_duplicate_symbol(sym_A);
-   check_duplicate_symbol(sym_LO);
-   check_duplicate_symbol(sym_FUN);
-   check_duplicate_symbol(sym_RO);
-   check_duplicate_symbol(sym_X);
-   check_duplicate_symbol(sym_B);
-
-   loop(l, local_vars.size())
-      check_duplicate_symbol(local_vars[l]);
-
-   loop(l, label_values.size())
-      check_duplicate_symbol(label_values[l].sym);
-}
-//-----------------------------------------------------------------------------
-void
-UserFunction_header::check_duplicate_symbol(const Symbol * sym)
-{
-   if (sym == 0)   return;   // unused symbol
-
-int count = 0;
-   if (sym == sym_Z)     ++count;
-   if (sym == sym_A)     ++count;
-   if (sym == sym_LO)    ++count;
-   if (sym == sym_FUN)   ++count;
-   if (sym == sym_RO)    ++count;
-   if (sym == sym_X)     ++count;
-   if (sym == sym_B)     ++count;
-
-   loop(l, local_vars.size())   if (sym == local_vars[l])   ++count;
-
-   loop(l, label_values.size())   if (sym == label_values[l].sym)   ++count;
-
-   if (count <= 1)   return;   // OK
-
-   CERR << "Duplicate symbol ";
-   sym->print(CERR);
-   CERR << "!!!" << endl;
-
-   DEFN_ERROR;
-}
-//-----------------------------------------------------------------------------
-UCS_string
-UserFunction_header::lambda_header(Fun_signature sig, const UCS_string & fname)
-{
-UCS_string u;
-
-   if (sig & SIG_Z)      u.append_utf8("λ←");
-   if (sig & SIG_A)      u.append_utf8("⍺ ");
-   if (sig & SIG_LO)   { u.append_utf8("(⍶ ");
-                         u.append(fname); 
-                         if (sig & SIG_RO)  u.append_utf8(" ⍹ ");
-                         u.append_utf8(")");
-                       }
-                    else
-                       {
-                         u.append(fname); 
-                       }
-   if (sig & SIG_X)      u.append_utf8("[χ]");
-   if (sig & SIG_B)      u.append_utf8(" ⍵");
-
-   return u;
-}
-//-----------------------------------------------------------------------------
-void
-UserFunction_header::print_properties(ostream & out, int indent) const
-{
-UCS_string ind(indent, UNI_ASCII_SPACE);
-   if (is_operator())   out << "Operator " << function_name << endl;
-   else                 out << "Function " << function_name << endl;
-
-   if (sym_Z)    out << ind << "Result:         " << *sym_Z  << endl;
-   if (sym_A)    out << ind << "Left Argument:  " << *sym_A  << endl;
-   if (sym_LO)   out << ind << "Left Op Arg:    " << *sym_LO << endl;
-   if (sym_RO)   out << ind << "Right Op Arg:   " << *sym_RO << endl;
-   if (sym_B)    out << ind << "Right Argument: " << *sym_B  << endl;
-
-   if (local_vars.size())
-      {
-        out << ind << "Local Variables:";
-        loop(l, local_vars.size())   out << " " << *local_vars[l];
-        out << endl;
-      }
-
-   if (label_values.size())
-      {
-        out << ind << "Labels:        ";
-        loop(l, label_values.size())
-           {
-             if (l)   out << ",";
-             out << " " << *label_values[l].sym
-                 << "=" << label_values[l].line;
-           }
-        out << endl;
-      }
-}
-//-----------------------------------------------------------------------------
-void
-UserFunction_header::eval_common()
-{
-   Log(LOG_UserFunction__enter_leave)   CERR << "eval_common()" << endl;
-
-   if (Z())   Z()->push();
-
-   // push local variables...
-   //
-   loop(l, local_vars.size())   local_vars[l]->push();
-
-   // push labels...
-   //
-   loop(l, label_values.size())
-       label_values[l].sym->push_label(label_values[l].line);
-}
-//=============================================================================
-UserFunction::UserFunction(const UCS_string txt,
-                           int & error_line, const char * & error_cause,
-                           bool keep_existing, const char * loc,
-                           const UTF8_string & _creator)
+UserFunction::UserFunction(const UCS_string txt, const char * loc,
+                           const UTF8_string & _creator, bool tolerant,
+                           bool macro)
   : Function(ID::USER_SYMBOL, TOK_FUN2),
-    Executable(txt, loc),
-    header(txt),
-    creator(_creator)
+    Executable(txt, true, PM_FUNCTION, loc),
+    header(txt, macro),
+    creator(_creator),
+    error_line(0),   // assume header is wrong
+    error_info("Unspecified")
 {
+   if (header.get_error())
+      {
+        error_info = header.get_error_info();
+        return;
+      }
+
    set_creation_time(now());
 
    exec_properties[0] = 0;
@@ -489,13 +59,12 @@ UserFunction::UserFunction(const UCS_string txt,
    exec_properties[2] = 0;
    exec_properties[3] = 0;
 
-   line_starts.push_back(Function_PC_0);   // will be set later.
+   line_starts.append(Function_PC_0);   // will be set later.
 
    if (header.get_error() != E_NO_ERROR)   // bad header
       {
-        error_line = 0;
-        error_cause = header.get_error_cause();
-        DEFN_ERROR;
+        error_info = header.get_error_info();
+        return;
       }
 
    // set Function::tag
@@ -506,47 +75,25 @@ UserFunction::UserFunction(const UCS_string txt,
    else if (header.B())    tag = TOK_FUN1;
    else                    tag = TOK_FUN0;
 
-
-   // check that the function can be defined.
-   //
-const char * why_not = header.FUN()->cant_be_defined();
-   if (why_not)
+   parse_body(loc, tolerant, macro);
+   if (error_line > 0)
       {
-         error_cause = why_not;
-         DEFN_ERROR;
-      }
-     
-Function * old_function = header.FUN()->get_function();
-   if (old_function && keep_existing)
-      {
-        error_cause = "function exists";
-        DEFN_ERROR;
+        error_info = "Error in function body";
+        return;
       }
 
-   parse_body(error_line, loc);
-
-   header.check_duplicate_symbols();
-   if (error_line > 0)   error_cause = "Error in function body";
-
-   if (header.LO())   header.FUN()->set_nc(NC_OPERATOR, this);
-   else               header.FUN()->set_nc(NC_FUNCTION, this);
-
-   if (old_function)
-      {
-        const UserFunction * old_ufun = old_function->get_ufun1();
-        Assert(old_ufun);
-        delete old_ufun;
-      }
-
-   error_line = -1;   // assume no error
+   error_line = -1;   // no error
+   error_info = 0;
 }
 //-----------------------------------------------------------------------------
-UserFunction::UserFunction(Fun_signature sig, const UCS_string & fname,
-                           const UCS_string & text, const Token_string & bdy)
+UserFunction::UserFunction(Fun_signature sig, int lambda_num,
+                           const UCS_string & text, Token_string & lambda_body)
   : Function(ID::USER_SYMBOL, TOK_FUN0),
-    Executable(sig, fname, text, LOC),
-    header(sig, fname),
-    creator(UNI_LAMBDA)
+    Executable(sig, lambda_num, text, LOC),
+    header(sig, lambda_num),
+    creator(UNI_LAMBDA),
+    error_line(0),
+    error_info("Unspecified")
 {
    set_creation_time(now());
 
@@ -557,7 +104,8 @@ UserFunction::UserFunction(Fun_signature sig, const UCS_string & fname,
 
    if (header.get_error() != E_NO_ERROR)   // bad header
       {
-        DEFN_ERROR;
+        error_info = header.get_error_info();
+        return;
       }
 
    if      (header.RO())   tag = TOK_OPER2;
@@ -566,15 +114,20 @@ UserFunction::UserFunction(Fun_signature sig, const UCS_string & fname,
    else if (header.B())    tag = TOK_FUN1;
    else                    tag = TOK_FUN0;
 
-   parse_body_line(Function_Line_0, bdy, false, LOC);
-   setup_lambdas();
-   line_starts.push_back(Function_PC(bdy.size() - 1));
-
-   if (header.FUN())
+   while (lambda_body.size() > 2 &&
+          lambda_body.last().get_tag() == TOK_SYMBOL &&
+          lambda_body[lambda_body.size() - 2].get_tag() == TOK_SEMICOL)
       {
-        if (header.LO())   header.FUN()->set_nc(NC_OPERATOR, this);
-        else               header.FUN()->set_nc(NC_FUNCTION, this);
+        header.add_local_var(lambda_body.last().get_sym_ptr());
+        lambda_body.pop();   // varname
+        lambda_body.pop();   // semicolon
       }
+
+   parse_body_line(Function_Line_0, lambda_body, false, false, LOC);
+   setup_lambdas();
+   line_starts.append(Function_PC(lambda_body.size() - 1));
+   error_line = -1;   // no error
+   error_info = 0;
 }
 //-----------------------------------------------------------------------------
 UserFunction::~UserFunction()
@@ -592,6 +145,7 @@ UserFunction::eval_()
    if (header.B())   SYNTAX_ERROR;   // not defined niladic
 
    Workspace::push_SI(this, LOC);
+   if (header.Z())   header.Z()->push();
 
    header.eval_common();
 
@@ -611,9 +165,10 @@ UserFunction::eval_B(Value_P B)
 
    Workspace::push_SI(this, LOC);
 
-   if (header.B())   header.B()->push_value(B);
-   if (header.X())   header.X()->push();
+   if (header.Z())   header.Z()->push();
    if (header.A())   header.A()->push();
+   if (header.X())   header.X()->push();
+   if (header.B())   header.B()->push_value(B);
 
    header.eval_common();
 
@@ -633,9 +188,10 @@ UserFunction::eval_XB(Value_P X, Value_P B)
 
    Workspace::push_SI(this, LOC);
 
-   if (header.B())   header.B()->push_value(B);
-   if (header.X())   header.X()->push_value(X);
+   if (header.Z())   header.Z()->push();
    if (header.A())   header.A()->push();
+   if (header.X())   header.X()->push_value(X);
+   if (header.B())   header.B()->push_value(B);
 
    header.eval_common();
 
@@ -652,13 +208,15 @@ UserFunction::eval_AB(Value_P A, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   if (header.LO())    SYNTAX_ERROR;   // defined as operator
+   if (header.LO())    SYNTAX_ERROR;    // defined as operator
+   if (!header.A())    VALENCE_ERROR;   // monadic
 
    Workspace::push_SI(this, LOC);
 
-   if (header.B())   header.B()->push_value(B);
-   if (header.X())   header.X()->push();
+   if (header.Z())   header.Z()->push();
    if (header.A())   header.A()->push_value(A);
+   if (header.X())   header.X()->push();
+   if (header.B())   header.B()->push_value(B);
 
    header.eval_common();
 
@@ -675,13 +233,15 @@ UserFunction::eval_AXB(Value_P A, Value_P X, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   if (header.LO())    SYNTAX_ERROR;   // defined as operator
+   if (header.LO())    SYNTAX_ERROR;    // defined as operator
+   if (!header.A())    VALENCE_ERROR;   // monadic
 
    Workspace::push_SI(this, LOC);
 
-   if (header.B())   header.B()->push_value(B);
-   if (header.X())   header.X()->push_value(X);
+   if (header.Z())   header.Z()->push();
    if (header.A())   header.A()->push_value(A);
+   if (header.X())   header.X()->push_value(X);
+   if (header.B())   header.B()->push_value(B);
 
    header.eval_common();
 
@@ -702,11 +262,12 @@ UserFunction::eval_LB(Token & LO, Value_P B)
 
    Workspace::push_SI(this, LOC);
 
-                header.B()->push_value(B);
-   if (header.X())   header.X()->push();
-   if (header.A())   header.A()->push();
+   if (header.Z())         header.Z()->push();
+   if (header.A())         header.A()->push();
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
+   if (header.X())         header.X()->push();
+                           header.B()->push_value(B);
 
    header.eval_common();
 
@@ -728,11 +289,12 @@ UserFunction::eval_LXB(Token & LO, Value_P X, Value_P B)
 
    Workspace::push_SI(this, LOC);
 
-                header.B()->push_value(B);
-   if (header.X())   header.X()->push_value(X);
-   if (header.A())   header.A()->push();
+   if (header.Z())         header.Z()->push();
+   if (header.A())         header.A()->push();
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
+   if (header.X())         header.X()->push_value(X);
+                           header.B()->push_value(B);
 
    header.eval_common();
 
@@ -750,17 +312,18 @@ UserFunction::eval_ALB(Value_P A, Token & LO, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   if (!header.A())    SYNTAX_ERROR;   // monadic function called dyadically
-   if (header.RO())    SYNTAX_ERROR;   // defined as dyadic operator
+   if (header.RO())    SYNTAX_ERROR;    // defined as dyadic operator
+   if (!header.A())    VALENCE_ERROR;   // monadic
 
    Workspace::push_SI(this, LOC);
 
-   if (header.X())   header.X()->push();
+   if (header.X())         header.X()->push();
 
-   header.B()->push_value(B);
-   header.A()->push_value(A);
+   if (header.Z())         header.Z()->push();
+                           header.A()->push_value(A);
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
+                           header.B()->push_value(B);
 
    header.eval_common();
 
@@ -779,17 +342,17 @@ UserFunction::eval_ALXB(Value_P A, Token & LO, Value_P X, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   if (!header.A())    SYNTAX_ERROR;   // monadic function called dyadically
-   if (header.RO())    SYNTAX_ERROR;   // defined as dyadic operator
+   if (header.RO())    SYNTAX_ERROR;    // defined as dyadic operator
+   if (!header.A())    VALENCE_ERROR;   // monadic
 
    Workspace::push_SI(this, LOC);
 
-   header.B()->push_value(B);
-   if (header.X())   header.X()->push();
-   header.A()->push_value(A);
-   if (header.X())   header.X()->push_value(X);
+   if (header.Z())         header.Z()->push();
+                           header.A()->push_value(A);
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
+   if (header.X())         header.X()->push_value(X);
+                           header.B()->push_value(B);
 
    header.eval_common();
 
@@ -811,14 +374,15 @@ UserFunction::eval_LRB(Token & LO, Token & RO, Value_P B)
 
    Workspace::push_SI(this, LOC);
 
-                 header.B()->push_value(B);
-   if (header.X())    header.X()->push();
-   if (header.A())    header.A()->push();
+   if (header.Z())         header.Z()->push();
+   if (header.A())         header.A()->push();
 
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
    if (RO.is_function())   header.RO()->push_function(RO.get_function());
    else                    header.RO()->push_value(RO.get_apl_val());
+   if (header.X())         header.X()->push();
+                           header.B()->push_value(B);
 
    header.eval_common();
 
@@ -841,13 +405,14 @@ UserFunction::eval_LRXB(Token & LO, Token & RO, Value_P X, Value_P B)
 
    Workspace::push_SI(this, LOC);
 
-                header.B()->push_value(B);
-   if (header.X())   header.X()->push_value(X);
-   if (header.A())   header.A()->push();
+   if (header.Z())         header.Z()->push();
+   if (header.A())         header.A()->push();
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
    if (RO.is_function())   header.RO()->push_function(RO.get_function());
+   if (header.X())         header.X()->push_value(X);
    else                    header.RO()->push_value(RO.get_apl_val());
+                           header.B()->push_value(B);
 
    header.eval_common();
 
@@ -866,19 +431,20 @@ UserFunction::eval_ALRB(Value_P A, Token & LO, Token & RO, Value_P B)
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   if (!header.A())    SYNTAX_ERROR;    // defined monadic called dyadic
-   if (!header.RO())    SYNTAX_ERROR;   // defined monadic op called dyadic
+   if (!header.RO())    SYNTAX_ERROR;   // defined monadic op called dyadically
+   if (!header.A())    VALENCE_ERROR;   // monadic
 
    Workspace::push_SI(this, LOC);
 
-   if (header.X())      header.X()->push();
+   if (header.Z())         header.Z()->push();
 
-                 header.B()->push_value(B);
-                 header.A()->push_value(A);
+                           header.A()->push_value(A);
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
    if (RO.is_function())   header.RO()->push_function(RO.get_function());
    else                    header.RO()->push_value(RO.get_apl_val());
+   if (header.X())         header.X()->push();
+                           header.B()->push_value(B);
 
    header.eval_common();
 
@@ -899,18 +465,19 @@ UserFunction::eval_ALRXB(Value_P A, Token & LO, Token & RO,
              << Token(TOK_APL_VALUE1, B) << ")" << endl;
       }
 
-   if (!header.A())    SYNTAX_ERROR;    // defined monadic called dyadic
-   if (!header.RO())    SYNTAX_ERROR;   // defined monadic op called dyadic
+   if (!header.RO())   SYNTAX_ERROR;   // defined monadic op called dyadically
+   if (!header.A())    VALENCE_ERROR;   // monadic
 
    Workspace::push_SI(this, LOC);
 
-                 header.B()->push_value(B);
-   if (header.X())    header.X()->push_value(X);
-                 header.A()->push_value(A);
+   if (header.Z())         header.Z()->push();
+                           header.A()->push_value(A);
    if (LO.is_function())   header.LO()->push_function(LO.get_function());
    else                    header.LO()->push_value(LO.get_apl_val());
    if (RO.is_function())   header.RO()->push_function(RO.get_function());
    else                    header.RO()->push_value(RO.get_apl_val());
+   if (header.X())         header.X()->push_value(X);
+                           header.B()->push_value(B);
 
    header.eval_common();
 
@@ -975,7 +542,7 @@ DynArray(bool, ts_lines, line_starts.size());
    loop(ll, line_count)
       {
         Function_Line line = lines[ll];
-        if (line >= 1 && line < line_starts.size())   ts_lines[line] = true;
+        if (line >= 1 && line < (int)line_starts.size())  ts_lines[line] = true;
       }
 
    if (stop)
@@ -983,7 +550,7 @@ DynArray(bool, ts_lines, line_starts.size());
         stop_lines.clear();
         loop(ts, line_starts.size())
            {
-             if (ts_lines[ts])   stop_lines.push_back((Function_Line)ts);
+             if (ts_lines[ts])   stop_lines.append((Function_Line)ts);
            }
       }
    else
@@ -991,19 +558,18 @@ DynArray(bool, ts_lines, line_starts.size());
         trace_lines.clear();
         loop(ts, line_starts.size())
            {
-             if (ts_lines[ts])   trace_lines.push_back((Function_Line)ts);
+             if (ts_lines[ts])   trace_lines.append((Function_Line)ts);
            }
       }
 
-int error_line = -1;
-   parse_body(error_line, LOC);
+   parse_body(LOC, false, false);
 }
 //-----------------------------------------------------------------------------
 void
-UserFunction::parse_body(int & error_line, const char * loc)
+UserFunction::parse_body(const char * loc, bool tolerant, bool macro)
 {
    line_starts.clear();
-   line_starts.push_back(Function_PC_0);   // will be set later.
+   line_starts.append(Function_PC_0);   // will be set later.
 
    clear_body();
 
@@ -1029,8 +595,8 @@ UserFunction::parse_body(int & error_line, const char * loc)
                 }
            }
 
-        error_line = l;
-        line_starts.push_back(Function_PC(body.size()));
+        error_line = l;   // assume error
+        line_starts.append(Function_PC(body.size()));
 
         if (stop_line)
            {
@@ -1039,10 +605,29 @@ UserFunction::parse_body(int & error_line, const char * loc)
              body.append(Token(TOK_END, tr));
            }
 
-        const UCS_string & text = get_text(l);
-        parse_body_line(Function_Line(l), text, trace_line, loc);
+        const UCS_string & line = get_text(l);
+        ErrorCode ec = E_SYNTAX_ERROR;
+        try {
+              ec = parse_body_line(Function_Line(l), line, trace_line,
+                                   tolerant, loc, macro);
+
+              if (tolerant && ec != E_NO_ERROR)
+                 {
+                   UCS_string new_line = "## ";
+                   new_line.append(line);
+                   text[l] = new_line;
+                   CERR << "WARNING: SYNTAX ERROR in function "
+                        << header.get_name() << endl;
+                 }
+            }
+        catch(Error err)
+            {
+              return;
+            }
       }
-      setup_lambdas();
+
+   error_line = -1;   // OK
+   setup_lambdas();
 
    Log(LOG_UserFunction__fix)
       {
@@ -1136,21 +721,21 @@ UCS_string ucs(utf);
    close(in);
 
 int error_line = -1;
-   fun = fix(ucs, error_line, false, LOC, filename);
+   fun = fix(ucs, error_line, false, LOC, filename, false);
 }
 //-----------------------------------------------------------------------------
 Function_PC
 UserFunction::pc_for_line(Function_Line line) const
 {
-   if (line < 1)                     return Function_PC(body.size() - 1);
-   if (line >= line_starts.size())   return Function_PC(body.size() - 1);
+   if (line < 1)                          return Function_PC(body.size() - 1);
+   if (line >= (int)line_starts.size())   return Function_PC(body.size() - 1);
    return line_starts[line];
 }
 //-----------------------------------------------------------------------------
 UserFunction *
-UserFunction::fix(const UCS_string & text, int & error_line,
+UserFunction::fix(const UCS_string & text, int & err_line,
                   bool keep_existing, const char * loc,
-                  const UTF8_string & creator)
+                  const UTF8_string & creator, bool tolerant)
 {
    Log(LOG_UserFunction__fix)
       {
@@ -1158,84 +743,170 @@ UserFunction::fix(const UCS_string & text, int & error_line,
              <<  "------------------- UserFunction::fix() --" << endl;
       }
 
-   if (Workspace::SI_top())   Workspace::SI_top()->set_safe_execution(true);
+UserFunction * ufun = new UserFunction(text, loc, creator, tolerant, false);
+const char * info = ufun->get_error_info();
+   err_line = ufun->get_error_line();
 
-UserFunction * fun = 0;
-const char * error_cause = 0;
-   try
+   if (info || err_line != -1)   // something went wrong
       {
-        fun = new UserFunction(text, error_line, error_cause,
-                               keep_existing, loc, creator);
-      }
-   catch (Error err)
-      {
-        err.print(CERR);
-        if (Workspace::SI_top())
-           Workspace::SI_top()->set_safe_execution(false);
-
-        if (error_cause)
+        if (info)
            {
-             Log(LOG_UserFunction__fix)
-                CERR << "Error: " << error_cause << endl;
-             Workspace::more_error() = UCS_string(error_cause);
+             Log(LOG_UserFunction__fix)   CERR << "Error: " << info << endl;
+             Workspace::more_error() = UCS_string(info);
            }
 
-         if (error_line == 0)
+         if (err_line == 0)
            {
              Workspace::more_error().append_utf8(" (function header)");
+             Log(LOG_UserFunction__fix) CERR << "Bad header line" <<  endl;
            }
-         else if (error_line > 0)
+         else if (err_line > 0)
            {
              Workspace::more_error().append_utf8(" (function line [");
-             Workspace::more_error().append_number(error_line);
-             Workspace::more_error().append_utf8("])");
+             Workspace::more_error().append_number(err_line);
+             Workspace::more_error().append_utf8("] of:\n");
+             Workspace::more_error().append(text);
+
+             Log(LOG_UserFunction__fix)
+                CERR << "Bad function line: " << err_line << endl;
            }
-                
+
+        delete ufun;
         return 0;
       }
-   catch (...)
+
+Symbol * symbol = Workspace::lookup_symbol(ufun->header.get_name());
+Function * old_function = symbol->get_function();
+   if (old_function && keep_existing)
       {
-        CERR << __FUNCTION__ << "caught something unexpected at " LOC << endl;
-        if (Workspace::SI_top())
-           Workspace::SI_top()->set_safe_execution(false);
+        err_line = 0;
+        delete ufun;
         return 0;
       }
 
+   // check that the function can be defined (e.g. is not on the )SI stack)
+   //
+   if (old_function && symbol->cant_be_defined())
+      {
+        err_line = 0;
+        delete ufun;
+        return 0;
+      }
+     
+   if (old_function)
+      {
+        const UserFunction * old_ufun = old_function->get_ufun1();
+        Assert(old_ufun);
+        delete old_ufun;
+      }
+
+   // bind function to symbol
+   //
+   if (ufun->header.LO())   ufun->header.FUN()->set_nc(NC_OPERATOR, ufun);
+   else                     ufun->header.FUN()->set_nc(NC_FUNCTION, ufun);
+
    Log(LOG_UserFunction__fix)
       {
-        CERR << " addr " << (const void *)fun << endl;
-        fun->print(CERR);
+        CERR << " addr " << (const void *)ufun << endl;
+        ufun->print(CERR);
       }
 
-   if (Workspace::SI_top())
-      Workspace::SI_top()->set_safe_execution(false);
+   return ufun;
+}
+//-----------------------------------------------------------------------------
+UserFunction *
+UserFunction::fix_lambda(Symbol & var, const UCS_string & text)
+{
+   /* Example: consider {⍺+⍵;LOCAL}
 
-   // UserFunction::UserFunction() sets error line to -1 if successful.
-   // We may have created a UserFunction but with an error. In this case
-   // we delete it here.
+      text a normal (non-lambda) function like:
+
+      λ←⍺ λ1 ⍵;LOCAL
+      λ← ⍺+⍵
+
+       which has a slightly different lambda body like:
+
+       λ← ⍺+⍵;LOCAL
+
+    */
+int signature = SIG_FUN | SIG_Z;
+int t = 0;
+
+ShapeItem semi = -1;
+   while (t < text.size())
+       {
+         switch(text[t++])
+            {
+              case UNI_CHI:             signature |= SIG_X;    continue;
+              case UNI_OMEGA:           signature |= SIG_B;    continue;
+              case UNI_ALPHA_UNDERBAR:  signature |= SIG_LO;   continue;
+              case UNI_OMEGA_UNDERBAR:  signature |= SIG_RO;   continue;
+              case UNI_ALPHA:           signature |= SIG_A;    continue;
+
+              case UNI_ASCII_SEMICOLON: if (semi == -1)   semi = t - 1;
+                                        continue;
+                                        
+              case UNI_ASCII_LF:        break;   // header done
+              default:                  continue;
+            }
+
+         break;   // header done
+       }
+
+   // discard leading spaces
    //
-   if (error_line != -1)
+   while (t < text.size() && text[t] == UNI_ASCII_SPACE)   ++t;
+
+UCS_string body_text;
+   for (; t < text.size(); ++t)   body_text.append(text[t]);
+
+   while (body_text.last() == UNI_ASCII_LF)  body_text.pop();
+
+Token_string body;
+   {
+     Token ret_lambda(TOK_RETURN_SYMBOL, &Workspace::get_v_LAMBDA());
+     body.append(ret_lambda);
+     const int64_t trace = 0;
+     Token tok_endl(TOK_ENDL, trace);
+     body.append(tok_endl);
+   }
+
+   if (semi != -1)
       {
-        Log(LOG_UserFunction__fix)
-           CERR << "Line: " << error_line << endl;
-        delete fun;
+        for (ShapeItem s = semi; text[s] != UNI_ASCII_LF; ++s)
+           {
+             body_text.append(text[s]);
+           }
+      }
+
+const Parser parser(PM_FUNCTION, LOC, false);
+const ErrorCode ec = parser.parse(body_text, body);
+   if (ec)
+      {
+        CERR << "Parsing '" << body_text << "' failed" << endl;
         return 0;
       }
 
-   return fun;
+UserFunction * ufun = new UserFunction((Fun_signature)signature, 0,
+                                       body_text, body);
+   return ufun;
 }
 //-----------------------------------------------------------------------------
 void
 UserFunction::destroy()
 {
    // delete will call ~Executable(), which releases the values owned by body.
-   delete this;
+   //
+   if (is_lambda())   decrement_refcount(LOC);
+   else               delete this;
 }
 //-----------------------------------------------------------------------------
 ostream &
 UserFunction::print(ostream & out) const
 {
-   return out << header.get_name();
+   out << header.get_name();
+   return out;
+
 /*
    out << "Function header:" << endl;
    if (header.Z())     out << "Result:         " << *header.Z()   << endl;
@@ -1278,7 +949,8 @@ const Function_Line line = get_line(pc);
 Function_Line
 UserFunction::get_line(Function_PC pc) const
 {
-   Assert(pc >= 0);
+   Assert(pc >= -1);
+   if (pc < 0)   pc = Function_PC_0;
 
    // search line_starts backwards until a line with non-greater pc is found.
    //
