@@ -54,9 +54,10 @@ UserPreferences uprefs;
 void
 UserPreferences::usage(const char * prog)
 {
-const char * prog1 = strrchr(prog, '/');
-   if (prog1)   ++prog1;
-   else         prog1 = prog;
+   {
+     const char * prog1 = strrchr(prog, '/');
+     if (prog1)   prog = prog1 + 1;
+   }
 
 char cc[4000];
    snprintf(cc, sizeof(cc),
@@ -115,7 +116,7 @@ char cc[4000];
 "    --TS                 append to (rather than override) summary.log\n"
 "    -v, --version        show version information and exit\n"
 "    -w milli             wait milli milliseconds at startup\n"
-"    --                   end of options for %s\n", prog1);
+"    --                   end of options for %s\n", prog);
    CERR << cc << endl;
 }
 //-----------------------------------------------------------------------------
@@ -171,14 +172,14 @@ const size_t len = read(fd, buf, sizeof(buf));
    if (len != 2)        return false;
    if (buf[0] != '#')   return false;
    if (buf[1] != '!')   return false;
-   return true;;
+   return true;
 }
 //-----------------------------------------------------------------------------
 bool
 UserPreferences::parse_argv_1()
 {
 bool log_startup = false;
-   for (int a = 1; a < expanded_argv.size(); )
+   for (size_t a = 1; a < expanded_argv.size(); )
        {
          const char * opt = expanded_argv[a++];
          const char * val = (a < expanded_argv.size()) ? expanded_argv[a] : 0;
@@ -221,9 +222,23 @@ UserPreferences::parse_argv_2(bool logit)
    //
    // /usr/bin/apl -apl-options... -- -script-options ./scriptname
    //
-   for (int a = 1; a < expanded_argv.size(); )
+
+   // if GNU APL is started as aplscript, then --script is implied
+   //
+   {
+      const char * prog = expanded_argv[0];
+      if (strlen(prog) >= 6 && !strcmp("script", (prog + strlen(prog) - 6)))
+         {
+            do_CONT = false;               // --noCONT
+            do_not_echo = true;            // -noCIN
+            do_Color = false;              // --noColor
+            silent = true;                 // --silent
+         }
+   }
+
+   for (size_t a = 1; a < expanded_argv.size(); )
        {
-         if (a == script_argc)   { ++a;   continue; }   // skip scriptname
+         if ((int)a == script_argc)   { ++a;   continue; }   // skip scriptname
 
          const char * opt = expanded_argv[a++];
          const char * val = (a < expanded_argv.size()) ? expanded_argv[a] : 0;
@@ -295,8 +310,9 @@ UserPreferences::parse_argv_2(bool logit)
                    CERR << "-f without argument" << endl;
                    exit(a);
                  }
-              InputFile fam(UTF8_string(val), 0, false,
-                                        !do_not_echo, true, false);
+
+              const UTF8_string & filename(val);
+              InputFile fam(filename, 0, false, !do_not_echo, true, no_LX);
               InputFile::files_todo.push_back(fam);
               continue;
             }
@@ -519,8 +535,8 @@ UserPreferences::parse_argv_2(bool logit)
                          break;           // inner for
 
                        }
-                    InputFile fam(UTF8_string(expanded_argv[a]), 0, true,
-                                          true, false, false);
+                    const UTF8_string & filename = expanded_argv[a];
+                    InputFile fam(filename, 0, true, true, false, no_LX);
                     InputFile::files_todo.push_back(fam);
                   }
 
@@ -544,7 +560,7 @@ UserPreferences::parse_argv_2(bool logit)
               if (!val)
                  {
                    CERR << "--TM without test mode" << endl;
-                   exit(a);;
+                   exit(a);
                  }
               const int mode = atoi(val);
               IO_Files::test_mode = IO_Files::TestMode(mode);
@@ -594,10 +610,9 @@ UserPreferences::parse_argv_2(bool logit)
    // if running from a script then make the script the first input file
    if (script_argc != 0)
       {
-        InputFile fam(UTF8_string(expanded_argv[script_argc]), 0,
-                                        false, !do_not_echo, true, false);
+        const UTF8_string & filename = expanded_argv[script_argc];
+        InputFile fam(filename, 0, false, !do_not_echo, true, no_LX);
         InputFile::files_todo.insert(InputFile::files_todo.begin(), fam);
-
       }
 
    // count number of testfiles
@@ -718,7 +733,7 @@ UserPreferences::show_version(ostream & out)
          out << *bt << endl;
        }
 
-   out << endl;
+   out << "   Archive SVN:    " << ARCHIVE_SVN << endl;
 
    Output::set_color_mode(Output::COLM_OUTPUT);
 }
@@ -825,7 +840,7 @@ UserPreferences::open_user_file(const char * fname, char * filename,
 {
    if (sys)   // eg. /etc/gnu-apl.d/preferences
       {
-        snprintf(filename, PATH_MAX,
+        snprintf(filename, APL_PATH_MAX,
                  "%s/gnu-apl.d/%s", Makefile__sysconfdir, fname);
       }
    else       // try $HOME/.gnu_apl
@@ -838,20 +853,20 @@ UserPreferences::open_user_file(const char * fname, char * filename,
              return 0;
            }
 
-        snprintf(filename, PATH_MAX, "%s/.gnu-apl/%s", HOME, fname);
-        filename[PATH_MAX] = 0;
+        snprintf(filename, APL_PATH_MAX, "%s/.gnu-apl/%s", HOME, fname);
+        filename[APL_PATH_MAX] = 0;
 
         // check that $HOME/.gnu-apl/preferences exist and fall back to
         // $HOME/.config gnu-apl/preferences if not
         //
         if (access(filename, F_OK) != 0)   // file does not exit
            {
-             snprintf(filename, PATH_MAX,
+             snprintf(filename, APL_PATH_MAX,
                       "%s/.config/gnu-apl/%s", HOME, fname);
            }
       }
 
-   filename[PATH_MAX] = 0;
+   filename[APL_PATH_MAX] = 0;
 
 FILE * f = fopen(filename, "r");
    if (f == 0)
@@ -871,7 +886,7 @@ FILE * f = fopen(filename, "r");
 void
 UserPreferences::read_config_file(bool sys, bool log_startup)
 {
-char filename[PATH_MAX + 1];
+char filename[APL_PATH_MAX + 1];
 
 FILE * f = open_user_file("preferences", filename, sys, log_startup);
    if (f == 0)   return;
@@ -1148,7 +1163,7 @@ int file_profile = 0;   // the current profile in the preferences file
             }
          else if (!strcasecmp(opt, "KEYBOARD_LAYOUT_FILE"))
             {
-              keyboard_layout_file = UTF8_string(arg);;
+              keyboard_layout_file = UTF8_string(arg);
             }
          else if (!strcasecmp(opt, "CONTROL-Ds-TO-EXIT"))
             {
@@ -1162,7 +1177,7 @@ int file_profile = 0;   // the current profile in the preferences file
 void
 UserPreferences::read_threshold_file(bool sys, bool log_startup)
 {
-char filename[PATH_MAX + 1];
+char filename[APL_PATH_MAX + 1];
 
 FILE * f = open_user_file("parallel_thresholds", filename, sys, log_startup);
    if (f == 0)   return;
@@ -1219,7 +1234,7 @@ int line = 0;
 
          enum { _B = 1, _AB = 2 };
 #define perfo_1(bif, ab, _name, th)   if (!strncmp(param, #bif, strlen(#bif))) \
-   set_threshold(*Bif_ ## bif::fun, ab, i_ab, value);
+   set_threshold(Bif_ ## bif::fun, ab, i_ab, value);
 
 #define perfo_2(bif, ab, _name, thr)  perfo_1(bif, ab, _name, thr)
 #define perfo_3(bif, ab, _name, thr)  perfo_1(bif, ab, _name, thr)
@@ -1236,14 +1251,14 @@ int line = 0;
 }
 //-----------------------------------------------------------------------------
 void
-UserPreferences::set_threshold(Function & fun, int ab, int i_ab,
+UserPreferences::set_threshold(Function * fun, int ab, int i_ab,
                                ShapeItem threshold)
 {
-   if (&fun == 0)    return;
-   if (ab != i_ab)   return;
+   if (fun == 0)    return;
+   if (ab != i_ab)  return;
 
-   if (ab == 1)   fun.set_monadic_threshold(threshold);
-   else           fun.set_dyadic_threshold(threshold);
+   if (ab == 1)   fun->set_monadic_threshold(threshold);
+   else           fun->set_dyadic_threshold(threshold);
 }
 //-----------------------------------------------------------------------------
 
