@@ -30,12 +30,14 @@
 #include "IntCell.hh"
 #include "IO_Files.hh"
 #include "LvalCell.hh"
+#include "Macro.hh"
 #include "Output.hh"
 #include "PointerCell.hh"
 #include "Parallel.hh"
 #include "PrintOperator.hh"
 #include "SystemVariable.hh"
 #include "UCS_string.hh"
+#include "UserFunction.hh"
 #include "Value.icc"
 #include "ValueHistory.hh"
 #include "Workspace.hh"
@@ -47,9 +49,11 @@ ShapeItem Value::total_ravel_count = 0;
 
 void * Value::deleted_values = 0;
 int Value::deleted_values_count = 0;
+uint64_t Value::fast_new = 0;
+uint64_t Value::slow_new = 0;
 
 //-----------------------------------------------------------------------------
-void
+inline void
 Value::init_ravel()
 {
    owner_count = 0;
@@ -60,13 +64,6 @@ Value::init_ravel()
    if (Quad_SYL::value_count_limit &&
        Quad_SYL::value_count_limit < value_count)
       {
-        --value_count;
-
-        // make sure that the value is properly initialized
-        //
-        ravel = short_value;
-        loop(l, SHORT_VALUE_LENGTH_WANTED)   new (ravel + l)   IntCell(42);
-
         Workspace::more_error() = UCS_string(
 "the system limit on the APL value count (as set in ⎕SYL) was reached\n"
 "(and to avoid lock-up, the limit in ⎕SYL was automatically cleared).");
@@ -74,7 +71,8 @@ Value::init_ravel()
         // reset the limit so that we don't get stuck here.
         //
         Quad_SYL::value_count_limit = 0;
-        attention_raised = interrupt_raised = true;
+        set_attention_raised(LOC);
+        set_interrupt_raised(LOC);
       }
 
 const ShapeItem length = shape.get_volume();
@@ -101,7 +99,8 @@ const ShapeItem length = shape.get_volume();
              // reset the limit so that we don't get stuck here.
              //
              Quad_SYL::ravel_count_limit = 0;
-             attention_raised = interrupt_raised = true;
+             set_attention_raised(LOC);
+             set_interrupt_raised(LOC);
            }
 
         try
@@ -126,6 +125,11 @@ const ShapeItem length = shape.get_volume();
         ravel = short_value;
       }
 
+   // init the first ravel element to (prototype) 0 so that we can avoid
+   // many empty checks all over the place
+   //
+   new (ravel)   IntCell(0);
+
    check_ptr = (const char *)this + 7;
 }
 //-----------------------------------------------------------------------------
@@ -138,6 +142,18 @@ Value::Value(const char * loc)
    init_ravel();
 }
 //-----------------------------------------------------------------------------
+Value::Value(const Cell & cell, const char * loc)
+   : DynamicObject(loc, &all_values),
+     flags(VF_NONE),
+     valid_ravel_items(0)
+{
+   ADD_EVENT(this, VHE_Create, 0, loc);
+   init_ravel();
+
+   get_ravel(0).init(cell, *this, loc);
+   check_value(LOC);
+}
+//-----------------------------------------------------------------------------
 Value::Value(ShapeItem sh, const char * loc)
    : DynamicObject(loc, &all_values),
      shape(sh),
@@ -148,6 +164,16 @@ Value::Value(ShapeItem sh, const char * loc)
    init_ravel();
 }
 //-----------------------------------------------------------------------------
+Value::Value(const Shape & sh, const char * loc)
+   : DynamicObject(loc, &all_values),
+     shape(sh),
+     flags(VF_NONE),
+     valid_ravel_items(0)
+{
+   ADD_EVENT(this, VHE_Create, 0, loc);
+   init_ravel();
+}
+//-----------------------------------------------------------------------------
 Value::Value(const UCS_string & ucs, const char * loc)
    : DynamicObject(loc, &all_values),
      shape(ucs.size()),
@@ -157,15 +183,8 @@ Value::Value(const UCS_string & ucs, const char * loc)
    ADD_EVENT(this, VHE_Create, 0, loc);
    init_ravel();
 
-   if (ucs.size())
-      {
-        loop(l, ucs.size())   new (next_ravel()) CharCell(ucs[l]);
-      }
-   else   // empty string: set prototype
-      {
-        new (&get_ravel(0)) CharCell(UNI_ASCII_SPACE);
-      }
-
+   new (&get_ravel(0)) CharCell(UNI_ASCII_SPACE);   // prototype
+   loop(l, ucs.size())   new (next_ravel()) CharCell(ucs[l]);
    set_complete();
 }
 //-----------------------------------------------------------------------------
@@ -178,16 +197,8 @@ Value::Value(const UTF8_string & utf, const char * loc)
    ADD_EVENT(this, VHE_Create, 0, loc);
    init_ravel();
 
-   if (utf.size())
-      {
-        loop(l, utf.size())
-            new (next_ravel()) CharCell((Unicode)(utf[l] & 0xFF));
-      }
-   else   // empty string: set prototype
-      {
-        new (&get_ravel(0)) CharCell(UNI_ASCII_SPACE);
-      }
-
+   new (&get_ravel(0)) CharCell(UNI_ASCII_SPACE);   // prototype
+   loop(l, utf.size())   new (next_ravel()) CharCell((Unicode)(utf[l] & 0xFF));
    set_complete();
 }
 //-----------------------------------------------------------------------------
@@ -200,38 +211,46 @@ Value::Value(const CDR_string & ui8, const char * loc)
    ADD_EVENT(this, VHE_Create, 0, loc);
    init_ravel();
 
-   if (ui8.size())
-      {
-        loop(l, ui8.size())   new (&get_ravel(l)) CharCell(Unicode(ui8[l]));
-      }
-   else   // empty string: set prototype
-      {
-        new (&get_ravel(0)) CharCell(UNI_ASCII_SPACE);
-      }
+   new (&get_ravel(0)) CharCell(UNI_ASCII_SPACE);   // prototype
+   loop(l, ui8.size())   new (next_ravel()) CharCell(Unicode(ui8[l]));
+   set_complete();
+}
+//-----------------------------------------------------------------------------
+Value::Value(const PrintBuffer & pb, const char * loc)
+   : DynamicObject(loc, &all_values),
+     shape(pb.get_height(), pb.get_width(0)),
+     flags(VF_NONE),
+     valid_ravel_items(0)
+{
+   ADD_EVENT(this, VHE_Create, 0, loc);
+   init_ravel();
+
+   new (&get_ravel(0)) CharCell(UNI_ASCII_SPACE);   // prototype
+
+const ShapeItem height = pb.get_height();
+const ShapeItem width = pb.get_width(0);
+
+   loop(y, height)
+   loop(x, width)   next_ravel()->init(CharCell(pb.get_char(x, y)), *this, loc);
 
    set_complete();
 }
 //-----------------------------------------------------------------------------
-Value::Value(const Shape & sh, const char * loc)
-   : DynamicObject(loc, &all_values),
-     shape(sh),
-     flags(VF_NONE),
-     valid_ravel_items(0)
-{
-   ADD_EVENT(this, VHE_Create, 0, loc);
-   init_ravel();
-}
-//-----------------------------------------------------------------------------
-Value::Value(const Cell & cell, const char * loc)
+Value::Value(const char * loc, const Shape * sh)
    : DynamicObject(loc, &all_values),
+     shape((ShapeItem)sh->get_rank()),
      flags(VF_NONE),
      valid_ravel_items(0)
 {
    ADD_EVENT(this, VHE_Create, 0, loc);
    init_ravel();
 
-   get_ravel(0).init(cell, *this);
-   check_value(LOC);
+   get_ravel(0).init(IntCell(0), *this, loc);   // prototype
+
+   loop(r, sh->get_rank())
+       next_ravel()->init(IntCell(sh->get_shape_item(r)), *this, loc);
+
+   set_complete();
 }
 //-----------------------------------------------------------------------------
 Value::~Value()
@@ -243,9 +262,17 @@ const ShapeItem length = nz_element_count();
 
    if (get_pointer_cell_count() > 0)
       {
-
         Cell * cZ = &get_ravel(0);
-        loop(c, length)   cZ++->release(LOC);
+        if (is_complete())   // OK to release
+           {
+             loop(c, length)   cZ++->release(LOC);
+           }
+        else
+           {
+             // the last ravel item could be corrupt
+             loop(c, valid_ravel_items - 1)   cZ++->release(LOC);
+           }
+
       }
 
    --value_count;
@@ -273,7 +300,8 @@ const ShapeItem ec = element_count();
         new (ret->next_ravel())   LvalCell(&cell, this);
       }
 
-   if (ec == 0)   new (&ret->get_ravel(0))   LvalCell(0, 0);
+   // prototype
+   if (ec == 0)   new (&ret->get_ravel(0))   LvalCell(&get_ravel(0), this);
 
    ret->check_value(LOC);
    return ret;
@@ -291,6 +319,7 @@ Cell * C = &get_ravel(0);
    // cellowner: the owner of the cells that this points to
    //
 Value * cellowner = get_lval_cellowner();
+
 const int src_incr = new_value->is_scalar() ? 0 : 1;
 
    // if this is a scalar and new_value is not, then enclose new_value
@@ -315,12 +344,14 @@ const int src_incr = new_value->is_scalar() ? 0 : 1;
       {
         if (!C->is_lval_cell())   LEFT_SYNTAX_ERROR;
 
-        Cell * dest = C++->get_lval_value();   // can be 0!
-        if (dest)   dest->release(LOC);   // free sub-values etc (if any)
+        if (Cell * dest = C++->get_lval_value())   // dest can be 0!
+           {
+             dest->release(LOC);   // free sub-values etc (if any)
 
-        // erase the pointee when overriding a pointer-cell.
-        //
-        if (dest)   dest->init(*src, *this);
+             // erase the pointee when overriding a pointer-cell.
+             //
+             dest->init(*src, *cellowner, LOC);
+           }
         src += src_incr;
       }
 }
@@ -330,31 +361,37 @@ Value::get_lval_cellowner() const
 {
 const ShapeItem ec = nz_element_count();
 
+   // find the first lval cell with a non-0 owner
+   //
+const Cell * C = &get_ravel(0);
    loop(e, ec)
       {
-        const Cell & cell = get_ravel(e);
-        if (cell.is_lval_cell())   return cell.get_cell_owner();
-        if (!cell.is_pointer_cell())   return 0;
-        return  cell.get_pointer_value()->get_lval_cellowner();
+        if (C->is_lval_cell())
+           return ((const LvalCell *)C)->get_cell_owner();
 
+        if (C->is_pointer_cell())
+           return  C->get_pointer_value()->get_lval_cellowner();
+
+        ++C;
       }
 
    return 0;
 }
 //-----------------------------------------------------------------------------
 bool
-Value::is_or_contains(const Value & value) const
+Value::is_or_contains(const Value * val, const Value & sub)
 {
-   if (this == 0)        return false;   // not a valie value
+   if (val == 0)        return false;   // not a valid value
 
-   if (this == &value)   return true;
+   if (val == &sub)   return true;
 
-const Cell * C = &get_ravel(0);
-   loop(e, nz_element_count())
+const Cell * C = &val->get_ravel(0);
+   loop(e, val->nz_element_count())
       {
         if (C->is_pointer_cell())
            {
-             if (C->get_pointer_value()->is_or_contains(value))   return true;;
+             if (is_or_contains(C->get_pointer_value().get(), sub))
+                return true;
            }
         ++C;
       }
@@ -660,7 +697,7 @@ ShapeItem ec = element_count();
             }
          else
             {
-              dest++->init(cell, dest_owner);
+              dest++->init(cell, dest_owner, LOC);
             }
        }
 }
@@ -681,10 +718,8 @@ Value::is_apl_char_vector() const
 }
 //-----------------------------------------------------------------------------
 bool
-Value::is_char_vector() const
+Value::is_char_array() const
 {
-   if (get_rank() != 1)   return false;
-
 const Cell * C = &get_ravel(0);
    loop(c, nz_element_count())   // also check prototype
       if (!C++->is_character_cell())   return false;   // not char
@@ -692,14 +727,6 @@ const Cell * C = &get_ravel(0);
 }
 //-----------------------------------------------------------------------------
 bool
-Value::is_char_scalar() const
-{
-   if (get_rank() != 0)   return false;
-
-   return get_ravel(0).is_character_cell();
-}
-//-----------------------------------------------------------------------------
-bool
 Value::NOTCHAR() const
 {
    // always test element 0.
@@ -743,7 +770,7 @@ int error_count = 0;
 }
 //-----------------------------------------------------------------------------
 bool
-Value::is_int_vector(APL_Float qct) const
+Value::is_int_vector() const
 {
    if (get_rank() != 1)   return false;
 
@@ -757,15 +784,7 @@ Value::is_int_vector(APL_Float qct) const
 }
 //-----------------------------------------------------------------------------
 bool
-Value::is_int_scalar(APL_Float qct) const
-{
-   if (get_rank() != 0)   return false;
-
-   return get_ravel(0).is_near_int();
-}
-//-----------------------------------------------------------------------------
-bool
-Value::is_complex(APL_Float qct) const
+Value::is_complex() const
 {
 const ShapeItem ec = nz_element_count();
 
@@ -964,7 +983,7 @@ const ShapeItem ec_z = Z->element_count();
 
    if (ec_z == 0)   // empty result
       {
-        Z->set_default(*this);
+        Z->set_default(*this, LOC);
         Z->check_value(LOC);
         return Z;
       }
@@ -973,10 +992,10 @@ const ShapeItem ec_z = Z->element_count();
    // We go from lower indices to higher indices in IX, which
    // means from higher indices to lower indices in this and Z
    
-   loop(z, ec_z)   Z->get_ravel(z).init(get_ravel(mult.next()), Z.getref());
+   loop(z, ec_z)
+       Z->next_ravel()->init(get_ravel(mult.next()), Z.getref(), LOC);
 
    Assert(mult.done());
-
    Z->check_value(LOC);
    return Z;
 }
@@ -984,14 +1003,29 @@ const ShapeItem ec_z = Z->element_count();
 Value_P
 Value::index(Value_P X) const
 {
+const ShapeItem max_idx = element_count();
+const APL_Integer qio = Workspace::get_IO();
+
+   // important special case: scalar X
+   //
+   if (get_rank() == 1 && (!!X) && X->is_scalar())
+      {
+        const APL_Integer idx = X->get_ravel(0).get_near_int() - qio;
+        if (idx >= 0 && idx < max_idx)
+           {
+             Value_P Z(LOC);
+             Z->next_ravel()->init(get_ravel(idx), Z.getref(), LOC);
+             Z->check_value(LOC);
+             return Z;
+           }
+      }
+
    if (get_rank() != 1)   RANK_ERROR;
 
-   if (!X)   return clone(LOC);
+   if (!X)   return clone(LOC);   // elided index
 
 const Shape shape_Z(X->get_shape());
 Value_P Z(shape_Z, LOC);
-const ShapeItem max_idx = element_count();
-const APL_Integer qio = Workspace::get_IO();
 
 const Cell * cI = &X->get_ravel(0);
 
@@ -1000,61 +1034,55 @@ const Cell * cI = &X->get_ravel(0);
          const ShapeItem idx = cI++->get_near_int() - qio;
          if (idx < 0 || idx >= max_idx)
             {
+              UCS_string & t4 = Workspace::more_error();
+              t4.clear();
+              t4.append_utf8("⎕IO=");
+              t4.append_number(qio);
+              t4.append_utf8(" offending index=");
+              t4.append_number(idx);
+              t4.append_utf8(" max index=");
+              t4.append_number(max_idx);
               Z->rollback(Z->valid_ravel_items, LOC);
               INDEX_ERROR;
             }
 
-         Z->next_ravel()->init(get_ravel(idx), Z.getref());
+         Z->next_ravel()->init(get_ravel(idx), Z.getref(), LOC);
       }
 
-   Z->set_default(*this);
-
+   Z->set_default(*this, LOC);
    Z->check_value(LOC);
    return Z;
 }
 //-----------------------------------------------------------------------------
-/**
-Value_P
-Value::index(Token & IX) const
-{
-   if (IX.get_tag() == TOK_AXES)     return index(IX.get_apl_val());
-   if (IX.get_tag() == TOK_INDEX)    return index(IX.get_index_val());
-
-   // not supposed to happen
-   //
-   Q1(IX.get_tag());  FIXME;
-}
-**/
-//-----------------------------------------------------------------------------
 Rank
-Value::get_single_axis(Rank max_axis) const
+Value::get_single_axis(const Value * val, Rank max_axis)
 {
-   if (this == 0)   AXIS_ERROR;
+   if (val == 0)   AXIS_ERROR;
 
-   if (!is_scalar_or_len1_vector())     AXIS_ERROR;
+   if (!val->is_scalar_or_len1_vector())     AXIS_ERROR;
 
-   if (!get_ravel(0).is_near_int())   AXIS_ERROR;
+   if (!val->get_ravel(0).is_near_int())   AXIS_ERROR;
 
    // if axis becomes (signed) negative then it will be (unsigned) too big.
    // Therefore we need not test for < 0.
    //
-const unsigned int axis = get_ravel(0).get_near_int() - Workspace::get_IO();
+const int axis = val->get_ravel(0).get_near_int() - Workspace::get_IO();
    if (axis >= max_axis)   AXIS_ERROR;
 
    return axis;
 }
 //-----------------------------------------------------------------------------
 Shape
-Value::to_shape() const
+Value::to_shape(const Value * val)
 {
-   if (this == 0)   INDEX_ERROR;   // elided index ?
+   if (val == 0)   INDEX_ERROR;   // elided index ?
 
-const ShapeItem xlen = element_count();
+const ShapeItem xlen = val->element_count();
 const APL_Integer qio = Workspace::get_IO();
 
 Shape shape;
      loop(x, xlen)
-        shape.add_shape_item(get_ravel(x).get_near_int() - qio);
+        shape.add_shape_item(val->get_ravel(x).get_near_int() - qio);
 
    return shape;
 }
@@ -1102,8 +1130,8 @@ const ShapeItem len_B = B->element_count();
 
 Value_P Z(len_A + len_B, LOC);
 
-   loop(a, len_A)   Z->next_ravel()->init(A->get_ravel(a), Z.getref());
-   loop(b, len_B)   Z->next_ravel()->init(B->get_ravel(b), Z.getref());
+   loop(a, len_A)   Z->next_ravel()->init(A->get_ravel(a), Z.getref(), LOC);
+   loop(b, len_B)   Z->next_ravel()->init(B->get_ravel(b), Z.getref(), LOC);
 
    Z->check_value(LOC);
    new (&result) Token(TOK_APL_VALUE3, Z);
@@ -1126,11 +1154,11 @@ Value::glue_strand_closed(Token & result, Value_P A, Value_P B,
 const ShapeItem len_A = A->element_count();
 Value_P Z(len_A + 1, LOC);
 
-   loop(a, len_A)   Z->next_ravel()->init(A->get_ravel(a), Z.getref());
+   loop(a, len_A)   Z->next_ravel()->init(A->get_ravel(a), Z.getref(), LOC);
 
    if (B->is_simple_scalar())
       {
-        Z->next_ravel()->init(B->get_ravel(0), Z.getref());
+        Z->next_ravel()->init(B->get_ravel(0), Z.getref(), LOC);
       }
    else
       {
@@ -1160,14 +1188,14 @@ Value_P Z(len_B + 1, LOC);
 
    if (A->is_simple_scalar())
       {
-        Z->next_ravel()->init(A->get_ravel(0), Z.getref());
+        Z->next_ravel()->init(A->get_ravel(0), Z.getref(), LOC);
       }
    else
       {
         new (Z->next_ravel()) PointerCell(A, Z.getref());
       }
 
-   loop(b, len_B)   Z->next_ravel()->init(B->get_ravel(b), Z.getref());
+   loop(b, len_B)   Z->next_ravel()->init(B->get_ravel(b), Z.getref(), LOC);
 
    Z->check_value(LOC);
    new (&result) Token(TOK_APL_VALUE3, Z);
@@ -1188,7 +1216,7 @@ Value::glue_closed_closed(Token & result, Value_P A, Value_P B,
 Value_P Z(2, LOC);
    if (A->is_simple_scalar())
       {
-        Z->next_ravel()->init(A->get_ravel(0), Z.getref());
+        Z->next_ravel()->init(A->get_ravel(0), Z.getref(), LOC);
       }
    else
       {
@@ -1197,7 +1225,7 @@ Value_P Z(2, LOC);
 
    if (B->is_simple_scalar())
       {
-        Z->next_ravel()->init(B->get_ravel(0), Z.getref());
+        Z->next_ravel()->init(B->get_ravel(0), Z.getref(), LOC);
       }
    else
       {
@@ -1225,7 +1253,8 @@ Value::check_value(const char * loc)
 uint32_t error_count = 0;
 const Cell * C = &get_ravel(0);
 
-    loop(c, nz_element_count())
+const ShapeItem ec = nz_element_count();
+    loop(c, ec)
        {
          const CellType ctype = C->get_cell_type();
          switch(ctype)
@@ -1283,7 +1312,7 @@ int size = 16 + 4*get_rank() + 4*ec;   // top_level size
    loop(e, ec)
       {
         const Cell & cell = get_ravel(e);
-        if (cell.is_character_cell() || cell.is_numeric())
+        if (cell.is_simple_cell())
            {
              // a non-pointer sub value consisting of its own 16 byte header,
              // and 1-16 data bytes, padded up to 16 bytes,
@@ -1329,7 +1358,7 @@ int size = 0;
    loop(e, ec)
       {
         const Cell & cell = get_ravel(e);
-        if (cell.is_character_cell() || cell.is_numeric())
+        if (cell.is_simple_cell())
            {
              size += cell.CDR_size();
            }
@@ -1412,7 +1441,7 @@ const Cell & cell_0 = get_ravel(0);
 ostream &
 Value::print(ostream & out) const
 {
-PrintContext pctx = Workspace::get_PrintContext();
+PrintContext pctx = Workspace::get_PrintContext(PR_APL);
    if (get_rank() == 0)   // scalar
       {
         pctx.set_style(PR_APL_MIN);
@@ -1420,7 +1449,7 @@ PrintContext pctx = Workspace::get_PrintContext();
    else if (get_rank() == 1)   // vector
       {
         if (element_count() == 0 &&   // empty vector
-            (get_ravel(0).is_character_cell() || get_ravel(0).is_numeric()))
+            (get_ravel(0).is_simple_cell()))
            {
              return out << endl;
            }
@@ -1476,7 +1505,7 @@ UCS_string ind(indent, UNI_ASCII_SPACE);
 void
 Value::debug(const char * info) const
 {
-const PrintContext pctx = Workspace::get_PrintContext();
+const PrintContext pctx = Workspace::get_PrintContext(PR_APL);
 PrintBuffer pb(*this, pctx, 0);
    pb.debug(CERR, info);
 }
@@ -1488,7 +1517,7 @@ Value::print_boxed(ostream & out, const char * info) const
 
 const PrintContext pctx(PST_NONE);
 
-Value_P Z = Quad_CR::do_CR(4, *this, pctx);
+Value_P Z = Quad_CR::do_CR(4, this, pctx);
    out << *Z << endl;
    return out;
 }
@@ -1522,41 +1551,68 @@ const ShapeItem name_len = get_cols();
              if (n == 0 && Avec::is_quad(uni))   // leading ⎕
                 {
                   name.append(uni);
+                  continue;
                 }
-             else if (Avec::is_symbol_char(uni))   // valid symbol char
+
+             if (Avec::is_symbol_char(uni))   // valid symbol char
                 {
                   name.append(uni);
+                  continue;
                 }
-             else if (uni == UNI_ASCII_SPACE)
-                {
-                  // skip space at nidx and subsequent spaces
-                  //
-                  while (nidx < end && get_ravel(nidx).get_char_value()
-                                  == UNI_ASCII_SPACE)   ++nidx;
 
-                  if (nidx == end)   break;   // only spaces (no second name)
-
-                  name.clear();
-                  if (last)
-                     {
-                       // if 'last' is true then to_varnames() was called from
-                       // ⎕SVO and the line may contains two variable names.
-                       // Return the second i.e. last) one)
-                       //
-
-
-                       // another name following
-                       //
-                       name.clear();   // discard first name
-                       name.append(get_ravel(nidx++).get_char_value());
-                       last = false;
-                     }
-                }
-             else  // neither space nor valid variable name character
+             // end of (first) name reached. At this point we expect either
+             // spaces until 'end' or some spaces and another name.
+             //
+             if (uni != UNI_ASCII_SPACE)
                 {
                   name.clear();
                   break;
                 }
+
+             // we have reached the end of the first name. At this point
+             // there could bei:
+             //
+             // 1. spaces until 'end' (= one name), or
+             // 2. a second name (alias)
+
+             // skip spaces from nidx and subsequent spaces
+             //
+             while (nidx < end &&
+                    get_ravel(nidx).get_char_value() == UNI_ASCII_SPACE)
+                   ++nidx;
+
+             if (nidx == end)   break;   // only spaces (no second name)
+
+             // at this point we maybe have the start of a second name, which
+             // is an error (if last is false) or not. In both cases the first
+             // name can be ignored.
+             //
+             name.clear();
+             if (!last)   break;   // error
+
+             // 'last' is true thus to_varnames() was called from ⎕SVO and
+             // the line may contains two variable names.
+             // Return the second i.e. last) one)
+             //
+             last = false;
+             while (nidx < end)
+                {
+                  const Unicode uni = get_ravel(nidx++).get_char_value();
+                  if (Avec::is_symbol_char(uni))   // valid symbol char
+                     {
+                       name.append(uni);
+                     }
+                  else if (uni == UNI_ASCII_SPACE)
+                     {
+                       break;
+                     }
+                  else
+                     {
+                       name.clear();   // error
+                       break;
+                     }
+                }
+             break;
            }
       }
 }
@@ -1613,16 +1669,24 @@ const Cell & first = get_ravel(0);
       {
         Value_P B0 = first.get_pointer_value();
         Value_P Z(B0->get_shape(), loc);
-        const ShapeItem ec_Z =  Z->nz_element_count();
+        const ShapeItem ec_Z =  Z->element_count();
 
-        loop(z, ec_Z)   Z->get_ravel(z).init_type(B0->get_ravel(z), Z.getref());
+        if (ec_Z == 0)
+           {
+             Z->get_ravel(0).init_type(B0->get_ravel(0), Z.getref(), LOC);
+           }
+
+        loop(z, ec_Z)
+            Z->next_ravel()->init_type(B0->get_ravel(z), Z.getref(), LOC);
+        Z->set_complete();
         return Z;
       }
    else
       {
         Value_P Z(loc);
 
-        Z->get_ravel(0).init_type(first, Z.getref());
+        Z->next_ravel()->init_type(first, Z.getref(), LOC);
+        Z->set_complete();
         return Z;
       }
 
@@ -1643,7 +1707,7 @@ const Cell * src = &get_ravel(0);
 Cell * dst = &ret->get_ravel(0);
 const ShapeItem count = nz_element_count();
 
-   loop(c, count)   dst++->init(*src++, ret.getref());
+   loop(c, count)   dst++->init(*src++, ret.getref(), LOC);
 
    ret->check_value(LOC);
 
@@ -1786,6 +1850,7 @@ int count = 0;
    //
    mark_all_dynamic_values();
    Workspace::unmark_all_values();
+   Macro::unmark_all_macros();
 
    // print all values that are still marked
    //
@@ -1830,7 +1895,7 @@ Value::print_stale_info(ostream & out, const DynamicObject * dob)
       {
         print_structure(out, 0, 0);
         const PrintContext pctx(PST_NONE);
-        Value_P Z = Quad_CR::do_CR(7, *this, pctx);
+        Value_P Z = Quad_CR::do_CR(7, this, pctx);
         Z->print(out);
         out << endl;
       }
