@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-2016  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
 #include <errno.h>
 #include <string.h>
+#include <sys/stat.h>
+
 #include <fstream>
 
 using namespace std;
@@ -32,6 +34,7 @@ using namespace std;
 #include "Output.hh"
 #include "Quad_FX.hh"
 #include "Quad_TF.hh"
+#include "SystemVariable.hh"
 #include "UserFunction.hh"
 #include "UserPreferences.hh"
 #include "Workspace.hh"
@@ -49,6 +52,25 @@ DynamicObject DynamicObject::all_index_exprs(LOC);
 //
 Workspace Workspace::the_workspace;
 
+//-----------------------------------------------------------------------------
+Workspace::Workspace()
+   : WS_name("CLEAR WS"),
+//   prompt("-----> "),
+     prompt("      "),
+     top_SI(0)
+{
+#define ro_sv_def(x, str, _txt)                                   \
+   if (*str) { UCS_string q(UNI_Quad_Quad);   q.append_utf8(str); \
+   distinguished_names.add_variable(q, ID::x, &v_ ## x); }
+
+#define rw_sv_def ro_sv_def
+
+#define sf_def(x, str, _txt)                                      \
+   if (*str) { UCS_string q(UNI_Quad_Quad);   q.append_utf8(str); \
+   distinguished_names.add_function(q, ID::x, x::fun); }
+
+#include "SystemVariable.def"
+}
 //-----------------------------------------------------------------------------
 void
 Workspace::push_SI(Executable * fun, const char * loc)
@@ -64,7 +86,8 @@ Workspace::push_SI(Executable * fun, const char * loc)
         "(and to avoid lock-up, the limit in ⎕SYL was automatically cleared).");
 
         Quad_SYL::si_depth_limit = 0;
-        attention_raised = interrupt_raised = true;
+        set_attention_raised(LOC);
+        set_interrupt_raised(LOC);
       }
 
    the_workspace.top_SI = new StateIndicator(fun, SI_top());
@@ -112,7 +135,7 @@ Workspace::get_RL(uint64_t mod)
    // we discard random numbers >= max_rand in order to avoid a bias
    // towards small numbers
    //
-const uint64_t max_rand = 0xFFFFFFFFFFFFFFFFULL - (0xFFFFFFFFFFFFFFFF % mod);
+const uint64_t max_rand = 0xFFFFFFFFFFFFFFFFULL - (0xFFFFFFFFFFFFFFFFULL % mod);
 uint64_t rand = the_workspace.v_Quad_RL.get_random();
 
    do { rand ^= rand >> 11;
@@ -131,7 +154,7 @@ Workspace::clear_error(const char * loc)
    for (StateIndicator * si = the_workspace.SI_top(); si; si = si->get_parent())
        {
          si->get_error().init(E_NO_ERROR, LOC);
-         if (si->get_executable()->get_parse_mode() == PM_FUNCTION)   break;
+         if (si->get_parse_mode() == PM_FUNCTION)   break;
        }
 }
 //-----------------------------------------------------------------------------
@@ -140,7 +163,7 @@ Workspace::SI_top_fun()
 {
    for (StateIndicator * si = SI_top(); si; si = si->get_parent())
        {
-         if (si->get_executable()->get_parse_mode() == PM_FUNCTION)   return si;
+         if (si->get_parse_mode() == PM_FUNCTION)   return si;
        }
 
    return 0;   // no context wirh parse mode PM_FUNCTION
@@ -215,7 +238,17 @@ Workspace::lookup_existing_name(const UCS_string & name)
 
    // user defined variable or function
    //
-   return the_workspace.symbol_table.lookup_existing_name(name);
+Symbol * sym = the_workspace.symbol_table.lookup_existing_symbol(name);
+   if (sym == 0)   return 0;
+
+   switch(sym->get_nc())
+      {
+        case NC_VARIABLE: return sym;
+
+        case NC_FUNCTION:
+        case NC_OPERATOR: return sym->get_function();
+        default:          return 0;
+      }
 }
 //-----------------------------------------------------------------------------
 Symbol *
@@ -237,7 +270,7 @@ Workspace::lookup_existing_symbol(const UCS_string & symbol_name)
 }
 //-----------------------------------------------------------------------------
 Token
-Workspace::get_quad(UCS_string ucs, int & len)
+Workspace::get_quad(const UCS_string & ucs, int & len)
 {
    if (ucs.size() == 0 || !Avec::is_quad(ucs[0]))
      {
@@ -245,172 +278,36 @@ Workspace::get_quad(UCS_string ucs, int & len)
        return Token();
      }
 
-Unicode av_0 = (ucs.size() > 1) ? ucs[1] : Invalid_Unicode;
-Unicode av_1 = (ucs.size() > 2) ? ucs[2] : Invalid_Unicode;
-Unicode av_2 = (ucs.size() > 3) ? ucs[3] : Invalid_Unicode;
-Unicode av_3 = (ucs.size() > 4) ? ucs[4] : Invalid_Unicode;
-Unicode av_4 = (ucs.size() > 5) ? ucs[5] : Invalid_Unicode;
+UCS_string name(UNI_Quad_Quad);
+   len = 1;
 
-   // lowercase to uppercase
-   //
-   if (av_0 >= 'a' && av_0 <= 'z')   av_0 = Unicode(av_0 - 0x20);
-   if (av_1 >= 'a' && av_1 <= 'z')   av_1 = Unicode(av_1 - 0x20);
-   if (av_2 >= 'a' && av_2 <= 'z')   av_2 = Unicode(av_2 - 0x20);
-   if (av_3 >= 'a' && av_3 <= 'z')   av_3 = Unicode(av_3 - 0x20);
-   if (av_4 >= 'a' && av_4 <= 'z')   av_4 = Unicode(av_4 - 0x20);
+SystemName * longest = 0;
 
-#define var(t, l) { len = l + 1; \
-   return Token(TOK_Quad_ ## t, &the_workspace.v_Quad_ ## t); }
-
-#define f0(t, l) { len = l + 1; \
-   return Token(TOK_Quad_ ## t, Quad_ ## t::fun); }
-
-#define f1(t, l) { len = l + 1; \
-   return Token(TOK_Quad_ ## t, Quad_ ## t::fun); }
-
-#define f2(t, l) { len = l + 1; \
-   return Token(TOK_Quad_ ## t, Quad_ ## t::fun); }
-
-   switch(av_0)
+   for (ShapeItem u = 1; u < ucs.size(); ++u)
       {
-        case UNI_ASCII_A:
-             if (av_1 == UNI_ASCII_F)        f1 (AF, 2)
-             if (av_1 == UNI_ASCII_I)        var(AI, 2)
-             if (av_1 == UNI_ASCII_R)
-                {
-                  if (av_2 == UNI_ASCII_G)   var(ARG, 3)
-                }
-             if (av_1 == UNI_ASCII_T)        f2 (AT, 2)
-             if (av_1 == UNI_ASCII_V)        var(AV, 2)
-             break;
+        Unicode uni = ucs[u];
+        if (!Avec::is_symbol_char(uni))   break;
 
-        case UNI_ASCII_C:
-             if (av_1 == UNI_ASCII_R)        f1 (CR, 2)
-             if (av_1 == UNI_ASCII_T)        var(CT, 2)
-             break;
+        if (uni >= 'a' && uni <= 'z')   uni = Unicode(uni - 0x20);
+        name.append(uni);
+        SystemName * dn =
+               the_workspace.distinguished_names.lookup_existing_symbol(name);
 
-        case UNI_ASCII_D:
-             if (av_1 == UNI_ASCII_L)        f1 (DL, 2)
-             break;
-
-        case UNI_ASCII_E:
-             if (av_1 == UNI_ASCII_A)        f2 (EA, 2)
-             if (av_1 == UNI_ASCII_C)        f1 (EC, 2)
-             if (av_1 == UNI_ASCII_M)        var(EM, 2)
-             if (av_1 == UNI_ASCII_N)
-                {
-                  if (av_2 == UNI_ASCII_V)   f1(ENV, 3)
-                }
-             if (av_1 == UNI_ASCII_S)        f2 (ES, 2)
-             if (av_1 == UNI_ASCII_T)        var(ET, 2)
-             if (av_1 == UNI_ASCII_X)        f1 (EX, 2)
-             break;
-
-        case UNI_ASCII_F:
-             if (av_1 == UNI_ASCII_C)        var(FC, 2)
-             if (av_1 == UNI_ASCII_X)        f2 (FX, 2)
-             break;
-
-        case UNI_ASCII_I:
-             if (av_1 == UNI_ASCII_N)
-                {
-                  if (av_2 == UNI_ASCII_P)   f2(INP, 3)
-                  break;
-                }
-
-             if (av_1 == UNI_ASCII_O)        var(IO, 2)
-             break;
-
-        case UNI_ASCII_L:
-             if (av_1 == UNI_ASCII_C)        var(LC, 2)
-             if (av_1 == UNI_ASCII_X)        var(LX, 2)
-                                             var(L, 1)
-             break;
-
-        case UNI_ASCII_N:
-             if (av_1 == UNI_ASCII_A)        f2 (NA, 2);
-             if (av_1 == UNI_ASCII_C)        f2 (NC, 2);
-             if (av_1 == UNI_ASCII_L)        f1 (NL, 2)
-             break;
-
-        case UNI_ASCII_P:
-             if (av_1 == UNI_ASCII_P)        var(PP, 2)
-             if (av_1 == UNI_ASCII_R)        var(PR, 2)
-             if (av_1 == UNI_ASCII_S)        var(PS, 2)
-             if (av_1 == UNI_ASCII_W)        var(PW, 2)
-             break;
-
-        case UNI_ASCII_R:
-             if (av_1 == UNI_ASCII_L)        var(RL, 2)
-                                             var(R, 1)
-             break;
-
-        case UNI_ASCII_S:
-             if (av_1 == UNI_ASCII_I)      f1 (SI, 2)
-             if (av_1 == UNI_ASCII_T)
-                {
-                  if (av_2 == UNI_ASCII_O &&
-                      av_3 == UNI_ASCII_P)   f2 (STOP, 4)
-                  break;
-                }
-             if (av_1 == UNI_ASCII_V)
-                {
-                  if (av_2 == UNI_ASCII_C)   f2 (SVC, 3)
-                  if (av_2 == UNI_ASCII_E)   var(SVE, 3)
-                  if (av_2 == UNI_ASCII_O)   f2 (SVO, 3)
-                  if (av_2 == UNI_ASCII_Q)   f2 (SVQ, 3)
-                  if (av_2 == UNI_ASCII_R)   f1 (SVR, 3)
-                  if (av_2 == UNI_ASCII_S)   f1 (SVS, 3)
-                  break;
-                }
-             if (av_1 == UNI_ASCII_Y)
-                {
-                  if (av_2 == UNI_ASCII_L)   var(SYL, 3)
-                  break;
-                }
-             break;
-
-        case UNI_ASCII_T:
-             if (av_1 == UNI_ASCII_C)        var(TC, 2)
-             if (av_1 == UNI_ASCII_F)        f2 (TF, 2)
-             if (av_1 == UNI_ASCII_R)
-                {
-                  if (av_2 == UNI_ASCII_A &&
-                      av_3 == UNI_ASCII_C &&
-                      av_4 == UNI_ASCII_E)   f2 (TRACE, 5)
-                  break;
-                }
-             if (av_1 == UNI_ASCII_S)        var(TS, 2)
-             if (av_1 == UNI_ASCII_Z)        var(TZ, 2)
-             break;
-
-        case UNI_ASCII_U:
-             if (av_1 == UNI_ASCII_C &&
-                 av_2 == UNI_ASCII_S)        f1 (UCS, 3)
-             if (av_1 == UNI_ASCII_L)        var(UL, 2)
-             break;
-
-        case UNI_ASCII_W:
-             if (av_1 == UNI_ASCII_A)        var(WA, 2)
-             break;
-
-        case UNI_ASCII_X:
-                                             var(X, 1)
-             break;
-
-        default: break;
+        if (dn)   // ss is a valid distinguished name
+           {
+             longest = dn;
+             len = name.size();
+           }
       }
 
-   var(Quad, 0);
+   if (longest == 0)   return Token(TOK_Quad_Quad, &the_workspace.v_Quad_Quad);
 
-#undef var
-#undef f0
-#undef f1
-#undef f2
+   if (longest->get_variable())   return longest->get_variable()->get_token();
+   else                           return longest->get_function()->get_token();
 }
 //-----------------------------------------------------------------------------
 StateIndicator *
-Workspace:: oldest_exec(const Executable * exec)
+Workspace::oldest_exec(const Executable * exec)
 {
 StateIndicator * ret = 0;
 
@@ -511,8 +408,8 @@ Workspace::unmark_all_values()
 
    // unmark system variables
    //
-#define rw_sv_def(x, txt) the_workspace.v_ ## x.unmark_all_values();
-#define ro_sv_def(x, txt) the_workspace.v_ ## x.unmark_all_values();
+#define rw_sv_def(x, _str, _txt) the_workspace.v_ ## x.unmark_all_values();
+#define ro_sv_def(x, _str, _txt) the_workspace.v_ ## x.unmark_all_values();
 #include "SystemVariable.def"
    the_workspace.v_Quad_Quad .unmark_all_values();
    the_workspace.v_Quad_QUOTE.unmark_all_values();
@@ -539,8 +436,8 @@ int count = 0;
 
    // system variabes
    //
-#define rw_sv_def(x, txt) count += get_v_ ## x().show_owners(out, value);
-#define ro_sv_def(x, txt) count += get_v_ ## x().show_owners(out, value);
+#define rw_sv_def(x, _str, _txt) count += get_v_ ## x().show_owners(out, value);
+#define ro_sv_def(x, _str, _txt) count += get_v_ ## x().show_owners(out, value);
 #include "SystemVariable.def"
    count += the_workspace.v_Quad_Quad .show_owners(out, value);
    count += the_workspace.v_Quad_QUOTE.show_owners(out, value);
@@ -602,8 +499,8 @@ Workspace::clear_WS(ostream & out, bool silent)
 
    // clear the value stacks of read/write system variables...
    //
-#define rw_sv_def(x, txt) get_v_ ## x().destruct();
-#define ro_sv_def(x, txt)
+#define rw_sv_def(x, _str, _txt) get_v_ ## x().destruct();
+#define ro_sv_def(x, _str, _txt)
 #include "SystemVariable.def"
 
    // at this point all values should have been gone.
@@ -611,12 +508,15 @@ Workspace::clear_WS(ostream & out, bool silent)
    //
 // Value::erase_all(out);
 
-#define rw_sv_def(x, txt) new  (&get_v_ ##x()) x;
-#define ro_sv_def(x, txt)
+#define rw_sv_def(x, _str, _txt) new  (&get_v_ ##x()) x;
+#define ro_sv_def(x, _str, _txt)
 #include "SystemVariable.def"
 
    get_v_Quad_RL().reset_seed();
 
+   // close open files in ⎕FIO
+   Quad_FIO::fun->clear();
+
    set_WS_name(UCS_string("CLEAR WS"));
    if (!silent)   out << "CLEAR WS" << endl;
 }
@@ -713,7 +613,7 @@ ofstream outf(filename.c_str(), ofstream::out);
    if (!outf.is_open())   // open failed
       {
         CERR << "Unable to )SAVE workspace '" << wname
-             << "'." << strerror(errno) << endl;
+             << "'. " << strerror(errno) << endl;
         return;
       }
 
@@ -728,17 +628,17 @@ XML_Saving_Archive ar(outf);
      const YMDhmsu time(now() + 1000000*offset);
      const char * tz_sign = (offset < 0) ? "" : "+";
 
-     out << setfill('0') << time.year  << "-"
-         << setw(2)      << time.month << "-"
-         << setw(2)      << time.day   << "  " 
-         << setw(2)      << time.hour  << ":"
-         << setw(2)      << time.minute << ":"
-         << setw(2)      << time.second << " (GMT"
-         << tz_sign      << offset/3600 << ")"
-         << setfill(' ');
+     ostringstream os;
+     os << setfill('0') << time.year  << "-"
+        << setw(2)      << time.month << "-"
+        << setw(2)      << time.day   << "  " 
+        << setw(2)      << time.hour  << ":"
+        << setw(2)      << time.minute << ":"
+        << setw(2)      << time.second << " (GMT"
+        << tz_sign      << offset/3600 << ")";
 
-     if (name_from_WSID)   out << " " << the_workspace.WS_name;
-     out << endl;
+     if (name_from_WSID)   os << " " << the_workspace.WS_name;
+     out << os.str() << endl;
    }
 }
 //-----------------------------------------------------------------------------
@@ -794,9 +694,30 @@ const int err = rename(filename, backup_filename.c_str());
 //-----------------------------------------------------------------------------
 void
 Workspace::load_DUMP(ostream & out, const UTF8_string & filename, int fd,
-                     bool with_LX)
+                     LX_mode with_LX, bool silent,
+                     vector<UCS_string> * object_filter)
 {
-   out << "loading )DUMP file " << filename << "..." << endl;
+   Log(LOG_command_IN)
+      CERR << "loading )DUMP file " << filename << "..." << endl;
+
+   {
+     struct stat st;
+     fstat(fd, &st);
+     const int offset = Quad_TZ::compute_offset();  // call BEFORE localtime() !
+     tm * t = localtime(&st.st_mtime);
+     const char * tz_sign = offset < 0 ? "" : "+";
+
+      out << "DUMPED "
+          << setfill('0') << (1900 + t->tm_year) << "-"
+          << setw(2)      << (t->tm_mon + 1)     << "-"
+          << setw(2)      << t->tm_mday          << " "
+          << setw(2)      << t->tm_hour          << ":"
+          << setw(2)      << t->tm_min           << ":"
+          << setw(2)      << t->tm_sec           << " (GMT"
+          << tz_sign      << offset/3600 << ")"
+          << setfill(' ') <<  endl;
+   }
+
 FILE * file = fdopen(fd, "r");
 
    // make sure that filename is not already open (which would indicate
@@ -812,11 +733,56 @@ FILE * file = fdopen(fd, "r");
       }
 
 InputFile fam(filename, file, false, false, true, with_LX);
+   if (object_filter)
+      {
+        loop(o, object_filter->size())
+            fam.add_filter_object((*object_filter)[o]);
+      }
    InputFile::files_todo.insert(InputFile::files_todo.begin(), fam);
 }
 //-----------------------------------------------------------------------------
+/// a streambuf that escapes certain HTML characters
+class HTML_streambuf : public streambuf
+{
+public:
+   /// constructor
+   HTML_streambuf(ofstream & outf)
+   : out_file(outf)
+   {}
+
+   /// overloaded streambuf::overflow()
+   virtual int overflow(int c)
+      {
+        switch(c & 0xFF)
+           {
+              case '#':  out_file << "&#35";    break;
+              case '%':  out_file << "&#37;";   break;
+              case '&':  out_file << "&#38;";   break;
+              case '<':  out_file << "&lt;";    break;
+              case '>':  out_file << "&gt;";    break;
+              default:   out_file << (char)(c & 0xFF);
+           }
+        return 0;
+      }
+
+protected:
+   /// the output file
+   ofstream & out_file;
+};
+
+/// a stream that escapes certain HTML characters
+class HTML_stream : public ostream
+{
+public:
+   /// constructor
+   HTML_stream(HTML_streambuf * html_out)
+   : ostream(html_out)
+   {}
+};
+//-----------------------------------------------------------------------------
 void
-Workspace::dump_WS(ostream & out, vector<UCS_string> & lib_ws)
+Workspace::dump_WS(ostream & out, vector<UCS_string> & lib_ws, bool html,
+                   bool silent)
 {
    // )DUMP
    // )DUMP wsname
@@ -836,11 +802,11 @@ Workspace::dump_WS(ostream & out, vector<UCS_string> & lib_ws)
    // at this point, lib_ws.size() is 1 or 2.
 
 LibRef libref = LIB_NONE;
-UCS_string wname = lib_ws.back();
+const UCS_string wname = lib_ws.back();
    if (lib_ws.size() == 2)   libref = (LibRef)(lib_ws.front().atoi());
+const char * extension = html ? ".html" : ".apl";
 UTF8_string filename = LibPaths::get_lib_filename(libref, wname, false,
-                                                  ".apl", 0);
-
+                                                  extension, 0);
    if (wname.compare(UCS_string("CLEAR WS")) == 0)   // don't save CLEAR WS
       {
         COUT << "NOT DUMPED: THIS WS IS " << wname << endl;
@@ -858,25 +824,73 @@ UTF8_string filename = LibPaths::get_lib_filename(libref, wname, false,
       }
 
 ofstream outf(filename.c_str(), ofstream::out);
+
    if (!outf.is_open())   // open failed
       {
         CERR << "Unable to )DUMP workspace '" << wname
-             << "'." << strerror(errno) << endl;
+             << "': " << strerror(errno) << endl;
         return;
       }
 
+HTML_streambuf hout_buf(outf);
+HTML_stream hout(&hout_buf);
+ostream * sout = &outf;
+   if (html)   sout = &hout;
    // print header line, workspace name, time, and date to outf
    //
+const APL_time_us offset = get_v_Quad_TZ().get_offset();
+const YMDhmsu time(now() + 1000000*offset);
+const char * tz_sign = (offset < 0) ? "" : "+";
    {
-     const APL_time_us offset = get_v_Quad_TZ().get_offset();
-     const YMDhmsu time(now() + 1000000*offset);
-     const char * tz_sign = (offset < 0) ? "" : "+";
 
-     outf << "#!" << LibPaths::get_APL_bin_path()
-          << "/" << LibPaths::get_APL_bin_name()
-          << " --script" << endl
-          << endl
-          << " ⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝" << endl
+     if (html)
+        {
+          outf <<
+"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\""               << endl <<
+"                      \"http://www.w3.org/TR/html4/strict.dtd\">"  << endl <<
+"<html>"                                                            << endl <<
+"  <head>"                                                          << endl <<
+"    <title>" << wname << ".apl</title> "                           << endl <<
+"    <meta http-equiv=\"content-type\" "                            << endl <<
+"          content=\"text/html; charset=UTF-8\">"                   << endl <<
+"    <meta name=\"author\" content=\"??????\">"                     << endl <<
+"    <meta name=\"copyright\" content=\"&copy; " << time.year
+                                                 <<" by ??????\">"  << endl <<
+"    <meta name=\"date\" content=\"" << time.year << "-"
+                                     << time.month << "-"
+                                     << time.day << "\">"           << endl <<
+"    <meta name=\"description\""                                    << endl <<
+"          content=\"??????\">"                                     << endl <<
+"    <meta name=\"keywords\" lang=\"en\""                           << endl <<
+"          content=\"??????, APL, GNU\">"                           << endl <<
+" </head>"                                                          << endl <<
+" <body><pre>"                                                      << endl <<
+"⍝"                                                                 << endl <<
+"⍝ Author:      ??????"                                             << endl <<
+"⍝ Date:        " << time.year << "-"
+                  << time.month << "-"
+                  << time.day                                       << endl <<
+"⍝ Copyright:   Copyright (C) " << time.year << " by ??????"        << endl <<
+"⍝ License:     GPL see http://www.gnu.org/licenses/gpl-3.0.en.html"<< endl <<
+"⍝ Support email: ??????@??????"                                    << endl <<
+"⍝ Portability:   L3 (GNU APL)"                                     << endl <<
+"⍝"                                                                 << endl <<
+"⍝ Purpose:"                                                        << endl <<
+"⍝ ??????"                                                          << endl <<
+"⍝"                                                                 << endl <<
+"⍝ Description:"                                                    << endl <<
+"⍝ ??????"                                                          << endl <<
+"⍝"                                                                 << endl <<
+                                                                       endl;
+        }
+     else
+        {
+          outf << "#!" << LibPaths::get_APL_bin_path()
+               << "/" << LibPaths::get_APL_bin_name()
+               << " --script" << endl;
+        }
+
+     *sout << " ⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝" << endl
           << "⍝" << endl
           << "⍝ " << wname << " "
           << setfill('0') << time.year  << "-"
@@ -894,18 +908,35 @@ ofstream outf(filename.c_str(), ofstream::out);
 
 int function_count = 0;
 int variable_count = 0;
-   the_workspace.symbol_table.dump(outf, function_count, variable_count);
+   the_workspace.symbol_table.dump(*sout, function_count, variable_count);
 
    // system variables
    //
-#define ro_sv_def(x, txt)
-#define rw_sv_def(x, txt) if (ID:: x != ID::Quad_SYL) { get_v_ ## x().dump(outf);   ++variable_count; }
+#define ro_sv_def(x, _str, _txt)
+#define rw_sv_def(x, _str, _txt) if (ID:: x != ID::Quad_SYL) \
+   { get_v_ ## x().dump(*sout);   ++variable_count; }
 #include "SystemVariable.def"
 
-   COUT << "DUMPED WORKSPACE '" << wname << "'" << endl
-        << " TO FILE '" << filename << "'" << endl
-        << " (" << function_count << " FUNCTIONS, " << variable_count
-        << " VARIABLES)" << endl;
+   if (html)   outf << endl << "⍝ EOF </pre></body></html>" << endl;
+
+   if (silent)
+      {
+        out << setfill('0') << time.year        << "-"
+            << setw(2)      << time.month       << "-"
+            << setw(2)      << time.day         << " "
+            << setw(2)      << time.hour        << ":"
+            << setw(2)      << time.minute      << ":"
+            << setw(2)      << time.second      << " (GMT"
+            << tz_sign      << offset/3600 << ")"
+            << setfill(' ') <<  endl;
+      }
+   else
+      {
+        out << "DUMPED WORKSPACE '" << wname << "'" << endl
+            << " TO FILE '" << filename << "'" << endl
+            << " (" << function_count << " FUNCTIONS, " << variable_count
+            << " VARIABLES)" << endl;
+      }
 }
 //-----------------------------------------------------------------------------
 // )LOAD WS, set ⎕LX of loaded WS on success
@@ -936,18 +967,23 @@ XML_Loading_Archive in(filename.c_str(), dump_fd);
         Log(LOG_command_IN)   out << "LOADING " << wname << " from file '"
                                   << filename << "' ..." << endl;
 
-        load_DUMP(out, filename, dump_fd, true);   // closes dump_fd
+        load_DUMP(out, filename, dump_fd, do_LX, silent, 0);   // closes dump_fd
 
-        // )DUMP files have no )WSID so create on from the filename
+        // )DUMP files have no )WSID so create one from the filename
         //
         const char * wsid_start = strrchr(filename.c_str(), '/');
         if (wsid_start == 0)   wsid_start = filename.c_str();
         else                   ++wsid_start;   // skip /
-        const char * wsid_end = filename.c_str() + filename.size() - 4;
+        const char * wsid_end = filename.c_str() + filename.size();
+        if (wsid_end > (wsid_start - 4) &&
+           wsid_end[-4] == '.' &&
+           wsid_end[-3] == 'a' &&
+           wsid_end[-2] == 'p' &&
+           wsid_end[-1] == 'l')   wsid_end -= 4;   // skip .apl extension
         const UTF8_string wsid_utf8((const UTF8 *)wsid_start,
                                     wsid_end - wsid_start);
         const UCS_string wsid_ucs(wsid_utf8);
-        wsid(out, wsid_ucs);
+        wsid(out, wsid_ucs, true);
 
         // we cant set ⎕LX because it was not executed yet.
         return;
@@ -1016,7 +1052,8 @@ int dump_fd = -1;
 XML_Loading_Archive in(filename.c_str(), dump_fd);
    if (dump_fd != -1)
       {
-        load_DUMP(out, filename, dump_fd, false);   // closes dump_fd
+        load_DUMP(out, filename, dump_fd, no_LX, false, &lib_ws_objects);
+        // load_DUMP closes dump_fd
         return;
       }
 
@@ -1036,7 +1073,7 @@ XML_Loading_Archive in(filename.c_str(), dump_fd);
 }
 //-----------------------------------------------------------------------------
 void
-Workspace::wsid(ostream & out, UCS_string arg)
+Workspace::wsid(ostream & out, UCS_string arg, bool silent)
 {
    while (arg.size() && arg[0] <= ' ')       arg.remove_front();
    while (arg.size() && arg.back() <= ' ')   arg.erase(arg.size() - 1);
@@ -1056,7 +1093,7 @@ Workspace::wsid(ostream & out, UCS_string arg)
            }
       }
 
-   out << "WAS " << the_workspace.WS_name << endl;
+   if (!silent)   out << "WAS " << the_workspace.WS_name << endl;
    the_workspace.WS_name = arg;
 }
 //-----------------------------------------------------------------------------
