@@ -2,7 +2,7 @@
     This file is part of GNU APL, a free implementation of the
     ISO/IEC Standard 13751, "Programming Language APL, Extended"
 
-    Copyright (C) 2008-2015  Dr. Jürgen Sauermann
+    Copyright (C) 2008-201c  Dr. Jürgen Sauermann
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,43 +32,55 @@
 #include "Common.hh"
 #include "IO_Files.hh"
 #include "LibPaths.hh"
-#include "LineInput.hh"
-#include "Logging.hh"
+#include "Macro.hh"
 #include "makefile.h"
 #include "Output.hh"
 #include "NativeFunction.hh"
-#include "Parallel.hh"
-#include "Prefix.hh"
-#include "ProcessorID.hh"
-#include "Quad_SVx.hh"
-#include "ValueHistory.hh"
 #include "Workspace.hh"
 #include "UserPreferences.hh"
-#include "Value.icc"
 
-#ifndef PATH_MAX
-#define PATH_MAX 4096
-#endif
+/** @file **/
 
+/** \mainpage GNU APL
+
+   GNU APL is a free interpreter for the programming language APL.
+
+   GNU APL tries to be compatible with both the \b ISO \b standard \b 13751
+   (aka. Programming Language APL, Extended) and to \b IBM \b APL2.
+
+   It is \b NOT meant to be a vehicle for implementing new features to the
+   APL language.
+ **/
+
+/// when this file  was built
 static const char * build_tag[] = { BUILDTAG, 0 };
 
 //-----------------------------------------------------------------------------
 
+/// old sigaction argument for ^C
 static struct sigaction old_control_C_action;
+
+/// new sigaction argument for ^C
 static struct sigaction new_control_C_action;
 
 //-----------------------------------------------------------------------------
+/// old sigaction argument for segfaults
 static struct sigaction old_SEGV_action;
+
+/// new sigaction argument for segfaults
 static struct sigaction new_SEGV_action;
 
+/// signal handler for segfaults
 static void
 signal_SEGV_handler(int)
 {
    CERR << "\n\n====================================================\n"
            "SEGMENTATION FAULT" << endl;
 
+#if PARALLEL_ENABLED
    CERR << "thread: " << (const void *)pthread_self() << endl;
    Thread_context::print_all(CERR);
+#endif // PARALLEL_ENABLED
 
    Backtrace::show(__FILE__, __LINE__);
 
@@ -77,22 +89,29 @@ signal_SEGV_handler(int)
    // count errors
    IO_Files::assert_error();
 
-     Command::cmd_OFF(3);
+   Command::cmd_OFF(3);
 }
 //-----------------------------------------------------------------------------
+/// old sigaction argument for SIGUSR1
+static struct sigaction old_USR1_action;
+
+/// new sigaction argument for SIGUSR1
+static struct sigaction new_USR1_action;
+
+/// signal handler for SIGUSR1
 static void
 signal_USR1_handler(int)
 {
    CERR << "Got signal USR1" << endl;
 }
-
-static struct sigaction old_USR1_action;
-static struct sigaction new_USR1_action;
-
 //-----------------------------------------------------------------------------
+/// old sigaction argument for SIGTERM
 static struct sigaction old_TERM_action;
+
+/// new sigaction argument for SIGTERM
 static struct sigaction new_TERM_action;
 
+/// signal handler for SIGTERM
 static void
 signal_TERM_handler(int)
 {
@@ -101,19 +120,29 @@ signal_TERM_handler(int)
    raise(SIGTERM);
 }
 //-----------------------------------------------------------------------------
+#if PARALLEL_ENABLED
+/// old sigaction argument for ^\,
 static struct sigaction old_control_BSL_action;
+
+/// new sigaction argument for ^\.
 static struct sigaction new_control_BSL_action;
 
+/// signal handler for ^\.
 static void
 control_BSL(int sig)
 {
    CERR << endl << "^\\" << endl;
    Thread_context::print_all(CERR);
 }
+#endif // PARALLEL_ENABLED
 //-----------------------------------------------------------------------------
+/// old sigaction argument for SIGHUP
 static struct sigaction old_HUP_action;
+
+/// new sigaction argument for SIGHUP
 static struct sigaction new_HUP_action;
 
+/// new signal handler for SIGHUP
 static void
 signal_HUP_handler(int)
 {
@@ -122,6 +151,7 @@ signal_HUP_handler(int)
    raise(SIGHUP);
 }
 //-----------------------------------------------------------------------------
+/// print argc and argv[]
 static void
 show_argv(int argc, const char ** argv)
 {
@@ -149,7 +179,7 @@ const char * lines[] =
   ""                                                                      ,
   c1                                                                      ,
   ""                                                                      ,
-  "Copyright (C) 2008-2015  Dr. Jürgen Sauermann"                         ,
+  "Copyright (C) 2008-2016  Dr. Jürgen Sauermann"                         ,
   "Banner by FIGlet: www.figlet.org"                                      ,
   ""                                                                      ,
   "This program comes with ABSOLUTELY NO WARRANTY;"                       ,
@@ -183,6 +213,7 @@ const int left_pad = (80 - len)/2;
        }
 }
 //-----------------------------------------------------------------------------
+/// initialize the interpreter
 int
 init_apl(int argc, const char * argv[])
 {
@@ -207,35 +238,40 @@ const bool log_startup = uprefs.parse_argv_1();
    uprefs.read_threshold_file(true,  log_startup);  // dito parallel_thresholds
    uprefs.read_threshold_file(false, log_startup);  // dito parallel_thresholds
 
-   // struct sigaction differs between GNU/Linux and other systems, which
-   // causes direct bracket assignment to not compile on some machines.
+   // struct sigaction differs between GNU/Linux and other systems, which causes
+   // compile errors for direct curly bracket assignment on some systems
    //
    // We therefore memset everything to 0 and then set the handler (which
-   // should be compatible on GNU/Linux and other systems.
+   // should compile on GNU/Linux and also on other systems.
    //
-   memset(&new_control_C_action,   0, sizeof(struct sigaction));
-   memset(&new_USR1_action,        0, sizeof(struct sigaction));
-   memset(&new_SEGV_action,        0, sizeof(struct sigaction));
-   memset(&new_TERM_action,        0, sizeof(struct sigaction));
-   memset(&new_control_BSL_action, 0, sizeof(struct sigaction));
-   memset(&new_HUP_action,         0, sizeof(struct sigaction));
+   memset(&new_control_C_action, 0, sizeof(struct sigaction));
+   memset(&new_USR1_action,      0, sizeof(struct sigaction));
+   memset(&new_SEGV_action,      0, sizeof(struct sigaction));
+   memset(&new_TERM_action,      0, sizeof(struct sigaction));
+   memset(&new_HUP_action,       0, sizeof(struct sigaction));
 
    new_control_C_action.sa_handler = &control_C;
-   new_control_BSL_action.sa_handler = &control_BSL;
-   new_USR1_action .sa_handler = &signal_USR1_handler;
-   new_SEGV_action .sa_handler = &signal_SEGV_handler;
-   new_TERM_action .sa_handler = &signal_TERM_handler;
-   new_HUP_action  .sa_handler = &signal_HUP_handler;
+   new_USR1_action     .sa_handler = &signal_USR1_handler;
+   new_SEGV_action     .sa_handler = &signal_SEGV_handler;
+   new_TERM_action     .sa_handler = &signal_TERM_handler;
+   new_HUP_action      .sa_handler = &signal_HUP_handler;
 
-   sigaction(SIGINT,  &new_control_C_action,   &old_control_C_action);
-   sigaction(SIGQUIT, &new_control_BSL_action, &old_control_BSL_action);
-   sigaction(SIGUSR1, &new_USR1_action,        &old_USR1_action);
-   sigaction(SIGSEGV, &new_SEGV_action,        &old_SEGV_action);
-   sigaction(SIGTERM, &new_TERM_action,        &old_TERM_action);
+   sigaction(SIGINT,  &new_control_C_action, &old_control_C_action);
+   sigaction(SIGUSR1, &new_USR1_action,      &old_USR1_action);
+   sigaction(SIGSEGV, &new_SEGV_action,      &old_SEGV_action);
+   sigaction(SIGTERM, &new_TERM_action,      &old_TERM_action);
    sigaction(SIGHUP,  &new_HUP_action,       &old_HUP_action);
 
+#if PARALLEL_ENABLED
+   memset(&new_control_BSL_action, 0, sizeof(struct sigaction));
+   new_control_BSL_action.sa_handler = &control_BSL;
+   sigaction(SIGQUIT, &new_control_BSL_action, &old_control_BSL_action);
+#endif
+
    uprefs.parse_argv_2(log_startup);
 
+   Macro::init_macros();
+
    if (uprefs.emacs_mode)
       {
         UCS_string info;
@@ -360,6 +396,7 @@ const bool log_startup = uprefs.parse_argv_1();
    return 0;
 }
 //-----------------------------------------------------------------------------
+/// dito.
 int
 main(int argc, const char *argv[])
 {
